#!/usr/bin/perl
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


  use strict;
  use Date::Parse;
  use Math::BigInt;
  use File::Compare;
  use File::Copy;

  # set unbuffered stdout
  $| = 1;

  # get cmd line params
  my $version="$ENV{version}";
  my $hea = $ENV{HEA};
  my $hmc_user = $ENV{HMC_USER};
  my $webdir = $ENV{WEBDIR};
  my $bindir = $ENV{BINDIR};
  my $basedir = $ENV{INPUTDIR};
  my $tmpdir = "$basedir/tmp";
  if (defined $ENV{TMPDIR}) {
    $tmpdir = $ENV{TMPDIR};
  }
  my $rrdtool = $ENV{RRDTOOL};
  my $DEBUG = $ENV{DEBUG};
  my $pic_col = $ENV{PICTURE_COLOR};
  my $STEP = $ENV{SAMPLE_RATE};
  my $HWINFO = $ENV{HWINFO};
  my $managed_systems_exclude =  $ENV{MANAGED_SYSTEMS_EXCLUDE};
  my $SYS_CHANGE = $ENV{SYS_CHANGE};
  my $STEP_HEA=$ENV{STEP_HEA};;
  my $upgrade=$ENV{UPGRADE};;
  my $new_change="$tmpdir/$version-run";

  my $wrkdir = "$basedir/data"; 

  if ( -f "$bindir/premium.pl" ) {
    require "$bindir/premium.pl";
  }
  else {
    require "$bindir/standard.pl";
  }

  # Global definitions
  my $type_sam = "";
  my $step = "";
  my $YEAR_REFRESH = 72000 ;  # 20 hour, minimum time in sec when yearly graphs are updated (refreshed)
  my $MONTH_REFRESH = 20000 ; # 5 hour, minimum time in sec when monthly graphs are updated (refreshed)
  my $WEEK_REFRESH = 7000;    # minimum time in sec when weekly  graphs are updated (refreshed)
  my $PARALLELIZATION = 5;
  my @lpar_trans = "";  # lpar translation names/ids for IVM systems

  if ( $upgrade == 1 ) {
    $YEAR_REFRESH = 0;  # to force creation of trend graphs
  }

  # 48 colors like for HMC bellow + 216 basic HTML colors x 6 == 1296 + 48
my @color=("#FF0000","#0000FF","#00FF00","#FFFF00","#FF3399","#00FFFF","#999933","#0099CC","#3300CC",
"#804000", "#004000", "#004040", "#000080", "#000040", "#400040", "#400080", "#000000", "#808000", "#808040", "#808080",
"#408080", "#C0C0C0", "#400040", "#AAAAAA",
"#CC0000","#FF0000", "#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#000000","#FFFF00","#333300","#FFFFFF","#330000","#999990","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600");

  my @color_lpar="";
  my $color_indx = 0;



  my $lpar_v = premium ();
  print "LPAR2RRD custom $lpar_v version $version\n" if $DEBUG ;

  my $act_time_txt = localtime();
  my $act_time = time();
  my $step_new = 60; # -PH solve it somehow


  if ( ! -d "$webdir" ) {
     error("Pls set correct path to Web server pages, it does not exist here: $webdir".__FILE__.":".__LINE__) && return 0;
  }

  # defining global variables
  my $pool_suffix = "__pool__";
  my @cfg_list = "";
  my @groups = ""; # uniq list of group
  my @groups_type = ""; # LPAR/POOL
  my @groups_server = "";
  my @groups_lpar = "";
  my @groups_pool = "";
  my @groups_name = "";
  my @lpar_all_list = ""; # list of all lpars under data/*
  my @lpar_all_list_org = ""; # list of all lpars under data/*, complete path include suffix
  my @pool_all_list = ""; # list of all pools under data/*
  my @pool_all_list_org = ""; # list of all pools under data/*
  my @server_list_all = ""; # list of all server
  my @lpars_graph = "";
  my @lpars_graph_name = "";
  my @lpars_graph_server = "";
  my @lpars_graph_type = ""; # LPAR/POOL
  my $lpar_graph_indx = 0;
  my $search_all_lpars=0;

  my $cfg_update_time = cfg_update();

  my $ret = cfg_load($basedir);
  if ( $ret == 0 ) {
    # no groups are configurd, exiting
    print "No custom groups configured\n";
    exit (0);
  }

  # start RRD via a pipe
  use RRDp;
  RRDp::start "$rrdtool";

  find_rrd_files();

  # close RRD pipe
  RRDp::end;

  exit (0);


sub cfg_update {
  my $cfg = "$basedir/etc/custom_groups.cfg";

  if  ( ! -f $cfg ) {
    # cfg does not exist
    error ("custom : custom cfg files does not exist: $cfg");
    exit 1;
  }

  my $png_time = (stat("$cfg"))[9];

  return $png_time;
}


sub cfg_load {
  my $basedir = shift;
  my $cfg = "$basedir/etc/custom_groups.cfg";
  my $cfg_indx = 0;
  my $groups_indx = 0;
  my $groups_name_indx = 0;

  if  ( ! -f $cfg ) {
    # cfg does not exist
    error ("custom : custom cfg files does not exist: $cfg");
    exit 1;
  }

  my $group_prev = "";

  open (FHR, "< $cfg");

  foreach my $line (<FHR>) {
    chomp ($line);
    $line =~ s/\\:/===========doublecoma=========/g;  # workround for lpars/pool/groups with double coma inside the name
    $line =~ s/ *$//g; # delete spaces at the end
    if ( $line =~ m/^$/ || ($line !~ m/^POOL/ && $line !~ m/^LPAR/) || $line =~ m/^#/ || $line !~ m/:/ || $line =~ m/:$/ || $line =~ m/: *$/) {
      next;
    }
    #print "99 $line\n";
    # place cfg into an array
    $cfg_list[$cfg_indx] = $line;
    $cfg_indx++;
    # --> list of groups here
    (my $type, my $server, my $name, my $group_act ) = split(/:/,$line);
    if ( $type eq '' || $server eq '' || $name eq '' || $group_act eq '' ) {
      error ("custom : syntax issue in $cfg: $line");
      next; 
    }
    $server =~ s/===========doublecoma=========/:/g;
    $name =~ s/===========doublecoma=========/:/g;
    $name =~ s/\//\&\&1/g;
    $group_act =~ s/===========doublecoma=========/:/g;
    if ( $group_prev eq '' || $group_prev !~ m/^$group_act$/ ) {
      # create list of groups
      my $exist=0;
      foreach my $gr (@groups) {
        if ( $gr =~ m/^$group_act$/ ) {
          $exist=1;
          last;
        }
      }
      if ( $exist == 0 ) {
        $groups[$groups_name_indx] = $group_act;
        $group_prev = $group_act;
        $groups_name_indx++;
      }
    } 
    $groups_type[$groups_indx] = $type;
    $groups_server[$groups_indx] = $server;
    $groups_lpar[$groups_indx] = $name;
    $groups_name[$groups_indx] = $group_act;
    #print "18 $groups_type[$groups_indx] $groups_server[$groups_indx] $groups_lpar[$groups_indx] $groups_name[$groups_indx]\n";
    $groups_indx++;
  }
  close (FHR);

  #foreach my $a (@groups) {
  #  print "66 $a\n";
  #}

  return $cfg_indx;
}


sub load_all_lpars {
  my $lpar_indx = 0;
  my $pool_indx = 0;
  my $server_indx = 0;

  # goes through all hmc/server and find all lpars
  foreach my $server (<$wrkdir/*>) {
    if ( -l "$server" ) {
      # avoid symlinks
      next;
    }
    if ( -f "$server" ) {
      # avoid regular files
      next;
    }


    # open first available pool mapping file
    my @map = "";
    my $map_file = "";
    my $map_file_final = "";
    my $map_file_tstamp = 0;

    my $server_space = $server;
    if ( $server =~ m/ / ) { # workaround for server name with a space inside, nothing else works, grrr
      $server_space = "\"".$server."\"";
    }

    foreach my $map_file (<$server_space/*/cpu-pools-mapping.txt>) {
      my $tstamp = (stat("$map_file"))[9];
      if ( $tstamp > $map_file_tstamp ) {
        $map_file_tstamp = $tstamp;
        $map_file_final = $map_file;
      }
    }
    if ( $map_file_tstamp > 0 ) {
      open (FHP,"< $map_file_final");
      @map = <FHP>;
      close (FHP);
    }

    foreach my $lpar_fullp (<$server_space/*/*rrm>) {
      my $pool=0;
      my $lpar_fullp_org = $lpar_fullp; # keep original path 
      $lpar_fullp =~ s/\.rrm//;
      $lpar_fullp =~ s/\.rrh//;

      if ( $lpar_fullp =~ m/SharedPool0$/ || $lpar_fullp =~ m/\/mem$/ || $lpar_fullp =~ m/\/mem-pool$/ || $lpar_fullp =~ m/\/cod$/ ) {
        next; #exclude SharedPool0 and memory and CoD
      }

      # translate shared pool names 
      if ( $lpar_fullp =~ m/SharedPool[1-9]/ ) {
        # shared pools
        $pool=1;

        # basename without direct function
        my $sh_pool_in = $lpar_fullp;
        my @link_base = split(/\//,$sh_pool_in);
        foreach my $m (@link_base) {
          $sh_pool_in = $m; 
        }
        $lpar_fullp =~ s/$sh_pool_in$//; # dirname
        $sh_pool_in =~ s/^SharedPool//;

        my $found=0;
        my $map_rows=0;
        foreach my $line (@map) {
          chomp ($line);
          $map_rows++;
          if ( $line !~ m/^[0-9].*,/ ) {
            #something wrong , ignoring
    	    next;
          }
          (my $pool_indx_new, my $pool_name_new) = split(/,/,$line);
          if ($pool_indx_new == $sh_pool_in ) {
            $lpar_fullp .= $pool_name_new; 
            $found++;
            last;
          }
        }
        if ($found == 0 ) {
          if ($map_rows > 0 ) {
            print "cgraph error   : could not found name for shared pool : $lpar_fullp/SharedPool$sh_pool_in\n";
            #error ("custom : could not found name for shared pool : $lpar_fullp/SharedPool$sh_pool_in");
            $lpar_fullp .= "SharedPool$sh_pool_in";
          }
          else {
            print "cgraph error   : Pool mapping table is either empty or does not exist : $map_file_final\n";
            #error ("custom : Pool mapping table is either empty or does not exist : $map_file_final");
            $lpar_fullp .= "SharedPool$sh_pool_in";
          }
        }
      }

      # translate default pool
      if ( $lpar_fullp =~ m/\/pool$/ ) {
        $lpar_fullp =~ s/\/pool$/\/all_pools/;
        $pool=1;
      }  
 
      #print "$lpar_fullp\n";

      if ( $pool == 1 ) {
        # it is a pool
        $pool_all_list[$pool_indx] = $lpar_fullp;
        $pool_all_list[$pool_indx] =~ s/$wrkdir//g;
        $pool_all_list_org[$pool_indx] = $lpar_fullp_org;
        $pool_all_list_org[$pool_indx] =~ s/$wrkdir//g;
        $pool_indx++;
      }
      else {
        # place lpars into an array
        #print "099 $lpar_indx: $lpar_fullp_org - $lpar_fullp\n";
        $lpar_all_list_org[$lpar_indx] = $lpar_fullp_org;
        $lpar_all_list_org[$lpar_indx] =~ s/$wrkdir//g;
        $lpar_all_list[$lpar_indx] = $lpar_fullp;
        $lpar_all_list[$lpar_indx] =~ s/$wrkdir//g;
        $lpar_indx++;
      }
      #print "33 $lpar_fullp\n";
    }
    $server_list_all[$server_indx] = $server;

    # basename without direct function
    my @link_l = split(/\//,$server_list_all[$server_indx]);
    foreach my  $m (@link_l) {
      $server_list_all[$server_indx] = $m; #lpar name, basename from $file
    }

    $server_indx++;

  }

  # sorting lpars/pools per lpar/pool due to dual HMC ....
  @lpar_all_list = sort { (split '/', $a)[3] cmp (split '/', $b)[3] } @lpar_all_list;
  @lpar_all_list_org = sort { (split '/', $a)[3] cmp (split '/', $b)[3] } @lpar_all_list_org;
  @pool_all_list = sort { (split '/', $a)[3] cmp (split '/', $b)[3] } @pool_all_list;
  @pool_all_list_org = sort { (split '/', $a)[3] cmp (split '/', $b)[3] } @pool_all_list_org;
  @server_list_all = sort { lc $a cmp lc $b } @server_list_all;

  #foreach my $a (@lpar_all_list) {
  #  print "001 $a\n";
  #}
  return 1;
}

sub find_rrd_files {
  my $group_type_prev = "";

  foreach my $group (@groups) {
    #print "G: $group\n";
    my $indx = 0;
    @lpars_graph = "";
    @lpars_graph_name = "";
    @lpars_graph_server = "";
    $lpar_graph_indx = 0; #must be global
    $group_type_prev = "";
    my @all_list = "";
    my @all_list_org = "";

    foreach my $group_act (@groups_name) {
      if ( $group_act !~ m/^$group$/ ) {
        $indx++;
        next;
      }

      # check if it is the same type LPAR/POOL, if note then return
      #print "00 $group_type_prev -- $groups_type[$indx] --\n";
      if ( ! $group_type_prev eq '' && $group_type_prev !~ m/$groups_type[$indx]/ ) {
        print "creating cgraph: group mismatch $group_act contains LPAR and POOL directives\n";
        my $indx_prev=$indx;
        $indx_prev--;
        error ("custom : group mismatch $group_act contains LPAR ($groups_lpar[$indx])and POOL ($groups_lpar[$indx]) directives");
        last;
      }
      $group_type_prev = $groups_type[$indx];

      #print "42 $groups_server[$indx] --  $group_act\n";
      #if ( $groups_server[$indx] =~ m/\*/ ) {
	# here it must find all server where particular lpar is running
        # and fill in lpar_graph table by by them
        my @server_list = "";

        # goes through all data and put into memory list of all rrm/h $lpar_all_list[] and $pool_all_list[]
        # do it just onece!!!
        if ( $search_all_lpars == 0) {
          $search_all_lpars++;
          load_all_lpars();
        }

        #print "43 $group_act\n";
        # goes through all servers
        foreach my $server (@server_list_all) {
          #print "44 $group_act : $server\n";
          if ( $server =~ m/^$groups_server[$indx]$/ ) {
            #print "45 $group_act : $server\n";

            my $lpar_prev = "";

            # set the source, pool or lpar
            if ( $groups_type[$indx] =~ m/POOL/ ) {
              @all_list = @pool_all_list;
              @all_list_org = @pool_all_list_org;
            }
            else {
              @all_list = @lpar_all_list;
              @all_list_org = @lpar_all_list_org;
            }

            # goes through all lpars/pools
            my $server_prev = ""; # dual HMC exclude ...
            my $lpar_line_prev = "";
            my $index_org = -1;
            foreach my $lpar_line_part (@all_list) {
              $index_org++;
#print "066 $index: $lpar_line_part : $all_list_org[$index] : $all_list[$index]\n";
              my $lpar_line = $wrkdir.$lpar_line_part;
              my $lpar_line_all = $lpar_line;
              #print "46 $lpar_line - $wrkdir/$server/*/$groups_lpar[$indx] - $group_act : $server\n";
              if ( $lpar_line =~ m/^$wrkdir\/$server\/.*\/$groups_lpar[$indx]$/ ) {
                #print "47 $lpar_line : $groups_server[$indx] ; $groups_lpar[$indx]\n";

                my $name_full = $lpar_line;
                # basename without direct function
                my $lpar_a = $name_full;
                my @base = split(/\//,$lpar_a);
                foreach my $m (@base) {
                  $lpar_a = $m; #lpar name, basename from $file
                }
               
                 
                $name_full =~ s/\/$lpar_a$//;
                # basename without direct function
                my $hmc_a = $name_full;
                @base = split(/\//,$hmc_a);
                foreach my $m (@base) {
                  $hmc_a = $m; #lpar name, basename from $file
                }
               
                $name_full =~ s/\/$hmc_a$//;
                # basename without direct function
                my $server_a = $name_full;
                @base = split(/\//,$server_a);
                foreach my $m (@base) {
                  $server_a = $m; #lpar name, basename from $file
                }
                
                # dual HMC exclude
                if ( ! $server_prev eq '' ) {
		  #print "48 $group : $server_a - $server_prev - $hmc_a - $lpar_a \n";
                  if ( ! $server_prev eq '' && $server_prev =~ m/^$server_a$/ && $lpar_prev =~ m/^$lpar_a$/ ) {
		    #print "49 $server_a - $hmc_a - $lpar_a \n";
 
                    # *_org keep original full path of the .rrm file
                    my $lpar_time = (stat("$wrkdir/$all_list_org[$index_org]"))[9];
                    my $lpar_prev_time = (stat("$wrkdir/$lpar_line_prev"))[9];
                    #print "000 $index_org: $lpar_line_prev : $lpar_time : $all_list_org[$index_org] : $lpar_prev_time \n";

		    # find out the HMC with most recent data, if it is the second one listed then change the previous record
                    if ( ! $lpar_prev_time eq '' && $lpar_time > $lpar_prev_time ) {
                      #print "001 change $lpar_line $lpar_line_prev : $lpar_a $server_a\n";
                      # replace previous record for dual HMC with the fresh one
                      $lpar_graph_indx--;
                      $lpars_graph[$lpar_graph_indx] = $lpar_line;
                      $lpars_graph_name[$lpar_graph_indx] = $lpar_a;
                      $lpars_graph_server[$lpar_graph_indx] = $server_a;
                      $lpars_graph_type[$lpar_graph_indx] = $groups_type[$indx];
                      $lpar_graph_indx++;

                      $lpar_line_prev = $all_list_org[$index_org]; # full original path
                      $lpar_prev=$lpar_a;
                      $server_prev=$server_a;
                    }
                    next;  # dual HMC??
                  }
                }

                #fill_in_lpar_table($indx,$group_type_prev,$server_a,$lpar_a);
		# fill in the array with list of lpars/pools prepared for graphing
                $lpars_graph[$lpar_graph_indx] = $lpar_line;
                $lpars_graph_name[$lpar_graph_indx] = $lpar_a;
                $lpars_graph_server[$lpar_graph_indx] = $server_a;
                $lpars_graph_type[$lpar_graph_indx] = $groups_type[$indx];
                $lpar_graph_indx++;

                $lpar_line_prev = $all_list_org[$index_org]; # full original path
                $lpar_prev=$lpar_a;
                $server_prev=$server_a;
              }
            }
          }
        }
      $indx++;
    }
    if ( ! $group_type_prev eq '' ) {
      if ( $lpar_graph_indx == 0 ) {
        # nothing has been found
        print "cgraph no found: could not identify any lpar or pool for group: \"$group\"\n";
        error ("custom group: could not identify any lpar or pool for group: \"$group\"");
      }
      else {
        #print "77 $group: $lpar_graph_indx\n";
        graph_it($group);
      }
    }
  }
  return 0;
}

sub graph_it {
  my $group = shift;
  my $type_sam = "m"; # temp

  if ( $type_sam =~ "d") {
    draw_graph ($group,"year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$type_sam);
  }
  else {
    draw_graph ($group,"day","d","MINUTE:60:HOUR:2:HOUR:4:0:%H",$type_sam);
    draw_graph ($group,"week","w","HOUR:8:DAY:1:DAY:1:86400:%a",$type_sam);
    draw_graph ($group,"4 weeks","m","DAY:1:DAY:2:DAY:7:0:%U",$type_sam);
    draw_graph ($group,"year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$type_sam);
  }
  return 0;
}

sub draw_graph {
  my $group    = shift;
  my $text     = shift;
  my $type     = shift;
  my $xgrid    = shift;
  my $type_sam = shift;
  my $name     = "$webdir/custom/$group/$type";

  if ( ! -d "$webdir/custom" ) {
     print "mkdir          : custom $webdir/custom\n" if $DEBUG ;
     mkdir("$webdir/custom", 0755) || error("Cannot mkdir $webdir/custom: $!".__FILE__.":".__LINE__) && return 0;
     touch ("$webdir/custom");
  }
  if ( ! -d "$webdir/custom/$group" ) {
     print "mkdir          : custom $webdir/custom/$group\n" if $DEBUG ;
     mkdir("$webdir/custom/$group", 0755) || error("Cannot mkdir $webdir/custom/$group: $!".__FILE__.":".__LINE__) && return 0;
     touch ("$webdir/custom/$group");
  }


  my $png_time = 0;
  if ( -f "$name.png" ) {
    $png_time = (stat("$name.png"))[9];
  }

  if ( $cfg_update_time > $png_time ) {
    # cfg has been changed since last actualization, refresh everything to keep custom page up-to-date
    print "cgraph         : custom graph cfg change, redraw everything\n" if $DEBUG ;
    $png_time = 0; # this does the job
    touch ("custom:draw_graph $cfg_update_time > $png_time");
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "y" && -f "$name.png" && $upgrade == 0 ) {
    if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
      print "creating cgraph: custom:$group:$type_sam:$type no update\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "m" && -f "$name.png" && $upgrade == 0 ) {
    if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
      print "creating cgraph: custom:$group:$type_sam:$type no update\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "w" && -f "$name.png" && $upgrade == 0 ) {
    if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
      print "creating cgraph: custom:$group:$type_sam:$type no update\n" if $DEBUG ;
      return 0;
    }
  }

  # Multiview chart
  print "creating cgraph: custom:$group:$type_sam:$type\n" if $DEBUG ;
  multiview($group,$name,$type,$type_sam,$act_time,$step_new,$text,$xgrid);

  print "creating cgraph: custom memory:$group:$type_sam:$type\n" if $DEBUG ;
  $name .= "-mem";
  multiview_mem($group,$name,$type,$type_sam,$act_time,$step_new,$text,$xgrid);
  print "creating cgraph: custom memory os :$group:$type_sam:$type\n" if $DEBUG ;
  $name .= "-os";
  multiview_mem($group,$name,$type,$type_sam,$act_time,$step_new,$text,$xgrid);

  return 0;
}


sub multiview {
  my $group = shift;
  my $name = shift;
  my $type = shift;
  my $type_sam = shift;
  my $act_time = shift ;
  my $step_new = shift;
  my $text = shift;
  my $xgrid = shift;
  my $req_time = 0;
  my $comm = "COMM ";
  $color_indx = 0; # clear color index
  my $list = "$webdir/custom/$group/list.txt";
  my $lim = ".li";
  my $updated = "";


  # open file for writing list of lpars
  open(FHL, "> $list") || error("Can't open $list : $!".__FILE__.":".__LINE__) && return 0;

  my $header = "Custom group $group: last $text";

  my $file = "";
  my $i = 0;
  my $lpar = "";
  my $lparn = "";
  my $cmd = "";
  my $cmd_sum = "";
  my $j = 0;

  if ( "$type" =~ "d" ) {
    $req_time = $act_time - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time - 2764800;
  }

  # $cmd_sum gets summary value 
  $cmd_sum .= "graph \\\"$tmpdir/name.png\\\"";
  $cmd_sum .= " --start now-1$type";
  $cmd_sum .= " --end now-1$type+1$type";
  my $cmd_sum_max = $cmd_sum;

  my $lim_org = $lim;
  $lim =~ s/\.//;
  $lim =~ s/i/l/;

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"CPU cores\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " COMMENT:\\\"Utilization in CPU cores                       \\l\\\"";
  if ( $lpars_graph_type[0] =~ /POOL/ ) {
    $cmd .= " COMMENT:\\\"  Server                       Pool       Max pool     avrg     max\\l\\\"";   
  }
  else {
    $cmd .= " COMMENT:\\\"  Server                       LPAR                    avrg     max\\l\\\"";   
  }


  my $gtype="AREA";
  my $index=0;
  my $sh_pools = 0; # identificator of shared pool
  my $pool_indx = 0;

  my @lpars_graph_local = @lpars_graph;

  # do not sort it HERE!! there is several arrays, sorting lpars_graph is not enough!!! --PH
  foreach my $file (@lpars_graph_local) {

    if ( $file eq '' ) {
      $index++;
      next; #some trash
    }

    if ( $file !~ m/\.rr[m,h,d]$/ ) {
      # if pool the  translate back to full path rrm
      if ( $lpars_graph_type[$index] =~ /POOL/ ) {
        # translate default pool
        if ( $file =~ m/\/all_pools$/ ) {
          $file =~ s/\/all_pools$/\/pool/;
        }  
        else { # shared pools
  
          # translate shared pool names 
          # basename without direct function
          my $sh_pool_in = $file;
          my @link_base = split(/\//,$sh_pool_in);
          foreach my $m (@link_base) {
            $sh_pool_in = $m; 
          }
          $file =~ s/\/$sh_pool_in$//; # dirname
          $sh_pool_in =~ s/^SharedPool//;
     
          # open available pool mapping file
          my @map = "";
          my $map_file_final = "";

          my $file_space = $file;
          if ( $file =~ m/ / ) { # workaround for server name with a space inside, nothing else works, grrr
            $file_space = "\"".$file."\"";
          }

          foreach my $map_file (<$file_space/cpu-pools-mapping.txt>) {
            $map_file_final = $map_file;
            open (FHP,"< $map_file");
            @map = <FHP>;
            close (FHP);
            last;
          }
  
          my $found=0;
          my $map_rows=0;
          foreach my $line (@map) {
            chomp ($line);
            $map_rows++;
            if ( $line !~ m/^[0-9].*,/ ) {
              #something wrong , ignoring
              next;
            }
            (my $pool_indx_new, my $pool_name_new) = split(/,/,$line);
            if ($pool_name_new =~ m/^$sh_pool_in$/ ) {
              $file .= "/SharedPool".$pool_indx_new; 
              $found++;
              last;
            }
          }
          if ($found == 0 ) {
            if ($map_rows > 0 ) {
              print "cgraph error 1 : could not found name for shared pool : $file/SharedPool$sh_pool_in\n";
              error ("custom 1: could not found name for shared pool : $file/SharedPool$sh_pool_in");
              next;
            }
            else {
              print "cgraph error 1 : Pool mapping table is either empty or does not exist : $map_file_final\n";
              error ("custom 1: Pool mapping table is either empty or does not exist : $map_file_final");
              next;
            }
          }
        }
      }

      my $file_org = $file;
      $file .= ".rrm";
      if ( ! -f $file ) {
        $file = $file_org;
        $file =~ s/\.rrm$/\.rrh/;
        #$file .= ".rrh";
      }
    }
    if ( "$type" =~ "d" ) {
      my $file_org = $file;
      if ($file !~ m/\.rr[m,d]$/ ) {
        $file .= ".rrd";
        if ( ! -f $file ) {
          $file = $file_org;
          $file =~ s/\.rrd$/\.rrm/;
          $file =~ s/\.rrh$/\.rrm/;
          #$file .= ".rrm";
        }
      }
    }

    if ( ! -f $file ) {
      #print "cgraph error   : could not found rrdtool file for: $file : $type\n";
      #error ("custom : could not found rrdtool file for: $file : $type");
      #if ( $file =~ m/^\.rr[h|m|d]\.rr/ ) {
	# just some workaround to delete files .rr* which time to time are created in the server dir
      #  $file =~ s/\.rr[h|d|m]$//;
      #  print "cgraph error   : removing $file : $type\n";
      #  error ("custom : removing $file : $type");
      #  unlink ($file);
      #}
      $index++;
      next;
    }
    #print "12 $file $lpars_graph_type[$index]\n";

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = (stat("$file"))[9];
    if ( $rrd_upd_time < $req_time ) {
      $index++;
      next;
    }

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar = $lpars_graph_server[$index];
    $lpar =~ s/\&\&1/\//g;
    for(my $k = length($lpar); $k < 25; $k++) {
      $lpar .= " ";
    }
    $lparn .= " ";
    $lpar .= "   ";
    $lpar .= $lpars_graph_name[$index];
    for(my $k = length($lpar); $k < 50; $k++) {
      $lpar .= " ";
    }

    print FHL "$file\n";

    #print "11 $file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 1);

    if ( (length($lpar_v) + 1 ) == length($comm) && length($lparn) == length($comm) ) {
      copy("$basedir/html/$lim_org", "$webdir/custom/$group/$lim");
      last;
    }

    if ($type =~ "d" && $i == 0 ) {
      RRDp::cmd qq(last "$file");
      my $last_tt = RRDp::read;
      chomp ($$last_tt);
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $updated =" COMMENT:\\\"  Updated\\\: $l \\\"";
    }


    # bulid RRDTool cmd
    if ( $lpars_graph_type[$index] =~ /LPAR/ ) {
      $cmd .= " DEF:cap${i}=\\\"$file\\\":capped_cycles:AVERAGE";
      $cmd .= " DEF:uncap${i}=\\\"$file\\\":uncapped_cycles:AVERAGE";
      $cmd .= " DEF:ent${i}=\\\"$file\\\":entitled_cycles:AVERAGE";
      $cmd .= " DEF:cur${i}=\\\"$file\\\":curr_proc_units:AVERAGE";
  
      $cmd .= " CDEF:tot${i}=cap${i},uncap${i},+";
      $cmd .= " CDEF:util${i}=tot${i},ent${i},/";
      $cmd .= " CDEF:utiltotu${i}=util${i},cur${i},*";
      $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";

      $cmd_sum .= " DEF:cap${i}=\\\"$file\\\":capped_cycles:AVERAGE";
      $cmd_sum .= " DEF:uncap${i}=\\\"$file\\\":uncapped_cycles:AVERAGE";
      $cmd_sum .= " DEF:ent${i}=\\\"$file\\\":entitled_cycles:AVERAGE";
      $cmd_sum .= " DEF:cur${i}=\\\"$file\\\":curr_proc_units:AVERAGE";
  
      $cmd_sum .= " CDEF:tot${i}=cap${i},uncap${i},+";
      $cmd_sum .= " CDEF:util${i}=tot${i},ent${i},/";
      $cmd_sum .= " CDEF:utiltotu${i}=util${i},cur${i},*";
      $cmd_sum .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";
      $cmd .= " $gtype:utiltot${i}$color[$color_indx]:\\\"$lpar\\\"";
    }
    else { 
      # POOLs
      my $lpar_cut = substr($lpar,0,35);
      $sh_pools = 1;
      if ( $file =~ /SharedPool[1-9]/ ) {
        # Shared pools 1 - X
        $cmd .= " DEF:max${i}=\\\"$file\\\":max_pool_units:AVERAGE";
        $cmd .= " DEF:totcyc${i}=\\\"$file\\\":total_pool_cycles:AVERAGE";
        $cmd .= " DEF:uticyc${i}=\\\"$file\\\":utilized_pool_cyc:AVERAGE";

        $cmd .= " CDEF:cpuutil${i}=uticyc${i},totcyc${i},/";
        $cmd .= " CDEF:utiltotu${i}=cpuutil${i},max${i},*";
        $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";

        $cmd_sum .= " DEF:max${i}=\\\"$file\\\":max_pool_units:AVERAGE";
        $cmd_sum .= " DEF:totcyc${i}=\\\"$file\\\":total_pool_cycles:AVERAGE";
        $cmd_sum .= " DEF:uticyc${i}=\\\"$file\\\":utilized_pool_cyc:AVERAGE";

        $cmd_sum .= " CDEF:cpuutil${i}=uticyc${i},totcyc${i},/";
        $cmd_sum .= " CDEF:utiltotu${i}=cpuutil${i},max${i},*";
        $cmd_sum .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";
        $cmd_sum_max .= " DEF:max${i}=\\\"$file\\\":max_pool_units:AVERAGE";
        $cmd_sum_max .= " CDEF:maxnull${i}=max${i},UN,0,max${i},IF";

        # this must be here!!!
        #print "001 $lpar - $lpar_cut\n";
        $cmd .= " $gtype:utiltot${i}$color[$color_indx]:\\\"$lpar_cut    \\\"";
        $cmd .= " GPRINT:max${i}:AVERAGE:\\\"%3.0lf      \\\"";
        $cmd_sum_max .= " PRINT:max${i}:AVERAGE:\\\"%3.0lf  \\\"";
      }
      else {
        # default pool
        $cmd .= " DEF:totcyc${i}=\\\"$file\\\":total_pool_cycles:AVERAGE";
        $cmd .= " DEF:uticyc${i}=\\\"$file\\\":utilized_pool_cyc:AVERAGE";
        $cmd .= " DEF:cpu${i}=\\\"$file\\\":conf_proc_units:AVERAGE";
        $cmd .= " DEF:cpubor${i}=\\\"$file\\\":bor_proc_units:AVERAGE";

        $cmd .= " CDEF:totcpu${i}=cpu${i},cpubor${i},+";
        $cmd .= " CDEF:cpuutil${i}=uticyc${i},totcyc${i},/";
        $cmd .= " CDEF:utiltotu${i}=cpuutil${i},totcpu${i},*";
        $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";

        $cmd_sum .= " DEF:totcyc${i}=\\\"$file\\\":total_pool_cycles:AVERAGE";
        $cmd_sum .= " DEF:uticyc${i}=\\\"$file\\\":utilized_pool_cyc:AVERAGE";
        $cmd_sum .= " DEF:cpu${i}=\\\"$file\\\":conf_proc_units:AVERAGE";
        $cmd_sum .= " DEF:cpubor${i}=\\\"$file\\\":bor_proc_units:AVERAGE";

        $cmd_sum .= " CDEF:totcpu${i}=cpu${i},cpubor${i},+";
        $cmd_sum .= " CDEF:cpuutil${i}=uticyc${i},totcyc${i},/";
        $cmd_sum .= " CDEF:utiltotu${i}=cpuutil${i},totcpu${i},*";
        $cmd_sum .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";

        $cmd .= " $gtype:utiltot${i}$color[$color_indx]:\\\"$lpar_cut    \\\"";   # MAX allocated
        $cmd .= " GPRINT:totcpu${i}:AVERAGE:\\\"%3.0lf      \\\"";

        $cmd_sum_max .= " DEF:cpu${i}=\\\"$file\\\":conf_proc_units:AVERAGE";
        $cmd_sum_max .= " DEF:cpubor${i}=\\\"$file\\\":bor_proc_units:AVERAGE";
        $cmd_sum_max .= " CDEF:totcpu${i}=cpu${i},cpubor${i},+";
        $cmd_sum_max .= " PRINT:totcpu${i}:AVERAGE:\\\"%3.0lf  \\\"";
      }
    }

    # put carriage return after each second lpar in the legend
    if ($j == 1) {
      $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%5.2lf  \\\"";
      $cmd_sum .= " PRINT:utiltot${i}:AVERAGE:\\\"%5.2lf  \\\"";
      $cmd .= " GPRINT:utiltot${i}:MAX:\\\"%5.2lf \\l\\\"";
      $j = 0;
    }
    else {
      $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%5.2lf  \\\"";
      $cmd_sum .= " PRINT:utiltot${i}:AVERAGE:\\\"%5.2lf  \\\"";
      $cmd .= " GPRINT:utiltot${i}:MAX:\\\"%5.2lf \\l\\\"";
	# --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $j++
    }
    $gtype="STACK";
    $i++;
    $color_indx++;
    $index++;
    if ( ! get_lpar_num() && $index == 5 ) {
      last;
    }

  }
  $cmd_sum =~ s/\\"/"/g;

  close (FHL);
  my $FH;
  my $tmp_file_sum="$tmpdir/custom-group-$group-$type-sum.cmd";

  # Find out total value
  open(FH, "> $tmp_file_sum") || error("Can't open $tmp_file_sum : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd_sum\n";

  $ret  = `$rrdtool - < "$tmp_file_sum" 2>&1`;
  #print "000 $ret\n";
  my $total = 0;
  foreach my $ret_line (split(/\n/,$ret)) {
    chomp($ret_line);
    #print "110 $ret_line\n";
    if ( $ret_line =~ m/:/ || $ret_line =~ m/x/ ) {
      next;
    }
    $ret_line =~ s/0x0//g;
    $ret_line =~ s/ //g;
    $ret_line =~ s/OK.*$//g;
    chomp($ret_line); # must be here as well !!
    if ( $ret_line eq '' ) {
      next;
    }
    $ret_line =~ s/ //g;
    my $ret_digit = isdigit($ret_line);
    #print "111 $ret_line - ret_digit = $ret_digit\n";
    if ( $ret_digit == 0 ) {
      next;
    }
    $total = $total + $ret_line;
    #print "112 $ret_line : $total \n";
  }
  #print "113 $total \n";
  close (FH);


  # Find out total value of max_pool_units for Pools
  #print "109 $sh_pools\n";
  my $total_max = 0;
  if ( $sh_pools == 1 ) {
    $cmd_sum_max =~ s/\\"/"/g;
    open(FH, "> $tmp_file_sum") || error("Can't open $tmp_file_sum : $!".__FILE__.":".__LINE__) && return 0;
    print FH "$cmd_sum_max\n";

    $ret  = `$rrdtool - < "$tmp_file_sum" 2>&1`;
    foreach my $ret_line (split(/\n/,$ret)) {
      chomp($ret_line);
      #print "110 $ret_line\n";
      if ( $ret_line =~ m/:/ ) {
        next;
      }
      $ret_line =~ s/0x0//g;
      if ( $ret_line eq '' ) {
        next;
      }
      #my $len = length($ret_line);
      #my $digit = substr($ret_line,1,$len);
      my $digit = $ret_line;
      if ( $digit eq '' ) {
        next;
      }
      #print "111 $ret_line : $digit\n";
      $digit =~ s/ //g;
      my $ret_digit = isdigit($digit);
      if ( $ret_digit == 0 ) {
        next;
      }
      $total_max = $total_max + $digit;
      #print "112 $ret_line : $digit : $total_max \n";
    }
    #print "113 $total_max \n";
    close (FH);
  }
  
  # print out total value only for shared pools
  if ( $total_max > 0 ) {
      $cmd .= " LINE2:$total_max#000000:\\\"Total max pool units                      $total_max\\l\\\"";
  }

  $cmd .= " COMMENT:\\\"  Total average                                        $total\\l\\\"";
  $cmd .= " $updated";

  if ( "$type" =~ "d" ) {
    if ($j == 1) {
      $cmd .= " COMMENT:\\\" \\l\\\"";
    }
    #if ( $lpars_graph_type[$index] =~ /LPAR/ ) {
    #  $cmd .= " COMMENT:\\\"\(Note that for CPU dedicated LPARs is always shown their whole entitlement\)\\\"";
    #}
  }
  $cmd .= " HRULE:0#000000";
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  my $tmp_file="$tmpdir/custom-group-$group-$type.cmd";
  open(FH, "> $tmp_file") || error("Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
  
  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("custom : Multi graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err, my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("custom: $cmd : Multi graph rrdtool error : $ret");
    }
  }
  #unlink ("$tmp_file"); # keep it on purpose for detail pop-up

  close (FH);

  if ( ! get_lpar_num() ) {
    if ( $index > 4 ) {
      if ( ! -f "$webdir/custom/$group/$lim" ) {
        copy("$basedir/html/$lim_org", "$webdir/custom/$group/$lim");
      }
    }
  }

  return 0;
}

sub multiview_mem {
  my $group = shift;
  my $name = shift;
  my $type = shift;
  my $type_sam = shift;
  my $act_time = shift ;
  my $step_new = shift;
  my $text = shift;
  my $xgrid = shift;
  my $req_time = 0;
  my $comm = "COMM ";
  $color_indx = 0; # clear color index
  my $list = "$webdir/custom/$group/list.txt";
  my $lim = ".li";


  # open file for writing list of lpars
  open(FHL, "> $list") || error("Can't open $list : $!".__FILE__.":".__LINE__) && return 0;

  my $header = "Custom group $group: last $text";

  my $file = "";
  my $i = 0;
  my $lpar = "";
  my $lparn = "";
  my $cmd = "";
  my $cmd_sum = "";
  my $j = 0;

  if ( "$type" =~ "d" ) {
    $req_time = $act_time - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time - 2764800;
  }

  # $cmd_sum gets summary value 
  $cmd_sum .= "graph \\\"$tmpdir/name.png\\\"";
  $cmd_sum .= " --start now-1$type";
  $cmd_sum .= " --end now-1$type+1$type";
  my $cmd_sum_max = $cmd_sum;
  my $index=0;

  my $lim_org = $lim;
  $lim =~ s/\.//;
  $lim =~ s/i/l/;

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"GB\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  if ( $name =~ m/-os$/ ) {
    $cmd .= " COMMENT:\\\"Used memory in GBytes                          \\l\\\"";
  }
  else {
    $cmd .= " COMMENT:\\\"Allocation in GBytes                          \\l\\\"";
  }
  if ( $lpars_graph_type[$index] =~ /POOL/ ) { 
    $cmd .= " COMMENT:\\\"  Server                       Pool        Max pool     avrg     max\\l\\\"";   
  }
  else {
    $cmd .= " COMMENT:\\\"  Server                       LPAR                     avrg     max\\l\\\"";   
  }


  my $gtype="AREA";
  my $sh_pools = 0; # identificator of shared pool
  my @lpars_graph_local = @lpars_graph;
  my $updated = " ";

  # do not sort it HERE!! there is several arrays, sorting lpars_graph is not enough!!! --PH
  foreach my $file_tmp (@lpars_graph_local) {
    my $file = $file_tmp; # must be here to allocate new space and do not make changes in @lpars_graph

    #print "000 $file\n";
    if ( $file eq '' ) {
      $index++;
      next; #some trash
    }

    # note there is already no suffix, but do bellow to be sure
    $file =~ s/\.rrm$//;
    $file =~ s/\.rrd$//;
    $file =~ s/\.rrh$//;
    $file =~ s/all_pools$/pool/;

    if ( $lpars_graph_type[$index] =~ /POOL/ && $name =~ m/-os$/ ) {
      return 1; # no OS pool based graphs 
    }


    # pool names are already translated here!!!
    if ( $lpars_graph_type[$index] =~ /POOL/ && $file !~ m/\/pool$/ ) {
      $index++;
      next; # include only real lpars
    }

    if ( $file =~ m/\/mem$/ ) {
      $index++;
      next; # include only real lpars
    }

    if ( $lpars_graph_type[$index] =~ /LPAR/ ) {
      if ( $name =~ m/-os$/ ) {
        # OS mem graphs
        $file =~ s/$/\.mmm/;
        if ( ! -f $file ) {
          $index++;
          next;
        }
        if ( $file =~ m/\/pool\.r/ || $file =~ m/\/pool$/ ) {
          # exclude pool memory from OS memory 
          $index++;
          next;
        }
      }
      else {
        # note there is already no suffix, but do bellow to be sure
        $file =~ s/$/\.rmm/;
        if ( ! -f $file ) {
          $file =~ s/\.rmm$/\.rsm/;
          if ( ! -f $file ) {
            $index++;
            next;
          }
        }
      }
    }
    else {
      if ( $name =~ m/-os$/ ) {
        # exlude for mem OS agent 
        $index++;
        next;
      }
      # conversion to memory db files
      # pools : just default pool ann mem allocation
      $file =~ s/\/pool$/\/mem/;
      $file =~ s/$/\.rrm/;

      if ( ! -f $file ) {
        $file =~ s/\.rrm/\.rrh/;
        if ( ! -f $file ) {
          $index++;
          next;
        }
      }
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = (stat("$file"))[9];
    if ( $rrd_upd_time < $req_time ) {
      $index++;
      next;
    }

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar = $lpars_graph_server[$index];
    $lpar =~ s/\&\&1/\//g;
    for(my $k = length($lpar); $k < 25; $k++) {
      $lpar .= " ";
    }
    $lparn .= " ";
    $lpar .= "   ";
    $lpar .= $lpars_graph_name[$index];
    for(my $k = length($lpar); $k < 50; $k++) {
      $lpar .= " ";
    }

    print FHL "$file\n";

    #print "11 $file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 1);

    if ( (length($lpar_v) + 1 ) == length($comm) && length($lparn) == length($comm) ) {
      copy("$basedir/html/$lim_org", "$webdir/custom/$group/$lim");
      last;
    }

    if ($type =~ "d" && $i == 0 ) {
      RRDp::cmd qq(last "$file");
      my $last_tt = RRDp::read;
      chomp ($$last_tt);
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $updated =" COMMENT:\\\"  Updated\\\: $l \\\"";
    }

    # bulid RRDTool cmd
    if ( $lpars_graph_type[$index] =~ /LPAR/ ) {
      if ( $name =~ m/-os$/ ) {
        # OS memory - agent
        $cmd .= " DEF:cur${i}=\\\"$file\\\":nuse:AVERAGE";
        $cmd .= " DEF:in_use_clnt${i}=\\\"$file\\\":in_use_clnt:AVERAGE";
        $cmd .= " CDEF:cur_real${i}=cur${i},in_use_clnt${i},-";
        $cmd .= " CDEF:curg${i}=cur_real${i},1048576,/";
        $cmd_sum .= " DEF:cur${i}=\\\"$file\\\":nuse:AVERAGE";
        $cmd_sum .= " DEF:in_use_clnt${i}=\\\"$file\\\":in_use_clnt:AVERAGE";
        $cmd_sum .= " CDEF:cur_real${i}=cur${i},in_use_clnt${i},-";
        $cmd_sum .= " CDEF:curg${i}=cur_real${i},1048576,/";
      }
      else {
        $cmd .= " DEF:cur${i}=\\\"$file\\\":curr_mem:AVERAGE";
        $cmd .= " CDEF:curg${i}=cur${i},1024,/";
        $cmd_sum .= " DEF:cur${i}=\\\"$file\\\":curr_mem:AVERAGE";
        $cmd_sum .= " CDEF:curg${i}=cur${i},1024,/";
      }
      $cmd .= " $gtype:curg${i}$color[$color_indx]:\\\"$lpar\\\"";
      $cmd_sum .= " $gtype:curg${i}$color[$color_indx]:\\\"$lpar\\\"";
    }
    else { 
      # POOLs
      # default pool only
        $cmd .= " DEF:tot${i}=\\\"$file\\\":conf_sys_mem:AVERAGE";
        $cmd .= " DEF:free${i}=\\\"$file\\\":curr_avail_mem:AVERAGE";
        $cmd .= " CDEF:totg${i}=tot${i},1024,/";
        $cmd .= " CDEF:freeg${i}=free${i},1024,/";
        $cmd .= " CDEF:curg${i}=totg${i},freeg${i},-";
        $cmd .= " $gtype:curg${i}$color[$color_indx]:\\\"$lpar\\\"";
        $cmd_sum .= " DEF:tot${i}=\\\"$file\\\":conf_sys_mem:AVERAGE";
        $cmd_sum .= " DEF:free${i}=\\\"$file\\\":curr_avail_mem:AVERAGE";
        $cmd_sum .= " CDEF:totg${i}=tot${i},1024,/";
        $cmd_sum .= " CDEF:freeg${i}=free${i},1024,/";
        $cmd_sum .= " CDEF:curg${i}=totg${i},freeg${i},-";
        $cmd_sum .= " $gtype:curg${i}$color[$color_indx]:\\\"$lpar\\\"";
    }

    $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"%5.2lf\\\"";
    $cmd .= " GPRINT:curg${i}:MAX:\\\"%5.2lf\\l\\\"";
    $cmd_sum .= " PRINT:curg${i}:AVERAGE:\\\"%5.2lf\\\"";

    $gtype="STACK";
    $i++;
    $color_indx++;
    $index++;
    if ( ! get_lpar_num() && $index == 5 ) {
      last;
    }

  }
  close (FHL);

  if ( $i == 0 && $type =~ m/^d$/ ) {
    if ( $lpars_graph_type[0] !~ /POOL/ && $name !~ m/-os$/ ) { # $index = 0 on purpose
      error ("creating cgraph: custom memory:$group:$type_sam:$type - not identified any source");
    }
    return 0; # nothing has been found
  }

  $cmd_sum =~ s/\\"/"/g;

  my $FH;
  my $tmp_file_sum="$tmpdir/custom-group-mem-$group-$type-sum.cmd";
  if ( $name =~ m/-os$/ ) {
    $tmp_file_sum="$tmpdir/custom-group-mem-os-$group-$type-sum.cmd";
  }

  # Find out total value
  open(FH, "> $tmp_file_sum") || error("Can't open $tmp_file_sum : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd_sum\n";

  $ret  = `$rrdtool - < "$tmp_file_sum" 2>&1`;
  #print "$name: $cmd_sum - $ret\n";
  my $total = 0;
  foreach my $ret_line (split(/\n/,$ret)) {
    chomp($ret_line);
    #print "110 $ret_line\n";
    if ( $ret_line =~ m/:/ || $ret_line =~ m/x/ ) {
      next;
    }
    $ret_line =~ s/0x0//g;
    $ret_line =~ s/ //g;
    $ret_line =~ s/OK.*$//g;
    chomp($ret_line); # must be here as well !!
    if ( $ret_line eq '' ) {
      next;
    }
    my $ret_digit = isdigit($ret_line);
    #print "111 $ret_line - ret_digit = $ret_digit\n";
    if ( $ret_digit == 0 ) {
      next;
    }
    $total = $total + $ret_line;
    #print "112 $ret_line : $total \n";
  }
  #print "112 $total \n\n\n";
  close (FH);


  $cmd .= " COMMENT:\\\"  Total                                                $total\\l\\\"";

  $cmd .= " $updated";
  $cmd .= " HRULE:0#000000";
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  my $tmp_file="$tmpdir/custom-group-mem-$group-$type.cmd";
  if ( $name =~ m/-os$/ ) {
    $tmp_file="$tmpdir/custom-group-mem-os-$group-$type.cmd";
  }
  open(FH, "> $tmp_file") || error("Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
  
  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("custom mem: Multi graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err, my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("custom mem: $cmd : Multi graph rrdtool error : $ret");
    }
  }
  #unlink ("$tmp_file"); # keep it on purpose for detail pop-up

  close (FH);

  if ( ! get_lpar_num() ) {
    if ( $index > 4 ) {
      if ( ! -f "$webdir/custom/$group/$lim" ) {
        copy("$basedir/html/$lim_org", "$webdir/custom/$group/$lim");
      }
    }
  }

  return 0;
}

# error handling
sub error
{
  my $text = shift;
  my $act_time = localtime();
  chomp ($text);

  print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
}

sub touch
{
  my $text = shift;

  my $version="$ENV{version}";
  my $basedir = $ENV{INPUTDIR};
  my $new_change="$tmpdir/$version-run";
  my $DEBUG = $ENV{DEBUG};

  if ( ! -f $new_change ) {
   `touch $new_change`; # say install_html.sh that there was any change
   if ( $text eq '' ) {
     print "touch          : custom groups $new_change\n" if $DEBUG ;
   }
   else {
     print "touch          : custom groups $new_change : $text\n" if $DEBUG ;
   }
  }

  return 0
}


sub isdigit
{
  my $digit = shift;

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;

  if (length($digit_work) == 0) {
    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  #error ("there was expected a digit but a string is there, field: $text , value: $digit");
  return 0;
}

