#!/usr/bin/perl
use strict;
use Date::Parse;
use Math::BigInt;
use Time::Local;
use POSIX;
use File::Copy;
use File::Compare;

# it runs only if not exist tmp/dent-run or it has previous day timestamp

# set unbuffered stdout
$| = 1;

# get cmd line params
my $version="$ENV{version}";
my $webdir = $ENV{WEBDIR};
my $bindir = $ENV{BINDIR};
my $basedir = $ENV{INPUTDIR};
my $tmpdir = "$basedir/tmp";
if (defined $ENV{TMPDIR}) {
  $tmpdir = $ENV{TMPDIR};
}
my $rrdtool = $ENV{RRDTOOL};
my $DEBUG = $ENV{DEBUG};
# $DEBUG = 3;
my $managed_systems_exclude =  $ENV{MANAGED_SYSTEMS_EXCLUDE};
my $upgrade=$ENV{UPGRADE};
my $wrkdir = "$basedir/data";
my $dent_log="$basedir/logs/daily_lpar_check.log"; # log output
my $dent_html="$webdir/daily_lpar_check.html"; # html ouput of the last run
my $dent_run="$tmpdir/dent-run";
my $filelpst = "$tmpdir/daily_lpar_check.txt";
my $filelpstlog = "$basedir/logs/daily_lpar_check.log";
my $actprogsize = -s "$basedir/bin/daily_lpar_check.pl";

my $act_time = localtime();

# at first check whether it is a first run after the midnight
if ( ! -f $dent_run ) {
  `touch $dent_run`; # first run after the install
  print "lpar_check     : first run after install 01\n";
}
else {
  my $run_time = (stat("$dent_run"))[9];
  (my $sec,my $min,my $h,my $aday,my $m,my $y,my $wday,my $yday,my $isdst) = localtime(time());
  ($sec,$min,$h,my $png_day,$m,$y,$wday,$yday,$isdst) = localtime($run_time);
  if ( $aday == $png_day ) {
                # If it is the same day then do not update except upgrade
     if ( $upgrade ==  0 ) {
        print "lpar_check     : not this time $aday == $png_day\n";
        exit (0); # run just once a day per timestamp on $webdir/daily_lpar_check.html 
                  # and  $tmpdir/daily_lpar_check.txt
     }
     else {
          print "lpar_check     : run it as first run after the upgrade : $upgrade\n";
     }
  }
  else {
       print "lpar_check     : first run after the midnight 02: $aday != $png_day\n";
       `touch $dent_run`; 
  }
}

 hlink_check();

 open (VYFILE, ">$filelpst") || error("Cannot open: $!".__FILE__.":".__LINE__) && exit 0;
 print VYFILE "InFo_Lpar Activity Table\n";

read_lpars();

 print VYFILE "InFo_Lpar Activity Table Prog Size $actprogsize\n";
 close (VYFILE);
 copy("$filelpst","$filelpstlog") || error("Cannot copy $filelpstlog: $!".__FILE__.":".__LINE__) && exit 0;

# create web page

  open (FHW, "> $dent_html") || error("Cannot open $dent_html: $!".__FILE__.":".__LINE__) && exit 0;
  print_head ();

  open (FH, "< $filelpst") || error("Cannot read $filelpst: $!".__FILE__.":".__LINE__) && exit 0;
  my @lines = <FH>;
  close (FH);

  my $count = 0;
  while (my $line = shift @lines) {

# looking for lpar data

    if ($line =~ m/InFo_cfg/) {
       my @lpar_info = split(',',$line);
       print FHW "<tr><td>$lpar_info[1]</td><td>$lpar_info[2]</td><td>non actual</td><td>$lpar_info[4]</td><td>server</td></tr>\n";
       $count = $count + 1;
    }

    if (!($line =~ m/InFo/)) {
       my @lpar_info = split(',',$line);
       if ($lpar_info[3] ne "OK") {
          my $type = "CPU";
          if ($line =~ m/poool/) { $type = "Pool"; }
          if ($line =~ m/AllCPUPools/) { $type = "All CPU Pools"; }
          if ($line =~ m/\.mmm/) { $type = "OS MEM"; }
          if ($line =~ m/agent/) { $type = "OS agent"; }
          $lpar_info[2] =~ s/\.rrm$//;
          $lpar_info[2] =~ s/\.rrh$//;
          $lpar_info[2] =~ s/\.mmm$//;
          # if there is a "/" in lpar name, make subst only for print
          $lpar_info[2] =~ s/&&1/\//g;

          print FHW "<tr><td>$lpar_info[0]</td><td>$lpar_info[1]</td><td>$lpar_info[2]</td><td>$lpar_info[4]</td><td>$type</td></tr>\n";
          $count = $count + 1;
       }
    }
  }
 print FHW "</table></center><br><br><br><br><br>";
 
 if ($count == 0) { print FHW "Congrats,all LPARs and CPU pools are regularly updated.<br>\n";
  }

# looking for other comments 
#  open FH,"< $filelpst" or die "Cannot read the file $filelpst: $!\n";
#  @lines = <FH>;
#  close (FH);
#
#  while (my $line = shift @lines) {
#    if ($line =~ m/InFo/)  {
#       print FHW "$line<br>\n";
#    }
#  } 
  print FHW "Report has been created at: $act_time<br>\n";
  print FHW "Table above shows lpars and servers not updated within last 24 hours.<br>\n"; 
  print FHW "Lpars and servers not updated more than 30 days are ignored.<br>\n";
  print FHW "</body></html>\n";
  close (FHW);

exit(0);

sub print_head {

  print FHW "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">
<HTML>
<HEAD>
  <TITLE>LPAR2RRD</TITLE>
  <META HTTP-EQUIV=\"pragma\" CONTENT=\"no-cache\">
  <META HTTP-EQUIV=\"Expires\" CONTENT=\"NOW\">
  <META HTTP-EQUIV=\"last modified\" CONTENT=\"NOW\">
<style>
<!--
a {text-decoration: none}
-->
</style>
</HEAD>
<BODY BGCOLOR=\"#D3D2D2\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#0000FF\" ALINK=\"#FF0000\" >\n
<center><h3>Data health check: LPARs and servers not being updated</h3></center>
<center><table border=\"1\">
<tr><th>SERVER&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th><th>HMC&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th><th>LPAR/CPU Pool/MEM&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th><th>last update&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th><th>Type&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th></tr>";

  return(0);
}

sub read_lpars {
  my $act_time_u = time();

  foreach my $server_all (<$wrkdir/*>) {
    my $server = basename ($server_all);
#    print "00 $server_all\n";
    if ( -l $server_all ) { next; }
    if ( $server_all =~ m/\.rrx$/ ) {print VYFILE "InFo_serv,$server,found database,server\n"; next; }
    if ( ! -d $server_all ) {print VYFILE "InFo_serv,$server,non-existent,server\n"; next; }
 
    #test for empty server directory - see 2nd line and end of next foreach
    my $emptydir = "empty";

   my $server_all_space = $server_all;
   $server_all_space = "\"".$server_all."\"" if $server_all =~ m/ /; 
  
   foreach my $hmc_all (<$server_all_space/*>) {
      $emptydir = $hmc_all;
      my $hmc = basename ($hmc_all);
      #print "02 $hmc,$hmc_all\n";
      if ( ! -d $hmc_all ) { print VYFILE "InFo_hmc,$server,non-existent,in $server_all \n"; next; }
      if ( ! -f "$hmc_all/config.cfg" ) 
         { print VYFILE "InFo_hmc,$server,non-existent,config.cfg in $server_all \n"; next;
      }
      my $mon_sec = 30*86400;
      if ( ($act_time_u - (stat("$hmc_all/config.cfg"))[9]) > $mon_sec)
           { print VYFILE "InFo_hmc,$server,older 30 days,config.cfg in $server_all \n"; next; };
      my $hmc_all_time = (stat("$hmc_all/config.cfg"))[9];
      my($mmin,$hhour,$dd,$mm,$yy) = (localtime($hmc_all_time))[1,2,3,4,5];
      $yy = $yy + 1900;
      $mm = $mm + 1;
      if ($mmin < 10) { $mmin = "0".$mmin };
      if ($hhour < 10) { $hhour = "0".$hhour };
      my $update_d = "$hhour:$mmin $dd.$mm.$yy";
      if ( ($act_time_u - $hmc_all_time) > 86400) 
         { print VYFILE "InFo_cfg,$server,$hmc,non-actual,$update_d \n"; 
      };  #  next; ???

      my $hmc_all_space = $hmc_all;
      $hmc_all_space = "\"".$hmc_all."\"" if $hmc_all =~ m/ /;
 
      my $lpar_mem_name = ""; #do not test lpar.mmm if lpar/mem.mmm is OK

      foreach my $lpar_all (<$hmc_all_space/*>) {

        if ( -d "$lpar_all" ) {        #  lpar directory
           my $lpar_all_space = $lpar_all;
           $lpar_all_space = "\"".$lpar_all."\"" if $lpar_all =~ m/ /;
             
           foreach my $file_tst (<$lpar_all_space/*.mmm>) {
              my $file_name = basename($file_tst);
              if ( $file_name ne "mem.mmm" ) { next; } 
              print "testing        : $file_tst\n" ;
              my $lpar = basename($lpar_all);
              my $file_tst_ttime=(stat("$file_tst"))[9];
              my($mmin,$hhour,$dd,$mm,$yy) = (localtime($file_tst_ttime))[1,2,3,4,5];
              $yy = $yy + 1900;
              $mm = $mm + 1;
              if ($mmin < 10) { $mmin = "0".$mmin };
              if ($hhour < 10) { $hhour = "0".$hhour };
              my $update_d = "$hhour:$mmin $dd.$mm.$yy";
              if ( ($act_time_u - ($file_tst_ttime)) > (86400))
                 { my $file_name = basename($file_tst);
                   print VYFILE "$server,$hmc,$lpar,agent older 1 day,$update_d\n"; 
                 next;
              };
              $lpar_mem_name = $lpar; 
           } 
           next; 
        }
        if (!(($lpar_all =~ m/mmm$/) or ($lpar_all =~ m/rrh$/) or ($lpar_all =~ m/rrm$/)))  {
           #print "this is to omit, $lpar_all\n";
           next; 
        }

        #print VYFILE "InFo_lpar,$lpar_all,,$lpar_mem_name,,\n"; 
                     #do not test lpar.mmm if lpar/mem.mmm is OK        
        if ($lpar_all =~ m/mmm$/ && $lpar_mem_name ne "") { 
            if ( -1 != index($lpar_all,$lpar_mem_name)) {
                $lpar_mem_name = "";
                next
            }
        }

        my $lpar = basename($lpar_all);
        my $lpar_name_ttime=(stat("$lpar_all"))[9];
        my($mmin,$hhour,$dd,$mm,$yy) = (localtime($lpar_name_ttime))[1,2,3,4,5];
        $yy = $yy + 1900;
        $mm = $mm + 1;
        if ($mmin < 10) { $mmin = "0".$mmin };
        if ($hhour < 10) { $hhour = "0".$hhour };
        my $update_d = "$hhour:$mmin $dd.$mm.$yy";
        if ( ($act_time_u - ($lpar_name_ttime)) > (30*86400)) 
           { print VYFILE "InFo_lpar,$server,$hmc,$lpar,older 30 days,$update_d\n"; next; 
        };
        my $lpar_all_name = $lpar_all;
        $lpar_all_name =~ s/\.rrm$//;
        $lpar_all_name =~ s/\.rrh$//;
        $lpar_all_name =~ s/\.mmm$//;
 print "testing        : $server, $hmc, $lpar\n";
        my $lpar_name  = $lpar;
 #print "X01,$lpar_name,\n";
        $lpar_name =~ s/\.rrm$//;
        $lpar_name =~ s/\.rrh$//;
        $lpar_name =~ s/\.mmm$//;
 #print "X02,$lpar_name,\n";

# if both .rrh and .rrm, take the newer from them
        my $last_letter = substr($lpar_all,length($lpar_all)-1,1);
        if ($last_letter eq "h"){
            if ( -f "$lpar_all_name.rrm" ) {                              #print "found -h and -m\n";
                my $lpar_name_ttimem = (stat("$lpar_all_name.rrm"))[9];
                if ($lpar_name_ttimem > $lpar_name_ttime ) { next ; }
             }
        }
        if ($last_letter eq "m"){
            if ( -f "$lpar_all_name.rrh" ) {                              #print "found -m and -h\n";
                my $lpar_name_ttimem = (stat("$lpar_all_name.rrh"))[9];
                if ($lpar_name_ttimem > $lpar_name_ttime ) { next ; }
             }
          }
        my $lpar_pool_alias = $lpar;
        if ( $lpar_name =~ m/^pool$/ ) {             # test for actual time
          $lpar_pool_alias =~ s/pool/AllCPUPools/g;
          if ( ($act_time_u - ($lpar_name_ttime)) > 86400) 
             { print VYFILE "$server,$hmc,$lpar_pool_alias,non_actual,$update_d,poool\n"; next;
          };
          print VYFILE "$server,$hmc,$lpar_pool_alias,OK,$update_d,poool\n";  
#print "001 pool : $lpar_name,$lpar_all,$lpar,$lpar_pool_alias,$server,$hmc,$hmc_all,$update_d\n";
          next;
        }
        if ( $lpar =~ m/^SharedPool0\.rrm/ ) { next; }  #print "    always leave out\n";
        if ( $lpar_name =~ m/^SharedPool/ ) {       # test for actual time

# change  Shared CPU pool alias  (if exists and it is not a default CPUpool)
           my $pool_id = $lpar;
           $pool_id =~ s/SharedPool//g;
           $pool_id =~ s/\.[^\.]*$//; 
           if ( -f "$hmc_all/cpu-pools-mapping.txt") {
              my $c_p_mapping_time=(stat("$hmc_all/cpu-pools-mapping.txt"))[9];
              if ( ($act_time_u - ($c_p_mapping_time)) > (30*86400))
                 { print VYFILE "InFo_map,$server,$hmc,cpu-pools-mapping.txt,older 30 days\n"; next;
              };
              if ( ($act_time_u - ($c_p_mapping_time)) > 86400)
                 { print VYFILE "InFo_map,$server,$hmc,cpu-pools-mapping.txt,older 24 hours\n";
              };
              open(FR, "<$hmc_all/cpu-pools-mapping.txt");
              foreach my $linep (<FR>) {
                  chomp ($linep);
                  (my $id, my $pool_name) = split(/,/,$linep);
                  if ( $id == $pool_id  ) {
                     $lpar_pool_alias = "$pool_name";
                     last;
                  }
              }
              close (FR);
              if ( $lpar_pool_alias eq $lpar )  { #SharedPool not in mapping? then leave out
                 next;
              }
           }
          if ( ($act_time_u - ($lpar_name_ttime)) > 86400)
             { print VYFILE "$server,$hmc,$lpar_pool_alias,non_actual,$update_d,poool\n"; next;
          };
          print VYFILE "$server,$hmc,$lpar_pool_alias,OK,$update_d,poool\n";
#print "001 SharedPool : $lpar_name,$lpar_all,$lpar_pool_alias,$server,$hmc,$hmc_all,$update_d\n";
          next;
        }
        if ( $lpar_all =~ /\.mmm$/ ) {               # test for actual time
          if ( ($act_time_u - ($lpar_name_ttime)) > 86400) 
             { print VYFILE "$server,$hmc,$lpar,non_actual,$update_d\n"; next;
          };
          print VYFILE "$server,$hmc,$lpar,OK,$update_d\n";
#print "001 mmm : $lpar_name,$lpar_all,$lpar,$server,$hmc,$hmc_all,$update_d\n";
          next;
        }
        if ( $lpar_name =~ m/^mem-pool$/ || $lpar_name =~ m/^mem$/ || $lpar_name =~ m/^cod$/ ) {
#           print "excluded $lpar_name\n";
           next; #exclude non lpars
        }
                                                      # test activity in config.cfg and actual time
         my $lpar_state = lpar_state_in_configcfg($lpar_name,$hmc_all);
#print "009 LPAR_state is,$lpar_state,\n";
         if ( $lpar_state eq "not_found" )  {
              print VYFILE "InFo_lpar,$server,not found,in config.cfg $lpar_all \n"; next;
         };
         if ( $lpar_state eq "Not Activated" )  {next; }   # always leave out

         if (!(( $lpar_state eq "Running" ) 
               or ( $lpar_state eq "Started" ))) {
             print VYFILE "$server,$hmc,$lpar,UNKNOWN_state,$update_d,$lpar_state\n"; next;
         }
         if ( ($act_time_u - ($lpar_name_ttime)) > 86400) 
            { print VYFILE "$server,$hmc,$lpar,non_actual,$update_d\n"; next; 
         };
         print VYFILE "$server,$hmc,$lpar,OK,$update_d\n";
# print "001 lpar : $lpar_name,$lpar_all,$lpar,$server,$hmc,$hmc_all,$update_d\n";
          next;
      }
    }
    if ($emptydir eq "empty" ) { print VYFILE "InFo_hmc,$server,non-existent,in $server_all \n"; }
  }
}

#</PRE><B>LPAR config:</B><PRE>
#name                                   aix4 test /%#;        pay att to "/"
#lpar_id                                5
#lpar_env                               aixlinux
#primary_state                          Started

sub lpar_state_in_configcfg {
  my $lpar = shift;
  my $hmc_all = shift;
 
  my ($line,$lpar_env,$lpar_state);

  my $filename = "$hmc_all/config.cfg";

  # if there is a "/" in lpar name, make subst
  $lpar =~ s/&&1/\//g;
  chomp($lpar); 
# print "008;lpar ;$lpar;$filename;\n";

  open (FH,"<$filename") || error("Cannot read $filename: $!".__FILE__.":".__LINE__) && return 0;
  my @lines = <FH>;
  close (FH);

  my $lpar_name = "";
  my $lpar_found = 0;
  my $rmc_state = "";
  my $number_bad = 0;

  while (my $line = shift @lines) {

# looking for lpar

    if ($line =~ m/>LPAR config:</)
    {
#        print $line;
         $line = shift @lines;
#print $line,$lpar;
         $line =~ s/^name                                   //;
         chomp($line);
         if ($line eq $lpar) 
         {
             $line = shift @lines; $line = shift @lines;
#             $lpar_env= (split(/ +/, $line))[1]; 
#             chomp($lpar_env);
             $line = shift @lines;
# print $line;
              $lpar_state= (split(/ +/, $line))[1];
              chomp($lpar_state);
# for Not Activated state
             if ( $lpar_state eq "Not" )  { $lpar_state .=" " . (split(/ +/, $line))[2]; chomp($lpar_state);  }
             $lpar_found = 1;
             last;
         }

    }
  }
  if ($lpar_found == 0) {$lpar_state = "not_found" } ;
#  print "ret lpar_state_in_configcfg,$lpar_state\n";
  return ($lpar_state);
 }

sub basename {
  my $full = shift;
  my $out = "";

  # basename without direct function
  my @base = split(/\//,$full);
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
}

# error handling
sub error
{
  my $text = shift;
  my $act_time = localtime();
  chomp ($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
}

#
### hard link check
#
#  algorithm:
 #  cycle on workdir/servers - take only directories
 #   |   cycle on workdir/servers/hmcs - take only directories
 #   |  - no dir for this server? error -> log -> next
 #   |  - one dir  - nothing to test  -> log -> next
 #   |  - cycle on two or more dirs -> first is hmc1, others hmcX   
 #   |   |   cycle on workdir/servers/hmc1/*.mmm - means 'lpar'.mmm = file
 #   |   |   |  - test-hlink( hmcX/, file)
 #   |   |   cycle on workdir/servers/hmc1/*/    
 #   |   |   |  -  if exists workdir/servers/hmc1/lpar/   -> check if  ..hmcX/lpar/  or create 
 #   |   |   |  -  cycle on workdir/servers/hmc1/*/*.mmm or *.cfg = file
 #   |   |   |  | -  test-hlink( hmcX/lpar/, file)
 #   --------------

# sub test-hlink (dirX, file)  
  #-	dirX/file exists? 
  #     o	no -  [A] make HL dir1/file -> dirX/file -> log ->  (if links>2 ? -> log ) -> return
  #     o	yes -  same inode? 
  #             	yes  -> ( if links>2 ? -> log ) -> return
  #                     no  -> move force = back-up existing dirX/file to dirX/file.bphl  -> log -> [A]
  #--------------------------- 


  #  ***     main    *** 
sub hlink_check
  {
  my $logfile = "$basedir/logs/thl.out";
  open (THL, ">> $logfile") || errorh("Cannot open for writing $logfile: $! ".__FILE__.":".__LINE__) && return 0;
  #write_log("testing hlink : started");
  print "testing hlink  : started\n";

  foreach my $server_ful (<$wrkdir/*>) {
     print "01 take item from server dir: $server_ful\n" if $DEBUG>2 ;
     if (   -l $server_ful ) { next; }
     if ( ! -d $server_ful ) { next; }
       
     opendir (my $dh, $server_ful) ||
        errorh("Cannot open dir for reading $server_ful: $! ".__FILE__.":".__LINE__) && next ;
 
     my @dirs = grep {-d "$server_ful/$_" && ! /^\.{1,2}$/} readdir($dh);
     closedir($dh);
     print "02 hmcs : @dirs\n" if $DEBUG>2 ;

     my $hmcount = @dirs;
     if ($hmcount == 1) {
        print "one hmc:@dirs for server: $server_ful 'Bye'\n" if $DEBUG>2 ;
        next;
     }
     if ($hmcount > 2) {
        print "only 2 hmcs not!$hmcount! possible\n" if $DEBUG>2 ;
        errorh("only 2 hmcs not!$hmcount! possible: @dirs ".__FILE__.":".__LINE__) ;
 #       next;
     }
     my $hmc1 = $dirs[0];
     for (my $i = 1; $i <= ($hmcount - 1); $i++) {
        my $hmc2 = $dirs[$i];
  
        my $server_ful_space = $server_ful;
        $server_ful_space = "\"".$server_ful."\"" if $server_ful =~ m/ /;  
        my $hmc1_space = $hmc1;
        $hmc1_space = "\"".$hmc1."\"" if $hmc1 =~ m/ /;
 
        foreach my $file_tst (<$server_ful_space/$hmc1_space/*.mmm>) {
           print "03 take item lpar.mmm: $file_tst\n" if $DEBUG>2 ;
           test_hlink("$server_ful/$hmc2", $file_tst);
        }
      
        opendir (my $dhl, "$server_ful/$hmc1/") ||
          errorh("Cannot open dir for reading $server_ful/$hmc1/: $! ".__FILE__.":".__LINE__) && next ;

        my @dirsl = grep {-d "$server_ful/$hmc1/$_" && ! /^\.{1,2}$/} readdir($dhl);
        closedir($dhl);
        print "04 lpars : @dirsl\n" if $DEBUG>2 ;
 
        foreach my $dirl (@dirsl) {
          my @dirl_part = split ( /\//,$dirl);
          my $lpar_dir = $dirl_part[-1];

          if ( ! -d "$server_ful/$hmc2/$lpar_dir" ) {
            mkdir ("$server_ful/$hmc2/$lpar_dir") || 
              errorh("Cannot mkdir $server_ful/$hmc2/$lpar_dir: $! ".__FILE__.":".__LINE__) && next ; 
            write_log("directory made: $server_ful/$hmc2/$lpar_dir");  
          }
          my $lpar_dir_space = $lpar_dir;
          $lpar_dir_space = "\"".$lpar_dir."\"" if $lpar_dir =~ m/ /;

          foreach my $file_tst (<$server_ful_space/$hmc1_space/$lpar_dir_space/*.mmm>) {
           print "05 take item : $file_tst\n" if $DEBUG>2 ;
           test_hlink("$server_ful/$hmc2/$lpar_dir", $file_tst);
          }
          foreach my $file_tst (<$server_ful_space/$hmc1_space/$lpar_dir_space/*.cfg>) {
           print "06 take item : $file_tst\n" if $DEBUG>2 ;
           test_hlink("$server_ful/$hmc2/$lpar_dir", $file_tst);
          } 
       }
     }
  }
  #write_log("testing hlink : finished");
  close (THL);
  print "testing hlink  : finished\n" ;

 return (0);
}

sub test_hlink 
{ 
   my $hmc2 = shift;
   my $file_ful = shift;

   my @file_ful_part = split ( /\//,$file_ful);
   my $file_name = $file_ful_part[-1];
 
   my $hlink;
   my ($inode_1, $inode_2);

   if ( ! -f "$hmc2/$file_name" ) {
     if (link ($file_ful, "$hmc2/$file_name" )) {
        write_log("hlink created for:$file_ful to:$hmc2/$file_name");
           # test number of links   
    #     $hlink = (stat("$hmc2/$file_name"))[3];
    #    if ( $hlink != 2 ) { 
    #       write_log("hlink num: $hlink for: $hmc2/$file_name");
    #    }
     return 1;
     }
     else {
       errorh("Cannot hlink create for:$file_ful to:$hmc2/$file_name : $! ".__FILE__.":".__LINE__) && return 0;
     }
   }
   $inode_1 = (stat("$file_ful"))[1];
   ($inode_2, $hlink) = (stat("$hmc2/$file_name"))[1,3]; 
   if ( $inode_1 == $inode_2 ) {
 #    if ( $hlink != 2 ) {
 #       write_log("hlink num: $hlink for: $hmc2/$file_name");
 #    }
     return 1;
   }
   if ( ! move ("$hmc2/$file_name", "$hmc2/$file_name.bphl")) {
     errorh("Cannot back up: $hmc2/$file_name : $! ".__FILE__.":".__LINE__) && return 0;
   }
   write_log("backed up $hmc2/$file_name");   
   if (link ($file_ful, "$hmc2/$file_name" )) {
      write_log("hlink created for:$file_ful to:$hmc2/$file_name");
         # test number of links
 #     $hlink = (stat("$hmc2/$file_name"))[3];
 #     if ( $hlink != 2 ) {
 #        write_log("hlink num: $hlink for: $hmc2/$file_name");
 #     }
   return 1;
   }
   else {
     errorh("Cannot hlink create for:$file_ful to:$hmc2/$file_name : $! ".__FILE__.":".__LINE__) && return 0;
   }
 } 

sub write_log
{
  my $text = shift;
  my $act_time = localtime();
  print THL "$act_time: $text\n";
}
  
    # error handling
sub errorh
{
  my $text = shift;
  my $act_time = localtime();
  chomp ($text);

  print STDERR "$act_time: $text : $!\n";
  print THL "$act_time: $text : $!\n";
  return 1;
}

 
