#!/usr/bin/perl
use strict;
use Date::Parse;
use Math::BigInt;
use Time::Local;
use POSIX;
use RRDp;

# it runs only if not exist tmp/lpars_count-run or it has previous day timestamp

# set unbuffered stdout
$| = 1;

# get cmd line params
my $version="$ENV{version}";
my $webdir = $ENV{WEBDIR};
my $bindir = $ENV{BINDIR};
my $basedir = $ENV{INPUTDIR};
my $tmpdir = "$basedir/tmp";
if (defined $ENV{TMPDIR}) {
  $tmpdir = $ENV{TMPDIR};
}
my $rrdtool = $ENV{RRDTOOL};
my $pic_col = $ENV{PICTURE_COLOR};
my $DEBUG = $ENV{DEBUG};
my $upgrade=$ENV{UPGRADE};
my $managed_systems_exclude =  $ENV{MANAGED_SYSTEMS_EXCLUDE};
my $wrkdir = "$basedir/data";
my $lpars_count_log="$basedir/logs/lpars_count.log"; # log output
my $lpars_count_run="$tmpdir/lpars_count-run";
my $filelpst = "$tmpdir/daily_lpar_check.txt";
my $act_time = localtime();

# at first check whether there was a first run of daily_lpar_check.pl after upgrade or midnight
if ( ! -f $filelpst ) {
  system "perl $bindir/daily_lpar_check.pl"; # first run after the upgrade
  print "lpars_count    : first run after install 01\n";
}
else { 
  my $run_time = (stat("$filelpst"))[9];
  (my $sec,my $min,my $h,my $aday,my $m,my $y,my $wday,my $yday,my $isdst) = localtime(time());
  ($sec,$min,$h,my $png_day,$m,$y,$wday,$yday,$isdst) = localtime($run_time);
  if ( $aday ne $png_day ) {
     # If it is not the same day then update daily_lpar_check 
     system "perl $bindir/daily_lpar_check.pl"; # first run after the midnight
     print "lpars_count    : first run after the midnight 01\n";
  }
}
if ( ! -f $lpars_count_run ) {
  `touch $lpars_count_run`; # first run after install
  print "lpars_count    : first run after install 01\n";
}
else {
  my $run_time = (stat("$lpars_count_run"))[9];
  (my $sec,my $min,my $h,my $aday,my $m,my $y,my $wday,my $yday,my $isdst) = localtime(time());
  ($sec,$min,$h,my $png_day,$m,$y,$wday,$yday,$isdst) = localtime($run_time);
  if ( $aday == $png_day ) {
                # If it is the same day then do not update except upgrade
     if ( $upgrade ==  0 ) {
        print "lpars_count    : not this time $aday == $png_day\n";
        exit (0); # run just once a day per timestamp on graphs
     }
     else {
          print "lpars_count    : run it as first run after the upgrade : $upgrade\n";
     }
  }
  else {
     print "lpars_count   : first run after the midnight 02: $aday != $png_day\n";
     `touch $lpars_count_run`; 
  }
}
# read lpars info, choose those OK, sort according to hmc

#open FH,"< $filelpst" or die "Cannot read the file $filelpst: $!\n";
  open (FH,"< $filelpst") || error("Cannot open $filelpst: $!".__FILE__.":".__LINE__) && return 0;
my @lines = <FH>;
close (FH);
#print join("", @lines);

my (@lines1,@lines2);
while (my $line = shift @lines) {

# looking for lpars OK

    if (!(($line =~ m/InFo/) || ($line =~ m/poool/) || ($line =~ m/\.mmm/))) {
       my @lpar_info = split(',',$line);
       if ($lpar_info[3] eq "OK") {
          push(@lines1,"$lpar_info[1],$lpar_info[0],$lpar_info[2]\n");
       }
    }
}
#print join("", @lines1);

my @sorted_lines = sort @lines1;
#print "after sort \n";
#print join("", @sorted_lines);

my $s_num = 0;
my $l_num = 0;
my $is_hmc = 0;
my ($hmc_name, $s_name);
my ($act_h,$act_s,$act_l);
while (my $line = shift @sorted_lines) {
      ($act_h,$act_s,$act_l) = split(',',$line);
      if ($is_hmc == 0) {
         $is_hmc = 1 ;
         $hmc_name = $act_h;
         $s_name = $act_s;
         $s_num++;
         $l_num++;
      }
      else {
         if ($hmc_name eq $act_h) {      # same hmc
            if ($s_name eq $act_s) {     # same server
               $l_num++;
            }
            else {                       # next server
               $s_name = $act_s;
               $s_num++;
               $l_num++;
            }
         }
         else {                          # next hmc 
#            print "lpars_count   :hmc $hmc_name has servers: $s_num, lpars: $l_num\n";
#$s_num =15 ; $l_num = 158;
            update_rrx($hmc_name, $s_num, $l_num);
            $hmc_name = $act_h;
            $s_name = $act_s;
            $s_num = 1;
            $l_num = 1;
         }
      }
}
if ($is_hmc ne 0) {
#   print "hmc: $hmc_name, servers: $s_num, lpars: $l_num\n";
   update_rrx($hmc_name, $s_num, $l_num);
}
print "lpars_count    : finished\n";

exit (0);

sub update_rrx {		# update (create if not exists) 
    my $hmc = shift;
    my $s_num = shift;
    my $l_num = shift;
    my $rrx_name = "$wrkdir/$hmc\.rrx";
RRDp::start "$rrdtool";

# test if exists database for lpars and servers for hmc
if ( ! -f $rrx_name ) {
  #initialise  rrx for servers and lpars count
  RRDp::cmd qq(create "$rrx_name"  --step "86400"
    "DS:servers:GAUGE:172800:0:48"
    "DS:lpars:GAUGE:172800:0:9999"
    "RRA:MAX:0.5:1:1500"
    "RRA:AVERAGE:0.5:1:1500"
    "RRA:LAST:0.5:1:1"
  );
  my $answer = RRDp::read;
  if ( $$answer ) {
    error ("unable to create $rrx_name: answer: $$answer");
    exit(1);
  }
}
RRDp::cmd qq(update "$rrx_name" N:$s_num:$l_num);
    my $answer = RRDp::read;
    if ( $$answer ) {
       error ("unable to update $rrx_name: answer: $$answer");
       exit(1);
    }
#print "updated $rrx_name, servers $s_num, lpars $l_num\n";

# month graph
my $where_is_graph = "$webdir/$hmc/lpars_count_m.png";
if ( ! -d "$webdir/$hmc" ) {
    error ("unable to find hmc directory $webdir/$hmc");
    exit(1);
}
my  $start_rrd = -(86400*31);
my  $l=$act_time;
    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
#    my $t ="COMMENT:Updated\\: $l ";
RRDp::cmd qq(graph "$where_is_graph"
        "--title" "$hmc totals : last month"
        "--start" "$start_rrd"
        "--end" "now"
        "--imgformat" "PNG"
        "--slope-mode"
        "--width=400"
        "--height=130"
        "--lower-limit=0"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--alt-autoscale-max"
        "--interlaced"
        "--font=DEFAULT:8:"
        "--font=TITLE:10:"
        "--vertical-label=Quantity"
        "--units-exponent=1.00"
        "--alt-y-grid"
"DEF:roundtrip=$rrx_name:servers:MAX"
"DEF:roundtrip1=$rrx_name:lpars:MAX"
"DEF:rts=$rrx_name:servers:LAST"
"DEF:rtl=$rrx_name:lpars:LAST"
"DEF:rtsa=$rrx_name:servers:AVERAGE"
"DEF:rtla=$rrx_name:lpars:AVERAGE"
"COMMENT:          "
"COMMENT:average  "
"COMMENT:maximum  "
"COMMENT:last  \\s"
"COMMENT:\\s"
"LINE1:roundtrip#FF0000:servers"
"GPRINT:rtsa:AVERAGE:%8.0lf"
"GPRINT:roundtrip:MAX:%9.0lf"
"GPRINT:rts:LAST:%6.0lf\\s"
"COMMENT:\\s"
"LINE1:roundtrip1#0000FF:lpars  "
"GPRINT:rtla:AVERAGE:%8.0lf"
"GPRINT:roundtrip1:MAX:%9.0lf"
"GPRINT:rtl:LAST:%6.0lf\\s"
"COMMENT:\\s"
"COMMENT:\\s"
"COMMENT:Updated\\: $l "
);
      $answer = RRDp::read;
      if ( $$answer =~ "ERROR" ) {
         error ( "Graph rrdtool error : $$answer");
        }
my $status = RRDp::end;
#print "graph $where_is_graph the answer is ,$answer, $$answer,  status je ,$status\n";
print "creating graph :$where_is_graph\n";
#}
# year graph
   $where_is_graph = "$webdir/$hmc/lpars_count_y.png";
if ( ! -d "$webdir/$hmc" ) {
    error ("unable to find hmc directory $webdir/$hmc");
    exit(1);
}
    $start_rrd = -(86400*365);
    $l=$act_time;
    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
#    my $t ="COMMENT:Updated\\: $l ";
RRDp::start "$rrdtool";
RRDp::cmd qq(graph "$where_is_graph"
        "--title" "$hmc totals : last year"
        "--start" "$start_rrd"
        "--end" "now"
        "--imgformat" "PNG"
        "--slope-mode"
        "--width=400"
        "--height=130"
        "--lower-limit=0"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--alt-autoscale-max"
        "--interlaced"
        "--font=DEFAULT:8:"
        "--font=TITLE:10:"
        "--vertical-label=Quantity"
        "--units-exponent=1.00"
        "--alt-y-grid"
"DEF:roundtrip=$rrx_name:servers:MAX"
"DEF:roundtrip1=$rrx_name:lpars:MAX"
"DEF:rts=$rrx_name:servers:LAST"
"DEF:rtl=$rrx_name:lpars:LAST"
"DEF:rtsa=$rrx_name:servers:AVERAGE"
"DEF:rtla=$rrx_name:lpars:AVERAGE"
"COMMENT:          "
"COMMENT:average  "
"COMMENT:maximum\\s"
"COMMENT:\\s"
"LINE1:roundtrip#FF0000:servers"
"GPRINT:rtsa:AVERAGE:%8.0lf"
"GPRINT:roundtrip:MAX:%9.0lf\\s"
"COMMENT:\\s"
"LINE1:roundtrip1#0000FF:lpars  "
"GPRINT:rtla:AVERAGE:%8.0lf"
"GPRINT:roundtrip1:MAX:%9.0lf\\s"
"COMMENT:\\s"
"COMMENT:\\s"
"COMMENT:Updated\\: $l "
);
      $answer = RRDp::read;
      if ( $$answer =~ "ERROR" ) {
         error ( "Graph rrdtool error : $$answer");
        }
   $status = RRDp::end;
#print "graph $where_is_graph the answer is ,$answer, $$answer,  status je ,$status\n";
print "creating graph :$where_is_graph\n";
}
# error handling
sub error
{
  my $text = shift;
  my $act_time = localtime();
  chomp ($text);
  
 # print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
}

