#!/usr/bin/perl


  #use strict;
  # no strict as definition of @inp does not work with it, o idea why ....

  #use lib qw (/opt/freeware/lib/perl/5.8.0);
  # no longer need to use "use lib qw" as the library PATH is already in PERL5LIB (lpar2rrd.cfg)

  use Env qw(QUERY_STRING);
  use Date::Parse;
  use Switch;


  #$QUERY_STRING .= ":.";

  my $inputdir = $ENV{INPUTDIR};
  my $webdir = $ENV{WEBDIR};
  my $wrkdir = "$inputdir/data";
  my $DEBUG = $ENV{DEBUG};
  my $errlog = $ENV{ERRLOG};

  my $bindir = $ENV{BINDIR};

  print_header();

  open(OUT, ">> $errlog") if $DEBUG == 2 ;

  (my $ftype) = split(/&/,$QUERY_STRING);

  $ftype =~ s/name=//;
  $ftype =~ s/:\.$//;

  my $done = 0;

  if ( $ftype =~ m/maincfg/ )  { print_plain ($ftype,"lpar2rrd.cfg","LPAR2RRD configuration file: ","",); $done=1; }
  if ( $ftype =~ m/favcfg/  )  { print_valid ($ftype,"favourites.cfg","Favourites configuration file: ","",); $done=1; }
  if ( $ftype =~ m/custcfg/ )  { print_valid ($ftype,"custom_groups.cfg","Custom Groups configuration file:","",); $done=1; }
  if ( $ftype =~ m/alrtcfg/ )  { print_valid ($ftype,"alert.cfg","Alert configuration file:","",); $done=1; }
  if ( $ftype =~ m/alhist/  )  { print_log   ($ftype,"alert_history.log","Alert history log (last 500 rows):","",); $done=1; }
  if ( $ftype =~ m/errlog/  )  { print_log   ($ftype,"error.log","LPAR2RRD error log (last 500 rows):","",); $done=1; }
  if ( $ftype =~ m/errcgi/  )  { print_log   ($ftype,"error-cgi.log","LPAR2RRD cgi-bin error log (last 500 rows):","",); $done=1; }
  if ( $ftype =~ m/entitle/ )  { print_log   ($ftype,"entitle-check.log","CPU Configuration Advisor log (last 1000 rows):","<a href=\"http://www.lpar2rrd.com/cpu_configuration_advisor.html\">How it works</a>"); $done=1; }

  if ( $done == 0 ) {
    error ("wrong parameter: $ftype : $QUERY_STRING");
  }

  print_trailer ();

  exit 0;



sub print_plain
{
  my $ftype = shift;
  my $file_base = shift;
  my $text = shift;
  my $file = $inputdir."/etc/".$file_base;

  if ( ! -f "$file" ) {
    print "<CENTER><B>$text $file</B></CENTER>";
    print "<B>File: $file does not exist</B>";
    return 0;
  }

  open(FH, "< $file") || error ("$0: Can't open $file") && return 0;
  print "<CENTER><B>$text $file</B></CENTER><PRE>";
  foreach my $line (<FH>) {
    if ( $line =~ m/^export / ) {
      next; # exclude exporting variables lines
    }
    print "$line";
  }
  return 1;
}

sub print_valid
{
  my $ftype = shift;
  my $file_base = shift;
  my $text = shift;
  my $file = $inputdir."/etc/".$file_base;

  if ( ! -f "$file" ) {
    print "<CENTER><B>$text $file</B></CENTER>";
    print "<B>File: $file does not exist</B>";
    return 0;
  }

  open(FH, "< $file") || error ("$0: Can't open $file") && return 0;
  print "<CENTER><B>$text $file</B></CENTER><PRE>";
  foreach my $line (<FH>) {
    if ( $line =~ m/^#LPAR/ || $line =~ m/^#POOL/ ) {
      next; # print only valid lines
    }
    if ( $ftype =~ m/custcfg/ || $ftype =~ m/favcfg/ ) {
      if ( $line =~ m/^LPAR/ || $line =~ m/^POOL/ ) {
        my $group = "";
        (my $type, my $server, my $lpar, $group) = split(/:/,$line);
        if ( $group =~ m/^$/ ) {
          next; # print only valid lines
        }
      }
    }
    print "$line";
  }
  return 1;
}

sub print_log
{
  my $ftype = shift;
  my $file_base = shift;
  my $text = shift;
  my $text_tail = shift;
  my $file = $inputdir."/logs/".$file_base;

  if ( ! -f "$file" ) {
    print "<CENTER><B>$text $file $text_tail</B></CENTER>";
    print "<B>File: $file does not exist</B>";
    return 0;
  }

  open(FH, "< $file") || error ("$0: Can't open $file") && return 0;
  print "<CENTER><B>$text $file $text_tail</B></CENTER><PRE>";

  my @lines = reverse <FH>;
  my $count = 0;
  foreach my $line (@lines) {
    $count++;
    if ( $count == 500 ) {
      last;
    }
    print "$line";
  }
  return 1;
}

sub print_trailer
{

  print "</pre></body></html>\n";

  return 1;
}

sub print_header
{

# print HTML header
print "Content-type: text/html

<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">
<HTML>
<HEAD>
  <TITLE>LPAR2RRD</TITLE>
  <META HTTP-EQUIV=\"pragma\" CONTENT=\"no-cache\">
  <META HTTP-EQUIV=\"Expires\" CONTENT=\"NOW\">
  <META HTTP-EQUIV=\"last modified\" CONTENT=\"NOW\">
<style>
<!--
a {text-decoration: none}
-->
</style>
</HEAD>
<BODY BGCOLOR=\"#D3D2D2\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#0000FF\" ALINK=\"#FF0000\" >
";

return 0;
}

# error handling
sub error
{
  my $text = shift;
  my $act_time = localtime();
  chomp ($text);

  print "<pre>\n\nERROR          : $text : $!\n</pre>";
  print STDERR "$act_time: $text : $!\n";

  return 1;
}

