#!/usr/bin/perl

use strict;
use Date::Parse;

my $DEBUG = $ENV{DEBUG};
my $errlog = $ENV{ERRLOG};
my $xport = $ENV{EXPORT_TO_CSV};
open(OUT, ">> $errlog")  if $DEBUG == 2 ;
my $lpm = $ENV{LPM};
my $basedir = $ENV{INPUTDIR};
my $wrkdir = $basedir."/data";

# set unbuffered stdout
$| = 1;

# print HTML header
print "Content-type: text/html\n\n";
#  print_head(0,$referer);
#foreach my $key (sort keys(%ENV)) {
  #print "$key = $ENV{$key}";
#} 

# get QUERY_STRING
use Env qw(QUERY_STRING);
print OUT "99 $QUERY_STRING\n" if $DEBUG == 2 ;
#print STDERR "99 $QUERY_STRING\n";

my $shour = "";
my $sday = "";
my $smon = "";
my $syear = "";
my $ehour = "";
my $eday = "";
my $emon = "";
my $eyear = "";
my $type = "";
my $height = "";
my $width = "";
my $yaxis = "";
my $host = "";
my $server = "";
my $referer = "na";
my @lpar_row = "";

# just for backward compatability if users use historical reporting for whatever
if ( $QUERY_STRING =~ m/referer=/ ) {
  # new way
  ($shour, $sday, $smon, $syear, $ehour, $eday, $emon,$eyear,$type, $height, $width, $yaxis, $host, $server, $referer, @lpar_row) = split(/&/,$QUERY_STRING);
}
else {
  ($shour, $sday, $smon, $syear, $ehour, $eday, $emon,$eyear,$type, $height, $width, $yaxis, $host, $server, @lpar_row) = split(/&/,$QUERY_STRING);
}

# no URL decode here, as it goes immediately into URL again

$host =~ s/HMC=//;
$server =~ s/MNAME=//;
$smon =~ s/start-mon=//;
$sday =~ s/start-day=//;
$shour =~ s/start-hour=//;
$syear =~ s/start-yr=//;
$emon =~ s/end-mon=//;
$eday =~ s/end-day=//;
$ehour =~ s/end-hour=//;
$eyear =~ s/end-yr=//;
$type =~ s/type=//;
$height =~ s/\+//;
$height =~ s/HEIGHT=//;
$width =~ s/\+//;
$width =~ s/WIDTH=//;
$yaxis =~ s/yaxis=//;
$referer =~ s/referer=//; # HTTP_REFERER
$referer  =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
$referer  =~ s/\+/ /g;

#`echo "$shour, $sday, $smon, $syear, $ehour, $eday, $emon,$eyear,$type, $height, $width, $yaxis, $host, $server, $referer" >/tmp/xx3`;
my $host_url = $host;
$host  =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
$host  =~ s/\+/ /g;
my $server_url = $server;
$server =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
$server =~ s/\+/ /g;
my $detail_yes = 1;
my $detail_no  = 0;
my $entitle = 1; # --PH dodelat



if ( $yaxis =~ m/r/ || $yaxis =~ m/s/ ) {
  print_head(0,$referer);
  not_implemented ();
  exit 0;
}

my $start=$syear.$smon.$sday;
my $end=$eyear.$emon.$eday;
my $human_start=$shour.":00:00 ".$sday.".".$smon.".".$syear;
my $human_end=$ehour.":00:00 ".$eday.".".$emon.".".$eyear;

my $start_unix = str2time($syear."-".$smon."-".$sday." ".$shour.":00:00");
my $end_unix = "";

# workaround for 24:00. If is used proper 00:00 of the next day then there are 2 extra records in cvs after the midnight
# looks like rrdtool issue
if ( $ehour == 24 ) {
  $end_unix = str2time($eyear."-".$emon."-".$eday." 23:59:00");
}
else {
  $end_unix = str2time($eyear."-".$emon."-".$eday." ".$ehour.":00:00");
}

my $l=length($start_unix);
print OUT "$human_start : $human_end : $start_unix : $end_unix : $l \n" if $DEBUG == 2 ;


if (length($start_unix) < 1) {
  print_head(0,$referer);
  print "<center><br>Start date (<B>$sday.$smon.$syear</B>) does not seem to be valid";
  print "</center></BODY></HTML>";
  exit (0);
}

if (length($end_unix) < 1) {
  print_head(0,$referer);
  print "<center><br>End date (<B>$eday.$emon.$eyear</B>) does not seem to be valid";
  print "</center></BODY></HTML>";
  exit (0);
}

if ($end_unix <= $start_unix) {
  print_head(0,$referer);
  print "<center><br>Start (<B>$human_start</B>) should be less than end (<B>$human_end</B>)";
  print "</center></BODY></HTML>";
  exit (0);
}

# check if agent data  --> then tabs in place
my $agent = 0;
my $agent_oscpu = 0;
my $agent_ame = 0;
my $agent_pgs = 0;
my $agent_mem = 0;
my $agent_san = 0;
my $agent_lan = 0;
my $agent_sea = 0;
foreach my $lpar (@lpar_row) {
  #print STDERR "088 $lpar :$lpar : \n";
  if ( $lpar =~ m/^LPAR=/ ) {
    my $lpar_tmp = $lpar; # must be like that to do not modify the original script
    $lpar =~ s/LPAR=//;
    $lpar =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
    $lpar =~ s/\+/ /g;
    my $lpar_slash = $lpar; #must be in separate env!!
    $lpar_slash =~ s/\//\&\&1/g; # replace for "/"
    if ( ! -d "$wrkdir/$server/$host/$lpar_slash" ) {
      $lpar = $lpar_tmp;
      next;
    }
    $agent++;
    if (  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","cpu" )) {
     $agent_oscpu++; 
    }
    if (  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","mem" )) {
     $agent_mem++; 
    }
    if (  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","pgs" )) {
      $agent_pgs++;
    }
    if (  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","ame" )) {
      $agent_ame++;
    }
    if (  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","san" )) {
      $agent_san++;
    }
    if (  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","lan" )) {
      $agent_lan++;
    }
    if (  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","sea" )) {
      $agent_sea++;
    }
    $lpar = $lpar_tmp;
    #print STDERR "004 1 $lpar_tmp :$lpar\n";
  }
}

my $html_base = "";
if ( $agent > 0) {
  $html_base = print_head(1,$referer);
}
else {
  $html_base = print_head(0,$referer);
}

if ( $agent > 0 ) {
  print "<div  id=\"tabs\"> <ul>\n";
  print "  <li><a href=\"#tabs-0\">CPU</a></li>\n";
  if ( $agent_oscpu > 0 ) {
    print "  <li><a href=\"#tabs-1\">CPU OS</a></li>\n";
  }
  if ( $agent_mem > 0 ) {
    print "  <li><a href=\"#tabs-2\">Memory</a></li>\n";
  }
  if ( $agent_pgs > 0 ) {
    print "  <li><a href=\"#tabs-3\">Paging 1</a></li>\n";
    print "  <li><a href=\"#tabs-4\">Paging 2</a></li>\n";
  }
  if ( $agent_lan > 0 ) {
    print "  <li><a href=\"#tabs-5\">LAN</a></li>\n";
  }
  if ( $agent_san > 0 ) {
    print "  <li><a href=\"#tabs-6\">SAN</a></li>\n";
    print "  <li><a href=\"#tabs-7\">SAN IOPS</a></li>\n";
  }
  if ( $agent_sea > 0 ) {
    print "  <li><a href=\"#tabs-8\">SEA</a></li>\n";
  }
  if ( $agent_ame > 0 ) {
    print "  <li><a href=\"#tabs-9\">AME</a></li>\n";
  }
  print "   </ul> \n";
}

#
# CPU
#

if ( $agent > 0) {
  print "<div id=\"tabs-0\">\n";
}

print "<table align=\"center\" summary=\"Graphs\">\n";

# Loop per each choosen lpar/CPU pool/Memory
foreach my $lpar (@lpar_row) {
  #print STDERR "099 $lpar :$lpar : \n";
  if ( $lpar =~ m/^LPAR=/ ) {
    my $lpar_tmp = $lpar; # must be like that to do not modify the original script
    $lpar =~ s/LPAR=//;
    #$lpar =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
    #$lpar =~ s/\+/ /g;
    
    my $lpar_url  = $lpar;
    #$lpar_url =~ s/\&\&1/\//g; # replace for "/"
    $lpar_url =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;
    #$lpar_slash =~ 's/ /%20/g' ;
    #$lpar_slash =~ 's/\#/\%23/g';
    my $host_url = $host;
    $host_url =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;
    my $server_url = $server;
    $server_url =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;

    print "<tr><td><img class=\"lazy\" data-src=\"/lpar2rrd-cgi/lpar2rrd-rep.sh?hmc=$host_url&mname=$server_url&lpar=$lpar_url&shour=$shour&smon=$smon&sday=$sday&syear=$syear&ehour=$ehour&emon=$emon&eday=$eday&eyear=$eyear&type=$type&height=$height&width=$width&yaxis=$yaxis&xport=0&none=none\" src=\"$html_base/jquery/images/loading.gif\"></td>\n";
    if ( $xport == 1 ) {
      print "<td valign=\"top\"><font size=-1><A HREF=\"/lpar2rrd-cgi/lpar2rrd-rep.sh?host=$host_url&mname=$server_url&lpar=$lpar_url&shour=$shour&smon=$smon&sday=$sday&syear=$syear&ehour=$ehour&emon=$emon&eday=$eday&eyear=$eyear&type=$type&height=$height&width=$width&yaxis=$yaxis&xport=1&none=none\">CSV</A></font></td></tr>\n";
    }
    else {
      print "</tr>\n";
    }
    $lpar = $lpar_tmp;
  }
}
print "</table>\n";

if ( $agent > 0) {
  print "</div>\n";
}
else {
  # Exit, no agent in place
  print "</BODY></HTML>";
  exit (0);
}

#
# CPU OS
#

if ( $agent_oscpu > 0 ) {
  print "<div id=\"tabs-1\">\n";
  my $item="oscpu";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
}

#
# OS mem
#

if ( $agent_mem > 0 ) {
  print "<div id=\"tabs-2\">\n";
  my $item="mem";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
}

#
# OS paging
#

if ( $agent_pgs > 0 ) {
  print "<div id=\"tabs-3\">\n";
  my $item="pg1";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
  print "<div id=\"tabs-4\">\n";
  my $item="pg2";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
}

#
# OS Ethernet
#

if ( $agent_lan > 0 ) {
  print "<div id=\"tabs-5\">\n";
  my $item="lan";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
}

#
# OS FC
#

if ( $agent_san > 0 ) {
  print "<div id=\"tabs-6\">\n";
  my $item="san1";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
  print "<div id=\"tabs-7\">\n";
  my $item="san2";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
}

#
# OS SEA
#

if ( $agent_sea > 0 ) {
  print "<div id=\"tabs-8\">\n";
  my $item="sea";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
}


#
# OS AME
#

if ( $agent_ame > 0 ) {
  print "<div id=\"tabs-9\">\n";
  my $item="ame";
  agent($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,\@lpar_row);
  print "</div>\n";
}


print "<br></BODY></HTML>";
close(OUT)  if $DEBUG == 2;

exit (0);

sub not_implemented {
  print "<br><br><center><strong>rPerf ans SAPs scaling is not implemented yet</strong>\n";
  print "</body></html>";

  return 0;
}


sub agent {
  my ($host_url,$server_url,$item,$type,$entitle,$detail_yes,$start_unix,$end_unix,$html_base,$lpar_row_tmp) = @_;
  my @lpar_row = @{$lpar_row_tmp};
  my $type_sam = "x"; # whatever, it is not significant for OS agent graphs

  
  print "<table align=\"center\" summary=\"$item\">\n";

  foreach my $lpar_tmp (@lpar_row) {
    my $lpar = $lpar_tmp;
    if ( $lpar =~ "LPAR=" ) {
      $lpar =~ s/LPAR=//;
      $lpar =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
      $lpar =~ s/\+/ /g;
      my $lpar_slash = $lpar;
      $lpar_slash =~ s/\//\&\&1/g; # replace for "/"
      my $lpar_url = $lpar;
      #print STDERR "005 $lpar: $wrkdir/$server/$host/$lpar_slash.mmm : lpar_url: $lpar_url\n";

      if ( -d "$wrkdir/$server/$host/$lpar_slash" ) {
        if ( $item =~ m/^oscpu$/ && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","cpu" )) {
          print_item ($host_url,$server_url,$lpar_url,$item,$type,$type_sam,$entitle,$detail_yes,$start_unix,$end_unix,$html_base);
          next;
        }
        if ( $item =~ m/^mem$/ && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","mem" )) {
          print_item ($host_url,$server_url,$lpar_url,$item,$type,$type_sam,$entitle,$detail_yes,$start_unix,$end_unix,$html_base);
          next;
        }
        if ( $item =~ m/^pg/ &&  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","pgs" )) {
          print_item ($host_url,$server_url,$lpar_url,$item,$type,$type_sam,$entitle,$detail_yes,$start_unix,$end_unix,$html_base);
          next;
        }
        if ( $item =~ m/^lan$/ &&  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","lan" )) {
          print_item ($host_url,$server_url,$lpar_url,$item,$type,$type_sam,$entitle,$detail_yes,$start_unix,$end_unix,$html_base);
          next;
        }
        if ( $item =~ m/^san/ &&  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","san" )) {
          print_item ($host_url,$server_url,$lpar_url,$item,$type,$type_sam,$entitle,$detail_yes,$start_unix,$end_unix,$html_base);
          next;
        }
        if ( $item =~ m/^sea$/ &&  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","sea" )) {
          print_item ($host_url,$server_url,$lpar_url,$item,$type,$type_sam,$entitle,$detail_yes,$start_unix,$end_unix,$html_base);
          next;
        }
        if ( $item =~ m/^ame$/ &&  test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash","ame" )) {
          print_item ($host_url,$server_url,$lpar_url,$item,$type,$type_sam,$entitle,$detail_yes,$start_unix,$end_unix,$html_base);
          next;
        }
      } 
    }
  }

  print "</table>\n";
  return 1;
}

sub type_sam
{
  my $host = shift;
  my $server = shift;
  my $lpar = shift;
  my $type = shift;

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\//\&\&1/g; # replace for "/"
  if ( $type =~ m/d/ ) {
    if ( -f "$wrkdir/$server/$host/$lpar_slash.rrd" ) {
      return "d";
    }
    return "m";
  }

  if ( -f "$wrkdir/$server/$host/$lpar_slash.rrm" ) {
    return "m";
  }

  if ( -f "$wrkdir/$server/$host/$lpar_slash.rrh" ) {
    return "h";
  }

  return "m";
}


# print a link with detail
sub print_item
{
  my ($host_url,$server_url,$lpar_url,$item,$time_graph,$type_sam,$entitle,$detail,$start_unix,$end_unix,$html_base) = @_;
  my $upper = 0; # same limit for all 4 graphs, not used in hist graphs

  $lpar_url =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;
  $host_url =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;
  $server_url =~s/([^A-Za-z0-9\+-_])/sprintf("%%%02X",ord($1))/seg;

  #print STDERR "88 $lpar_url\n";
  if ( $detail > 0 ) {
    print "<tr><td valign=\"top\"><a href=\"/lpar2rrd-cgi/detail-graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=1&upper=$upper&entitle=$entitle&sunix=$start_unix&eunix=$end_unix&height=$height&width=$width&none=none\" onClick=\"return popup(this, \'detail\')\"><font color=\"#C8C8C8\"><img class=\"lazy\" border=\"0\" data-src=\"/lpar2rrd-cgi/detail-graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=0&upper=$upper&entitle=$entitle&sunix=$start_unix&eunix=$end_unix&height=$height&width=$width&none=none\" src=\"$html_base/jquery/images/loading.gif\" ></font></a></td></tr>\n";
  }
  else {
    print "<tr><td align=\"center\" valign=\"top\" colspan=\"2\"><font color=\"#C8C8C8\"><img class=\"lazy\" border=\"0\" data-src=\"/lpar2rrd-cgi/detail-graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=0&upper=$upper&entitle=$entitle&sunix=$start_unix&eunix=$end_unix&height=$height&width=$width&none=none\"src=\"$html_base/jquery/images/loading.gif\" ></td></tr>\n";
  }
  return 1;
}

sub print_head
{
  my $agent = shift;
  my $referer = shift;

   my $html_base = html_base($referer); 


# print HTML header
print "<HTML><HEAD>
<META HTTP-EQUIV=\"pragma\" CONTENT=\"no-cache\">
<META HTTP-EQUIV=\"Expires\" CONTENT=\"NOW\">
<META HTTP-EQUIV=\"last modified\" CONTENT=\"NOW\">
<STYLE TYPE=\"text/css\">
<!--
.header, .header TD, .header TH
{
background-color:#D3D2F3;
}
-->
</STYLE>
<style>
<!--
a {text-decoration: none}
-->
</style>
<SCRIPT TYPE=\"text/javascript\">
<!--
function popup(mylink, windowname)
{
if (! window.focus)return true;
var href;
if (typeof(mylink) == 'string')
   href=mylink;
else
   href=mylink.href;
window.open(href, windowname, 'width=1329,height=670,scrollbars=yes');
return false;
}
//-->
</SCRIPT>
<link rel=\"stylesheet\" href=\"$html_base/jquery/jquery-ui-1.10.3.custom.min.css\">
<link rel=\"stylesheet\" href=\"$html_base/jquery/magnific-popup.css\">

<script src=\"$html_base/jquery/jquery-1.9.1.js\"></script>
<script src=\"$html_base/jquery/jquery-ui-1.10.3.custom.min.js\"></script>
<script src=\"$html_base/jquery/jquery.lazy.min.js\"></script>
<script src='$html_base/jquery/magnific-popup.min.js'></script>
<script>
\$( document ).ready(function() {
        \$( \"#tabs\" ).tabs({
                activate: function() {
                        \$(\"img.lazy\").lazy({
                                bind: \"event\",
                                effect: \"fadeIn\",
                                effectTime: 1000
                                });
                }
        });
        \$( \"img.lazy\" ).lazy({
                        effect: \"fadeIn\",
                        effectTime: 1000
                        }
        );

        \$( 'a.detail' ).magnificPopup({
                type: 'image',
                closeOnContentClick: true,
                closeBtnInside: false,
                fixedContentPos: true,
                mainClass: 'mfp-no-margins mfp-with-zoom', // class to remove default margin from left and right side
                image: {
                        verticalFit: true
                },
                zoom: {
                        enabled: true,
                        duration: 300 // don't foget to change the duration also in CSS
                }
        });
});


</script>


</HEAD>
<BODY BGCOLOR=\"#D3D2D2\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#0000FF\" ALINK=\"#FF0000\" >";

return $html_base;
}

sub html_base
{
    # Print link to full lpar cfg (must find out at first html_base
    # find out HTML_BASE
    # renove from the path last 3 things
    # http://nim.praha.cz.ibm.com/lpar2rrd/hmc1/PWR6B-9117-MMA-SN103B5C0%20ttt/pool/top.html
    # --> http://nim.praha.cz.ibm.com/lpar2rrd
    # my $refer = $ENV{HTTP_REFERER}; --> no, no, here is cgi-bin referer already
    my $referer = shift;
    my $html_base = "";
    my $base = 0;

    if ( $referer !~ m/^na$/ ) { # when is http refer pasing then it is enough ... via lpar search
      my @full_path = split(/\//, $referer);
      my $k = 0;
      foreach my $path (@full_path){
        $k++
      }
      $k--;
      $k--;
      $k--;
      my $j = 0;
      foreach my $path (@full_path){
        if ($j < $k) {
          if ( $j == 0 ) {
            $html_base .= $path;
          }
          else {
            $html_base .= "/".$path;
          }
          $j++;
        }
      }
    }
    else {
     $html_base = $referer;
    }
    return $html_base;
}

sub test_file_in_directory
{
# Use a regular expression to find files
#    beginning by $fpn
#    ending by .mmm
#    returns 0 (zero) or first filename found i.e. non zero

 my $dir=shift;
 my $fpn=shift;

 my $found = 0;
 if ( !-d $dir ) { return $found;}
 opendir (DIR, $dir) || error("Error in opening dir $dir $! :".__FILE__.":".__LINE__) && return 0;

 while (my $file = readdir(DIR)) {
     if ($file =~ m/^$fpn.*\.mmm$/) {
        #if ($file =~ m/\.mmm$/) {
           $found = "$dir/$file";
           last;
        #}
     }
 }
 closedir (DIR);
 return $found;
}

