#!/bin/ksh

PATH=$PATH:/usr/bin:/bin:/usr/local/bin:/opt/freeware/bin
export PATH

LANG=C
export LANG

# it is necessary as files need to be readable also for WEB server user
umask 022

# Load LPAR2RRD configuration
. `dirname $0`/etc/lpar2rrd.cfg

# Load "magic" setup
if [ -f `dirname $0`/etc/.magic ]; then
  . `dirname $0`/etc/.magic
fi

if [ "$INPUTDIR" = "." ]; then
   INPUTDIR=`pwd`
   export INPUTDIR
fi


#
# when WEBDIR_PUBLIC is defined then it creates a new (second) web root for public users (on demand extention for a customer)
# (without entitlement area in lpar graphs)
# run it from cron as follows
# 30 * * * * export WEBDIR_PUBLIC=/home/lpar2rrd/lpar2rrd/www_public; /home/lpar2rrd/lpar2rrd/load.sh > /home/lpar2rrd/lpar2rrd/load.out 2>&1 
#

ENTITLEMENT_LESS=0
TMPDIR=$INPUTDIR/tmp
if [ ! "$WEBDIR_PUBLIC"x = "x" ]; then
  ENTITLEMENT_LESS=1
  WEBDIR=$WEBDIR_PUBLIC
  TMPDIR=$WEBDIR-tmp # set up new separate $TMPDIR
  echo "Creating public web without entitlements here: $WEBDIR_PUBLIC"
  if [ ! -d "$WEBDIR_PUBLIC" ]; then
    mkdir $WEBDIR_PUBLIC 
  fi
  if [ ! -d "$TMPDIR" ]; then
    mkdir $TMPDIR
  fi
fi
export ENTITLEMENT_LESS WEBDIR TMPDIR

# ./load.sh html --> it runs only web creation code
HTML=0
if [ ! "$1"x = "x" -a "$1" = "html" ]; then
  HTML=1
  touch $TMPDIR/$version-run # force to run html part
  LPM=0 # switch off LPM search
fi

prg_name=`basename $0`
if [ -f "$TMPDIR/$prg_name.pid" ]; then
  PID=`cat "$TMPDIR/$prg_name.pid"|sed 's/ //g'`
  if [ ! "$PID"x = "x" ]; then 
    ps -ef|grep "$prg_name"|egrep -v "vi |vim "|awk '{print $2}'|egrep "^$PID$" >/dev/null
    if [ $? -eq 0 ]; then
      echo "There is already running another copy of $prg_name, exiting ..."
      d=`date`
      echo "$d: There is already running another copy of $prg_name, exiting ..." >> $ERRLOG
      exit 1
    fi
  fi
fi
# ok, there is no other copy of $prg_name
echo "$$" > "$TMPDIR/$prg_name.pid"


HOSTNAME=`uname -n`
export HOSTNAME

ID=`( lsattr -El sys0 -a systemid 2>/dev/null || hostid ) | sed 's/^.*,//' | awk '{print $1}'`
UN=`uname -a`
UNAME=`echo "$UN $ID"`
export UNAME

PERL5LIB=$PERL5LIB:$BINDIR
export PERL5LIB

UPGRADE=0
if [ ! -f $TMPDIR/$version ]; then
  if [ $DEBUG -gt 0 ]; then 
    echo ""
    echo "Looks like there has been an upgrade to $version or new installation"
    echo "Run time will be significantly longer this time"
    echo ""
    sleep 3
  fi
  UPGRADE=1
fi
export UPGRADE  # for lpar2rrd.pl and trends especially


cd $INPUTDIR

# Check if it runs under the right user
install_user=lpar2rrd
running_user=`id |awk -F\( '{print $2}'|awk -F\) '{print $1}'`
if [ ! "$install_user" = "$running_user" ]; then
  echo "You probably trying to run it under wrong user" 
  echo "LPAR2RRD files are owned by : $install_user"
  echo "You are : $running_user"
  echo "LPAR2RRD should run only under user which owns installed package"
  echo "Do you want to really continue? [n]:"
  read answer
  if [ "$answer"x = "x" -o "$answer" = "n" -o "$answer" = "N" ]; then
    exit
  fi
fi

if [ -f $ERRLOG ]; then
  ERR_START=`wc -l $ERRLOG |awk '{print $1}'`
else
  ERR_START=0
fi 

# Checks
if [ ! -f "$RRDTOOL" ]; then
  echo "Set correct path to RRDTOOL binary in lpar2rrd.cfg, it does not exist here: $RRDTOOL"
  exit 0
fi 
ok=0
for i in `echo "$PERL5LIB"|sed 's/:/ /g'` 
do
  if [ -f "$i/RRDp.pm" ]; then
    ok=1
  fi
done
if [ $ok -eq 0 ]; then
  echo "Set correct path to RRDp.pm Perl module in lpar2rrd.cfg"
  echo "it does not exist here : $PERL5LIB"
  echo ""
  echo "1. Assure it is installed rrdtool-perl module"
  echo "2. try to find it "
  echo "   find /usr -name RRDp.pm"
  echo "   find /opt -name RRDp.pm"
  echo "3. place path (directory) to etc/lpar2rrd.cfg, parametr PERL5LIB"
  exit 0
fi
if [ ! -f "$PERL" ]; then
  echo "Set correct path to Perl binary in lpar2rrd.cfg, it does not exist here: $PERL"
  exit 0
fi 
if [ x"$RRDHEIGHT" = "x" ]; then
  echo "RRDHEIGHT does not seem to be set up, correct it in lpar2rrd.cfg"
  exit 0
fi
if [ x"$RRDWIDTH" = "x" ]; then
  echo "RRDWIDTH does not seem to be set up, correct it in lpar2rrd.cfg"
  exit 0
fi
if [ x"$SAMPLE_RATE" = "x" ]; then
  echo "SAMPLE_RATE does not seem to be set up, correct it in lpar2rrd.cfg"
  exit 0
fi
if [ x"$HMC_USER" = "x" ]; then
  echo "HMC_USER does not seem to be set up, correct it in lpar2rrd.cfg"
  exit 0
fi
if [ x"$HMC_LIST" = "x" ]; then
  echo "HMC_LIST does not seem to be set up, correct it in lpar2rrd.cfg"
  exit 0
fi
if [ x"$DEBUG" = "x" ]; then
  echo "DEBUG does not seem to be set up, correct it in lpar2rrd.cfg"
  exit 0
fi
if [ ! -d "$WEBDIR" ]; then
  echo "Set correct path to WEBDIR in lpar2rrd.cfg, it does not exist here: $WEBDIR"
  exit 0
fi 
if [ x"$MANAGED_SYSTEMS_EXCLUDE" = "x" ]; then
  MANAGED_SYSTEMS_EXCLUDE=""
  export MANAGED_SYSTEMS_EXCLUDE
fi 

if [ "$HMC_LIST" = "hmc1 sdmc1 ivm1" ]; then
  echo "Set your HMC/IVM/SDMC list, parameter HMC_LIST in etc/lpar2rrd.cfg"
  exit 0
fi

PID_DAEMON="999999999"
if [ ! x"$LPAR2RRD_AGENT_DAEMON" = "x" -a ! x"$LPAR2RRD_AGENT_DAEMON_PORT" = "x" ]; then
  if [ $LPAR2RRD_AGENT_DAEMON -eq 1 ]; then
    # Start up LPAR2RRD agent daemon for receiving mem stats directly from LPARs
    # if it is already running then do nothing
    if [ -f "$TMPDIR/lpar2rrd-daemon.pid" ]; then
      PID=`cat "$TMPDIR/lpar2rrd-daemon.pid"|sed 's/ //g'`
      if [ ! "$PID"x = "x" ]; then 
        run=`ps -ef|grep lpar2rrd-daemon.pl| egrep -v "grep |vi | vim "|grep $PID|wc -l`
        if [ $run -eq 0 ]; then
          # start up the daemon
          echo "Starting LPAR2RRD daemon on port:$LPAR2RRD_AGENT_DAEMON_PORT "
          nohup $PERL $INPUTDIR/bin/lpar2rrd-daemon.pl 2>>$ERRLOG-daemon 1>>$INPUTDIR/logs/daemon.out &
          sleep 2 # must be here to get time for writing PID into $TMPDIR/lpar2rrd-daemon.pid
        fi
      fi
    else
      echo "Starting LPAR2RRD daemon on port:$LPAR2RRD_AGENT_DAEMON_PORT "
      nohup $PERL $INPUTDIR/bin/lpar2rrd-daemon.pl 2>>$ERRLOG-daemon 1>>$INPUTDIR/logs/daemon.out &
      sleep 2 # must be here to get time for writing PID into $TMPDIR/lpar2rrd-daemon.pid
    fi
    if [ -f "$TMPDIR/lpar2rrd-daemon.pid" ]; then
      PID_DAEMON=`cat "$TMPDIR/lpar2rrd-daemon.pid"|sed 's/ //g'`
    fi
    if [ `ps -ef|grep -v grep|grep $PID_DAEMON|wc -l` -eq 0 -o `ps -ef|grep -v grep|grep lpar2rrd-daemon|wc -l|sed 's/ //g'` -eq 0 ]; then
      echo "LPAR2RRD daemon is not running"
      sleep 1
    fi
  fi
fi 


#
# Load data from HMC(s) and create graphs
#

# you can set HMC_PARALLEL_RUN in etc/.magic to persist upgrades
if [ "$HMC_PARALLEL_RUN"x = "x" ]; then
  HMC_PARALLEL_RUN=1 # when > 1 then HMC are processed concurrently
fi


for HMC in $HMC_LIST
do
  export HMC
  if [ $HTML -eq 1 ]; then continue; fi # let run only web part

  if [ $HMC_PARALLEL_RUN -gt 1 ]; then
    while [ `jobs -l|grep -iv stop|grep -v $PID_DAEMON|wc -l` -ge $HMC_PARALLEL_RUN ];
    do
      echo "waiting ...."
      sleep 5
    done
  fi


  if [ $HMC_PARALLEL_RUN -eq 1 ]; then
    $PERL -w $BINDIR/lpar2rrd.pl 2>>$ERRLOG
  else
    $PERL -w $BINDIR/lpar2rrd.pl 2>>$ERRLOG  &
  fi
done

# wait until all HMC jobs finish
if [ $HMC_PARALLEL_RUN -gt 1 ]; then
  echo "jobs -l        : " # just print jobs status
  jobs -l

  for pid in `jobs -l |sed -e 's/+//g' -e 's/-//g'|awk '{print $2}'|egrep -iv "stop|daemon|rrdtool"`
  do
    if [ -f "$TMPDIR/lpar2rrd-daemon.pid" -a $pid -eq `cat $TMPDIR/lpar2rrd-daemon.pid` ]; then
      echo "Skipping daemon: $pid"
      continue
    fi
    echo "Waiting for HMC: $pid"
    wait $pid
  done
fi


#
# Create custom graphs
#

if [ $HTML -eq 0 ]; then
  $PERL -w $BINDIR/custom.pl 2>>$ERRLOG 
  ret=$?
  if [ ! $ret -eq 0 ]; then
    DATE=`date`
    echo "$DATE: an error in $PERL -w $BINDIR/custom.pl: $ret" >> $ERRLOG
  fi
fi

#
# Create favourites links 
#

if [ $HTML -eq 0 ]; then
  $PERL -w $BINDIR/fav.pl 2>>$ERRLOG 
  ret=$?
  if [ ! $ret -eq 0 ]; then
    DATE=`date`
    echo "$DATE: an error in $PERL -w $BINDIR/fav.pl: $ret" >> $ERRLOG
  fi
fi

#
# Check of lpars which CPU utilization overcome their CPU limits
# It runs just once a day (first run after the midnight
#
$PERL -w $BINDIR/max-check.pl 2>>$ERRLOG


#
# Check RMC connectivity of all lpars
# It runs just once a day (first run after the midnight)
#
$PERL -w $BINDIR/rmc.pl 2>>$ERRLOG


#
# Daily lpar data check
# It runs just once a day (first run after the midnight)
#
$PERL -w $BINDIR/daily_lpar_check.pl 2>>$ERRLOG

#
# Daily lpar count
#
$PERL -w $BINDIR/daily_lpars_count.pl 2>>$ERRLOG


#
#
# Install/Update web frontend
#

$BINDIR/install-html.sh 
ret=$?
if [ ! $ret -eq 0 ]; then
  echo "`date` : An error occured in install-html.sh, return code: $ret" >> $ERRLOG
fi




#
# Error handling
# 

if [ -f $ERRLOG ]; then
  ERR_END=`wc -l $ERRLOG |awk '{print $1}'`
else
  ERR_END=0
fi
ERR_TOT=`expr $ERR_END - $ERR_START`
if [ $ERR_TOT -gt 0 ]; then
  echo "An error occured in lpar2rrd, check $ERRLOG and output of load.sh"
  echo ""
  echo "$ tail -$ERR_TOT $ERRLOG"
  echo ""
  tail -$ERR_TOT $ERRLOG
  #date >> $ERRLOG
fi

if [ -d "$INPUTDIR/logs" ]; then
  if [ -f "$INPUTDIR/load.out" ]; then
    cp -p "$INPUTDIR/load.out" "$INPUTDIR/logs"
  fi
fi


# user defined script if needed
if [ `ls -l $INPUTDIR/bin/user_script*.sh 2>/dev/null|wc -l` -gt 0 ]; then
  for uscript in $INPUTDIR/bin/user_script*.sh 
  do
    echo "Start user script: $uscript"
    $uscript
  done 
fi

edate=`date`
echo "date end all   : $edate"
exit 0
