#!/bin/ksh
# TODO
# - prefix ";" in lpar pool names by "\:"


if [ ! -d "etc" ]; then
  if [ -d "../etc" ]; then
    cd ..
  else
    echo "problem with actual directory, assure you are in LPAR2RRD home"
    exit
  fi
fi

LHOME=`pwd|sed -e 's/scripts//' -e 's/\/\//\//' -e 's/\/$//'` # it must be here
pwd=`pwd`
cd data
cfg="$pwd/etc/alert.cfg"

DEFAULT=":1.0:0:::"

if [ -f "$cfg" ]; then
  echo "Alerting configuration update is running:"
  echo "  $cfg"
  echo ""
else
  echo "Alerting initial configuration is running:"
  echo "  $cfg"
  echo ""

cat << END > $cfg
# LPAR2RRD alert configuration file
# You can define here alerts based on CPU utilization of CPU pools or lpars
# Alerts are send via email, in the future also via SNMP TRAP
# Note this is not going to fully replace tools you already have in place for CPU alerting
# It is rather enhanced functionality especially for critical CPU pools and lpars
# It might have impact on performance of hosted lpar if this is used for hundreds of lpars or CPU pools
# for activation you have to add following job into your cron
END

echo "#   0,10,20,30,40,50 * * * * $LHOME/load_alert.sh > $LHOME/load_alert.out 2>&1" >> $cfg

cat << END >> $cfg

# here will be sent operation or configuration problems with alerting, not ALERTS!
EMAIL_ADMIN=lpar2rrd_admin@whatever.com

# alert history log
END

echo "ALERT_HISTORY=$LHOME/logs/alert_history.log" >> $cfg

cat << END >> $cfg

# use external script for alerting 
# it will be called once an alarm appears with those 7 parameters
# script.sh  [Critical|Warning]  [POOL|LPAR]  $server$  [$lpar$|$pool$]  $util_actual$  $util_max$  $util_min$
# you can use $LHOME/bin/external_alert_example.sh as an example
#EXTERN_ALERT=$LHOME/bin/external_alert_example.sh

# Nagios alerting
# Call this script from nrpe.cfg: $LHOME/bin/check_lpar2rrd
# More details on http://www.lpar2rrd.com/nagios.html
NAGIOS=0	# [0/1] on/off Nagios support for alerting

# include graphs into the email notification 0 - false, last 1 - X hours included in the graphs
EMAIL_GRAPH=8

# email groups, there are not strictly predefined names, use your ones, just start with "EMAIL_" except EMAIL_ADMIN name
# you can even directly place email address instead of the group name to LPAR/POOL alarm directives
#EMAIL=test@whatever.com         # default email group, used when no email group is specified for the directive
#EMAIL_GROUP1=dba@whatever.com,unix@whatever.com

# SNMP traps 
# not implemented yet
TRAP=your_snmp_trap_server
TRAP_UUID=XY

# default time in minutes which says how often you should be alerted
# you can specify per lpar or pool different value in "alert repeat time" column per each ALERT
REPEAT_DEFAULT=60

# default time in minutes for length of CPU peek 
# (the time when avg CPU utilization must be above of specified limit to generate an alert)
# you can change it per lpar/poll level in "time in min" column per each ALERT
# note it should not be shorter than time of scheduling alert script from crontab (10 minutes by default)
PEEK_TIME_DEFAULT=10 

# List of POOLs and LPARs below is created by the script :
#   scripts/update_cfg_alert.sh
# You can run it whenever to refresh list of your POOLs and LPARs

# default warning alert in percents
# will be generated if CPU utilization reaches CPU_WARNING in %
CPU_WARNING_ALERT=90


#Bellow example after uncommenting alerts for every server and lpar 
# if paging activity goes above 10kBytes per second in 10 minute average.
#Alerts will be send to email lpar2rrd@lpar2rrd.com
# You need to activate LPAR2RRD agents for SWAP alerting!!! http://www.lpar2rrd.com/agent.htm

#SWAP:server:lpar name:swapping in kB/sec::peek time in min:alert repeat time in min:email group
#========================================================================================================================
#SWAP:.*:.*:10:::lpar2rrd@lpar2rrd.com


#[LPAR|POOL]:server:[lpar name|pool name]:CPU util MAX:CPU util MIN:peek time in min:alert repeat time in min:email group
#========================================================================================================================

# all lines below were generated by scripts/update_cfg_alert.sh
# to add new server/lpar/pool use this script which does that automatically

END
fi

for server_space in `ls|sort|sed 's/ /+===============+/g'`
do
  server=`echo $server_space|sed 's/+===============+/ /g'`
  if [ -L "$server" ]; then
    continue
  fi
  if [ -d "$server" ]; then
    if [ ! "$1" = "update" ]; then
      echo "Checking $server" 1>&2
    fi
    cd "$server"
    egrep "POOL:$server:all_pools:" $cfg >/dev/null
    if [ ! $? -eq 0 ]; then
      echo "#POOL:$server:all_pools$DEFAULT"
    fi
    for lpar_full_space in `find . -mtime -5 -name "*.rr[h|m]"|sed 's/ /+===============+/g'` # takes only lpars/pools which have been updated in last 5 days
    do
      lpar_full=`echo $lpar_full_space|sed 's/+===============+/ /g'`
      lparm=`basename "$lpar_full" .rrm`
      lpar_slash=`basename "$lparm" .rrh`
      lpar=`echo $lpar_slash| sed 's/&&1/\//g'`
      if [ "$lpar"x = "x" -o "$lpar" = ".rrm" -o "$lpar" = ".rrh" ]; then
        continue # NULL lpar name 
      fi
      if [ "$lpar" = "pool" -o "$lpar" = "mem-pool" -o "$lpar" = "mem" ]; then
        continue # pool is already echoed before
      fi
      echo "$lpar"|egrep SharedPool >/dev/null
      if [ $? -eq 0 ]; then
        # it is a pool, now map it to name
        pool_inx=`echo "$lpar"|sed 's/SharedPool//'`
        pool=`egrep "^$pool_inx," */cpu-pools-mapping.txt|head -1|sed -e 's/.*cpu-pools-mapping\.txt://' -e 's/^[0-9],//' -e 's/^[0-9][0-9],//'`
        egrep "POOL:$server:$pool:" $cfg >/dev/null
        if [ ! $? -eq 0 ]; then
          echo "#1POOL:$server:$pool$DEFAULT"
        fi
      else
        egrep "LPAR:$server:$lpar:" $cfg >/dev/null
        if [ ! $? -eq 0 ]; then
          echo "#2LPAR:$server:$lpar$DEFAULT"
        fi
      fi
    done|sort|sed 's/#[1,2]/#/g'|uniq 
    echo ""
    cd ../
  fi
done >> $cfg
