#!/bin/ksh
#
# TODO
# - prefix ";" in lpar pool names by "\:"

if [ ! -d "etc" ]; then
  if [ -d "../etc" ]; then
    cd ..
  else
    echo "problem with actual directory, assure you are in LPAR2RRD home"
    exit
  fi
fi

LHOME=`pwd|sed -e 's/scripts//' -e 's/\/\//\//' -e 's/\/$//'` # it must be here
pwd=`pwd`
cd data
cfg="$pwd/etc/custom_groups.cfg"

if [ -f $cfg ]; then
  echo "Custom groups configuration update is running:"
  echo "  $cfg"
  echo ""

else 
  echo "Custom groups initial configuration is running:"
  echo "  $cfg"
  echo ""

cat << END > $cfg
# 
# LPAR2RRD custom groups configuration file
#
# It is a configuration file for creating custom groups of lpars
# they are shown separately in the gui and aggregated
# typical usage is for clusters or for lpars running one application 
#
# Usage
# below you see simple lpar and CPU pool list. 
# If you want to create a group called "cluster_one"
# then simply after the last double quote place your name of the group.
# When a lpar/pool needs to belong to more groups then create a new row with a new group name
# LPAR2RRD applies changes during the next run.
# You will see the new group under "Custom" page on the web
#
# There are no restrictions for group names, just name them as you like
#
# If you need to use a colon in lpar/pool/group name then prefix it by backslash
# Rows without group name are ignored
#
# You can use wild cards (regular expressions) in server name and pool/lpar name
# Examples
#  POOL:.*:.*:pool-all   --> group pool-all will contain all pools from all servers
#  LPAR:.*:.*:lpar-all   --> group lpar-all will contain all lpars from all servers
#  LPAR:p[6,7].*:vio.*:vio-p67  --> group vio-p67 will contain lpars started vio* from servers started with p6 or p7 string
#
# Note that special charaters in server/pool/lpar names like "*" or "."  must be prefixed with backslash like here "\."
#
# This file has been created by cmd:
#   scripts/update_cfg_custom-groups.sh
# To update it about new servers/pools/lpars simply run this script
# It keeps actual setup and appends here newly found servers/pools/lpars
#
# [POOL|LPAR]:server:[lpar_name|pool_name]:your_group_name
#=========================================================

END
fi


for server_space in `ls|sort|sed 's/ /+===============+/g'`
do
  
  server=`echo $server_space|sed 's/+===============+/ /g'`
  if [ -L "$server" ]; then
    continue
  fi

  if [ -d "$server" ]; then
    if [ ! "$1" = "update" ]; then
      echo "Checking for $server" 1>&2
    fi
    cd "$server"
    egrep "POOL:$server:all_pools:" $cfg >/dev/null
    if [ ! $? -eq 0 ]; then
      echo "POOL:$server:all_pools:"
    fi
    for lpar_full_space in `find . -mtime -5 -name "*.rr[h|m]"|sed 's/ /+===============+/g'` # takes only lpars/pools which have been updated in last 5 days
    do
      lpar_full=`echo $lpar_full_space|sed 's/+===============+/ /g'`
      lparm=`basename "$lpar_full" .rrm`
      lpar_slash=`basename "$lparm" .rrh`
      lpar=`echo $lpar_slash| sed 's/&&1/\//g'`
      if [ "$lpar"x = "x" -o "$lpar" = ".rrm" -o "$lpar" = ".rrh" ]; then
        continue # NULL lpar name
      fi
      if [ "$lpar" = "pool" -o "$lpar" = "mem-pool" -o "$lpar" = "mem" ]; then
        continue # pool is already echoed before
      fi
      echo "$lpar"|egrep SharedPool >/dev/null 2>&1
      if [ $? -eq 0 ]; then
        # it is a pool, now map it to name
        pool_inx=`echo "$lpar"|sed 's/SharedPool//'`
        pool=`egrep "^$pool_inx," */cpu-pools-mapping.txt|head -1|sed -e 's/.*cpu-pools-mapping\.txt://' -e 's/^[0-9],//' -e 's/^[0-9][0-9],//'`
        egrep "POOL:$server:$pool:" $cfg >/dev/null
        if [ ! $? -eq 0 ]; then
          echo "#1POOL:$server:$pool:"
        fi
      else
        lpar_name=`echo "$lpar"|sed 's/:/\\:/g'` # ":" in lpar names to "\:"
        egrep "LPAR:$server:$lpar_name:" $cfg >/dev/null
        if [ ! $? -eq 0 ]; then
          echo "#2LPAR:$server:$lpar_name:"
        fi
      fi
    done|sort|sed 's/^#[1,2]//g'|uniq
    echo ""
    cd ../
  fi
done >> $cfg

