#!/usr/bin/perl
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


  use strict;
  use Date::Parse;
  use Math::BigInt;
  use File::Compare;
  use LoadDataModule;
  use File::Copy;


  #use lib qw (/opt/freeware/lib/perl/5.8.0);
  # no longer need to use "use lib qw" as the library PATH is already in PERL5LIB env var (lpar2rrd.cfg)


  # set unbuffered stdout
  $| = 1;

  my $fast_processing = 1;	# do not create lpar graph in advance, greate then on demand via cgi-bin

  # get cmd line params
  my $version="$ENV{version}";
  my $host = $ENV{HMC};
  my $hea = $ENV{HEA};
  my $hmc_user = $ENV{HMC_USER};
  my $webdir = $ENV{WEBDIR};
  my $bindir = $ENV{BINDIR};
  my $basedir = $ENV{INPUTDIR};
  my $tmpdir = "$basedir/tmp";
  if (defined $ENV{TMPDIR}) {
    $tmpdir = $ENV{TMPDIR};
  }
  my $rrdtool = $ENV{RRDTOOL};
  my $DEBUG = $ENV{DEBUG};
  my $pic_col = $ENV{PICTURE_COLOR};
  my $STEP = $ENV{SAMPLE_RATE};
  my $HWINFO = $ENV{HWINFO};
  my $CONFIG_HISTORY = $basedir."/data"; # do not change that as subdir tree is not created automatically here but via main loop
  my $managed_systems_exclude =  $ENV{MANAGED_SYSTEMS_EXCLUDE};
  my $SYS_CHANGE = $ENV{SYS_CHANGE};
  my $STEP_HEA=$ENV{STEP_HEA};;
  my $upgrade=$ENV{UPGRADE};;
  my $SSH = $ENV{SSH}." -q -o ConnectTimeout=80 -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey";
  # do not use: PasswordAuthentication=no due to reflection ssh which does not support that!!!
  # uncoment&adjust if you want to use your own ssh identification file
  #my $SSH = $ENV{SSH}." -i ".$ENV{HOME}."/.ssh/lpar2rrd -o ConnectTimeout=5 -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey";
  my $lpm = $ENV{LPM};
  my $h = $ENV{HOSTNAME};
  my $new_change="$tmpdir/$version-run";

  #print "++ $host $hmc_user $basedir $webdir $STEP\n";
  my $wrkdir = "$basedir/data"; 

  if ( -f "$bindir/premium.pl" ) {
    require "$bindir/premium.pl";
  }
  else {
    require "$bindir/standard.pl";
  }

  # Global definitions
  my $input = "";
  my $input_mem = "";
  my $input_pool = "";
  my $input_pool_mem = "";
  my $input_pool_sh = "";
  my $input_cod = "";
  my $input_shell = "";
  my $input_mem_shell = "";
  my $input_pool_shell = "";
  my $input_pool_mem_shell = "";
  my $input_pool_sh_shell = "";
  my $input_cod_shell = "";
  my $pool_list_file = "pool_list.txt";
  my $loadhours = "";
  my $loadmins  = "";
  my $type_sam = "";
  my $managedname = "";
  my $step = "";
  my $NO_TIME_MULTIPLY = 6; # increased since 3.63
  my $no_time = $STEP * $NO_TIME_MULTIPLY; # says the time interval when RRDTOOL consideres a gap in input data 6 mins now!
  # looks like 60sec sample rate is stored on HMC just for last 2 days (there in no doc so far)
  # INIT_LOAD_IN_HOURS_BACK should be generally high enough  (one year back =~ 9000 hours), it is in hours!!!
  my $INIT_LOAD_IN_HOURS_BACK="9000";
  #my $INIT_LOAD_IN_HOURS_BACK="49";
  my $YEAR_REFRESH = 72000 ;  # 20 hour, minimum time in sec when yearly graphs are updated (refreshed)
  my $MONTH_REFRESH = 20000 ; # 5 hour, minimum time in sec when monthly graphs are updated (refreshed)
  my $WEEK_REFRESH = 7000;    # minimum time in sec when weekly  graphs are updated (refreshed)
  my $UPDATE_DAILY=86400;     # for update config (documentation) files
  my $CFG_REFRESH = 7000;     # update of lpar cfg
  my $PARALLELIZATION = 5;
  #$UPDATE_DAILY=60;
  my @pool_list = "";
  # Random colors for disk charts
  my @managednamelist_un="";
  my @managednamelist="";
  my $HMC=1;	# if HMC then 1, if IVM/SDMC then 0
  my $SDMC=0;
  my $IVM=0;
  my $FSM=0; # so far only for setting lslparutil AMS parames
  my @lpar_trans = "";  # lpar translation names/ids for IVM systems
  my $MAX_ROWS = 10000;
  my $MAX_ROWS_WORKAROUND = 100000; # some HMCs have a problem with lslparutil -s d -r lpar/sys -h XY, solution is to put there 
				    # one more argument -n whatever_high_enough
				    # HMC returns either HSCL8016 or "No resupts found"
  my $restrict_rows = " -n $MAX_ROWS"; # workaround for HMC < 7.2.3 to do not exhaust memory
  my $timerange = " --minutes 120 "; # it must be global var, placing there some default just to be sure ...

  # last timestamp files --> must be for each load separated
  my $last_file="last.txt";
  my $last_file_pool="last-pool.txt";
  my $last_file_mem="last-mem.txt";
  my $last_file_sh_mem="last-sh-mem.txt";
  my $last_file_sh_pool="last-sh-pool.txt";
  my $last_file_cod="last-cod.txt";
  my $last_rec = "";
  my $sec = "";
  my $ivmmin = "";
  my $ivmh = "";
  my $ivmd = "";
  my $ivmm = "";
  my $ivmy = "";
  my $wday = "";
  my $yday = "";
  my $isdst = "";

  my $server_count = 0;
  my @pid = "";
  my $cycle_count = 1;
  my @poollistagg = "";
  my $DELETED_LPARS_TIME = 8640000; # 10 days
  my @lpm_excl_vio = "";
  my $run_topten = 0;
  my $view_suff= "view";
  my $mem_params = "";
  my @keep_color_lpar  = "";
  my @keep_color_pools = "";
  my @keep_color_mem = "";
  my @keep_color_ams = "";

  

  if ( $upgrade == 1 ) {
    $YEAR_REFRESH = 0;  # to force creation of trend graphs
  }

  # 48 colors like for HMC bellow + 216 basic HTML colors x 6 == 1296 + 48
  my @color=("#FF0000","#0000FF","#00FF00","#FFFF00","#FF3399","#00FFFF","#999933","#0099CC","#3300CC",
"#FF8080", "#FFFF80", "#80FF80", "#00FF80", "#80FFFF", "#0080FF", "#FF80C0", "#FF80FF", "#FF0000", "#FFFF00", "#80FF00",
"#00FF40", "#00FFFF", "#0080C0", "#8080C0", "#FF00FF", "#804040", "#FF8040", "#00FF00", "#008080", "#004080", "#8080FF",
"#800040", "#FF0080", "#800000", "#FF8000", "#008000", "#008040", "#0000FF", "#0000A0", "#800080", "#8000FF", "#400000",
"#804000", "#004000", "#004040", "#000080", "#000040", "#400040", "#400080", "#000000", "#808000", "#808040", "#808080",
"#408080", "#C0C0C0", "#400040", "#AAAAAA",
"#CC0000","#FF0000", "#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#000000","#FFFF00","#333300","#FFFFFF","#330000","#999990","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600",
"#CC0000","#FF0000","#CC00CC","#FF00CC","#0000CC","#0000FF","#00CCCC","#00FFFF","#00CC00","#00FF00",
"#CCCC00","#FFFF00","#000000","#999999","#FFFFFF","#330000","#333300","#336600","#FFCC00","#339900",
"#FF9900","#33CC00","#FF6600","#33FF00","#FF3300","#66FF00","#660000","#663300","#66CC00","#669900",
"#666600","#330033","#FFFF33","#333333","#FF0033","#336633","#FFCC33","#339933","#FF9933","#33CC33",
"#FF6633","#33FF33","#FF3333","#66FF33","#660033","#663333","#66CC33","#669933","#666633","#330066",
"#FFFF66","#333366","#FF0066","#336666","#FFCC66","#339966","#FF9966","#33CC66","#FF6666","#33FF66",
"#FF3366","#66FF66","#660066","#663366","#66CC66","#669966","#666666","#330099","#FFFF99","#333399",
"#FF0099","#336699","#FFCC99","#339999","#FF9999","#33CC99","#FF6699","#33FF99","#FF3399","#66FF99",
"#660099","#663399","#66CC99","#669999","#666699","#3300CC","#FFFFCC","#3333CC","#3366CC","#FFCCCC",
"#3399CC","#FF99CC","#33CCCC","#FF66CC","#33FFCC","#FF33CC","#66FFCC","#6600CC","#6633CC","#66CCCC",
"#6699CC","#6666CC","#3300FF","#FFFFFF","#3333FF","#FF00FF","#3366FF","#FFCCFF","#3399FF","#FF99FF",
"#33CCFF","#FF66FF","#33FFFF","#FF33FF","#66FFFF","#6600FF","#6633FF","#66CCFF","#6699FF","#6666FF",
"#CCFFFF","#0033FF","#CC00FF","#0066FF","#CCCCFF","#0099FF","#CC99FF","#00CCFF","#CC66FF","#CC33FF",
"#99FFFF","#9900FF","#9933FF","#99CCFF","#9999FF","#9966FF","#CCFFCC","#0033CC","#0066CC","#CCCCCC",
"#0099CC","#CC99CC","#CC66CC","#00FFCC","#CC33CC","#99FFCC","#9900CC","#9933CC","#99CCCC","#9999CC",
"#9966CC","#000099","#CCFF99","#003399","#CC0099","#006699","#CCCC99","#009999","#CC9999","#00CC99",
"#CC6699","#00FF99","#CC3399","#99FF99","#990099","#993399","#99CC99","#999999","#996699","#000066",
"#CCFF66","#003366","#CC0066","#006666","#CCCC66","#009966","#CC9966","#00CC66","#CC6666","#00FF66",
"#CC3366","#99FF66","#990066","#993366","#99CC66","#999966","#996666","#000033","#CCFF33","#003333",
"#CC0033","#006633","#CCCC33","#009933","#CC9933","#00CC33","#CC6633","#00FF33","#CC3333","#99FF33",
"#990033","#993333","#99CC33","#999933","#996633","#CCFF00","#003300","#006600","#009900","#CC9900",
"#CC6600","#CC3300","#99FF00","#990000","#993300","#99CC00","#999900","#996600");

  # 48 colors for CPU totaly per HMC (there is general limit max 48 managed systems per HMC so it is ok)
  my @color_hmc=("#FF8080", "#FFFF80", "#80FF80", "#00FF80", "#80FFFF", "#0080FF", "#FF80C0", "#FF80FF", "#FF0000", "#FFFF00", "#80FF00",
"#00FF40", "#00FFFF", "#0080C0", "#8080C0", "#FF00FF", "#804040", "#FF8040", "#00FF00", "#008080", "#004080", "#8080FF",
"#800040", "#FF0080", "#800000", "#FF8000", "#008000", "#008040", "#0000FF", "#0000A0", "#800080", "#8000FF", "#400000",
"#804000", "#004000", "#004040", "#000080", "#000040", "#400040", "#400080", "#000000", "#808000", "#808040", "#808080",
"#408080", "#C0C0C0", "#400040", "#AAAAAA");

  my @color_lpar="";
  my $color_indx = 0;

  my @fav_lpar = "";
  my @fav_server = "";
  
  load_favourites (); # load favourites.cfg into mem (graph of lpars in favorites has to be created regulary)

  my $ret = graph_without_load_data ();
  if ( $ret == 1 ) {
    exit 0
  }

  my $prem = premium ();
  print "LPAR2RRD $prem version $version\n" if $DEBUG ;
  print "Host           : $h\n" if $DEBUG ;
  print "HMC            : $host\n" if $DEBUG ;
  my $date = "";
  my $act_time = localtime();
  print "date start     : $host $act_time\n" if $DEBUG ;


  if ( ! -d "$webdir" ) {
    error(" Pls set correct path to Web server pages, it does not exist here: $webdir".__FILE__.":".__LINE__) && return 0;
  }

  cfg_config_change ();

  # start RRD via a pipe
  use RRDp;
  RRDp::start "$rrdtool";

  print "RRDTool version: $RRDp::VERSION \n";
  #my $perlv= $^V;
  #my $perlv2= $];
  print "Perl version   : $] \n";

  load_hmc();
  # close RRD pipe
  RRDp::end;

  $date = localtime();
  print "date end       : $host $date\n" if $DEBUG ;

  exit (0);

sub load_hmc  {
 
  my $timeout=120;
  my $model = "";
  my $serial = "";
  my $line = "";
  my $hmcv = "";

  # set alarm on first SSH command to make sure it does not hang
  eval {
    local $SIG{ALRM} = sub {die "$act_time: died in SIG ALRM";};
    alarm($timeout);
    # get list of serveres managed through HMC
    @managednamelist_un=`$SSH $hmc_user\@$host "lssyscfg -r sys -F name,type_model,serial_num" 2>\&1`;
    alarm (0);

    if ($managednamelist_un[0] eq '' ) {
      error ("HMC/SDMC/IVM name: $host either has not been resolved or ssh key based access is not allowed or other comunnication error");
      exit (1);
    }

    if ($managednamelist_un[0] =~ "no address associated with hostname" || $managednamelist_un[0] =~ "Could not resolve hostname" ) {
      error ("HMC/SDMC/IVM : $managednamelist_un[0]");
      exit (1);
    }

    # sorting non case sensitive
    @managednamelist = sort { lc($a) cmp lc($b) } @managednamelist_un;
  };

  if ($@) {
    if ($@ =~ /died in SIG ALRM/) {
      error ("$SSH $hmc_user\@$host command timed out after : $timeout seconds\nCheck why it takes so long, might be some network or authentification problems on the HMC\n# ssh $hmc_user\@$host \"lssyscfg -r sys -F name\"\nContinue with the other HMCs");
      exit (1);
    }
    else {
      error ("lpar2rrd failed: $@");
      exit (1);
    }
  }

  my $managed_ok; 
  my $managedname_exl = "";
  my @m_excl = "";
  my $once = 0;
  my $hmcv_num = "";

  # get data for all managed system which are conected to HMC
  foreach $line (@managednamelist) {
    chomp($line);

    if ( $line =~ m/Error:/ || $line =~ m/Permission denied/ ) {
      error ("problem connecting to $host : $line");
      exit (1);
    }

    if ( $line !~ ".*,.*,.*" ){
      # it must look like : PWR6A-9117-MMA-SN103A4B0,9117-MMA,103A4B0, --> exclude banners
      next;
    }

    if ($line =~ /No results were found/) {
      print "$host does not contain any managed system\n" if $DEBUG ;
      return 0;
    }
    ($managedname, $model, $serial) = split(/,/,$line);

    if ( is_IP($managedname) == 1 ) {
      next; # wrong entry from the HMC, a problem of the HMC or unconfigured server yet
    }

    print "managed system : $host:$managedname (type_model*serial : $model*$serial)\n" if $DEBUG ;

    rename_server($host,$managedname,$model,$serial);
   
    # create sym link model*serial for recognizing of renamin managed systems
    # it must be here due to skipping some server (exclude, not running utill collection) and saving cfg
    if (! -d "$wrkdir" ) {
      print "mkdir          : $host:$managedname $wrkdir\n" if $DEBUG ;
      LoadDataModule::touch ("$host:$managedname $wrkdir");
      mkdir("$wrkdir", 0755) || error(" Cannot mkdir $wrkdir: $!".__FILE__.":".__LINE__) && return 0;
    } 

    if (! -d "$wrkdir/$managedname" ) {
      print "mkdir          : $host:$managedname $wrkdir/$managedname\n" if $DEBUG ;
      LoadDataModule::touch ("$wrkdir/$managedname");
      mkdir("$wrkdir/$managedname", 0755) || error(" Cannot mkdir $wrkdir/$managedname: $!".__FILE__.":".__LINE__) && return 0;
    } 

    if (! -l "$wrkdir/$model*$serial" ) {
      print "ln -s          : $host:$managedname $wrkdir/$managedname $wrkdir/$model*$serial \n" if $DEBUG ;
      LoadDataModule::touch ("$wrkdir/$model*$serial");
      symlink("$wrkdir/$managedname","$wrkdir/$model*$serial") || error(" Cannot ln -s $wrkdir/$managedname $wrkdir/$model*$serial: $!".__FILE__.":".__LINE__) && return 0;
    }

    if (! -d "$wrkdir/$managedname/$host" ) {
      print "mkdir          : $host:$managedname $wrkdir/$managedname/$host\n" if $DEBUG ;
      LoadDataModule::touch ("$wrkdir/$managedname/$host");
      mkdir("$wrkdir/$managedname/$host", 0755) || error(" Cannot mkdir $wrkdir/$managedname/$host: $!".__FILE__.":".__LINE__) && return 0;
    } 

    $managed_ok=1;
    @pool_list = ""; # clean pool_list for each managed name
    if ( $managed_systems_exclude ne '' ) {
      @m_excl = split(/:/,$managed_systems_exclude);
      foreach $managedname_exl (@m_excl) {
        chomp($managedname_exl);
        if ( $managedname =~ m/^$managedname_exl$/ ) {
          $managed_ok=0; 
        }
      }
    }
    if ( $managed_ok == 0 ) {
      print "managed system : $host:$managedname is excluded in load.sh, continuing with the others ...\n" if $DEBUG ;
      save_cfg_data ($managedname,localtime(),$upgrade); # it is necessary to have all server in cfg page
      next;
    }


    # Check whether utilization data collection is enabled
    $step =`$SSH $hmc_user\@$host "lslparutil -r config -m \\"$managedname\\" -F sample_rate"` ;
    chomp($step);

    if ( $once == 0 ) {
      # Find out version of HMC (just first 3 digits and figure out if 
      # it is bigger than 733 what is the version since are supported
      # enhanced sampe rates
      my @sourcev = `$SSH $hmc_user\@$host "lshmc -v" 2>/dev/null|egrep "RM |DS "|tail -2`;
      #chomp($sourcev[0]);

      my $ivmv=1;
      my $hmcv_print = "";
      foreach my $mmm (@sourcev) {
        $ivmv=0;
      }
      if ( $ivmv == 1 ) {
          $hmcv="0"
      }
      else {
        if ( ! $sourcev[1] eq '' ) {
          $hmcv = "$sourcev[1]";
          chomp($hmcv);
          $hmcv =~ s/\.//g;
          $hmcv =~ s/^\*RM //g;
          $hmcv_print = "$hmcv";
          $hmcv =~ s/V//g;
          $hmcv =~ s/R//g;
        }
        else {
          $hmcv="0"
        }
      }

      if ( ! $sourcev[0] eq '' ) {
        $sourcev[0] =~ s/^\*DS //;
      }

      if ( ! $sourcev[0] eq '' && ( $sourcev[0] =~ m/SDMC/ || $sourcev[0] =~ m/FSM/ )) {
        $HMC = 0; # looks like SDMC system
        $SDMC=1; # lshmc is on new SDMC available
        if ( $sourcev[0] =~ m/SDMC/ ) {
          print "Looks like SDMC: $host:$hmcv\n" if $DEBUG ;
        }
        else {
          $FSM=1;
          print "Looks like FSM : $host:$hmcv\n" if $DEBUG ;
        }
      }

      if ( $ivmv == 1 || length($hmcv) < 3) {
        # old way, keeping it there ....
        $HMC = 0; # looks like IVM system
	$hmcv_num = 700;
	$SDMC=`$SSH $hmc_user\@$host "if test -r /usr/smrshbin ; then echo \"PLATFORM=1\" ; else echo \"PLATFORM=0\" ; fi"|grep PLATFORM`;
        chomp ($SDMC);
        $SDMC =~ s/PLATFORM=//g;
        if ( $SDMC == 1 ) {
          print "Looks like SDMC: $host $hmcv_print\n" if $DEBUG ;
        }
        else {
          $IVM=1;
          print "Looks like IVM : $host $hmcv_print\n" if $DEBUG ;
        }

      }
      else {
        $hmcv =~ s/ //g;
        $hmcv_num = substr($hmcv,0,3);
        #print "DEBUG : $hmcv_num $hmcv\n";
        print "HMC version    : $host $hmcv_print\n" if $DEBUG ;
      }
      $once = 1;

      if ( $HMC == 1 && ($hmcv_num > 734 || $FSM == 1 )) {
        # set params for lslparutil and AMS available from HMC V7R3.4.0 Service Pack 2 (05-21-2009)
        $mem_params = ",mem_mode,curr_mem,phys_run_mem,curr_io_entitled_mem,mapped_io_entitled_mem,mem_overage_cooperation";
      }
      else {
        if ( $HMC == 0 ) {
          $mem_params = ",mem_mode,curr_mem,phys_run_mem,curr_io_entitled_mem,mapped_io_entitled_mem,mem_overage_cooperation";
        }
        else {
          print "HMC version    : $host $hmcv $hmcv_num - no AMS ready : $HMC - $SDMC\n" if $DEBUG ;
        }
      }
    }

    my $check=$HMC+$IVM+$SDMC;
    if ( $check != 1 ) {
      error ("Wrongly identified source, exiting, contact the support: $HMC - $IVM - $SDMC");
      exit (0);
    }

    if ($HMC == 0 ) {
      # IVM/SDMC translation ids to names
      @lpar_trans=`$SSH $hmc_user\@$host "lssyscfg -r lpar -m \\"$managedname\\" -F lpar_id,name" 2>\&1| egrep "^[0-9]"`;
      # save that to lpar_trans.txt
      open FH, ">$wrkdir/$managedname/$host/lpar_trans.txt" or error("can't open '$wrkdir/$managedname/$host/lpar_trans.txt': $!".__FILE__.":".__LINE__) && return 0;
      foreach ( @lpar_trans )
      {
        print FH $_;
      }
      close FH;
    }

    if ( $step != $STEP ) {
       print "*****WARNING*****WARNING*****WARNING*****\n";
       if ( $step == 0 ) {
         print "Utilization data collection is disabled for managed system : $host:$managedname to enable it run : \n";
         print "ssh hscroot\@$host \"chlparutil -r config -m $managedname -s $STEP\"\n";
         print "*****WARNING*****WARNING*****WARNING*****\n";
         # go for next managed system / hmc server
         save_cfg_data ($managedname,localtime(),$upgrade); # it is necessary to have all server in cfg page
         next;    
       }
       else {

	  if ( $hmcv_num >= 733 ) {
             print "Utilization data collection is set to \"$step\" for managed system : $host:$managedname\n";
	     print "lpar2rrd tool is configured for $STEP seconds interval\n";
	     print "Your HMC supports it as its version is higher than 7.3.3\n";
	     print "Ignore this message if you want to use 3600s sample rate anyway\n";
       	     print "ssh hscroot\@$host \"chlparutil -r config -m $managedname -s $STEP\"\n";
             print "*****WARNING*****WARNING*****WARNING*****\n";
             # go for next managed system / hmc server
             #next;    
	  }
       }
    }

      
    if ($IVM == 1) { 
      # create a file "IVM" to be able install_html check whether it is IVM or not
      if ( ! -f "$wrkdir/$managedname/$host/IVM" ) {
        open(FH, "> $wrkdir/$managedname/$host/IVM") || error(" Can't create $wrkdir/$managedname/$host/IVM : $!".__FILE__.":".__LINE__) && return 0;
        print FH "IVM\n";
        close (FH);
      }
    }
    else {
      # just to be sure, sometimes happens than HMC based is signed for ever as IVM due to some error
      if ( -f "$wrkdir/$managedname/$host/IVM" ) {
        unlink("$wrkdir/$managedname/$host/IVM");
      }
    }
    LoadDataModule::smdc_touch ($SDMC,$wrkdir,$managedname,$host,$act_time);

    $input = "$wrkdir/$managedname/$host/in";
    $input_mem = "$wrkdir/$managedname/$host/mem.in";
    $input_pool = "$wrkdir/$managedname/$host/pool.in";
    $input_pool_mem = "$wrkdir/$managedname/$host/mem_sh.in";
    $input_pool_sh = "$wrkdir/$managedname/$host/pool_sh.in";
    $input_cod = "$wrkdir/$managedname/$host/cod.in";
    @color_lpar="";
    $color_indx = 0;

    # _shell variables for usage in shell cmd lines, must be to avoid a roblem with spaces in managed names
    # usage only for shell, means in ssh ...., perl natively has no problem with that
    $input_shell = $input;
    $input_mem_shell = $input_mem;
    $input_pool_shell = $input_pool;
    $input_pool_mem_shell = $input_pool_mem;
    $input_pool_sh_shell = $input_pool_sh;
    $input_cod_shell = $input_cod;
    $input_shell =~ s/ /\\ /g;
    $input_mem_shell =~ s/ /\\ /g;
    $input_pool_shell =~ s/ /\\ /g;
    $input_pool_mem_shell =~ s/ /\\ /g;
    $input_pool_sh_shell =~ s/ /\\ /g;
    $input_cod_shell =~ s/ /\\ /g;

    # allow collection data : chlparutil -r config  -m managed_system -s 3600

    # for 1hour sample rate --> suffix "h", for 1min and other suffix "m"
    if ($step == 3600) {
      $type_sam="h";
    }
    else { 
      $type_sam="m"; 
    }

    # must be here otherwise en error for 1h sample rates when creating RRD DB
    # it was fixed in 2.05
    $no_time = $step * $NO_TIME_MULTIPLY;  

    $loadhours = 0;  # must be here before rrd_check

    rrd_check($managedname);

    print "sample rate    : $host:$managedname $step seconds\n" if $DEBUG ;

    # here must be local time on HMC, need Unix time and date in text to have complete time
    my $date = `$SSH $hmc_user\@$host "export LANG=en_US; date \'+%m/%d/%Y %H:%M:%S\'"`;
    chomp($date);
    my $t=str2time($date); # ignore date +%s on purpose!!!
    #print "DATE: $t -- $date \n" ;

    # set defaults for IVM
    ($sec,$ivmmin,$ivmh,$ivmd,$ivmm,$ivmy,$wday,$yday,$isdst) = localtime($t-432000);  # just last 5 days for IVM
    $ivmy += 1900;
    $ivmm += 1;

    my $last_rec_file = "";

    my $where = "file";
    if (! $loadhours ) { # all except the initial load --> check rrd_check
      if ( -f "$wrkdir/$managedname/$host/$last_file" ) {
        $where = "$last_file";
        # read timestamp of last record 
        # this is main loop how to get corectly timestamp of last record!!!
        open(FHLT, "< $wrkdir/$managedname/$host/$last_file") || error(" Can't open $wrkdir/$managedname/$host/$last_file: $!".__FILE__.":".__LINE__) && return 0;
        foreach my $line1 (<FHLT>) {
          chomp($line1);
          $last_rec_file=$line1 ;
        }
        print "last rec 1     : $host:$managedname $last_rec_file $wrkdir/$managedname/$host/$last_file\n";
        close(FHLT);
	my $ret = substr($last_rec_file,0,1);
        if ($last_rec_file eq '' || $ret =~ /\D/) {
          # in case of an issue with last file, remove it and use default 140 min? for further run ...
          error ("Wrong input data, deleting file : $wrkdir/$managedname/$host/$last_file : $last_rec_file");
          unlink ("$wrkdir/$managedname/$host/$last_file");

          # place ther last 2h when an issue with last.txt
          $loadhours = 2;
          $loadmins  = 120;
          $last_rec = $t-7200;
	}
	else {
          $last_rec=str2time($last_rec_file);
          $loadhours = sprintf ("%.0f", ($t - $last_rec)/3600);
          $loadhours++;
          $loadmins  = sprintf ("%.0f", ($t - $last_rec)/60+5); # +5mins to be sure
        }
      }
      else {
        # old not accurate way how to get last time stamp, keeping it here as backup if above temp fails for any reason
        if ( -f "$wrkdir/$managedname/$host/mem.rr$type_sam" ) {
          $where = "mem.rr$type_sam";
          # find out last record in the db (hourly)
          RRDp::cmd qq(last "$wrkdir/$managedname/$host/mem.rr$type_sam" );
          my $last_rec_raw = RRDp::read;    
          chomp ($$last_rec_raw);
          $last_rec=$$last_rec_raw;
          $loadhours = sprintf ("%.0f", ($t - $last_rec)/3600);
          $loadhours++;
          $loadmins  = sprintf ("%.0f", ($t - $last_rec)/60+5); # +5mins to be sure
	  (my $sec,$ivmmin,$ivmh,$ivmd,$ivmm,$ivmy,my $wday,my $yday,my $isdst) = localtime($last_rec);
	  $ivmy += 1900;
	  $ivmm += 1;
          if ( $loadhours < 0 ) { # Do not know why, but it sometimes happens!!!
            if ( -f "$wrkdir/$managedname/$host/pool.rr$type_sam" ) {
              # find out last record in the db (hourly)
              RRDp::cmd qq(last "$wrkdir/$managedname/$host/pool.rr$type_sam" );
              my $last_rec_raw = RRDp::read;    
              chomp ($$last_rec_raw);
              $last_rec=$$last_rec_raw;
              $loadhours = sprintf ("%.0f", ($t - $last_rec)/3600);
              $loadhours++;
              $loadmins  = sprintf ("%.0f", ($t - $last_rec)/60+5); # +5mins to be sure
              error ("++2 $loadhours -- $last_rec -- $t");
            }
          }
        }
        else {
          if ( -f "$wrkdir/$managedname/$host/pool.rr$type_sam" ) {
            $where = "pool.rr$type_sam";
            # find out last record in the db (hourly)
            RRDp::cmd qq(last "$wrkdir/$managedname/$host/pool.rr$type_sam" );
            my $last_rec_raw = RRDp::read;    
            chomp ($$last_rec_raw);
            $last_rec=$$last_rec_raw;
            $loadhours = sprintf ("%.0f", ($t - $last_rec)/3600);
            $loadhours++;
            $loadmins  = sprintf ("%.0f", ($t - $last_rec)/60+5); # +5mins to be sure
          }
          else {
            $where = "init";
	    $loadmins  = $INIT_LOAD_IN_HOURS_BACK*60;
	    $loadhours = $INIT_LOAD_IN_HOURS_BACK;
	  }
        }
      }
    }
    else {
      $where = "init";
      my $loadsecs = $INIT_LOAD_IN_HOURS_BACK * 3600;
      $last_rec=$t-$loadsecs;
    }

    if ( $loadhours <= 0 || $loadmins <= 0) {	 # something wrong is here
      error ("Last rec issue: $last_file:  $loadhours - $loadmins -  $last_rec -- $last_rec_file : $date : $t : 01");
      # place some reasonable defaults
      $loadhours = 2;
      $loadmins = 120;
      $last_rec = time();
      $last_rec = $last_rec - 7200;
    }

    ($sec,$ivmmin,$ivmh,$ivmd,$ivmm,$ivmy,$wday,$yday,$isdst) = localtime($last_rec);
    $ivmy += 1900;
    $ivmm += 1;
    print "last rec 2     : $host:$managedname min:$loadmins , hour:$loadhours, $ivmm/$ivmd/$ivmy $ivmh:$ivmmin : $where\n" if $DEBUG ;

    my $ret =  save_cfg_data ($managedname,$date,$upgrade);
    if ( $ret == 2 ) {
      next; # server is switched off ???
    }

    lpm_exclude_vio($host,$managedname,$wrkdir);

    #hmc_load_data ($t,$loadhours,$managedname,$sec,$ivmmin,$ivmh,$ivmd,$ivmm,$ivmy,$wday,$yday,$isdst,$last_rec,$t,$hmcv_num);
    hmc_load_data ($t,$managedname,$host,$last_rec,$t,$hmcv_num);
    $date = localtime();
    print "date load      : $host:$managedname $date\n" if $DEBUG ;
   } 
   for (my $j=0; $j < $server_count; $j++) {
     print "Wait for chld  : $host $j \n" if $DEBUG ;
     waitpid($pid[$j],0);
   }
   print "All chld finish: $host \n" if $DEBUG ;

   # Include HW info
   if ( $HWINFO ) {
     get_cfg_server ();  # must be here , it goes through servers itself
   }

   my $time = time();
   # Total CPU per HMC --> must be exactly here
   @managednamelist = sort { lc($a) cmp lc($b) } @managednamelist_un; # just to be sure!!! 
	# against: semi-panic: attempt to dup freed string at ....
   frame_multi ("day","m","d","MINUTE:60:HOUR:2:HOUR:4:0:%H");
   frame_multi ("week","m","w","HOUR:8:DAY:1:DAY:1:86400:%a");
   frame_multi ("4 weeks","m","m","DAY:1:DAY:2:DAY:7:0:%U");
   frame_multi ("year","m","y","MONTH:1:MONTH:1:MONTH:1:0:%b");
   multiview_hmc($host,"lpar-multi","d","m",$time,"day","MINUTE:60:HOUR:2:HOUR:4:0:%H");
   multiview_hmc($host,"lpar-multi","w","m",$time,"week","HOUR:8:DAY:1:DAY:1:86400:%a");
   multiview_hmc($host,"lpar-multi","m","m",$time,"4 weeks","DAY:1:DAY:2:DAY:7:0:%U");
   multiview_hmc($host,"lpar-multi","y","m",$time,"year","MONTH:1:MONTH:1:MONTH:1:0:%b");

   return 0;
}

# This is jus for internal debug purposes
sub graph_without_load_data  {

  return 0;

  if ( $DEBUG == 2 ) {
    $step = 60;
    $STEP = 60;
    # start RRD via a pipe
    use RRDp;
    RRDp::start "$rrdtool";

    # find out existing RRD databases for particular hostname
    opendir(DIR, "$wrkdir/") || error(" directory does not exists : $wrkdir".__FILE__.":".__LINE__) && return 0;
    my @managednames = readdir(DIR);
    closedir(DIR);

    foreach my $managedname_act (@managednames) {
      chomp($managedname_act);
      if ( $managedname_act =~ m/^\./ ) {
        next;
      }
      $managedname = $managedname_act;
      #print "Server :  $managedname\n";
      opendir(DIR, "$wrkdir/$managedname") || error(" directory does not exists : $wrkdir/$managedname".__FILE__.":".__LINE__) && return 0;
      my @hosts = readdir(DIR);
      closedir(DIR);
      foreach my $host_act (@hosts) {
        chomp($host_act);
        if ( $host_act =~ m/^\./ ) {
          next;
        }
        $host = $host_act;
        if ( -f "$wrkdir/$managedname/$host/IVM" ) {
          $IVM=1;
          $HMC=0;
        }
        else {
          $IVM=0;
          $HMC=1;
        }
        print "HMC $host : $managedname : IVM:$IVM HMC:$HMC\n";
        $type_sam="m";
        rrd_find ($managedname,$host);
        $type_sam="d";
        rrd_find ($managedname,$host);
        #hea_run  ($managedname);
        #fcs_run  ($managedname);
      }
    }

    # HMC multi ...
    #@managednamelist = ("POWER520-1,,","POWER520-2,,","POWER520-3,,","POWER520-4,,","POWER750-1,,","POWER770-1,,","POWER770-2,,","POWER770-3,,","POWER770-4,,","POWER770-5,,");
    #my @hosts = ("hmca","hmcb");

    #my $host_act = "";
    #foreach $host_act (@hosts) {
    #  chomp ($host_act);
    #  $host=$host_act;
    #  $type_sam="m";
    #  my $ret_multi = frame_multi ("day","m","d","MINUTE:60:HOUR:2:HOUR:4:0:%H",@managednamelist);
    #  if ( $ret_multi == 0 ) {
    #    frame_multi ("week","m","w","HOUR:8:DAY:1:DAY:1:86400:%a",@managednamelist);
    #    frame_multi ("4 weeks","m","m","DAY:1:DAY:2:DAY:7:0:%U",@managednamelist);
    #    frame_multi ("year","m","y","MONTH:1:MONTH:1:MONTH:1:0:%b",@managednamelist);
    #  }
    #}

    # close RRD pipe
    RRDp::end;
    return 1
  }
  return 0
}

sub load_data_and_graph 
{
  # there is no on purpose checking return code to avoid that a problem with one source affects everything

  LoadDataModule::load_data ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file,$no_time,$SSH,$hmc_user);

  if ( $HMC == 1 || $SDMC == 1 ) { # IVM does nt have it
     LoadDataModule::load_data_pool_sh ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_sh_pool,$input_pool_sh,$SSH,$hmc_user,\@pool_list,$pool_list_file,$no_time);
  }

  LoadDataModule::load_data_pool_mem ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_sh_mem,$input_pool_mem,$no_time); # it has to be always last one !!!

  LoadDataModule::load_data_mem ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_mem,$input_mem,$no_time);

  LoadDataModule::load_data_cod ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_cod,$input_cod,$no_time);

  # Load & crate graphs
  LoadDataModule::load_data_pool ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_pool,$input_pool,$no_time); # it has to be always last one !!!


  rrd_find ($managedname,$host);
  hea_run  ($managedname);
  fcs_run  ($managedname);

    
  return 0;
}

sub hmc_load_data 
{
  my $hmc_utime = shift;
  #my $loadhours = shift; # it must be GLOBAL variable
  my $managedname = shift;
  my $host = shift;
  #my $sec = shift;
  #my $ivmmin = shift;
  #my $ivmh = shift;
  #my $ivmd = shift;
  #my $ivmm = shift;
  #my $ivmy = shift;
  #my $wday = shift;
  #my $yday = shift;
  #my $isdst = shift;
  my $last_rec = shift;
  my $t = shift;
  my $hmcv_num = shift;


  if ( $loadhours == $INIT_LOAD_IN_HOURS_BACK ) {
    # just to be sure ....
    $loadmins  = $INIT_LOAD_IN_HOURS_BACK*60;
  }

  $run_topten = open_topten_files($managedname,$host,$act_time);

  if ($loadhours <= 0 || $loadmins <= 0) {	# workaround as this sometimes is negative , need to check it out ...
    if ( ! $last_rec eq '' ) {
      error ("$act_time: time issue 1   : $host:$managedname hours:$loadhours mins:$loadmins Last saved record (HMC lslparutil time) : $last_rec ; HMC time : $t");
    }
    $loadhours = 3;    
    $loadmins  = 140; 
  }

  # workaround for HMC < 7.7.2 when it crashes in usage --minutes
  $timerange = " --minutes ".$loadmins." ";
  #$timerange = " -h ".$loadhours." " ;
  # -h obviously did not help ......

  my $loadsecs = $loadmins * 60;
  my $hmc_start_utime = $hmc_utime - $loadsecs;

  if ( $loadmins  > 0 ) {

    if ( $loadhours != $INIT_LOAD_IN_HOURS_BACK ) {
      print "download data  : $host:$managedname last $loadmins  minute(s) ($loadhours hours)\n" if $DEBUG ;
    }

    #
    # Load lpar data - it must be always first due to last file already loaded (last.txt)
    #

    print "fetching HMC   : $host:$managedname lpar data\n" if $DEBUG ;
      if ($step == 3600) {
        if ($loadhours >= 1) {
         if ( $HMC == 1 ){
	    if ( $hmcv_num < 700 ) {
              `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r lpar -h $loadhours -m \\"$managedname\\" -F time,lpar_name,curr_proc_units,curr_procs,curr_sharing_mode,entitled_cycles,capped_cycles,uncapped_cycles  --filter \"event_types=sample\"" > $input_shell-$type_sam` ;
	    }
	    else {
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r lpar -h $loadhours $restrict_rows -m \\"$managedname\\" -F time,lpar_name,curr_proc_units,curr_procs,curr_sharing_mode,entitled_cycles,capped_cycles,uncapped_cycles,shared_cycles_while_active$mem_params --filter \"event_types=sample\"" > $input_shell-$type_sam` ;
	    }
	  }
	  if ( $IVM == 1 ){
           `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r lpar --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin  -m \\"$managedname\\" -F time,lpar_id,curr_proc_units,curr_procs,curr_sharing_mode,entitled_cycles,capped_cycles,uncapped_cycles,donated_cycles$mem_params" > $input_shell-$type_sam` ;
	  }
	  if ( $SDMC == 1 ){
	   LoadDataModule::sdmc_lpar_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_shell-$type_sam",$type_sam,0,"",$mem_params);
	   # firt 0 means that t is called from alerting --> HMC data input file must have "alert" suffix"
	   # here must be 1
	  }
	}
      }
      else {
	  if ( $HMC == 1 ){
	    #
	    # here is the workaround for HMC < 7.2.3 for the problem with exhasting the memory during lslparutil
	    #
	    my $rowCount = $MAX_ROWS;    
	    my $rowTotal = 0;    
            my $mins = 0;
            my $line2 = "";
            my $repeat = 0;
            while ( $rowCount == $MAX_ROWS ) { 
	      $repeat++;
              if ($rowTotal == 0) {
                #print "--- $timerange  : $restrict_rows\n" if $DEBUG ;
                print "fetching HMC 0 : $host:$managedname $timerange  : $restrict_rows\n" if $DEBUG ;
                `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s s -r lpar $timerange $restrict_rows -m \\"$managedname\\" -F time,lpar_name,curr_proc_units,curr_procs,curr_sharing_mode,entitled_cycles,capped_cycles,uncapped_cycles,shared_cycles_while_active$mem_params  --filter \"event_types=sample\"" > $input_shell-$type_sam` ;
              }
	      else {
                (my $time_end, my $lpar, my $curr_proc_units, my $curr_procs, my $curr_sharing_mode, my $entitled_cycles,
                   my $capped_cycles, my $uncapped_cycles, my $shared_cycles_while_active) = split(/,/,$line2);
	        my $unixt = str2time($time_end);

                #print "--- $unixt - $rowCount : $line2 \n" if $DEBUG ;
	        (my $sec,my $hmcmin,my $hmch,my $hmcd,my $hmcm,my $hmcy,my $wday,my $yday,my $isdst) = localtime($unixt);
    		$hmcy += 1900;
    		$hmcm++; # month + 1, it starts from 0 in unix

                if ( $unixt < $hmc_start_utime ) {
                  print "end time reach : $host:$managedname $hmcy-$hmcm-$hmcd $hmch:$hmcmin(to be downloaded) - $time_end(downloaded): $unixt < $hmc_start_utime\n";
                  last; #already Rdownloaded required time range 
                }

                print "fetching HMC   : $host:$managedname lpar data : next $MAX_ROWS : $hmcy-$hmcm-$hmcd $hmch:$hmcmin - $time_end\n" if $DEBUG ;
                `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s s -r lpar $restrict_rows --endyear $hmcy --endmonth $hmcm --endday $hmcd --endhour $hmch --endminute $hmcmin -m \\"$managedname\\" -F time,lpar_name,curr_proc_units,curr_procs,curr_sharing_mode,entitled_cycles,capped_cycles,uncapped_cycles,shared_cycles_while_active$mem_params  --filter \"event_types=sample\"" >> $input_shell-$type_sam` ;
	      }
              open(FH, "< $input-$type_sam") || error(" Can't open $input-$type_sam : $!".__FILE__.":".__LINE__) && return 0;
	      $rowCount=0;    
              while (my $l = <FH>) {
		chomp($l);
                $rowCount++;
                $line2 = $l;
	      }
	      close(FH);
	      $rowTotal = $rowCount;
	      $rowCount = $rowCount / $repeat;
	    }
            print "fetched        : $host:$managedname $rowTotal rows\n" if $DEBUG ;
	  }
	  if ( $IVM == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r lpar --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin -m \\"$managedname\\" -F time,lpar_id,curr_proc_units,curr_procs,curr_sharing_mode,entitled_cycles,capped_cycles,uncapped_cycles,donated_cycles$mem_params" > $input_shell-$type_sam` ;
	  }
	  if ( $SDMC == 1 ){
	     LoadDataModule::sdmc_lpar_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_shell-$type_sam",$type_sam,0,"",$mem_params);
	     # first 0 means that t is called from alerting --> HMC data input file must have "alert" suffix"
	     # here must be 1
	  }
      }
  
      #
      # Load CPU pool data
      #
    
      print "fetching HMC   : $host:$managedname pool data\n" if $DEBUG ;
      # refresh last timestamp by actual one for that , it modifies $loadhours, $loadmins and IVM/SDMC ... days ...
      last_timestamp ($wrkdir,$managedname,$host,$last_file_pool,"pool",$t); 

      if ($step == 3600) {
 	if ($loadhours >= 1){
	  if ( $HMC == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r pool  -h $loadhours $restrict_rows  -m \\"$managedname\\" -F time,total_pool_cycles,utilized_pool_cycles,configurable_pool_proc_units,borrowed_pool_proc_units  --filter \"event_types=sample\"" > $input_pool_shell-$type_sam` ;
            print "fetching HMC   : $host:$managedname shared pool data\n" if $DEBUG ;
            last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_pool,"SharedPool0",$t); 
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r procpool  -h $loadhours $restrict_rows -m \\"$managedname\\" -F time,shared_proc_pool_id,total_pool_cycles,utilized_pool_cycles  --filter \"event_types=sample\"" > $input_pool_sh_shell-$type_sam` ;
	  }
	  if ( $IVM == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r pool --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin  -m \\"$managedname\\" -F time,total_pool_cycles,utilized_pool_cycles,configurable_pool_proc_units,borrowed_pool_proc_units" > $input_pool_shell-$type_sam` ;
	  }
	  if ( $SDMC == 1 ){
	    LoadDataModule::sdmc_pool_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_shell-$type_sam",$type_sam);
            print "fetching HMC   : $host:$managedname shared pool data\n" if $DEBUG ;
            last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_pool,"SharedPool0",$t); 
	    LoadDataModule::sdmc_procpool_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_sh_shell-$type_sam",$type_sam);
	  }
	}
      }
      else {
	  if ( $HMC == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s s -r pool $timerange $restrict_rows -m \\"$managedname\\" -F time,total_pool_cycles,utilized_pool_cycles,configurable_pool_proc_units,borrowed_pool_proc_units  --filter \"event_types=sample\"" > $input_pool_shell-$type_sam` ;

            print "fetching HMC   : $host:$managedname shared pool data\n" if $DEBUG ;
            last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_pool,"SharedPool0",$t); 
            print "fetching HMC   : $host:$managedname shared pool data\n" if $DEBUG ;
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s s -r procpool $timerange $restrict_rows -m \\"$managedname\\" -F time,shared_proc_pool_id,total_pool_cycles,utilized_pool_cycles  --filter \"event_types=sample\"" > $input_pool_sh_shell-$type_sam` ;
	  }
	  if ( $IVM == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r pool --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin  -m \\"$managedname\\" -F time,total_pool_cycles,utilized_pool_cycles,configurable_pool_proc_units,borrowed_pool_proc_units" > $input_pool_shell-$type_sam` ;
	  }
	  if ( $SDMC == 1 ){
	    LoadDataModule::sdmc_pool_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_shell-$type_sam",$type_sam,0);
            print "fetching HMC   : $host:$managedname shared pool data\n" if $DEBUG ;
            last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_pool,"SharedPool0",$t); 
	    LoadDataModule::sdmc_procpool_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_sh_shell-$type_sam",$type_sam,0);
	  }
      }

      #
      # Load memory data
      #
 
      print "fetching HMC   : $host:$managedname mem data\n" if $DEBUG ;
      # refresh last timestamp by actual one for that , it modifies $loadhours, $loadmins and IVM/SDMC ... days ...
      last_timestamp ($wrkdir,$managedname,$host,$last_file_mem,"mem",$t);

      if ($step == 3600) {
 	if ($loadhours >= 1){
	  if ( $HMC == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r sys -h $loadhours  $restrict_rows -m \\"$managedname\\" -F time,curr_avail_sys_mem,configurable_sys_mem,sys_firmware_mem --filter \"event_types=sample\"" > $input_mem_shell-$type_sam` ;
            if ( $hmcv_num > 734 || $FSM == 1 ) {
              # set params for lslparutil and AMS available from HMC V7R3.4.0 Service Pack 2 (05-21-2009)
              last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_mem,"mem-pool",$t);
              `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r mempool -h $loadhours  $restrict_rows -m \\"$managedname\\" -F time,curr_pool_mem,lpar_curr_io_entitled_mem,lpar_mapped_io_entitled_mem,lpar_run_mem,sys_firmware_pool_mem --filter \"event_types=sample\"" > $input_pool_mem_shell-$type_sam` ;
            }
	  }
	  if ( $IVM == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r sys --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin -m \\"$managedname\\" -F time,curr_avail_sys_mem,configurable_sys_mem,sys_firmware_mem " > $input_mem_shell-$type_sam` ;
             last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_mem,"mem-pool",$t);
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r mempool --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin -m \\"$managedname\\" -F time,curr_pool_mem,lpar_curr_io_entitled_mem,lpar_mapped_io_entitled_mem,lpar_run_mem,sys_firmware_pool_mem " > $input_pool_mem_shell-$type_sam` ;
	  }
	  if ( $SDMC == 1 ){
	    LoadDataModule::sdmc_sys_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_mem_shell-$type_sam",$type_sam,0);
            last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_mem,"mem-pool",$t);
	    LoadDataModule::sdmc_pool_mem_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_mem_shell-$type_sam",$type_sam,0);
	  }
	}
      }
      else {
	  if ( $HMC == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s s -r sys $timerange $restrict_rows -m \\"$managedname\\" -F time,curr_avail_sys_mem,configurable_sys_mem,sys_firmware_mem --filter \"event_types=sample\"" > $input_mem_shell-$type_sam` ;
            if ( $hmcv_num > 734 || $FSM == 1 ) {
               last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_mem,"mem-pool",$t);
              `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s s -r mempool $timerange $restrict_rows -m \\"$managedname\\" -F time,curr_pool_mem,lpar_curr_io_entitled_mem,lpar_mapped_io_entitled_mem,lpar_run_mem,sys_firmware_pool_mem --filter \"event_types=sample\"" > $input_pool_mem_shell-$type_sam` ;
            }
	  }
	  if ( $IVM == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r sys --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin -m \\"$managedname\\" -F time,curr_avail_sys_mem,configurable_sys_mem,sys_firmware_mem" > $input_mem_shell-$type_sam` ;
            last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_mem,"mem-pool",$t);
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r mempool --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin -m \\"$managedname\\" -F time,curr_pool_mem,lpar_curr_io_entitled_mem,lpar_mapped_io_entitled_mem,lpar_run_mem,sys_firmware_pool_mem " > $input_pool_mem_shell-$type_sam` ;
	  }
          if ( $SDMC == 1 ){
            LoadDataModule::sdmc_sys_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_mem_shell-$type_sam",$type_sam,0);
            last_timestamp ($wrkdir,$managedname,$host,$last_file_sh_mem,"mem-pool",$t);
            LoadDataModule::sdmc_pool_mem_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_mem_shell-$type_sam",$type_sam,0);
          }
      }


      #
      # Load CoD data
      #
 
      print "fetching HMC   : $host:$managedname CoD data\n" if $DEBUG ;
      # refresh last timestamp by actual one for that , it modifies $loadhours, $loadmins and IVM/SDMC ... days ...
      last_timestamp ($wrkdir,$managedname,$host,$last_file_cod,"cod",$t);

      if ($step == 3600) {
 	if ($loadhours >= 1){
	  if ( $HMC == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r all -h $loadhours  $restrict_rows -m \\"$managedname\\" -F time,used_proc_min,unreported_proc_min --filter \"event_types=utility_cod_proc_usage\"" > $input_cod_shell-$type_sam` ;
	  }
          # not sure if IVM supports that event_types=utility_cod_proc_usage --> have not found in IVM man    
	  #if ( $IVM == 1 ){
          #  `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r all --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin -m \\"$managedname\\" -F time,used_proc_min,unreported_proc_min " > $input_cod_shell-$type_sam` ;
	  #}
	  if ( $SDMC == 1 ){
	    LoadDataModule::sdmc_cod_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_cod_shell-$type_sam",$type_sam,0);
	  }
	}
      }
      else {
	  if ( $HMC == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s s -r all $timerange $restrict_rows -m \\"$managedname\\" -F time,used_proc_min,unreported_proc_min --filter \"event_types=utility_cod_proc_usage\"" > $input_cod_shell-$type_sam` ;
	  }
          # not sure if IVM supports that event_types=utility_cod_proc_usage --> have not found in IVM man    
	  #if ( $IVM == 1 ){
          #  `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r all --startyear $ivmy --startmonth $ivmm --startday $ivmd --starthour $ivmh --startminute $ivmmin -m \\"$managedname\\" -F time,used_proc_min,unreported_proc_min" > $input_cod_shell-$type_sam` ;
	  #}
          if ( $SDMC == 1 ){
            LoadDataModule::sdmc_cod_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_cod_shell-$type_sam",$type_sam,0);
          }
      }





      if ( $cycle_count == $PARALLELIZATION ) {
        print "No fork        : $host:$managedname : $server_count\n" if $DEBUG ;
        load_data_and_graph ();
        $cycle_count = 0;
      }
      else {
        $pid[$server_count] = fork();
        if (not defined $pid[$server_count]) {
          error ("$host:$managedname could not fork");
        } 
        elsif ($pid[$server_count] == 0) {
          print "Fork           : $host:$managedname : $server_count\n" if $DEBUG ;
          RRDp::end;
          RRDp::start "$rrdtool";
          load_data_and_graph ();
          print "Fork exit      : $host:$managedname : $server_count\n" if $DEBUG ;
          RRDp::end;
          exit(0);
        } 
        print "Parent continue: $host:$managedname $pid[$server_count]\n";
        $server_count++;
      }
      $cycle_count++;
    }
    else {
      my $t1 = localtime($last_rec);
      my $t2 = localtime($t);
      error ("$act_time: time issue 2   : $host:$managedname hours:$loadhours mins:$loadmins Last saved record (HMC lslparutil time) : $last_rec ; HMC time : $t - $t1 - $t2");
    }

    if ( $YEAR_REFRESH == 0 ) {
      #just for testing purposes
      $type_sam="d";
      rrd_find ($managedname,$host);
    }
    
    if ( $HMC == 1 ) { 
      # IVM does not have daily stats
      # SDMC has that but it does not work properly, got a problem with that, switched to IVM style
      # 	--> it is ready dor such usage, to allow it put here : || $SDMC == 1 and same in draw_graphs

      # daily stats
      $type_sam="d";

      # count hours for day statistics
      # loadhours = $loadhours * 24;
      # find out last record in the db (hourly)
      $loadhours = 0;  # must be here before rrd_check
      rrd_check($managedname);

      # print "daily : -- $loadhours -- \n";
      # for daily stats is sufficent RRDp::cmd qq(last..)  stuff ...
      if ( ! $loadhours) { # all except the initial load --> check rrd_check
        if ( -f "$wrkdir/$managedname/$host/mem.rr$type_sam" ) {
          RRDp::cmd qq(last "$wrkdir/$managedname/$host/mem.rr$type_sam" );
          $last_rec = RRDp::read;    
          chomp ($$last_rec);
          $loadhours = sprintf ("%.0f", ($t - $$last_rec)/(3600*24));
          #print "-- $loadhours -- $$last_rec -- $t \n";
        }
        else {
          if ( -f "$wrkdir/$managedname/$host/pool.rr$type_sam" ) {
            RRDp::cmd qq(last "$wrkdir/$managedname/$host/pool.rr$type_sam" );
            $last_rec = RRDp::read;    
            chomp ($$last_rec);
            $loadhours = sprintf ("%.0f", ($t - $$last_rec)/(3600*24));
            #print "-- $loadhours -- $$last_rec -- $t \n";
          }
          else {
	    $loadhours = $INIT_LOAD_IN_HOURS_BACK;
          }
        }
      }
      # print "daily : -- $loadhours -- \n";
  
      if ( $loadhours < 0 ) {	# workaround as this sometimes is negative , need to check it out ...
        error ("$act_time:time issue day : $host:$managedname hours:$loadhours - $$last_rec - $t");
        $loadhours = 1;           # 1d+
      }
  
      if ( $loadhours > 1 ) {	# go ahead only once a day
  
        # Find out HMC version even here if $step != 3600 
        if ( $HMC == 1 && $step != 3600 ) {
          # Find out version of HMC (just first 3 digits and figure out if 
          # it is bigger than 733 what is the version since are supported
          # enhanced sampe rates
          my $hmcv = `$SSH $hmc_user\@$host "lshmc -v"|grep RM| tr -d \'[:alpha:]\'|tr -d \'[:punct:]\'`;
          $hmcv =~ s/ //g;
          $hmcv_num = substr($hmcv,0,3);
          #print "DEBUG : $hmcv_num $hmcv\n";
          print "HMC version    : $host:$hmcv" if $DEBUG ;
        }

        print "sample rate    : $host:$managedname day load\n" if $DEBUG ;
        print "download data  : $host:$managedname last $loadhours day(s)\n" if $DEBUG ;
        $loadhours = $loadhours*24; # conversion back to hours
        #print "daily : ++ $loadhours -- \n";
        # Load lpar data
        print "fetching HMC   : $host:$managedname lpar data\n" if $DEBUG ;
        if ( $HMC == 1 ){
	  if ( $hmcv_num < 700 ) {
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r lpar -h $loadhours  -m \\"$managedname\\" -F time,lpar_name,curr_proc_units,curr_procs,curr_sharing_mode,entitled_cycles,capped_cycles,uncapped_cycles  --filter \"event_types=sample\"" > $input_shell-$type_sam` ;
  
	  }
	  else {
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r lpar -h $loadhours -n $MAX_ROWS_WORKAROUND -m \\"$managedname\\" -F time,lpar_name,curr_proc_units,curr_procs,curr_sharing_mode,entitled_cycles,capped_cycles,uncapped_cycles,shared_cycles_while_active$mem_params  --filter \"event_types=sample\"" > $input_shell-$type_sam` ;
	  }
        }
        if ( $SDMC == 1 ){
         LoadDataModule::sdmc_lpar_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_shell-$type_sam",$type_sam,$loadhours,"",$mem_params);
	 # firt 0 means that t is called from alerting --> HMC data input file must have "alert" suffix"
	 # here must be 1
        }

      

	# on purpose here to have some timeout between lslparutil issuing due to a bug in the HMC
        #load_data ($managedname);
        LoadDataModule::load_data ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file,$no_time,$SSH,$hmc_user);
      
        # Load CPU pool data
        print "fetching HMC   : $host:$managedname pool data\n" if $DEBUG ;
        if ( $HMC == 1 ){
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r pool -h $loadhours -n $MAX_ROWS_WORKAROUND -m \\"$managedname\\" -F time,total_pool_cycles,utilized_pool_cycles,configurable_pool_proc_units,borrowed_pool_proc_units  --filter \"event_types=sample\"" > $input_pool_shell-$type_sam` ;
        }
        if ( $SDMC == 1 ){
         LoadDataModule::sdmc_pool_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_shell-$type_sam",$type_sam,$loadhours);
        }
        print "fetching HMC   : $host:$managedname shared pool data\n" if $DEBUG ;
        if ( $HMC == 1 ){
          `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r procpool -h $loadhours -n $MAX_ROWS_WORKAROUND -m \\"$managedname\\" -F time,shared_proc_pool_id,total_pool_cycles,utilized_pool_cycles  --filter \"event_types=sample\"" > $input_pool_sh_shell-$type_sam` ;
        }
        if ( $SDMC == 1 ){
         LoadDataModule::sdmc_procpool_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_sh-$type_sam",$type_sam,$timerange);
        }

        LoadDataModule::load_data_pool_sh ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_sh_pool,$input_pool_sh,$SSH,$hmc_user,\@pool_list,$pool_list_file,$no_time);

      
        # Load memory data
        print "fetching HMC   : $host:$managedname mem data\n" if $DEBUG ;
        if ( $HMC == 1 ){
          `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r sys -h $loadhours -n $MAX_ROWS_WORKAROUND -m \\"$managedname\\" -F time,curr_avail_sys_mem,configurable_sys_mem,sys_firmware_mem --filter \"event_types=sample\"" > $input_mem_shell-$type_sam` ;
          if ( $hmcv_num > 734 || $FSM == 1 ) {
            `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r mempool -h $loadhours -n $MAX_ROWS_WORKAROUND -m \\"$managedname\\" -F time,curr_pool_mem,lpar_curr_io_entitled_mem,lpar_mapped_io_entitled_mem,lpar_run_mem,sys_firmware_pool_mem --filter \"event_types=sample\"" > $input_pool_mem_shell-$type_sam` ;
          }
        }
        if ( $SDMC == 1 ){
         LoadDataModule::sdmc_sys_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_mem_shell-$type_sam",$type_sam,$loadhours);
         LoadDataModule::sdmc_pool_mem_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_pool_mem_shell-$type_sam",$type_sam,$loadhours);
        }

        # Load memory data
        print "fetching HMC   : $host:$managedname cod data\n" if $DEBUG ;
        if ( $HMC == 1 ){
          `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -s $type_sam -r all -h $loadhours -n $MAX_ROWS_WORKAROUND -m \\"$managedname\\" -F time,used_proc_min,unreported_proc_min --filter \"event_types=utility_cod_proc_usage\"" > $input_cod_shell-$type_sam` ;
        }
        if ( $SDMC == 1 ){
         LoadDataModule::sdmc_cod_load ($SSH,$hmc_user,$host,$ivmy,$ivmm,$ivmd,$ivmh,$ivmmin,$managedname,"$input_cod_shell-$type_sam",$type_sam,$loadhours);
        }

        

        LoadDataModule::load_data_mem ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_mem,$input_mem,$no_time);
  
        LoadDataModule::load_data_pool_mem ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_sh_mem,$input_pool_mem,$no_time);

        LoadDataModule::load_data_cod ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_cod,$input_cod,$no_time);

        # Load & crate graphs
	# on purpose here to have some timeout between lslparutil issuing due to a bug in the HMC
        LoadDataModule::load_data_pool ($managedname,$host,$wrkdir,$input,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file_pool,$input_pool,$no_time);
        rrd_find ($managedname,$host);
      }
      
      # Just in case there is not yearly chart and do not want to wait 1 day
      if ( ! -f "$webdir/$host/$managedname/pool/y.png" || $run_topten == 1 ) {	# go ahead only once a day
        # it must run here if topten otherviwe there is missing daily data for HMC stuff
        rrd_find ($managedname,$host);
      }
    }
   close_topten_files($run_topten);
}
  
sub draw_graph {
  my $text     = shift;
  my $type     = shift;
  my $xgrid    = shift;
  my $lpar     = shift;
  my $managedname = shift;
  my $type_sam = shift;
  my $name = "$webdir/$host/$managedname/$lpar/$type";
  my $aggname = "$webdir/$host/$managedname/pool-multi/$type";
  my $aggname_ams = "$webdir/$host/$managedname/mem-multi/$type"; #AMS
  my $aggname_mem = "$webdir/$host/$managedname/mem-agg/$type";
  my $poolname = "$webdir/$host/$managedname/pool/$type";
  my $memname = "$webdir/$host/$managedname/mem/$type";
  my $t="COMMENT: ";
  my $t2="COMMENT:\\n";
  my $step_new = $step;
  my $last ="COMMENT: ";
  my $rrd_time = "";
  my $png_time = 0;

  if ($lpar =~ m/^mem-pool$/ || $lpar =~ m/^cod$/) {
     return 0; # do not create graph for it
  }

  if (! -d "$webdir/$host" ) {
    print "mkdir          : $host:$managedname $webdir/$host\n" if $DEBUG ;
    mkdir("$webdir/$host", 0755) || error(" Cannot mkdir $webdir/$host: $!".__FILE__.":".__LINE__) && return 0;
  } 
  if (! -d "$webdir/$host/$managedname" ) {
    print "mkdir          : $host:$managedname $webdir/$host/$managedname\n" if $DEBUG ;
    mkdir("$webdir/$host/$managedname", 0755) || error(" Cannot mkdir $webdir/$host/$managedname: $!".__FILE__.":".__LINE__) && return 0;
  } 
  if (! -d "$webdir/$host/$managedname/$lpar" ) {
    my $lpar_slash = $lpar;
    $lpar_slash =~ s/\&\&1/\//g;
    print "mkdir          : $host:$managedname $webdir/$host/$managedname/$lpar_slash\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host/$managedname/$lpar");
    mkdir("$webdir/$host/$managedname/$lpar" , 0755) || error(" Cannot mkdir $webdir/$host/$managedname/$lpar: $!".__FILE__.":".__LINE__) && return 0;
  } 
  if (! -d "$webdir/$host/$managedname/pool" ) {
    print "mkdir          : $host:$managedname $webdir/$host/$managedname/pool\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host/$managedname/pool");
    mkdir("$webdir/$host/$managedname/pool" , 0755) || error(" Cannot mkdir $webdir/$host/$managedname/pool: $!".__FILE__.":".__LINE__) && return 0;
  } 
  if (! -d "$webdir/$host/$managedname/mem" ) {
    print "mkdir          : $host:$managedname $webdir/$host/$managedname/mem\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host/$managedname/mem");
    mkdir("$webdir/$host/$managedname/mem", 0755) || error(" Cannot mkdir $webdir/$host/$managedname/mem: $!".__FILE__.":".__LINE__) && return 0;
  } 
  if (! -d "$webdir/$host/$managedname/pool-multi" ) {
    print "mkdir          : $host:$managedname $webdir/$host/$managedname/pool-multi\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host/$managedname/pool-multi");
    mkdir("$webdir/$host/$managedname/pool-multi", 0755) || error(" Cannot mkdir $webdir/$host/$managedname/pool-multi: $!".__FILE__.":".__LINE__) && return 0;
  } 
  if (! -d "$webdir/$host/$managedname/mem-agg" ) {
    print "mkdir          : $host:$managedname $webdir/$host/$managedname/mem-agg\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host/$managedname/mem-agg");
    mkdir("$webdir/$host/$managedname/mem-agg", 0755) || error(" Cannot mkdir $webdir/$host/$managedname/mem-agg: $!".__FILE__.":".__LINE__) && return 0;
  } 

  my $act_time = time();
  #$step = $STEP;

  if ( $lpar =~  "SharedPool[0-9]" || $lpar =~  "SharedPool[0-9][0-9]" ) {
    my $rrd= "$wrkdir/$managedname/$host/$lpar.rr$type_sam";

    # do not update charts if there is not new data in RRD DB
    if ( -f "$rrd" ) {
      $rrd_time = (stat("$rrd"))[9];
      if ( ($act_time - $rrd_time) > $DELETED_LPARS_TIME ) { #10 days
        #error ("----$rrd");
        return 0;
      }
    }

    my $lpar_pool_alias = $lpar;
    $lpar_pool_alias =~ s/SharedPool/CPU pool /g;

    # add CPU pool alias into png header (if exists and it is not a default CPU pool)
    my $pool_id = $lpar;
    $pool_id =~ s/SharedPool//g;

    if ( -f "$wrkdir/$managedname/$host/cpu-pools-mapping.txt") {
      open(FR, "< $wrkdir/$managedname/$host/cpu-pools-mapping.txt");
      foreach my $linep (<FR>) {
        chomp ($linep);
        (my $id, my $pool_name) = split (/,/,$linep);
        if ( $id == $pool_id  ) {
          $lpar_pool_alias = "$pool_name";
          last;
        }
      }
      close (FR);
    }

    my $header = "CPU pool: $lpar_pool_alias : last $text";

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "y" && -f "$name.png" && $run_topten == 0) {
      my $png_time = (stat("$name.png"))[9];
      if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "m" && -f "$name.png" && $run_topten == 0 ) {
      my $png_time = (stat("$name.png"))[9];
      if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "w" && -f "$name.png" && $run_topten == 0 ) {
      my $png_time = (stat("$name.png"))[9];
      if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    print "creating graph : $host:$managedname:$lpar_pool_alias:$type_sam:$type\n" if $DEBUG ;

    #print "Graph pool :$lpar\n";
    if ($type =~ "d") {
      RRDp::cmd qq(last "$rrd");
      my $last_tt = RRDp::read;    
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $t ="COMMENT:Updated\\: $l ";

      # get LAST value from RRD
      RRDp::cmd qq(fetch "$rrd" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
      my $row = RRDp::read;
      chomp($$row);
      my @row_arr = split(/\n/,$$row);
      my $m = "";
      my $i = 0;
      foreach $m (@row_arr) {
       chomp($m);
       $i++;
       if ( $i == 3 ) {
         my @m_arr = split(/ /,$m);
         # go further ony if it is a digit (avoid it when NaNQ (== no data) is there)
         if ($m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ && $m_arr[4] =~ /\d/ ) {
           #print "m : $m\n" if $DEBUG ;
           #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
           my $total_pool_cycles = sprintf("%e",$m_arr[1]);
           my $utilized_pool_cyc = sprintf("%e",$m_arr[2]);
           my $max_pool_units  = sprintf("%e",$m_arr[3]);
           my $res_pool_units  = sprintf("%e",$m_arr[4]);
           if ( $total_pool_cycles != 0 ) {
             my $util = sprintf ("%.2f",($utilized_pool_cyc / $total_pool_cycles ) * $max_pool_units);
	     #print ("\n $util $curr_proc $entitled_cycles $capped_cycles $uncapped_cycles\n");
             
             $last ="COMMENT:Last utilization in CPU cores  $util";
           }
	 }
       }
      }
    };  

    if ( $type =~ "y" ) {
      $step_new=86400;
    }

    if ( ! -f "$rrd" ) {
       if ( $type_sam =~ "d" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one day!!!\n";
       }
       if ( $type_sam =~ "h" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one hour!!!\n";
       }
       if ( $type_sam =~ "m" ) {
         error ("$host:$managedname $act_time:$rrd does not exists");
       }
       return 0;
    }

    if ( $lpar =~  "SharedPool[1-9]" || $lpar =~  "SharedPool[1-9][0-9]" ) {
      #print "creating graph : $rrd\n" if $DEBUG ;
      #print "               : $name.png\n" if $DEBUG ;
      # SharedPool0
      RRDp::cmd qq(graph "$name.png"
        "--title" "$header"
        "--start" "now-1$type"
        "--end" "now-1$type+1$type"
        "--imgformat" "PNG"
	"--slope-mode"
        "--width=400"
        "--height=150"
        "--step=$step_new"
        "--lower-limit=0"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--vertical-label=Utilization in CPU cores"
        "--alt-autoscale-max"
        "--upper-limit=0.2"
        "--units-exponent=1.00"
        "--alt-y-grid"
        "--x-grid=$xgrid"
        "DEF:max=$rrd:max_pool_units:AVERAGE"
        "DEF:res=$rrd:res_pool_units:AVERAGE"
        "DEF:totcyc=$rrd:total_pool_cycles:AVERAGE"
        "DEF:uticyc=$rrd:utilized_pool_cyc:AVERAGE"
        "CDEF:max1=max,res,-"
        "CDEF:cpuutil=uticyc,totcyc,/"
        "CDEF:cpuutiltot=cpuutil,max,*"
        "CDEF:utilisa=cpuutil,100,*"
        "COMMENT:   Average   \\n"
        "AREA:res#00FF00: Reserved CPU cores      "
        "GPRINT:res:AVERAGE: %2.1lf"
        "$t2"
        "STACK:max1#FFFF00: Max CPU cores           "
        "GPRINT:max1:AVERAGE: %2.1lf"
        "$t2"
        "LINE1:cpuutiltot#FF0000: Utilization in CPU cores"
        "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
        "COMMENT:("
        "GPRINT:utilisa:AVERAGE: %2.1lf"
        "COMMENT:\%)"
        "$t2"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
        "VRULE:0#000000"
      );
      my $answer = RRDp::read; 
      if ( $$answer =~ "ERROR" ) {
        error (" $host:$managedname : Graph rrdtool error : $$answer ".__FILE__.":".__LINE__);
        if ( $$answer =~ "is not an RRD file" ) {
          (my $err,  my $file, my $txt) = split(/'/,$$answer);
          error ("Removing as it seems to be corrupted: $file");
          unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
        }
        else {
          print "Graph rrdtool error : $$answer";
        }
      }

      if ( $run_topten == 1 || $upgrade == 1 ) {
        # also do it after the upgrade (or fresh instal what is upgrade in fact as well
        # create a copy for views, all views have to be snapped at the same moment
        copy("$name.png", "$name-$view_suff.png");
      } 

      create_graph_pool_agg ($lpar_pool_alias,$lpar,$name,$managedname,$host,$type_sam,$act_time,$type,$xgrid,$step_new,$text);

      if ( $type =~ "y" ) {
        # Trend for yearly chart 1y back  + 1y forward
        print "creating graph : $host:$managedname:$lpar:$type_sam:$type:trend\n" if $DEBUG ;

        RRDp::cmd qq(graph "$name-trend.png"
          "--title" "$header - trends"
          "--start" "now-1$type"
          "--end" "now+1$type"
          "--imgformat" "PNG"
	  "--slope-mode"
          "--width=800"
          "--height=150"
          "--step=$step_new"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "--vertical-label=Utilization in CPU cores"
          "--upper-limit=0.2"
	  "--lower-limit=0"
          "--alt-autoscale-max"
          "--units-exponent=1.00"
          "--alt-y-grid"
          "--x-grid=$xgrid"
          "DEF:maxd=$rrd:max_pool_units:AVERAGE:start=-1m"
          "DEF:maxm=$rrd:max_pool_units:AVERAGE:start=-3m"
          "DEF:maxy=$rrd:max_pool_units:AVERAGE:start=-1y"
          "DEF:max=$rrd:max_pool_units:AVERAGE"
          "DEF:res=$rrd:res_pool_units:AVERAGE"
          "DEF:totcycd=$rrd:total_pool_cycles:AVERAGE:start=-1m"
          "DEF:totcycm=$rrd:total_pool_cycles:AVERAGE:start=-3m"
          "DEF:totcycy=$rrd:total_pool_cycles:AVERAGE:start=-1y"
          "DEF:totcyc=$rrd:total_pool_cycles:AVERAGE"
          "DEF:uticycd=$rrd:utilized_pool_cyc:AVERAGE:start=-1m"
          "DEF:uticycm=$rrd:utilized_pool_cyc:AVERAGE:start=-3m"
          "DEF:uticycy=$rrd:utilized_pool_cyc:AVERAGE:start=-1y"
          "DEF:uticyc=$rrd:utilized_pool_cyc:AVERAGE"
          "CDEF:max1=max,res,-"
          "CDEF:cpuutild=uticycd,totcycd,/"
          "CDEF:cpuutilm=uticycm,totcycm,/"
          "CDEF:cpuutily=uticycy,totcycy,/"
          "CDEF:cpuutil=uticyc,totcyc,/"
          "CDEF:cpuutiltotd=cpuutild,maxd,*"
          "CDEF:cpuutiltotm=cpuutilm,maxm,*"
          "CDEF:cpuutiltoty=cpuutily,maxy,*"
          "CDEF:cpuutiltot=cpuutil,max,*"
          "CDEF:utilisa=cpuutil,100,*"
          "VDEF:Dd=cpuutiltotd,LSLSLOPE"
          "VDEF:Hd=cpuutiltotd,LSLINT"
          "CDEF:cpuutiltottrendd=cpuutiltotd,POP,Dd,COUNT,*,Hd,+"
          "VDEF:Dm=cpuutiltotm,LSLSLOPE"
          "VDEF:Hm=cpuutiltotm,LSLINT"
          "CDEF:cpuutiltottrendm=cpuutiltotm,POP,Dm,COUNT,*,Hm,+"
          "VDEF:Dy=cpuutiltoty,LSLSLOPE"
          "VDEF:Hy=cpuutiltoty,LSLINT"
          "CDEF:cpuutiltottrendy=cpuutiltoty,POP,Dy,COUNT,*,Hy,+"
          "COMMENT:   Average   \\n"
          "AREA:res#00FF00: Reserved CPU cores      "
          "GPRINT:res:AVERAGE: %2.1lf"
          "$t2"
          "STACK:max1#FFFF00: Max CPU cores           "
          "GPRINT:max1:AVERAGE: %2.1lf"
          "$t2"
          "LINE1:cpuutiltot#FF0000: Utilization in CPU cores"
          "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
          "COMMENT:("
          "GPRINT:utilisa:AVERAGE: %2.1lf"
          "COMMENT:\%)"
          "$t2"
          "LINE2:cpuutiltottrendd#FFFF00: Utilization in CPU cores - last 1 month trend"
          "$t2"
          "LINE2:cpuutiltottrendm#80FFFF: Utilization in CPU cores - last 3 months trend"
          "$t2"
          "LINE2:cpuutiltottrendy#FF8080: Utilization in CPU cores - last year trend"
          "$last"
          "$t2"
          "$t"
          "HRULE:0#000000"
          "VRULE:0#000000"
        );
        my $answer = RRDp::read; 
        if ( $$answer =~ "ERROR" ) {
          error (" $host:$managedname Graph rrdtool error : $$answer");
          if ( $$answer =~ "is not an RRD file" ) {
            (my $err,  my $file, my $txt) = split(/'/,$$answer);
 	    error ("Removing as it seems to be corrupted: $file");
            unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
          }
          else {
            print "Graph rrdtool error : $$answer";
          }
        }
      }
    }
    else {
      #print "creating graph : $rrd\n" if $DEBUG ;
      #print "               : $name.png\n" if $DEBUG ;
      # except SharedPool0
      RRDp::cmd qq(graph "$name.png"
        "--title" "$header"
        "--start" "now-1$type"
        "--end" "now-1$type+1$type"
        "--imgformat" "PNG"
  	"--slope-mode"
        "--width=400"
        "--height=150"
        "--step=$step_new"
        "--lower-limit=0"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--vertical-label=Utilization in CPU cores"
        "--alt-autoscale-max"
        "--upper-limit=0.2"
        "--units-exponent=1.00"
        "--alt-y-grid"
        "--x-grid=$xgrid"
        "DEF:max=$rrd:max_pool_units:AVERAGE"
        "DEF:totcyc=$rrd:total_pool_cycles:AVERAGE"
        "DEF:uticyc=$rrd:utilized_pool_cyc:AVERAGE"
        "CDEF:cpuutil=uticyc,totcyc,/"
        "CDEF:cpuutiltot=cpuutil,max,*"
        "CDEF:utilisa=cpuutil,100,*"
        "COMMENT:   Average   \\n"
        "AREA:max#00FF00: Max CPU cores           "
        "GPRINT:max:AVERAGE: %2.1lf"
        "$t2"
        "LINE1:cpuutiltot#FF0000: Utilization in CPU cores"
        "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
        "COMMENT:("
        "GPRINT:utilisa:AVERAGE: %2.1lf"
        "COMMENT:\%)"
        "$t2"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
        "VRULE:0#000000"
      );
      my $answer = RRDp::read; 
      if ( $$answer =~ "ERROR" ) {
        error (" $host:$managedname Graph rrdtool error : $$answer");
        if ( $$answer =~ "is not an RRD file" ) {
          (my $err,  my $file, my $txt) = split(/'/,$$answer);
 	  error ("Removing as it seems to be corrupted: $file");
          unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
        }
        else {
          print "Graph rrdtool error : $$answer";
        }
      }

      if ( $run_topten == 1 || $upgrade == 1 ) {
        # also do it after the upgrade (or fresh instal what is upgrade in fact as well
        # create a copy for views, all views have to be snapped at the same moment
        copy("$name.png", "$name-$view_suff.png");
      } 

      if ( $type =~ "y" ) {
        # Trend for yearly chart 1y back  + 1y forward
        print "creating graph : $host:$managedname:$lpar:$type_sam:$type:trend\n" if $DEBUG ;

        RRDp::cmd qq(graph "$name-trend.png"
          "--title" "$header - trends"
          "--start" "now-1$type"
          "--end" "now+1$type"
          "--imgformat" "PNG"
	  "--slope-mode"
          "--width=800"
          "--height=150"
          "--step=$step_new"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "--vertical-label=Utilization in CPU cores"
          "--alt-autoscale-max"
	  "--lower-limit=0"
          "--units-exponent=1.00"
          "--alt-y-grid"
          "--x-grid=$xgrid"
          "DEF:maxd=$rrd:max_pool_units:AVERAGE:start=-1m"
          "DEF:maxm=$rrd:max_pool_units:AVERAGE:start=-3m"
          "DEF:maxy=$rrd:max_pool_units:AVERAGE:start=-1y"
          "DEF:max=$rrd:max_pool_units:AVERAGE"
          "DEF:totcycd=$rrd:total_pool_cycles:AVERAGE:start=-1m"
          "DEF:totcycm=$rrd:total_pool_cycles:AVERAGE:start=-3m"
          "DEF:totcycy=$rrd:total_pool_cycles:AVERAGE:start=-1y"
          "DEF:totcyc=$rrd:total_pool_cycles:AVERAGE"
          "DEF:uticycd=$rrd:utilized_pool_cyc:AVERAGE:start=-1m"
          "DEF:uticycm=$rrd:utilized_pool_cyc:AVERAGE:start=-3m"
          "DEF:uticycy=$rrd:utilized_pool_cyc:AVERAGE:start=-1y"
          "DEF:uticyc=$rrd:utilized_pool_cyc:AVERAGE"
          "CDEF:cpuutild=uticycd,totcycd,/"
          "CDEF:cpuutilm=uticycm,totcycm,/"
          "CDEF:cpuutily=uticycy,totcycy,/"
          "CDEF:cpuutil=uticyc,totcyc,/"
          "CDEF:cpuutiltotd=cpuutild,maxd,*"
          "CDEF:cpuutiltotm=cpuutilm,maxm,*"
          "CDEF:cpuutiltoty=cpuutily,maxy,*"
          "CDEF:cpuutiltot=cpuutil,max,*"
          "CDEF:utilisa=cpuutil,100,*"
          "VDEF:Dd=cpuutiltotd,LSLSLOPE"
          "VDEF:Hd=cpuutiltotd,LSLINT"
          "CDEF:cpuutiltottrendd=cpuutiltotd,POP,Dd,COUNT,*,Hd,+"
          "VDEF:Dm=cpuutiltotm,LSLSLOPE"
          "VDEF:Hm=cpuutiltotm,LSLINT"
          "CDEF:cpuutiltottrendm=cpuutiltotm,POP,Dm,COUNT,*,Hm,+"
          "VDEF:Dy=cpuutiltoty,LSLSLOPE"
          "VDEF:Hy=cpuutiltoty,LSLINT"
          "CDEF:cpuutiltottrendy=cpuutiltoty,POP,Dy,COUNT,*,Hy,+"
          "COMMENT:   Average   \\n"
          "AREA:max#00FF00: Max CPU cores           "
          "GPRINT:max:AVERAGE: %2.1lf"
          "$t2"
          "LINE1:cpuutiltot#FF0000: Utilization in CPU cores"
          "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
          "COMMENT:("
          "GPRINT:utilisa:AVERAGE: %2.1lf"
          "COMMENT:\%)"
          "$t2"
          "LINE2:cpuutiltottrendd#FFFF00: Utilization in CPU cores - last 1 month trend"
          "$t2"
          "LINE2:cpuutiltottrendm#80FFFF: Utilization in CPU cores - last 3 months trend"
          "$t2"
          "LINE2:cpuutiltottrendy#FF8080: Utilization in CPU cores - last year trend"
          "$last"
          "$t2"
          "$t"
          "HRULE:0#000000"
          "VRULE:0#000000"
        );
        my $answer = RRDp::read; 
        if ( $$answer =~ "ERROR" ) {
          error (" $host:$managedname Graph rrdtool error : $$answer");
          if ( $$answer =~ "is not an RRD file" ) {
            (my $err,  my $file, my $txt) = split(/'/,$$answer);
 	    error ("Removing as it seems to be corrupted: $file");
            unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
          }
          else {
            print "Graph rrdtool error : $$answer";
          }
        }
      }
    }
  }
  else {

  if ( $lpar eq  "pool" ) {
    my $rrd= "$wrkdir/$managedname/$host/pool.rr$type_sam";

    # do not update charts if there is not new data in RRD DB
    if ( -f "$rrd" ) {
      $rrd_time = (stat("$rrd"))[9];
      if ( ($act_time - $rrd_time) > $DELETED_LPARS_TIME ) { #10 days
        #error ("----$rrd");
        return 0;
      }
    }

    #print "creating graph : $rrd\n" if $DEBUG ;
    #print "               : $poolname.png\n" if $DEBUG ;
    my $header = "CPU pool : last $text";

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "y" && -f "$poolname.png" && $run_topten == 0 ) {
      my $png_time = (stat("$poolname.png"))[9];
      if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "m" && -f "$poolname.png" && $run_topten == 0 ) {
      my $png_time = (stat("$poolname.png"))[9];
      if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "w" && -f "$poolname.png" && $run_topten == 0 ) {
      my $png_time = (stat("$poolname.png"))[9];
      if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    # Multiview chart
    print "creating mgraph: $host:$managedname:multi:$type_sam:$type\n" if $DEBUG ;
    multiview($aggname,$type,$type_sam,$act_time,$managedname,$step_new,$text,$xgrid);

    # Multiview AMS chart
    multiview_mem($aggname_ams,$type,$type_sam,$act_time,$managedname,$step_new,$text,$xgrid);

    # Memory static aggregated
    multiview_mem_static($aggname_mem,$type,$type_sam,$act_time,$managedname,$step_new,$text,$xgrid);

    print "creating graph : $host:$managedname:$lpar:$type_sam:$type\n" if $DEBUG ;

    #print "Graph pool :$lpar\n";
    if ($type =~ "d") {
      RRDp::cmd qq(last "$rrd");
      my $last_tt = RRDp::read;    
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $t ="COMMENT:Updated\\: $l ";

      # get LAST value from RRD
      RRDp::cmd qq(fetch "$rrd" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
      my $row = RRDp::read;
      chomp($$row);
      my @row_arr = split(/\n/,$$row);
      my $m = "";
      my $i = 0;
      foreach $m (@row_arr) {
       chomp($m);
       $i++;
       if ( $i == 3 ) {
         my @m_arr = split(/ /,$m);
         # go further ony if it is a digit (avoid it when NaNQ (== no data) is there)
         if ($m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ && $m_arr[4] =~ /\d/ ) {
           #print "m : $m\n" if $DEBUG ;
           #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
           my $total_pool_cycles = sprintf("%e",$m_arr[1]);
           my $utilized_pool_cyc = sprintf("%e",$m_arr[2]);
           my $conf_proc_units   = sprintf("%e",$m_arr[3]);
           my $bor_proc_units    = sprintf("%e",$m_arr[4]);
           if ( $total_pool_cycles != 0 ) {
             my $util = sprintf ("%.2f",($conf_proc_units + $bor_proc_units) * ($utilized_pool_cyc / $total_pool_cycles));
	     #print ("\n $util $curr_proc $entitled_cycles $capped_cycles $uncapped_cycles\n");
             
             $last ="COMMENT:Last utilization in CPU cores  $util";
           }
	 }
       }
      }
    };  

    if ( $type =~ "y" ) {
      $step_new=86400;
    }

    if ( ! -f "$rrd" ) {
       if ( $type_sam =~ "d" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one day!!!\n";
       }
       if ( $type_sam =~ "h" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one hour!!!\n";
       }
       if ( $type_sam =~ "m" ) {
         error ("$host:$managedname $rrd does not exists");
       }
       return 0;
    }

    RRDp::cmd qq(graph "$poolname.png"
      "--title" "$header"
      "--start" "now-1$type"
      "--end" "now-1$type+1$type"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=400"
      "--height=150"
      "--step=$step_new"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--vertical-label=CPU cores"
      "--alt-autoscale-max"
      "--upper-limit=1.00"
      "--units-exponent=1.00"
      "--alt-y-grid"
      "--x-grid=$xgrid"
      "DEF:totcyc=$rrd:total_pool_cycles:AVERAGE"
      "DEF:uticyc=$rrd:utilized_pool_cyc:AVERAGE"
      "DEF:cpu=$rrd:conf_proc_units:AVERAGE"
      "DEF:cpubor=$rrd:bor_proc_units:AVERAGE"
      "CDEF:totcpu=cpu,cpubor,+"
      "CDEF:cpuutil=uticyc,totcyc,/"
      "CDEF:cpuutiltot=cpuutil,totcpu,*"
      "CDEF:utilisa=cpuutil,100,*"
      "COMMENT:   Average   \\n"
      "AREA:cpu#00FF00: Configured CPU cores    "
      "GPRINT:cpu:AVERAGE: %2.1lf"
      "$t2"
      "STACK:cpubor#FFFF00: Not assigned CPU cores  "
      "GPRINT:cpubor:AVERAGE: %2.1lf"
      "$t2"
      "LINE1:cpuutiltot#FF0000: Utilization in CPU cores"
      "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
      "COMMENT:("
      "GPRINT:utilisa:AVERAGE: %2.1lf"
      "COMMENT:\%)"
      "$t2"
      "$last"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
    my $answer = RRDp::read; 
    if ( $$answer =~ "ERROR" ) {
        error ("$host:$managedname Graph rrdtool error : $$answer");
        if ( $$answer =~ "is not an RRD file" ) {
          (my $err,  my $file, my $txt) = split(/'/,$$answer);
	  error ("Removing as it seems to be corrupted: $file");
          unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
        }
        else {
          print "Graph rrdtool error : $$answer";
        }
    }
    #chomp ($$answer);
    #print "answer: $$answer\n" if $$answer;

    if ( $run_topten == 1 || $upgrade == 1 ) {
      # also do it after the upgrade (or fresh instal what is upgrade in fact as well
      # create a copy for views, all views have to be snapped at the same moment
      copy("$poolname.png", "$poolname-$view_suff.png");
    } 

    if ( $type =~ "y" ) {
      # Trend for yearly chart 1y back  + 1y forward
      print "creating graph : $host:$managedname:$lpar:$type_sam:$type:trend\n" if $DEBUG ;
  
      RRDp::cmd qq(graph "$poolname-trend.png"
        "--title" "$header - trends"
        "--start" "now-1$type"
        "--end" "now+1$type"
        "--imgformat" "PNG"
	"--slope-mode"
        "--width=800"
        "--height=150"
        "--step=$step_new"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--vertical-label=CPU cores"
        "--alt-autoscale-max"
        "--upper-limit=1.00"
	"--lower-limit=0"
        "--units-exponent=1.00"
        "--alt-y-grid"
        "--x-grid=$xgrid"
        "DEF:totcycd=$rrd:total_pool_cycles:AVERAGE:start=-1m"
        "DEF:totcycm=$rrd:total_pool_cycles:AVERAGE:start=-3m"
        "DEF:totcycy=$rrd:total_pool_cycles:AVERAGE:start=-1y"
        "DEF:totcyc=$rrd:total_pool_cycles:AVERAGE"
        "DEF:uticycd=$rrd:utilized_pool_cyc:AVERAGE:start=-1m"
        "DEF:uticycm=$rrd:utilized_pool_cyc:AVERAGE:start=-3m"
        "DEF:uticycy=$rrd:utilized_pool_cyc:AVERAGE:start=-1y"
        "DEF:uticyc=$rrd:utilized_pool_cyc:AVERAGE"
        "DEF:cpud=$rrd:conf_proc_units:AVERAGE:start=-1m"
        "DEF:cpum=$rrd:conf_proc_units:AVERAGE:start=-3m"
        "DEF:cpuy=$rrd:conf_proc_units:AVERAGE:start=-1y"
        "DEF:cpu=$rrd:conf_proc_units:AVERAGE"
        "DEF:cpubord=$rrd:bor_proc_units:AVERAGE:start=-1m"
        "DEF:cpuborm=$rrd:bor_proc_units:AVERAGE:start=-3m"
        "DEF:cpubory=$rrd:bor_proc_units:AVERAGE:start=-1y"
        "DEF:cpubor=$rrd:bor_proc_units:AVERAGE"
        "CDEF:totcpud=cpud,cpubord,+"
        "CDEF:totcpum=cpum,cpuborm,+"
        "CDEF:totcpuy=cpuy,cpubory,+"
        "CDEF:totcpu=cpu,cpubor,+"
        "CDEF:cpuutild=uticycm,totcycd,/"
        "CDEF:cpuutilm=uticycm,totcycm,/"
        "CDEF:cpuutily=uticycy,totcycy,/"
        "CDEF:cpuutil=uticyc,totcyc,/"
        "CDEF:cpuutiltotd=cpuutild,totcpud,*"
        "CDEF:cpuutiltotm=cpuutilm,totcpum,*"
        "CDEF:cpuutiltoty=cpuutily,totcpuy,*"
        "CDEF:cpuutiltot=cpuutil,totcpu,*"
        "CDEF:utilisa=cpuutil,100,*"
        "VDEF:Dd=cpuutiltotd,LSLSLOPE"
        "VDEF:Hd=cpuutiltotd,LSLINT"
        "CDEF:cpuutiltottrendd=cpuutiltotd,POP,Dd,COUNT,*,Hd,+"
        "VDEF:Dm=cpuutiltotm,LSLSLOPE"
        "VDEF:Hm=cpuutiltotm,LSLINT"
        "CDEF:cpuutiltottrendm=cpuutiltotm,POP,Dm,COUNT,*,Hm,+"
	"VDEF:Dy=cpuutiltoty,LSLSLOPE"
	"VDEF:Hy=cpuutiltoty,LSLINT"
	"CDEF:cpuutiltottrendy=cpuutiltoty,POP,Dy,COUNT,*,Hy,+"
        "COMMENT:   Average   \\n"
        "AREA:cpu#00FF00: Configured CPU cores    "
        "GPRINT:cpu:AVERAGE: %2.1lf"
        "$t2"
        "STACK:cpubor#FFFF00: Not assigned CPU cores  "
        "GPRINT:cpubor:AVERAGE: %2.1lf"
        "$t2"
        "LINE1:cpuutiltot#FF0000: Utilization in CPU cores"
        "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
        "COMMENT:("
        "GPRINT:utilisa:AVERAGE: %2.1lf"
        "COMMENT:\%)"
        "$t2"
        "LINE2:cpuutiltottrendd#FFFF00: Utilization in CPU cores - last 1 month trend"
        "$t2"
        "LINE2:cpuutiltottrendm#80FFFF: Utilization in CPU cores - last 3 months trend"
        "$t2"
        "LINE2:cpuutiltottrendy#FF8080: Utilization in CPU cores - last year trend"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
        "VRULE:0#000000"
      );
      my $answer = RRDp::read; 
      if ( $$answer =~ "ERROR" ) {
          error ("$host:$managedname Graph rrdtool error : $$answer");
          if ( $$answer =~ "is not an RRD file" ) {
            (my $err,  my $file, my $txt) = split(/'/,$$answer);
	    error ("Removing as it seems to be corrupted: $file");
            unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
          }
          else {
            print "Graph rrdtool error : $$answer";
          }
      }
      #chomp ($$answer);
    } # end of yearly trend
  }
  else {

  if ( $lpar eq  "mem" ) {
    my $rrd= "$wrkdir/$managedname/$host/mem.rr$type_sam";

    # do not update charts if there is not new data in RRD DB
    if ( -f "$rrd" ) {
      $rrd_time = (stat("$rrd"))[9];
      if ( ($act_time - $rrd_time) > $DELETED_LPARS_TIME ) { #10 days
        #error ("----$rrd");
        return 0;
      }
    }

    my $header = "Memory usage: last $text";

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "y" && -f "$memname.png" && $run_topten == 0 ) {
      my $png_time = (stat("$memname.png"))[9];
      if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "m" && -f "$memname.png" && $run_topten == 0 ) {
      my $png_time = (stat("$memname.png"))[9];
      if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "w" && -f "$memname.png" && $run_topten == 0 ) {
      my $png_time = (stat("$memname.png"))[9];
      if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
        #print "creating graph : $host:$managedname:$lpar:$type_sam:$type no update\n" if $DEBUG ;
	return 0;
      }
    }

    print "creating graph : $host:$managedname:$lpar:$type_sam:$type\n" if $DEBUG ;

    #print "Graph mem  : $lpar\n";
    if ($type =~ "d") {
      RRDp::cmd qq(last "$rrd");
      my $last_tt = RRDp::read;    
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $t ="COMMENT:Updated\\: $l ";
    };  

    if ( $type =~ "y" ) {
      $step_new=86400;
    }

    if ( ! -f "$rrd" ) {
       if ( $type_sam =~ "d" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one day!!!\n";
       }
       if ( $type_sam =~ "h" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one hour!!!\n";
       }
       if ( $type_sam =~ "m" ) {
         error ("$rrd does not exists")
       }
       return 0;
    }

    RRDp::cmd qq(graph "$memname.png"
      "--title" "$header"
      "--start" "now-1$type"
      "--end" "now-1$type+1$type"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=400"
      "--height=150"
      "--step=$step_new"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--vertical-label=Memory in GBytes"
      "--upper-limit=1"
      "--alt-autoscale-max"
      "--base=1024"
      "--x-grid=$xgrid"
      "DEF:free=$rrd:curr_avail_mem:AVERAGE"
      "DEF:fw=$rrd:sys_firmware_mem:AVERAGE"
      "DEF:tot=$rrd:conf_sys_mem:AVERAGE"
      "CDEF:freeg=free,1024,/"
      "CDEF:fwg=fw,1024,/"
      "CDEF:totg=tot,1024,/"
      "CDEF:used=totg,freeg,-"
      "CDEF:used1=used,fwg,-"
      "COMMENT:   Average   \\n"
      "AREA:fwg#0080FF: Firmware memory"
      "GPRINT:fwg:AVERAGE: %4.2lf GB"
      "$t2"
      "STACK:used1#FF8080: Used memory    "
      "GPRINT:used1:AVERAGE: %4.2lf GB"
      "$t2"
      "STACK:freeg#00FF00: Free memory    "
      "GPRINT:freeg:AVERAGE: %4.2lf GB"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
    my $answer = RRDp::read; 
    if ( $$answer =~ "ERROR" ) {
        error ("$host:$managedname Graph rrdtool error : $$answer");
        if ( $$answer =~ "is not an RRD file" ) {
          (my $err,  my $file, my $txt) = split(/'/,$$answer);
          error ("Removing as it seems to be corrupted: $file");
          unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
        }
        else {
          print "Graph rrdtool error : $$answer";
        }
    }

    if ( $run_topten == 1 || $upgrade == 1 ) {
      # also do it after the upgrade (or fresh instal what is upgrade in fact as well
      # create a copy for views, all views have to be snapped at the same moment
      copy("$memname.png", "$memname-$view_suff.png");
    } 

    if ( $type =~ "y" ) {
      # Trend for yearly chart 1y back  + 1y forward
      print "creating graph : $host:$managedname:$lpar:$type_sam:$type:trend\n" if $DEBUG ;

      RRDp::cmd qq(graph "$memname-trend.png"
        "--title" "$header - trends"
        "--start" "now-1$type"
        "--end" "now+1$type"
        "--imgformat" "PNG"
	"--slope-mode"
        "--width=800"
        "--height=150"
        "--step=$step_new"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--vertical-label=Memory in GBytes"
        "--upper-limit=1"
	"--lower-limit=0"
        "--alt-autoscale-max"
        "--base=1024"
        "--x-grid=$xgrid"
        "DEF:freed=$rrd:curr_avail_mem:AVERAGE:start=-1m"
        "DEF:freem=$rrd:curr_avail_mem:AVERAGE:start=-3m"
        "DEF:freey=$rrd:curr_avail_mem:AVERAGE:start=-1y"
        "DEF:free=$rrd:curr_avail_mem:AVERAGE"
        "DEF:fwd=$rrd:sys_firmware_mem:AVERAGE:start=-1m"
        "DEF:fwm=$rrd:sys_firmware_mem:AVERAGE:start=-3m"
        "DEF:fwy=$rrd:sys_firmware_mem:AVERAGE:start=-1y"
        "DEF:fw=$rrd:sys_firmware_mem:AVERAGE"
        "DEF:totd=$rrd:conf_sys_mem:AVERAGE:start=-1m"
        "DEF:totm=$rrd:conf_sys_mem:AVERAGE:start=-3m"
        "DEF:toty=$rrd:conf_sys_mem:AVERAGE:start=-1y"
        "DEF:tot=$rrd:conf_sys_mem:AVERAGE"
        "CDEF:freegd=freed,1024,/"
        "CDEF:freegm=freem,1024,/"
        "CDEF:freegy=freey,1024,/"
        "CDEF:freeg=free,1024,/"
        "CDEF:fwgd=fwd,1024,/"
        "CDEF:fwgm=fwm,1024,/"
        "CDEF:fwgy=fwy,1024,/"
        "CDEF:fwg=fw,1024,/"
        "CDEF:totgd=totd,1024,/"
        "CDEF:totgm=totm,1024,/"
        "CDEF:totgy=toty,1024,/"
        "CDEF:totg=tot,1024,/"
        "CDEF:usedd=totgd,freegd,-"
        "CDEF:usedm=totgm,freegm,-"
        "CDEF:usedy=totgy,freegy,-"
        "CDEF:used=totg,freeg,-"
        "CDEF:used1d=usedd,fwgd,-"
        "CDEF:used1m=usedm,fwgm,-"
        "CDEF:used1y=usedy,fwgy,-"
        "CDEF:used1=used,fwg,-"
        "CDEF:usedtotd=used1d,fwgd,+"
        "CDEF:usedtotm=used1m,fwgm,+"
        "CDEF:usedtoty=used1y,fwgy,+"
        "CDEF:usedtot=used1,fwg,+"
        "VDEF:Dd=usedtotd,LSLSLOPE"
        "VDEF:Hd=usedtotd,LSLINT"
        "CDEF:usedtottrendd=usedtotd,POP,Dd,COUNT,*,Hd,+"
        "VDEF:Dm=usedtotm,LSLSLOPE"
        "VDEF:Hm=usedtotm,LSLINT"
        "CDEF:usedtottrendm=usedtotm,POP,Dm,COUNT,*,Hm,+"
        "VDEF:Dy=usedtoty,LSLSLOPE"
        "VDEF:Hy=usedtoty,LSLINT"
        "CDEF:usedtottrendy=usedtoty,POP,Dy,COUNT,*,Hy,+"
        "COMMENT:   Average   \\n"
        "AREA:fwg#0080FF: Firmware memory"
        "GPRINT:fwg:AVERAGE: %4.2lf GB"
        "$t2"
        "STACK:used1#FF8080: Used memory    "
        "GPRINT:used1:AVERAGE: %4.2lf GB"
        "$t2"
        "STACK:freeg#00FF00: Free memory    "
        "GPRINT:freeg:AVERAGE: %4.2lf GB"
        "$t2"
        "LINE2:usedtottrendd#FFFF00: Used memory total - last 1 month trend"
        "$t2"
        "LINE2:usedtottrendm#80FFFF: Used memory total - last 3 months trend"
        "$t2"
        "LINE2:usedtottrendy#FF8080: Used memory total - last year trend"
        "$t"
        "HRULE:0#000000"
        "VRULE:0#000000"
      );
      my $answer = RRDp::read; 
      if ( $$answer =~ "ERROR" ) {
          error ("$host:$managedname Graph rrdtool error : $$answer");
          if ( $$answer =~ "is not an RRD file" ) {
            (my $err,  my $file, my $txt) = split(/'/,$$answer);
            error ("Removing as it seems to be corrupted: $file");
            unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
          }
          else {
            print "Graph rrdtool error : $$answer";
          }
      }
    }
  }

  else { #lpars (everything except pool and mem)

    my $rrd= "$wrkdir/$managedname/$host/$lpar.rr$type_sam";

    # do not update charts if there is not new data in RRD DB
    if ( -f "$rrd" ) {
      $rrd_time = (stat("$rrd"))[9];
      if ( ($act_time - $rrd_time) > $DELETED_LPARS_TIME ) { #10 days
        #error ("----$rrd");
        return 0;
      }
    }

    my $lpar_slash = $lpar;
    $lpar_slash =~ s/\&\&1/\//g;   # to show slash and not &&1 which is general replacemnt for it
    my $header = "LPAR: $lpar_slash : last $text";

    # check whether it is configured as favourites or not
    # if favourites then create graphs regulary
    my $fav = 0;
    my $fav_indx = 0;
    foreach my $fav_line (@fav_lpar) {
      if ($lpar_slash =~ m/^$fav_line$/ && $managedname =~ m/^$fav_server[$fav_indx]/ ) {
        $fav=1;
        last;
      }
      $fav_indx++;
    }

    # do not update charts if there is not new data in RRD DB
    # change this to do it first time after the midnight
    if ( -f "$name.png" ) {
      $png_time = (stat("$name.png"))[9];
    }
    else {
      $png_time = 0;
    }

    # eonce a day must be created all graphs for top10 and views
    # $run_topten == 1

    if ( -f "$name.png" ) {
      if ( $run_topten == 0 ) {
        (my $sec,my $min,my $h,my $aday,my $m,my $y,my $wday,my $yday,my $isdst) = localtime($act_time);
        ($sec,$min,$h,my $png_day,$m,$y,$wday,$yday,$isdst) = localtime($png_time);
        if ( $aday == $png_day ) {
          # If it is the same day then do not update static graphs
          # static graps need to be updated due to views and top10
          if ( $fav == 0 ) {
	    return 0; # favorites have to be regulary updated
          }
        }
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "y" && -f "$name.png" && $run_topten == 0 ) {
      if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
        return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "m" && -f "$name.png" && $run_topten == 0 ) {
      if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
        return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "w" && -f "$name.png" && $run_topten == 0 ) {
      if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
        return 0;
      }
    }



    if ($type =~ "d") {
      RRDp::cmd qq(last "$rrd");
      my $last_tt = RRDp::read;    
      chomp ($$last_tt);
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $t ="COMMENT:Updated\\: $l ";

      # get LAST value from RRD
      RRDp::cmd qq(fetch "$rrd" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
      my $row = RRDp::read;
      chomp($$row);
      my @row_arr = split(/\n/,$$row);
      my $m = "";
      my $i = 0;

      foreach $m (@row_arr) {
        chomp($m);
         $i++;
         if ( $i == 3 ) {
           my @m_arr = split(/ /,$m);
           # go further ony if it is a digit (avoid it when NaNQ (== no data) is there)
           if ($m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ && $m_arr[4] =~ /\d/ ) {
             #print "m : $m\n" if $DEBUG ;
             #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
  
             my $entitled_cycles = sprintf("%e",$m_arr[2]);
             my $capped_cycles   = sprintf("%e",$m_arr[3]);
             my $uncapped_cycles = sprintf("%e",$m_arr[4]);
             my $curr_proc       = sprintf("%e",$m_arr[1]);
             if ( $entitled_cycles != 0 ) {
               my $util = sprintf ("%.2f",($capped_cycles + $uncapped_cycles) / $entitled_cycles) * $curr_proc;
	       #print ("\n $util $curr_proc $entitled_cycles $capped_cycles $uncapped_cycles\n");
             
               $last ="COMMENT:Last utilization in CPU cores  $util";
	     }
           }
         }
      }
    }  

    if ( $type =~ "y" ) {
      $step_new=86400;
    }

    if ( ! -f "$rrd" ) {
       if ( $type_sam =~ "d" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one day!!!\n";
       }
       if ( $type_sam =~ "h" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one hour!!!\n";
       }
       if ( $type_sam =~ "m" ) {
         print "\n$rrd does not exists\n";
         error ("$rrd does not exists");
       }
       return 0;
    }

    if ( $png_time <= $rrd_time || $YEAR_REFRESH == 0 || $run_topten == 1) {

     # LPM support
     # No LPM here, it has been shifted to detail-cgi.pl
     # normal non LPM stuff

      print "creating graph : $host:$managedname:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
      RRDp::cmd qq(graph "$name.png"
        "--title" "$header"
        "--start" "now-1$type"
        "--end" "now-1$type+1$type"
        "--imgformat" "PNG"
	"--slope-mode"
        "--width=400"
        "--height=150"
        "--step=$step_new"
        "--lower-limit=0.00"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--alt-autoscale-max"
        "--upper-limit=0.1"
        "--vertical-label=CPU cores"
        "--units-exponent=1.00"
	"--alt-y-grid"
        "--x-grid=$xgrid"
        "DEF:cur=$rrd:curr_proc_units:AVERAGE"
        "DEF:ent=$rrd:entitled_cycles:AVERAGE"
        "DEF:cap=$rrd:capped_cycles:AVERAGE"
        "DEF:uncap=$rrd:uncapped_cycles:AVERAGE"
        "CDEF:tot=cap,uncap,+"
        "CDEF:util=tot,ent,/"
        "CDEF:utilperct=util,100,*"
        "CDEF:utiltot=util,cur,*"
        "COMMENT:   Average   \\n"
        "AREA:cur#00FF00: Entitled processor cores"
        "GPRINT:cur:AVERAGE: %2.1lf"
        "$t2"
        "LINE1:utiltot#FF0000: Utilization in CPU cores"
        "GPRINT:utiltot:AVERAGE: %3.3lf"
        "PRINT:utiltot:AVERAGE: %3.3lf"
        "$t2"
        "COMMENT:Entitled CPU utilization   "
        "GPRINT:utilperct:AVERAGE: %2.1lf"
	"COMMENT:\%"
        "$t2"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
        "VRULE:0#000000"
      );
      my $answer = RRDp::read; 
      if ( $$answer =~ "ERROR" ) {
        error ("$host:$managedname Graph rrdtool error : $$answer");
        if ( $$answer =~ "is not an RRD file" ) {
          (my $err,  my $file, my $txt) = split(/'/,$$answer);
          error ("Removing as it seems to be corrupted: $file");
          unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
        }
        else {
          print "Graph rrdtool error : $$answer";;
        }
        return 0;
      }

      if ( $run_topten == 1 || $upgrade == 1 ) {
        # also do it after the upgrade (or fresh instal what is upgrade in fact as well
        # create a copy for views, all views have to be snapped at the same moment
        copy("$name.png", "$name-$view_suff.png");
      } 

      # parse graph output, at the second line is avrg CPU load for TOP10 page
      my $once = 0;
      foreach my $answ (split(/ /,$$answer)) {
        if ( $once == 1 ) {
          $answ =~ s/ //g;
          my $ret = substr($answ,0,1);
          if ($ret =~ /\D/) {
            next;
          }
          else {
            # exclude dedicated lpars
            my $cfg = "$wrkdir/$managedname/$host/cpu.cfg-$lpar";
            if ( -f $cfg ) {
              my $ded=0;
  	      open(FHC, "< $cfg") ;
              while (my $line = <FHC>) {
                chomp($line);
                if ($line =~ "curr_proc_mode") {
                  if ( $line =~ "ded") {
		    $ded=1;
                  }
                  last;
                }
	      }
              close (FHC);
              if ( $ded == 1 ) {
                next; #skip when dedicated
              }
            }
            chomp($answ);
            my $source_type=0;
            if ($HMC) {
              $source_type=1
            }
            if ($SDMC) {
              $source_type=2
            }
            if ( $type =~ "d" ) {
              print TOPD "$answ|$lpar|$managedname|$host|$source_type\n";
            }
            if ( $type =~ "w" ) {
              print TOPW "$answ|$lpar|$managedname|$host|$source_type\n";
            }
            if ( $type =~ "m" ) {
              print TOPM "$answ|$lpar|$managedname|$host|$source_type\n";
            }
            if ( $type =~ "y" ) {
              print TOPY "$answ|$lpar|$managedname|$host|$source_type\n";
            }
          }
        }
        else {
          if ( $answ =~ "ERROR" ) {
            error ("$host:$managedname Graph rrdtool error : $$answer");
            if ( $answ =~ "is not an RRD file" ) {
              (my $err,  my $file, my $txt) = split(/'/,$answ);
	      error ("Removing as it seems to be corrupted: $file");
              unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
            }
            else {
              print "Graph rrdtool error : $answ";;
            }
          }
        }
        if ( $run_topten == 0 ) {
          last; # skip when no topten
        }
        $once++;
      }

      if ( $type =~ "y" ) {
        # Trend for yearly chart 1y back  + 1y forward
        print "creating graph : $host:$managedname:$lpar:$type_sam:$type:trend\n" if $DEBUG ;
    
        RRDp::cmd qq(graph "$name-trend.png"
          "--title" "$header - trends"
          "--start" "now-1$type"
          "--end" "now+1$type"
          "--imgformat" "PNG"
	  "--slope-mode"
          "--width=800"
          "--height=150"
          "--step=$step_new"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "--upper-limit=0.1"
          "--lower-limit=0"
          "--alt-autoscale-max"
          "--vertical-label=CPU cores"
          "--units-exponent=1.00"
  	  "--alt-y-grid"
          "--x-grid=$xgrid"
          "DEF:curd=$rrd:curr_proc_units:AVERAGE:start=-1m"
          "DEF:curm=$rrd:curr_proc_units:AVERAGE:start=-3m"
          "DEF:cury=$rrd:curr_proc_units:AVERAGE:start=-1y"
          "DEF:cur=$rrd:curr_proc_units:AVERAGE"
          "DEF:entd=$rrd:entitled_cycles:AVERAGE:start=-1m"
          "DEF:entm=$rrd:entitled_cycles:AVERAGE:start=-3m"
          "DEF:enty=$rrd:entitled_cycles:AVERAGE:start=-1y"
          "DEF:ent=$rrd:entitled_cycles:AVERAGE"
          "DEF:capd=$rrd:capped_cycles:AVERAGE:start=-1m"
          "DEF:capm=$rrd:capped_cycles:AVERAGE:start=-3m"
          "DEF:capy=$rrd:capped_cycles:AVERAGE:start=-1y"
          "DEF:cap=$rrd:capped_cycles:AVERAGE"
          "DEF:uncapd=$rrd:uncapped_cycles:AVERAGE:start=-1m"
          "DEF:uncapm=$rrd:uncapped_cycles:AVERAGE:start=-3m"
          "DEF:uncapy=$rrd:uncapped_cycles:AVERAGE:start=-1y"
          "DEF:uncap=$rrd:uncapped_cycles:AVERAGE"
          "CDEF:totd=capd,uncapd,+"
          "CDEF:totm=capm,uncapm,+"
          "CDEF:toty=capy,uncapy,+"
          "CDEF:tot=cap,uncap,+"
          "CDEF:utild=totd,entd,/"
          "CDEF:utilm=totm,entm,/"
          "CDEF:utily=toty,enty,/"
          "CDEF:util=tot,ent,/"
          "CDEF:utilperct=util,100,*"
          "CDEF:utiltotd=utild,curd,*"
          "CDEF:utiltotm=utilm,curm,*"
          "CDEF:utiltoty=utily,cury,*"
          "CDEF:utiltot=util,cur,*"
          "VDEF:Dd=utiltotd,LSLSLOPE"
          "VDEF:Hd=utiltotd,LSLINT"
          "CDEF:utiltottrendd=utiltotd,POP,Dd,COUNT,*,Hd,+"
          "VDEF:Dm=utiltotm,LSLSLOPE"
          "VDEF:Hm=utiltotm,LSLINT"
          "CDEF:utiltottrendm=utiltotm,POP,Dm,COUNT,*,Hm,+"
          "VDEF:Dy=utiltoty,LSLSLOPE"
          "VDEF:Hy=utiltoty,LSLINT"
          "CDEF:utiltottrendy=utiltoty,POP,Dy,COUNT,*,Hy,+"
          "COMMENT:   Average   \\n"
          "AREA:cur#00FF00: Entitled processor cores"
          "GPRINT:cur:AVERAGE: %2.1lf"
          "$t2"
          "LINE1:utiltot#FF0000: Utilization in CPU cores"
          "GPRINT:utiltot:AVERAGE: %3.3lf"
          "$t2"
          "COMMENT:Entitled CPU utilization   "
          "GPRINT:utilperct:AVERAGE: %2.1lf"
  	  "COMMENT:\%"
          "$t2"
          "LINE2:utiltottrendd#FFFF00: Utilization in CPU cores - last 1 month trend"
          "$t2"
          "LINE2:utiltottrendm#80FFFF: Utilization in CPU cores - last 3 months trend"
          "$t2"
          "LINE2:utiltottrendy#FF8080: Utilization in CPU cores - last year trend"
          "$last"
          "$t2"
          "$t"
          "HRULE:0#000000"
          "VRULE:0#000000"
        );
        my $answer = RRDp::read; 
        if ( $$answer =~ "ERROR" ) {
          error ("$host:$managedname Graph rrdtool error : $$answer");
          if ( $$answer =~ "is not an RRD file" ) {
            (my $err,  my $file, my $txt) = split(/'/,$$answer);
	    error ("Removing as it seems to be corrupted: $file");
            unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
          }
          else {
            print "Graph rrdtool error : $$answer";;
          }
        }
      }
    }

    # AMS stuff

    if ( $lpar eq '' ) {
      return 0; # some workaround, it should not go here
    }

    $rrd= "$wrkdir/$managedname/$host/$lpar.rm$type_sam";
    $name = "$webdir/$host/$managedname/$lpar/$type-ams";
    # print "001 AMS $rrd\n";

    # do not update charts if there is not new data in RRD DB
    if ( -f "$rrd" ) {
      $rrd_time = (stat("$rrd"))[9];
      if ( ($act_time - $rrd_time) > $DELETED_LPARS_TIME ) { #10 days
        #error ("----$rrd");
        return 0;
      }
    }
    else {
      return 0; # No AMS in place
    }

    $lpar_slash = $lpar;
    $lpar_slash =~ s/\&\&1/\//g;   # to show slash and not &&1 which is general replacemnt for it
    $header = "LPAR Memory: $lpar_slash : last $text";

    if ( -f "$name.png" ) {
      $png_time = (stat("$name.png"))[9];
    }

    # do not update charts if there is not new data in RRD DB
    # change this to do it first time after the midnight
#    if ( -f "$name.png" ) {
#      my $png_time = (stat("$name.png"))[9];
#      (my $sec,my $min,my $h,my $aday,my $m,my $y,my $wday,my $yday,my $isdst) = localtime($act_time);
#      ($sec,$min,$h,my $png_day,$m,$y,$wday,$yday,$isdst) = localtime($png_time);
#      if ( $aday == $png_day ) {
#        # If it is the same day then do not update static graphs
#        # static graps need to be updated due to views and top10
#	return 0;
    #  }
    #}

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "y" && -f "$name.png" ) {
      if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
        return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "m" && -f "$name.png" ) {
      if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
        return 0;
      }
    }

    # do not update charts if there is not new data in RRD DB
    if ( $type =~ "w" && -f "$name.png" ) {
      if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
        return 0;
      }
    }

    if ($type =~ "d") {
      RRDp::cmd qq(last "$rrd");
      my $last_tt = RRDp::read;    
      chomp ($$last_tt);
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $t ="COMMENT:Updated\\: $l ";

      # get LAST value from RRD
      RRDp::cmd qq(fetch "$rrd" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
      my $row = RRDp::read;
      chomp($$row);
      my @row_arr = split(/\n/,$$row);
      my $m = "";
      my $i = 0;

      foreach $m (@row_arr) {
        chomp($m);
         $i++;
         if ( $i == 3 ) {
           my @m_arr = split(/ /,$m);
           # go further ony if it is a digit (avoid it when NaNQ (== no data) is there)
           if ($m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ ) {
             #print "m : $m\n" if $DEBUG ;
             #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
  
             #my $curr_mem = sprintf("%e",$m_arr[1]);
             #my $phys_run_mem    = sprintf("%e",$m_arr[2]);
             if ( $m_arr[2] > 0 ) {
               my $last_mem = sprintf( "%6.2d",$m_arr[2]) / 1024;
               my $last_mem_g = sprintf( "%.2f",$m_arr[2] / 1024);
               $last ="COMMENT:Last memory consuption in GB         $last_mem_g";
             }
           }
         }
      }
    };  

    if ( $type =~ "y" ) {
      $step_new=86400;
    }

    if ( ! -f "$rrd" ) {
       if ( $type_sam =~ "d" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one day!!!\n";
       }
       if ( $type_sam =~ "h" ) {
         print "\n$rrd does not exists, probably freshly enabled utilization data collection for that lpar/managed node, this message should not apear more than one hour!!!\n";
       }
       if ( $type_sam =~ "m" ) {
         print "\n$rrd does not exists\n";
         error ("$rrd does not exists");
       }
       return 0;
    }

    # do not update charts if there is not new data in RRD DB
    $rrd_time = (stat("$rrd"))[9];
    $png_time = "";
    if ( -f "$name.png" ) {
      $png_time = (stat("$name.png"))[9];
    } 
    else {
      $png_time = 0;
    }

    if ( $png_time <= $rrd_time || $YEAR_REFRESH == 0) {

     # LPM support
     $lpm = 0; # memory does not support AMS so far --PH
     if ( $lpm == 1) {
       my $lpm_count = 0;
       my $lpm_suff = "rrl";
       if ($type_sam =~ "d" ) {
         $lpm_suff = "rrk";
       }
       if ($type_sam =~ "h" ) {
         $lpm_suff = "rri";
       }
  
       # never use function "grab", grrrr ...
       my $managedname_space = $managedname;
       if ( $managedname =~ m/ / ) { # workaround for server name with a space inside, nothing else works, grrr
         $managedname_space = "\"".$managedname."\"";
       }
       my $host_space = $host;
       if ( $host =~ m/ / ) { # workaround for server name with a space inside, nothing else works, grrr
         $host_space = "\"".$host."\"";
       }
       my $lpar_space = $lpar;
       if ( $lpar =~ m/ / ) { # workaround for server name with a space inside, nothing else works, grrr
         $lpar_space = "\"".$lpar."\"";
       }

       foreach my $trash (<"$wrkdir/$managedname_space/$host_space/$lpar_space=====*=====*.$lpm_suff">) {
         $lpm_count++;
       }
       # if it is a VIO server then do not go for LPM
       # it is already being exlcuded in install-html.sh, this is just for sure
       foreach my $lpm_line (@lpm_excl_vio) {
         chomp($lpm_line);
         if ( "$lpm_line" =~ m/$lpar/ && length($lpm_line) == length($lpar)){
           print "LPM VIO exclude: $host:$managedname:$lpar_slash:$type_sam:$type - $lpm_line\n" if $DEBUG ;
           $lpm_count=0;
           last;
         }
       }

       if ( $lpm_count) {
         # LPM is detected

         # if daily does not exist then must be immediattely created even weekly/monthly/yeary
	 # if not then graphs would appear there with a delay
         if ( ! -f "$name-lpm.png" ) {
           LoadDataModule::touch ("$name-lpm.png");
           # it is necessary only if there is no upgrade running
           if ( $upgrade == 0 ) {
             lpm ("","","","","","","","",$wrkdir,$webdir,$managedname,$host,$lpar,$lpar_slash,"week",$type_sam,"w",$step_new,"HOUR:8:DAY:1:DAY:1:86400:%a",$act_time,$pic_col,$DEBUG,$rrdtool,$lpm_suff,\@color);
             lpm ("","","","","","","","",$wrkdir,$webdir,$managedname,$host,$lpar,$lpar_slash,"4 weeks",$type_sam,"m",$step_new,"DAY:1:DAY:2:DAY:7:0:%U",$act_time,$pic_col,$DEBUG,$rrdtool,$lpm_suff,\@color);
             if ( -f "$wrkdir/$managedname/$host/$lpar.rrd" ) {
               lpm ("","","","","","","","",$wrkdir,$webdir,$managedname,$host,$lpar,$lpar_slash,"year","d","y",$step_new,"MONTH:1:MONTH:1:MONTH:1:0:%b",$act_time,$pic_col,$DEBUG,$rrdtool,$lpm_suff,\@color);
             }
             else {
               lpm ("","","","","","","","",$wrkdir,$webdir,$managedname,$host,$lpar,$lpar_slash,"year",$type_sam,"y",$step_new,"MONTH:1:MONTH:1:MONTH:1:0:%b",$act_time,$pic_col,$DEBUG,$rrdtool,$lpm_suff,\@color);
             }
           }
         }

         # normal daily stuff
         lpm ("","","","","","","","",$wrkdir,$webdir,$managedname,$host,$lpar,$lpar_slash,$text,$type_sam,$type,$step_new,$xgrid,$act_time,$pic_col,$DEBUG,$rrdtool,$lpm_suff,\@color);
       }
     }
     # normal non LPM stuff

      print "graphing AMS   : $host:$managedname:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
      RRDp::cmd qq(graph "$name.png"
        "--title" "$header"
        "--start" "now-1$type"
        "--end" "now-1$type+1$type"
        "--imgformat" "PNG"
	"--slope-mode"
        "--width=400"
        "--height=150"
        "--step=$step_new"
        "--lower-limit=0.00"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--alt-autoscale-max"
        "--upper-limit=0.1"
        "--vertical-label= Memory in GB"
        "--units-exponent=1.00"
	"--alt-y-grid"
        "--x-grid=$xgrid"
        "DEF:ent=$rrd:curr_mem:AVERAGE"
        "DEF:cur=$rrd:phys_run_mem:AVERAGE"
        "DEF:io=$rrd:io_ent:AVERAGE"
        "CDEF:entg=ent,1024,/"
        "CDEF:curg=cur,1024,/"
        "CDEF:iog=io,1024,/"
        "COMMENT:   Average   \\n"
        "AREA:entg#00FF00: Current allocated memory    "
        "GPRINT:entg:AVERAGE: %6.2lf"
        "$t2"
        "LINE1:iog#0080FF: Current allocated IO memory "
        "GPRINT:iog:AVERAGE: %6.2lf"
        "PRINT:iog:AVERAGE: %6.2lf"
        "$t2"
        "LINE1:curg#FF0000: Actuall physical consumption"
        "GPRINT:curg:AVERAGE: %6.2lf"
        "PRINT:curg:AVERAGE: %6.2lf"
        "$t2"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
        "VRULE:0#000000"
      );
      my $answ = RRDp::read; 
      if ( $$answ =~ "ERROR" ) {
        error ("$host:$managedname Graph rrdtool error : $$answ");
        if ( $answ =~ "is not an RRD file" ) {
          (my $err,  my $file, my $txt) = split(/'/,$answ);
          error ("Removing as it seems to be corrupted: $file");
          unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
        }
        else {
          print "Graph rrdtool error : $answ";;
        }
      }

      if ( $run_topten == 1 || $upgrade == 1 ) {
        # also do it after the upgrade (or fresh instal what is upgrade in fact as well
        # create a copy for views, all views have to be snapped at the same moment
        copy("$name.png", "$name-$view_suff.png");
      } 

      if ( $type =~ "XXX" ) { # --PH not AMS trending yet
      #if ( $type =~ "y" ) {
        # Trend for yearly chart 1y back  + 1y forward
        print "creating graph : $host:$managedname:$lpar:$type_sam:$type:trend\n" if $DEBUG ;
    
        RRDp::cmd qq(graph "$name-trend.png"
          "--title" "$header - trends"
          "--start" "now-1$type"
          "--end" "now+1$type"
          "--imgformat" "PNG"
	  "--slope-mode"
          "--width=800"
          "--height=150"
          "--step=$step_new"
          "--color=BACK#$pic_col"
          "--color=SHADEA#$pic_col"
          "--color=SHADEB#$pic_col"
          "--color=CANVAS#$pic_col"
          "--upper-limit=0.1"
          "--lower-limit=0"
          "--alt-autoscale-max"
          "--vertical-label=CPU cores"
          "--units-exponent=1.00"
  	  "--alt-y-grid"
          "--x-grid=$xgrid"
          "DEF:curd=$rrd:curr_proc_units:AVERAGE:start=-1m"
          "DEF:curm=$rrd:curr_proc_units:AVERAGE:start=-3m"
          "DEF:cury=$rrd:curr_proc_units:AVERAGE:start=-1y"
          "DEF:cur=$rrd:curr_proc_units:AVERAGE"
          "DEF:entd=$rrd:entitled_cycles:AVERAGE:start=-1m"
          "DEF:entm=$rrd:entitled_cycles:AVERAGE:start=-3m"
          "DEF:enty=$rrd:entitled_cycles:AVERAGE:start=-1y"
          "DEF:ent=$rrd:entitled_cycles:AVERAGE"
          "DEF:capd=$rrd:capped_cycles:AVERAGE:start=-1m"
          "DEF:capm=$rrd:capped_cycles:AVERAGE:start=-3m"
          "DEF:capy=$rrd:capped_cycles:AVERAGE:start=-1y"
          "DEF:cap=$rrd:capped_cycles:AVERAGE"
          "DEF:uncapd=$rrd:uncapped_cycles:AVERAGE:start=-1m"
          "DEF:uncapm=$rrd:uncapped_cycles:AVERAGE:start=-3m"
          "DEF:uncapy=$rrd:uncapped_cycles:AVERAGE:start=-1y"
          "DEF:uncap=$rrd:uncapped_cycles:AVERAGE"
          "CDEF:totd=capd,uncapd,+"
          "CDEF:totm=capm,uncapm,+"
          "CDEF:toty=capy,uncapy,+"
          "CDEF:tot=cap,uncap,+"
          "CDEF:utild=totd,entd,/"
          "CDEF:utilm=totm,entm,/"
          "CDEF:utily=toty,enty,/"
          "CDEF:util=tot,ent,/"
          "CDEF:utilperct=util,100,*"
          "CDEF:utiltotd=utild,curd,*"
          "CDEF:utiltotm=utilm,curm,*"
          "CDEF:utiltoty=utily,cury,*"
          "CDEF:utiltot=util,cur,*"
          "VDEF:Dd=utiltotd,LSLSLOPE"
          "VDEF:Hd=utiltotd,LSLINT"
          "CDEF:utiltottrendd=utiltotd,POP,Dd,COUNT,*,Hd,+"
          "VDEF:Dm=utiltotm,LSLSLOPE"
          "VDEF:Hm=utiltotm,LSLINT"
          "CDEF:utiltottrendm=utiltotm,POP,Dm,COUNT,*,Hm,+"
          "VDEF:Dy=utiltoty,LSLSLOPE"
          "VDEF:Hy=utiltoty,LSLINT"
          "CDEF:utiltottrendy=utiltoty,POP,Dy,COUNT,*,Hy,+"
          "COMMENT:   Average   \\n"
          "AREA:cur#00FF00: Entitled processor cores"
          "GPRINT:cur:AVERAGE: %2.1lf"
          "$t2"
          "LINE1:utiltot#FF0000: Utilization in CPU cores"
          "GPRINT:utiltot:AVERAGE: %3.3lf"
          "$t2"
          "COMMENT:Entitled CPU utilization   "
          "GPRINT:utilperct:AVERAGE: %2.1lf"
  	  "COMMENT:\%"
          "$t2"
          "LINE2:utiltottrendd#80FFFF: Utilization in CPU cores - last 1 month trend"
          "$t2"
          "LINE2:utiltottrendm#80FFFF: Utilization in CPU cores - last 3 months trend"
          "$t2"
          "LINE2:utiltottrendy#FF8080: Utilization in CPU cores - last year trend"
          "$last"
          "$t2"
          "$t"
          "HRULE:0#000000"
          "VRULE:0#000000"
        );
        my $answer = RRDp::read; 
        if ( $$answer =~ "ERROR" ) {
          error ("$host:$managedname Graph rrdtool error : $$answer");
          if ( $$answer =~ "is not an RRD file" ) {
            (my $err,  my $file, my $txt) = split(/'/,$$answer);
	    error ("Removing as it seems to be corrupted: $file");
            unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
          }
          else {
            print "Graph rrdtool error : $$answer";;
          }
        }
      }
    }
  }
  }
  }  
  return 0;
}

# static memory aggregated
# if AMS in place that that info is in .rms AMS files
# when no AMS then .rsm

sub multiview_mem_static
{
  my $name = shift;
  my $type = shift;
  my $type_sam = shift;
  my $act_time = shift ;
  my $managedname = shift;
  my $step_new = shift;
  my $text = shift;
  my $xgrid = shift;
  my $req_time = 0;

  my $header = "Memory aggregated : last $text";

  opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0;
  my @files = "";
  if ( $type_sam =~ "m" ) {
    my @files_unsorted = grep(/\.r[m|s]m$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "h" ) {
    my @files_unsorted = grep(/\.r[m|s]h$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "d" ) {
    my @files_unsorted = grep(/\.r[m|s]d$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
    $req_time = $act_time - 31536000;
  }
  closedir(DIR);
  my $file = "";
  my $i = 0;
  my $lpar = "";
  my $cmd = "";
  my $j = 0;

  if ( "$type" =~ "d" ) {
    $req_time = $act_time - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time - 2764800;
  }
  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d"  || "$type" =~ "y" ) {
    $req_time = $act_time - 31536000;
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"Memory in GB\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " COMMENT:\\\"Current allocated memory in GB\\:    average\\l\\\"";
  my $gtype="AREA";
  $cmd .= " DEF:max=\\\"$wrkdir/$managedname/$host/mem.rr$type_sam\\\":conf_sys_mem:AVERAGE";
  $cmd .= " CDEF:maxg=max,1024,/";
  $cmd .= " DEF:fw=\\\"$wrkdir/$managedname/$host/mem.rr$type_sam\\\":sys_firmware_mem:AVERAGE";
  $cmd .= " CDEF:fwg=fw,1024,/";
  $cmd .= " $gtype:fwg#0080FF:\\\"Firmware                 \\\"";
  $cmd .= " GPRINT:fwg:AVERAGE:\\\"     %6.2lf \\l\\\"";
  $cmd .= " COMMENT:\\n";
  $gtype="STACK";

  my $found_at_least_one = 0;
  foreach $file (@files) {
    chomp($file);
    $found_at_least_one = 1;

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = (stat("$wrkdir/$managedname/$host/$file"))[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    $lpar = $file;
    $lpar =~ s/.rmh//;
    $lpar =~ s/.rmm//;
    $lpar =~ s/.rmd//;
    $lpar =~ s/.rsm//;
    $lpar =~ s/.rsh//;
    $lpar =~ s/.rsd//;
    my $lpar_space = $lpar;
    $lpar_space =~ s/\&\&1/\//g;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    for(my $k = length($lpar); $k < 25; $k++) {
      $lpar_space .= " ";
    }

    # Exclude pools and memory
    if ((("$lpar" eq  "pool") && (length($lpar) == 4)) || (( "$lpar" eq  "mem") && (length($lpar) == 3)) || ( $lpar =~  "SharedPool[0-9]" ) || ( $lpar =~  "SharedPool[1-9][0-9]") ) {
      next;
    }
 
    # Assure that lpar colors is same for each lpar through daily - yearly charts
    my $l;
    my $l_count = 0;
    my $found = -1;
    foreach $l (@keep_color_mem) {
      if ( $l eq '' || $l =~ m/^ $/ ) {
        next;
      }
      if ( $l =~ m/^$lpar$/ ) {
        $found = $l_count;
        last;
      }
      $l_count++;
    }
    if ( $found == -1 ) {
      $keep_color_mem[$l_count] = $lpar;
      $found = $l_count;
    }

    #print "$found $lpar $color[$found]\n";
    print "$wrkdir/$managedname/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);

    # bulid RRDTool cmd
    $cmd .= " DEF:cur${i}=\\\"$wrkdir/$managedname/$host/$file\\\":curr_mem:AVERAGE";
    $cmd .= " CDEF:curg${i}=cur${i},1024,/";
    $cmd .= " $gtype:curg${i}$color[$found]:\\\"$lpar_space\\\"";

    # put carriage return after each second lpar in the legend
    if ($j == 1) {
      $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     %6.2lf \\l\\\"";
      $j = 0;
    }
    else {
      $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     %6.2lf \\l\\\"";
	# --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $j++
    }
    $i++;
    $gtype="STACK";
  }

  if ( $found_at_least_one == 0 ) {
    return 0; # no source has been found
  }

  $cmd .= " COMMENT:\\n";
  $cmd .= " LINE2:maxg#000000:\\\"Available memory         \\\"";
  $cmd .= " GPRINT:maxg:AVERAGE:\\\"     %6.2lf \\l\\\"";
  $cmd .= " HRULE:0#000000";
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  my  $FH;
  my $tmp_file="$tmpdir/mem-agg-$host-$managedname-$type.cmd";
  open(FH, "> $tmp_file") || error(" Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
  
  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("$host : Multi graph mem aggregated rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err,  my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("$host: $cmd : Multi graph AMS rrdtool error : $ret");
    }
  }
  #unlink ("$tmp_file");
  # do not remove here, it is used by pop up detail

  if ( $run_topten == 1 || $upgrade == 1 ) {
    # also do it after the upgrade (or fresh instal what is upgrade in fact as well
    # create a copy for views, all views have to be snapped at the same moment
    copy("$name.png", "$name-$view_suff.png");
  } 

  return 0;
}


# AMS
sub multiview_mem
{
  my $name = shift;
  my $type = shift;
  my $type_sam = shift;
  my $act_time = shift ;
  my $managedname = shift;
  my $step_new = shift;
  my $text = shift;
  my $xgrid = shift;
  my $req_time = 0;

  my $header = "Active Memory Sharing : last $text";

  opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0;
  my @files = "";
  if ( $type_sam =~ "m" ) {
    my @files_unsorted = grep(/\.rmm$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "h" ) {
    my @files_unsorted = grep(/\.rmh$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "d" ) {
    my @files_unsorted = grep(/\.rmd$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
    $req_time = $act_time - 31536000;
  }
  closedir(DIR);
  my $file = "";
  my $i = 0;
  my $lpar = "";
  my $cmd = "";
  my $j = 0;

  if ( "$type" =~ "d" ) {
    $req_time = $act_time - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time - 2764800;
  }
  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d"  || "$type" =~ "y" ) {
    $req_time = $act_time - 31536000;
  }


  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"Memory in GB\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  if ( "$type" =~ "d" ) {
    $cmd .= " COMMENT:\\\"Current allocated memory in GB\\:    average    maximum     last\\l\\\"";
  }
  else {
    $cmd .= " COMMENT:\\\"Current allocated memory in GB\\:    average    maximum\\l\\\"";
  }
  my $gtype="AREA";
  if ( -f "$wrkdir/$managedname/$host/mem-pool.rr$type_sam" ) {
    $cmd .= " DEF:cur_pool=\\\"$wrkdir/$managedname/$host/mem-pool.rr$type_sam\\\":curr_pool_mem:AVERAGE";
    $cmd .= " CDEF:cur_poolg=cur_pool,1024,/";
  }

  my $found_at_least_one = 0;

  foreach $file (@files) {
    chomp($file);
    $found_at_least_one = 1;

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = (stat("$wrkdir/$managedname/$host/$file"))[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    $lpar = $file;
    $lpar =~ s/.rmh//;
    $lpar =~ s/.rmm//;
    $lpar =~ s/.rmd//;
    my $lpar_space = $lpar;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    for(my $k = length($lpar_space); $k < 25; $k++) {
      $lpar_space .= " ";
    }

    # Exclude pools and memory
    if ((("$lpar" eq  "pool") && (length($lpar) == 4)) || (( "$lpar" eq  "mem") && (length($lpar) == 3)) || ( $lpar =~  "SharedPool[0-9]" ) || ( $lpar =~  "SharedPool[1-9][0-9]") ) {
      next;
    }
 
    # Assure that lpar colors is same for each lpar through daily - yearly charts
    my $l;
    my $l_count = 0;
    my $found = -1;
    foreach $l (@keep_color_ams) {
      if ( $l eq '' || $l =~ m/^ $/ ) {
        next;
      }
      if ( $l =~ m/^$lpar$/ ) {
        $found = $l_count;
        last;
      }
      $l_count++;
    }
    if ( $found == -1 ) {
      $keep_color_ams[$l_count] = $lpar;
      $found = $l_count;
    }

    #print "$found $lpar $color[$found]\n";
    print "$wrkdir/$managedname/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);

    # bulid RRDTool cmd
    $cmd .= " DEF:cur${i}=\\\"$wrkdir/$managedname/$host/$file\\\":phys_run_mem:AVERAGE";
    $cmd .= " CDEF:curg${i}=cur${i},1024,/";
    $cmd .= " $gtype:curg${i}$color[$found]:\\\"$lpar_space\\\"";

    # put carriage return after each second lpar in the legend
    if ($j == 1) {
      $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     %6.2lf \\\"";
      if ( "$type" =~ "d" ) {
        $cmd .= " GPRINT:curg${i}:MAX:\\\" %6.2lf \\\"";
        $cmd .= " GPRINT:curg${i}:LAST:\\\" %6.2lf \\l\\\"";
      }
      else {
        $cmd .= " GPRINT:curg${i}:MAX:\\\" %6.2lf \\l\\\"";
      }
      $j = 0;
    }
    else {
      $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     %6.2lf \\\"";
      if ( "$type" =~ "d" ) {
        $cmd .= " GPRINT:curg${i}:MAX:\\\" %6.2lf \\\"";
        $cmd .= " GPRINT:curg${i}:LAST:\\\" %6.2lf \\l\\\"";
      }
      else {
        $cmd .= " GPRINT:curg${i}:MAX:\\\" %6.2lf \\l\\\"";
      }
	# --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $j++
    }
    $i++;
    $gtype="STACK";
  }

  if ( $found_at_least_one == 0 ) {
    return 0; # no source has been found
  }

  if ( -f "$wrkdir/$managedname/$host/mem-pool.rr$type_sam" ) {
    $cmd .= " COMMENT:\\n";
    $cmd .= " LINE2:cur_poolg#000000:\\\"Configured max mem pool  \\\"";
    $cmd .= " GPRINT:cur_poolg:AVERAGE:\\\"     %6.2lf \\l\\\"";
  }
  $cmd .= " HRULE:0#000000";
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  my  $FH;
  my $tmp_file="/var/tmp/lpar2rrd.tmp-$$";
  open(FH, "> $tmp_file") || error(" Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
  
  # Create directory here if necessary!!! not earlier 
  if (! -d "$webdir/$host/$managedname/mem-multi" ) {
    print "mkdir          : $host:$managedname $webdir/$host/$managedname/mem-multi\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host/$managedname/mem-multi");
    mkdir("$webdir/$host/$managedname/mem-multi", 0755) || error(" Cannot mkdir $webdir/$host/$managedname/mem-multi: $!".__FILE__.":".__LINE__) && return 0;
  } 

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("$host : Multi graph AMS rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err,  my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("$host: $cmd : Multi graph AMS rrdtool error : $ret");
    }
  }
  unlink ("$tmp_file");

  if ( $run_topten == 1 || $upgrade == 1 ) {
    # also do it after the upgrade (or fresh instal what is upgrade in fact as well
    # create a copy for views, all views have to be snapped at the same moment
    copy("$name.png", "$name-$view_suff.png");
  } 

  return 0;
}

# LPARs aggregated per a server or per a HMC
sub multiview
{
  my ($name,$type,$type_sam,$act_time,$managedname,$step_new,$text,$xgrid) = @_;
  my $req_time = 0;

  my $header = "LPARs aggregated : last $text";

  my $file = "";
  my $i = 0;
  my $lpar = "";
  my $cmd = "";
  my $j = 0;

  if ( "$type" =~ "d" ) {
    $req_time = $act_time - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time - 2764800;
  }
  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d"  || "$type" =~ "y" ) {
    $req_time = $act_time - 31536000;
  }


  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"CPU cores\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " COMMENT:\\\"Utilization in CPU cores\\:    average  maximum\\l\\\"";


  my $gtype="AREA";

  opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0;
  my @files = "";
  if ( $type_sam =~ "m" ) {
    my @files_unsorted = grep(/\.rrm$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "h" ) {
    my @files_unsorted = grep(/\.rrh$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "d" ) {
    my @files_unsorted = grep(/\.rrd$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
    $req_time = $act_time - 31536000;
  }
  closedir(DIR);

  foreach $file (@files) {
    chomp($file);

    if ($file =~ m/^\.rr/ ) {
      next; # avoid .rrm
    }

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = (stat("$wrkdir/$managedname/$host/$file"))[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    $lpar = $file;
    $lpar =~ s/.rrh//;
    $lpar =~ s/.rrm//;
    $lpar =~ s/.rrd//;
    my $lpar_space = $lpar;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    for(my $k = length($lpar_space); $k < 25; $k++) {
      $lpar_space .= " ";
    }

    # Exclude pools and memory
    if ($lpar =~ m/^cod$/ || $lpar =~ m/^pool$/ || $lpar =~ m/^mem$/ || $lpar =~  /^SharedPool[0-9]$/ || $lpar =~  m/^SharedPool[1-9][0-9]$/ || $lpar =~ m/^mem-pool$/) {
      next;
    }
 
    # Assure that lpar colors is same for each lpar through daily - yearly charts
    my $l;
    my $l_count = 0;
    my $found = -1;
    foreach $l (@keep_color_lpar) {
      if ( $l eq '' || $l =~ m/^ $/ ) {
        next;
      }
      if ( $l =~ m/^$lpar$/ ) {
        $found = $l_count;
        last;
      }
      $l_count++;
    }
    if ( $found == -1 ) {
      $keep_color_lpar[$l_count] = $lpar;
      $found = $l_count;
    }

    #print "$found $lpar $color[$found]\n";
    print "$wrkdir/$managedname/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);

    # bulid RRDTool cmd
    $cmd .= " DEF:cap${i}=\\\"$wrkdir/$managedname/$host/$file\\\":capped_cycles:AVERAGE";
    $cmd .= " DEF:uncap${i}=\\\"$wrkdir/$managedname/$host/$file\\\":uncapped_cycles:AVERAGE";
    $cmd .= " DEF:ent${i}=\\\"$wrkdir/$managedname/$host/$file\\\":entitled_cycles:AVERAGE";
    $cmd .= " DEF:cur${i}=\\\"$wrkdir/$managedname/$host/$file\\\":curr_proc_units:AVERAGE";

    $cmd .= " CDEF:tot${i}=cap${i},uncap${i},+";
    $cmd .= " CDEF:util${i}=tot${i},ent${i},/";
    $cmd .= " CDEF:utiltotu${i}=util${i},cur${i},*";
    $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";
    $cmd .= " $gtype:utiltot${i}$color[$found]:\\\"$lpar_space\\\"";

    # put carriage return after each second lpar in the legend
    if ($j == 1) {
      $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.2lf \\\"";
      $cmd .= " GPRINT:utiltot${i}:MAX:\\\" %3.2lf \\l\\\"";
      $j = 0;
    }
    else {
      $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.2lf \\\"";
      $cmd .= " GPRINT:utiltot${i}:MAX:\\\" %3.2lf \\l\\\"";
      #$cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.2lf \\t\\\"";
	# --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $j++
    }
    $gtype="STACK";
    $i++;
  }
  if ( "$type" =~ "d" ) {
    if ($j == 1) {
      $cmd .= " COMMENT:\\\" \\l\\\"";
    }
    $cmd .= " COMMENT:\\\"\(Note that for CPU dedicated LPARs is always shown their whole entitlement\)\\\"";
  }
  $cmd .= " HRULE:0#000000";
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  my  $FH;
  my $tmp_file="/var/tmp/lpar2rrd.tmp-$$";
  open(FH, "> $tmp_file") || error(" Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
  
  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("$host : Multi graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err,  my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("$host: $cmd : Multi graph rrdtool error : $ret");
    }
  }
  unlink ("$tmp_file");

  if ( $run_topten == 1 || $upgrade == 1 ) {
    # also do it after the upgrade (or fresh instal what is upgrade in fact as well
    # create a copy for views, all views have to be snapped at the same moment
    copy("$name.png", "$name-$view_suff.png");
  } 

  return 0;
}

sub draw_all {
  my $lpar = shift;
  my $managedname = shift;

  @keep_color_lpar  = "";
  @keep_color_pools = "";
  @keep_color_mem = "";
  @keep_color_ams = "";

  if ( $type_sam =~ "d") {
    draw_graph ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$lpar,$managedname,$type_sam);
  }
  else {
    draw_graph ("day","d","MINUTE:60:HOUR:2:HOUR:4:0:%H",$lpar,$managedname,$type_sam);
    draw_graph ("week","w","HOUR:8:DAY:1:DAY:1:86400:%a",$lpar,$managedname,$type_sam);
    draw_graph ("4 weeks","m","DAY:1:DAY:2:DAY:7:0:%U",$lpar,$managedname,$type_sam);
    if ( $IVM == 1 || $SDMC == 1 ) {
      draw_graph ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$lpar,$managedname,$type_sam);
    }
  }
  return 0;
}    

sub rrd_check {
  my $managedname = shift;

  # Check wheter do initial or normal load
  opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0 ;
  my @files = "";
  if ( $type_sam =~ "m" ) {
    my @files_unsorted = grep(/\.rrm$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "h" ) {
    my @files_unsorted = grep(/\.rrh$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "d" ) {
    my @files_unsorted = grep(/\.rrd$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  closedir(DIR);
  my $file = "";
  my $count = 0;

  foreach $file (@files) {
    $count++;
  }

  if ( $count == 0 ) {
    print "There is no RRD: $host:$managedname attempting to do initial load, be patient, it might take some time\n" if $DEBUG ;
    # get last 62ays (HMC keeps hourly history data just for 2 months, not more)
    # no idea for shother sampe rates, it is not documented yet
    # it is for initial load
    # daily data it keeps for last 2years and monthly for last 10years
    # let load far enough backward for initial load (value in hours or in days)
    $loadhours=$INIT_LOAD_IN_HOURS_BACK; 
    $loadmins=$INIT_LOAD_IN_HOURS_BACK*60;
  }
  return 0;
}

sub rrd_find {
  my $managedname = shift;
  my $host = shift;

  $date = localtime();
  print "date graph     : $host:$managedname $date\n" if $DEBUG ;

  # find out existing RRD databases for particular hostname
  opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0;
  my @files = "";
  if ( $type_sam =~ "m" ) {
    my @files_unsorted = grep(/\.rrm$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "h" ) {
    my @files_unsorted = grep(/\.rrh$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "d" ) {
    my @files_unsorted = grep(/\.rrd$/,readdir(DIR));
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  closedir(DIR);
  my $file = "";
  my $pid  = "";

  foreach $file (@files) {
     chomp($file);
     $file =~ s/.rrh$//;
     $file =~ s/.rrm$//;
     $file =~ s/.rrd$//;
     #print "Drawing charts for : $file\n" if $DEBUG ;
     draw_all("$file",$managedname);
  }
  return 0;
}

sub save_config {
  my $managedname = shift;
  my $date = shift;

  my $out_file = "$CONFIG_HISTORY/$managedname/$host/config.cfg";
  my $out_file_tmp = "$CONFIG_HISTORY/$managedname/$host/config.cfg-tmp";
  my $out_file_shell = $out_file;
  my $out_file_tmp_shell = $out_file_tmp;
  $out_file_tmp_shell =~ s/ /\\ /g;
  $out_file_shell =~ s/ /\\ /g;
  my $lpar = "";
  my $hw_cfg_lpar = "$bindir/hw_cfg_lpar.sh";
  my $hw_cfg_sys  = "$bindir/hw_cfg_sys.sh";

  #if ( ! -f "$hw_cfg_lpar" ) {
  #  $hw_cfg_lpar="true"
  #}
  #if ( ! -f "hw_cfg_sys" ) {
  #  $hw_cfg_lpar="true"
  #}

  # do not update cfg file if less than 1 day
  if ( -f "$out_file" ) {
    my $png_time  = (stat("$out_file"))[9];
    (my $sec,my $min,my $h,my $aday,my $m,my $y,my $wday,my $yday,my $isdst) = localtime(time());
    ($sec,$min,$h,my $png_day,$m,$y,$wday,$yday,$isdst) = localtime($png_time);
    if ( $aday == $png_day ) {
      return 0;
    }
    else {
      print "cfg refresh    : $aday != $png_day\n" if $DEBUG ;
    }
  }
  print "config save    : $host:$managedname once a day ... \n" if $DEBUG ;
  print "cfg save sys   : $host:$managedname \n" if $DEBUG ;

  if ( $IVM == 1 ) {
    # save CPU speed for CPU workload estimator
    `$SSH $hmc_user\@$host "ioscli lsdev -dev proc0 -attr frequency|tail -1" > $wrkdir/$managedname/$host/cpu_speed.txt`;
  }

  LoadDataModule::touch ("config save");
  # read hw_cfg_sys.sh
  open(FHS, "< $hw_cfg_sys") || error(" Cannot open  $hw_cfg_sys: $!".__FILE__.":".__LINE__) && return 0;
  my $cmd_sys = "";
  while (my $line = <FHS>) {
    $cmd_sys = $cmd_sys.$line;
  }
  close (FHS);
  $cmd_sys =~ s/\$managedname/$managedname/g;
  #$cmd_sys =~ s/\$managedname/\\"$managedname\\"/g;
  # with above uncomented do not work systems with space inside 
  #print "$cmd_sys\n";

  # read hw_cfg_lpar.sh
  open(FHL, "< $hw_cfg_lpar") || error(" Cannot open  $hw_cfg_lpar: $!".__FILE__.":".__LINE__) && return 0;
  my $cmd_lpar     = "";
  my $cmd_lpar_glb = "";
  while (my $line = <FHL>) {
    $cmd_lpar_glb = $cmd_lpar_glb.$line;
  }
  close (FHL);
  $cmd_lpar_glb =~ s/\$managedname/$managedname/g;
  #$cmd_lpar_glb =~ s/\$managedname/\\"$managedname\\"/g;
  # with above uncomented do not work systems with space inside 
  #print "$cmd_lpar_glb\n";

  # get list of lpars
  my @lparlist;
  if ( $IVM == 1 ) {
    # IVM does not support "sort"
    @lparlist=`$SSH $hmc_user\@$host "lssyscfg -r lpar -m \\"$managedname\\" -F name|sed 's/ /%20/g'"`;
  }
  else {
    @lparlist=`$SSH $hmc_user\@$host "lssyscfg -r lpar -m \\"$managedname\\" -F name|sort -f|sed 's/ /%20/g'"`;
  }

  `echo "<HTML> <HEAD> <TITLE>LPAR2RRD</TITLE> </HEAD> \
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > \
	<PRE>" > $out_file_tmp_shell`;

  #`ssh $hmc_user\@$host ". $hw_cfg_sys $managedname" | 
  # sed -e 's/sys_time=[0-9][0-9]\\\/[0-9][0-9]\\\/[0-9][0-9][0-9][0-9] [0-9][0-9]:[0-9][0-9]:[0-9][0-9],//g' \
  if ( $IVM == 1 ) {
    # IVM does not support "sort"
    $cmd_sys =~ s/|sort -nk 2 -t=//g;
    $cmd_sys =~ s/|sort -f//g;
    $cmd_sys =~ s/|sort//g;
  }

  `$SSH $hmc_user\@$host "$cmd_sys"  >> $out_file_tmp_shell`;

  foreach $lpar (@lparlist) {
    chomp($lpar);
    $cmd_lpar = $cmd_lpar_glb;
    # workaround for lpars which have a space within the name
    $cmd_lpar =~ s/\$lpar_space/$lpar/g;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g; # for those which have a slash
    $cmd_lpar =~ s/\$lpar/$lpar/g;
    if ( $IVM == 1 ) {
      # IVM does not support "sort"
      $cmd_lpar =~ s/|sort -nk 2 -t=//g;
      $cmd_lpar =~ s/|sort -f//g;
      $cmd_lpar =~ s/|sort//g;
    }
    print "cfg save       : $host:$managedname:$lpar\n";
    `$SSH $hmc_user\@$host "$cmd_lpar" 2>/dev/null| \
	sed -e 's/^lpar_name=.*,lpar_id=[1-9],//g'  -e 's/^lpar_name=.*,lpar_id=[1-9][0-9],//g'  | \
	sed -e 's/,lpar_name=.*,lpar_id=[1-9],/,/g' -e 's/,lpar_name=.*,lpar_id=[1-9][0-9],/,/g' >> $out_file_tmp_shell`;
	# must be 2 x sed, 4 x "-e" is does not work !!!
  }
  `echo "</PRE></BODY></HTML>" >> $out_file_tmp_shell`;
  
  print "cfg save form  : $host:$managedname \n" if $DEBUG ;
  # copy $out_file_tmp to $out_file and devide some very long rows into more rows to be better readable in the web
  open(FHR, "< $out_file_tmp") || error(" Cannot open  $out_file_tmp: $!".__FILE__.":".__LINE__) && return 0;
  open(FHW, "> $out_file")     || error(" Cannot open  $out_file: $!".__FILE__.":".__LINE__) && return 0; 
  my $count=0;
  my $count_prof=0;
  my $line_prev="";
  my $count_no_result=0;
  
  while (my $line = <FHR>) {
    chomp($line);

    # count == 2 is only for LPAR profiles  & CPU pools where can be more than 1 profile and it should be devided into more rows
    if ($count == 2) {
      if ( $line =~ m/^name=/ ) {
        if ($count_prof) {
          print FHW "\n\n";
        }
      }
      else {
        $count=0;
      }
      $count_prof++;
    }
    if ( $line =~ "LPAR profiles:") {
      $count=2;
      $count_prof=0;
      print FHW "$line\n";
      next;
    }

    if ( $line =~ "Memory - AMS:" || $line =~ "Memory - AMS - paging:" || $line =~ "Virtual VASI:" || $line =~ "Virtual slots per lpar:" || $line =~ "Physical IO per io pool level sys:" || $line =~ "Physical IO per io pool level pool:" || $line =~ "Ethernet:" || $line =~ "HEA logical:" || $line =~ "HEA physical per system:" || $line =~ "Logical HEA per system:" || $line =~ "Virtual SCSI:" || $line =~ "Virtual Ethernet:" || $line =~ "Logical HEA:" || $line =~ "CPU pools:" || $line =~ "Physical adapters:" || $line =~ "Physical IO per slot children:" || $line =~ "Physical IO per taggedio:" || $line =~ "HEA physical per port:" || $line =~ "HEA physical per port group:" || $line =~ "HEA logical per port:" || $line =~ "HCA adapters:" || $line =~ "SNI adapters:" || $line =~ "Virtual OptiConnec:" ) {
      $line_prev=$line;
      $count_no_result=1;
      next;
    }

    if ($count_no_result) {
      if ( $line =~ "HSCL" || $line =~ "VIOSE0" || $line =~ "The managed system does not support hardware discovery." || $line =~ "The managed system does not support multiple shared processor pools." || $line =~ "No results were found.") {
        $count_no_result=0;
        next;
      }
      print FHW "$line_prev\n";
      $count_no_result=0;
      if ( $line_prev =~ "Memory - AMS:" || $line_prev =~ "Memory - AMS - paging:" || $line_prev =~ "Virtual OptiConnec:" || $line_prev =~ "Virtual slots per lpar:" || $line_prev =~ "Physical IO per io pool level sys:" || $line_prev =~ "<B>Ethernet:" || $line_prev =~ "CPU pools:" || $line_prev =~ "CPU pool:" || $line_prev =~ "System overview:" || $line_prev =~ "CPU resources:" || $line_prev =~ "LPAR config:" || $line_prev =~ "LPAR profiles:" || $line_prev =~ "CPU resources:" || $line_prev =~ "CPU totally:" || $line_prev =~ "Memory resources "  || $line_prev =~ "Memory:" ) {
        $count=1;
      }
      if ( $line_prev =~ "CPU pools:") {
        $count=2;
        $count_prof=1;
      }
    }

    if ( $line =~ "Memory - AMS:" || $line =~ "Memory - AMS - paging:" || $line =~ "Virtual OptiConnec:" || $line =~ "Virtual slots per lpar:" || $line =~ "Physical IO per io pool level sys:" || $line =~ "<B>Ethernet:" || $line =~ "CPU pools:" || $line =~ "CPU pool:" || $line =~ "System overview:" || $line =~ "CPU resources:" || $line =~ "LPAR config:" || $line =~ "LPAR profiles:" || $line =~ "CPU resources:" || $line =~ "CPU totally:" || $line =~ "Memory resources "  || $line =~ "Memory:" ) {
      $count=1;
      print FHW "$line\n";
      next;
    }

    # Formating output 
    if ($count > 0 ) {
      my @list = split(/,/,$line);
      my $comma = 0;
      my $enter = 0;
      my $end_comma = 0;
      foreach my $item (@list) {
        chomp($item);
   	my $last = substr($item,1,1);
	if (($item =~ "=") && ($item !~ m/^"/) && ($item !~ m/"$/)) {
          (my $left_c, my $right_c) = split(/=/,$item);
	  my $result = "";
          if (($count == 2) && ($left_c =~ m/^name=/)) {
	    $result = sprintf ("%-36s = </PRE><B>%s</B><PRE>\n",$left_c,$right_c);
          }
	  else {
	    $result = sprintf ("%-36s = %s\n",$left_c,$right_c);
          }
	  if ($result =~ m/"$/){
	     $comma = 0;
	     $enter = 1;
	  }
          $result =~ s/=/ /g;
	  if ($enter == 1) {
	    $enter = 0;
            print FHW "$result";
	  }
	  else {
            print FHW "$result";
	  }
	  next;
	}
	else {
	  if (($comma == 0) && ($item =~ "^\"")) {
	    if ($item =~ "^\"") {
	      $comma = 1;
	    }
	    if ($item =~ "=") {
              (my $left_c, my $right_c) = split(/=/,$item);
	      my $result = sprintf ("%-36s = %s",$left_c,$right_c);
	      if ($result =~ m/"$/){
	        $comma = 0;
		$enter = 1;
	      }
              $result =~ s/^"//g;
              $result =~ s/"$//g;
              $result =~ s/ =/   /g;
              $result =~ s/=/ /g;
	      if ($enter == 1) {
		$enter = 0;
                print FHW "$result\n";
	      }
	      else {
                print FHW "$result";
	      }
	      next;
  	    }
	    else {
	      my $result = sprintf ("%-36s",$item);
	      if ($result =~ m/"$/){
	          $comma = 0;
		  $enter = 1;
	      }
              $result =~ s/^"//g;
              $result =~ s/"$//g;
              $result =~ s/ =/  =/g;
	      if ($enter == 1) {
		$enter = 0;
                print FHW "$result\n";
	      }
	      else {
                print FHW "$result";
	      }
	      next;
	   }
	  }
	  else {
      	    if (($comma == 1) && (($item !~ m/^"/) || ($item =~ m/""$/) || ($item =~ m/""""$/)) && ($item !~ m/=/)) {
	      if ($item =~ m/"$/){
	          $comma = 0;
	          $enter = 1;
	      }
              while ($item =~ m/^"/) {
                $item =~ s/^"//g;
	      }
              while ($item =~ m/"$/) {
                $item =~ s/"$//g;
	      }
              $item =~ s/ =/  =/g;
	      if ($enter == 1) {
		$enter = 0;
                print FHW "\n                                       $item\n";
	      }
	      else {
                print FHW "\n                                       $item";
	      }
	      next;
	    }
	    else {
      	      if (($comma == 1) && ($item =~ m/^"/)) {
	        if (($item =~ m/^""/) || ($item =~ m/^""""/)) {
	          if ($item =~ m/"$/){
	            $comma = 0;
		    $enter = 1;
		  }
                  $item =~ s/=/ /g;
	          if ($enter == 1) {
		    $enter = 0;
                    print FHW "$item\n";
	          }
	          else {
                    print FHW "$item";
	          }
		  next;
		}
                (my $left_c, my $right_c) = split(/=/,$item);
	        my $result = sprintf ("\n%-36s = %s",$left_c,$right_c);
	        if ($result =~ m/"$/){
	          $comma = 0;
		  $enter = 1;
	        }
                $result =~ s/^"//g;
                $result =~ s/"$//g;
                $result =~ s/ =/   /g;
                $result =~ s/=/ /g;
	        if ($enter == 1) {
		  $enter = 0;
                  print FHW "$result\n";
	        }
	        else {
                  print FHW "$result";
	        }
	        next;
	      }
	      else {
	        if ($item =~ m/"$/){
	          $comma = 0;
		  $enter = 1;
		}
                $item =~ s/^"//g;
                $item =~ s/"$//g;
                $item =~ s/ =/   /g;
                $item =~ s/=/ /g;
	        if ($enter == 1) {
		  $enter = 0;
                  print FHW "$item\n";
	        }
	        else {
                  print FHW "$item";
	        }
	        print FHW "$item\n";
	        next;
	      }
	    }
	  }
	  if ($item =~ m/"$/){
	      $comma = 0;
	      $enter = 1;
              $item =~ s/^"//g;
              $item =~ s/"$//g;
              $item =~ s/ =/   /g;
              $item =~ s/=/ /g;
	      if ($enter == 1) {
		$enter = 0;
                print FHW "$item\n";
	      }
	      else {
                print FHW "$item";
	      }
	      next;
	  }
	}
      }
      if ($comma == 1) {
        print FHW "\n";
      }
      if ( $count != 2 ) {
        print FHW "\n";
      }
      if ($count == 2) {
        next;
      }
      $count=0;
    }
    else {
      $line =~ s/,/ /g;
      print FHW "$line\n";
    }
  }
  print FHW "\n</PRE><A HREF=\"http://www.lpar2rrd.com\">Created by LPAR2RRD $version</A><PRE>\n";
  print FHW "$date\n";
  close (FHW);
  close (FHR);
  unlink("$out_file_tmp") || error(" Cannot rm $out_file_tmp : $!".__FILE__.":".__LINE__) && return 0;

  return 1;
}

sub save_cpu_cfg_global { 
  my $managedname = shift;
  my $time = shift;
  my $upgrade = shift;
  my $cpu_cfg = "$wrkdir/$managedname/$host/cpu.html";
  my $mem_cfg = "$wrkdir/$managedname/$host/mem.html";
  my $lpm_excl= "$wrkdir/$managedname/$host/lpm-exclude.txt";
  my $act_time = time();
  my @results="";

  if ( ! -d "$wrkdir/$managedname/$host" ) {
    error ("$wrkdir/$managedname/$host does not exist");
    return 0;
  }

  # do not update chart every run
  if ( -f "$cpu_cfg" && $upgrade == 0 ) {
    my $png_time = (stat("$cpu_cfg"))[9];
    (my $sec,my $min,my $h,my $aday,my $m,my $y,my $wday,my $yday,my $isdst) = localtime(time());
    ($sec,$min,$h,my $png_day,$m,$y,$wday,$yday,$isdst) = localtime($png_time);
    if ( $aday == $png_day ) {
      return 0;
    }
    else {
      print "cfg cpu refresh: $aday != $png_day\n" if $DEBUG ;
    }
  }

  print "fetching HMC   : $host:$managedname CPU/MEM global config\n" if $DEBUG ;
  if ( -f "$mem_cfg" ) {
    # backup old one, crate a new one a do diff, if there is any then --> install_html.sh
    if ( -f "$mem_cfg-old" ) {
      unlink("$mem_cfg-old");
    }
    rename("$mem_cfg","$mem_cfg-old");
  }

  if ( -f "$cpu_cfg" ) {
    # backup old one, crate a new one a do diff, if there is any then --> install_html.sh
    if ( -f "$cpu_cfg-old" ) {
      unlink("$cpu_cfg-old");
    }
    rename("$cpu_cfg","$cpu_cfg-old");
  }

  if ( $lpm == 1) {
    # creates file with list of VIO servers, they have to be excluded from LPM
    open(FHW, "> $lpm_excl") || error(" Cannot open  $lpm_excl: $!".__FILE__.":".__LINE__) && return 0;
    @results=`$SSH $hmc_user\@$host "export LANG=en_US; lssyscfg -m \\"$managedname\\" -r lpar -F name,lpar_env"`;
    foreach my $lpm_line (@results) {
      (my $lpar_name, my $lpar_env)  = split(/,/,$lpm_line);
      if ( $lpar_env =~ m/vioserver/ ) {
        print FHW "$lpar_name\n";
      } 
    }
    close(FHW);
  }

  if ( $IVM == 1 ) {
    # does not support curr_shared_proc_pool_name
    @results=`$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r proc --level lpar -F lpar_name,curr_proc_mode,curr_min_proc_units,curr_proc_units,curr_max_proc_units,curr_procs,curr_sharing_mode,curr_uncap_weight"`;
  }
  else {
    @results=`$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r proc --level lpar -F lpar_name,curr_proc_mode,curr_min_proc_units,curr_proc_units,curr_max_proc_units,curr_procs,curr_sharing_mode,curr_uncap_weight,curr_shared_proc_pool_name"`;
  }

  if ( $results[0] =~ "HSCL" || $results[0] =~ "VIOSE0" ) {
    # something wrong with the input data
    error ("$host:$managedname : $results[0] : lshwres -m $managedname -r proc --level lpar -F lpar_name,cur ...");
    return 1;
  }

  my $cpu_pool_err = 0; 
  if ( $results[0] =~ /An invalid attribute was entered/ ) { 
    # POWER5 do not support curr_shared_proc_pool_name
    #print "ERROR with $managedname : curr_shared_proc_pool_name is not valid for this frame.\n"; 
    $cpu_pool_err = 1; 
    @results=`$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r proc --level lpar -F lpar_name,curr_proc_mode,curr_min_proc_units,curr_proc_units,curr_max_proc_units,curr_procs,curr_sharing_mode,curr_uncap_weight"`;
  }
  my $report=FormatResults (@results);
  open(FHW, "> $cpu_cfg") || error(" Cannot open  $cpu_cfg: $!".__FILE__.":".__LINE__) && return 0;
  #print FHW "<BR><CENTER><TABLE border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n";
  print FHW "<BR><CENTER><TABLE border=\"1\" cellpadding=\"1\" cellspacing=\"1\">\n";
  if ( $cpu_pool_err == 1 ) {
     print FHW "<TR> <TH>LPAR name&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Mode&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Min&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Assigned&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Max&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Virtual&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Sharing&nbsp;&nbsp;&nbsp;<br>mode</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Uncap&nbsp;&nbsp;&nbsp;<br>weight</TH><TH align=\"center\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;OS&nbsp;&nbsp;&nbsp;</b></TH></TR>\n"; 
   } 
   else { 
    print FHW "<TR> <TH>LPAR name&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Mode&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Min&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Assigned&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Max&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Virtual&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Sharing&nbsp;&nbsp;&nbsp;<br>mode</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Uncap&nbsp;&nbsp;&nbsp;<br>weight</TH><TH align=\"center\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</TH><TH align=\"center\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;OS&nbsp;&nbsp;&nbsp;</b></TH></TR>\n";
  }

  # Check whether AIX or another OS due to "grep -p" which supports only AIX
  my $uname = `uname -a`;
  chomp($uname);
  my $aix = 0;
  my @u_a = split(/ /,$uname);
  foreach my $u (@u_a) {
    if ($u =~ "AIX") {
      $aix = 1;
    }
  }

  # Loop for adding OS 
  foreach my $line (split (/\n/,$report)) {
    my $os = "";
    my $lpar = "";
    chomp ($line);
    $line =~ s/0.0.0.0.0.0//g;
    if ( -f "$basedir/data/$managedname/$host/config.cfg" ) {
     (my $trash, $lpar) = split(/<TD/,$line);
      $lpar =~ s/<B>//;
      $lpar =~ s/<\/B><\/TD> //;
      $lpar =~ s/^>//;
      if ( $aix == 1) {
        # AIX with grep -p support
        $os = `egrep -p \"^name\.\*    $lpar\" $basedir/data/"$managedname"/"$host"/config.cfg|egrep \"^os_vers\"|head -1`;
      }
      else {
        $os = `egrep \"^name\.\*    $lpar\" $basedir/data/"$managedname"/"$host"/config.cfg|egrep \"^os_vers\"|head -1`;
      }
      chomp ($os);
      $os =~ s/^os_version//g;
      $os =~ s/  //g;
      $os =~ s/ /&nbsp;/g;
    }
    else {
      $os = "";
    }
    $os =~ s/Unknown//;
    #print "$basedir/data/$managedname/$host/config.cfg -- $line ++++  $lpar $os $aix -- \n";
    $line =~ s/<\/TR>//;
    print FHW "$line <TD align=\"center\">&nbsp;&nbsp;&nbsp;$os&nbsp;&nbsp;&nbsp;</TD></TR>\n";
  }
  print FHW "</TABLE></CENTER><BR><BR>\n";
  close (FHW);


  # Memory
  @results=`$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r mem --level lpar -F lpar_name,curr_min_mem,curr_mem,curr_max_mem,run_mem"`;
  $report=FormatResults (@results);
  open(FHW, "> $mem_cfg") || error(" Cannot open  $mem_cfg: $!".__FILE__.":".__LINE__) && return 0;
  print FHW "<BR><TABLE border=\"1\" cellpadding=\"1\" cellspacing=\"1\">\n";
  print FHW "<TR> <TH>LPAR name&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Min&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Assigned&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Max&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\">&nbsp;&nbsp;&nbsp;Running&nbsp;&nbsp;&nbsp;</TH></TR>\n";
  print FHW "$report\n";
  print FHW "</TABLE><br>(all in MB)<BR>\n";
  close (FHW);

  # if any change then  run install_html.sh
  if ( -f "$mem_cfg-old" ) {
    if (compare("$mem_cfg","$mem_cfg-old") != 0) {
     LoadDataModule::touch ("$mem_cfg-old");
    }
  }
  else {
   LoadDataModule::touch ("$mem_cfg-old 1");
  }

  if ( -f "$cpu_cfg-old" ) {
    if (compare("$cpu_cfg","$cpu_cfg-old") != 0) {
     LoadDataModule::touch ("$cpu_cfg-old");
    }
  }
  else {
   LoadDataModule::touch ("$cpu_cfg-old 1");
  }

}

sub FormatResults {
  my @results_unsort=@_;
  my $line = "";
  my $formated="";
  my @items1="";
  my $item="";

  my @results = sort { lc $a cmp lc $b } @results_unsort;
  foreach $line (@results) {
    chomp $line;
    @items1=split /,/,$line;
    $formated=$formated . "<TR>";
    my $col = 0;
    foreach $item (@items1) {
        if ($col == 0) {
          $formated=sprintf ("%s <TD><B>%s</B></TD>",$formated,$item);
        }
        else {
          $formated=sprintf ("%s <TD align=\"center\">%s</TD>",$formated,$item);
        }
        $col++;
    };
    $formated=$formated . "</TR>\n";
  };
  return $formated;
}

# function saves CPU config of each lpar, this is then shown in the main panel of each lpar
# it save also lpar cfg for all lpars into lpar.cfg
sub save_cpu_cfg { 
  my $managedname = shift;
  my $time = shift;
  my $upgrade = shift;
  my $cfg = "$wrkdir/$managedname/$host/cpu.cfg";
  my $cfg_lpar = "$wrkdir/$managedname/$host/lpar.cfg";
  my $cfg_shell = $cfg;
  my $cfg_lpar_shell = $cfg_lpar;
  $cfg_shell =~ s/ /\\ /g;
  $cfg_lpar_shell =~ s/ /\\ /g;
  my $act_time = time();
  my $serial = "";

  if ( ! -d "$wrkdir/$managedname/$host" ) {
    error ("$wrkdir/$managedname/$host does not exist");
    return 0;
  }

  # --> IT must not be here!!! lpar2rrd needs to have actuall data every run!!!
  # do not update chart every run
  #if ( -f "$cfg" ) {
  #  my $png_time = (stat("$cfg"))[9];
  #  if ( ($act_time - $png_time) < $CFG_REFRESH ) {
  #    #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
  #    return 0;
  #  }
  #}

  my @lparlist=`$SSH $hmc_user\@$host "lssyscfg -m \\"$managedname\\" -r lpar -F name,state,logical_serial_num" 2>&1`;
  if ($lparlist[0] =~ /Standby or Operating state/ ) {
    error ("$host:$managedname : $lparlist[0] : server looks like switched off");
    return 2;
  }

  if ($lparlist[0] =~ /HSCL/ || $lparlist[0] =~ /VIOSE0/) {
    error ("$host:$managedname : $lparlist[0] : lssyscfg -m $managedname  -r lpar -F name,state");
    return 1;
  }

  print "fetching HMC   : $host:$managedname CPU lpar config\n" if $DEBUG ;
  `$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r proc --level lpar" > $cfg_shell`;
  `$SSH $hmc_user\@$host "export LANG=en_US; lssyscfg -m \\"$managedname\\" -r lpar " > $cfg_lpar_shell`;
  
  open(FHR, "< $cfg") || error(" Cannot open  $cfg: $!".__FILE__.":".__LINE__) && return 0;
  while (my $line = <FHR>) {
    chomp($line);
    if ($line =~ /Standby or Operating state/ ) {
      error ("$host:$managedname : $line : server looks like switched off");
      return 2;
    }
    if ($line =~ /HSCL/ || $line =~ /VIOSE0/) {
      error ("$host:$managedname : $line : lshwres -m $managedname -r proc --level lpar");
      return 1;
    }

    if ($line =~ "lpar_name=" ) {
      my @list = split(/,/,$line);
      my $first = 0;
      my $l     = "";
      my $state = "";
      foreach my $item (@list) {
        chomp($item);
        $first++;
	if ($first == 1) {
	  (my $name, my $lpar) = split(/=/,$item);
          foreach $l (@lparlist) {
	    (my $name,  my $state_a, my $serial_new) = split(/,/,$l);
	      if ( $name =~ m/^$lpar$/ ) {
                $state = $state_a;
                $serial = $serial_new;
                last;
	      }
          }
          #print "$lpar $state\n";
	  $lpar =~ s/\//\&\&1/g;
          open(FHW, "> $cfg-$lpar")     || error(" Cannot open $cfg-$lpar: $!".__FILE__.":".__LINE__) && return 0; 
	  print FHW "<B>Current CPU configuration:</B><TABLE border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n";
	  my $result = sprintf ("<TR><TD><font size=\"-1\">state</font></TD><TD><font size=\"-1\">&nbsp;&nbsp;%s</font></TD></TR>\n",$state);
          print FHW "$result";
          chomp ($serial);
	  $result = sprintf ("<TR><TD><font size=\"-1\">logical_serial_num</font></TD><TD><font size=\"-1\">%s</font></TD></TR>\n",$serial);
          print FHW "$result";
	  next;
	}
	if ($item =~ m/^curr_/ && ($item !~ "_max") && ($item !~ "_min") ){
          (my $left_c, my $right_c) = split(/=/,$item);
	  #my $result = sprintf ("%-36s = %s",$left_c,$right_cp;
	  my $result = sprintf ("<TR><TD><font size=\"-1\">%s</font></TD><TD><font size=\"-1\">&nbsp;&nbsp;%s</font></TD></TR>\n",$left_c,$right_c);
          print FHW "$result";
        }
      }
      #print FHW "<TR><TD><font size=\"-1\">$time</font></TD></TR></TABLE>\n<BR>\n";
      print FHW "</TABLE>\n<font size=\"-1\">HMC time : $time</font><BR><BR>\n";
      close (FHW);
    }
  }
  close (FHR);
  return 0;
}

sub sys_change {
  my $managedname = shift;
  my $cmd = "export LANG=en_US; lslparutil -r all -s h  -m \"$managedname\" --filter \"event_types=config_change,state_change,utility_cod_proc_usage\" -n $SYS_CHANGE";

  if ( ! -d "$wrkdir/$managedname/$host" ) {
    error ("$wrkdir/$managedname/$host does not exist");
    return 0;
  }
  my $out="$wrkdir/$managedname/$host/change-state.txt";
  my $out1="$wrkdir/$managedname/$host/change-config.txt";

  `echo "<HTML> <HEAD> <TITLE>LPAR2RRD</TITLE> </HEAD> \
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > \
	<CENTER><B>Last $SYS_CHANGE changes in state</B></CENTER><PRE>" > "$out"`;
  `echo "<HTML> <HEAD> <TITLE>LPAR2RRD</TITLE> </HEAD> \
	<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > \
	<CENTER><B>Last $SYS_CHANGE changes in configuration</B></CENTER><PRE>" > "$out1"`;

  print "sys change     : $host:$managedname:$SYS_CHANGE\n" if $DEBUG ;
  `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r all -s h  -m \\"$managedname\\" --filter \"event_types=state_change\" -n $SYS_CHANGE" |sed -e 's/time=//g' -e 's/event_type=state_change//g'  -e 's/,/ /g'   >> "$out"`;
  `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r all -s h  -m \\"$managedname\\" --filter \"event_types=config_change\" -n $SYS_CHANGE"|sed -e 's/time=//g' -e 's/event_type=config_change//g' -e 's/,/ /g'   >> "$out1"`;
  `echo "</PRE></BODY></HTML>" >> "$out"`;
  `echo "</PRE></BODY></HTML>" >> "$out1"`;
}

# CPU pool total per HMC
sub frame_multi {
  my ($text,$type_sam,$type,$xgrid) = @_;

  my $file = "";
  my $i = 0;
  my $cmd = "";
  my $cmdq = "";
  my $j = 0;
  my $managed_ok; 
  my $managedname_exl = "";
  my $managedname = "";
  my @m_excl = "";
  my $req_time = 0;
  my $act_time = time();
  my $name = "$webdir/$host/pool-multi-$type";

  $step = $STEP;
  #if ( $type =~ "y" ) {
  #  $step=86400;
  #}

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "y" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $YEAR_REFRESH && $upgrade == 0 ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "m" && -f "$name.png" && $upgrade == 0 ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "w" && -f "$name.png" && $upgrade == 0 ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  print "creating m_hmc : $host:$type\n" if $DEBUG ;
  my $header = "$host aggregated : last $text";

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"CPU cores\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " COMMENT:\\\"Utilization in CPU cores\\:               average   maximum\\l\\\"";


  my $gtype="AREA";
  foreach my $line (@managednamelist) {
    chomp($line);
    if ( $line !~ ".*,.*,.*" ){
      # it must look like : PWR6A-9117-MMA-SN103A4B0,9117-MMA,103A4B0, --> exclude banners
      next;
    }
    if ($line =~ /No results were found/) {
      print "HMC : $host does not contain any managed system\n" if $DEBUG ;
      return 0;
    }
    ($managedname, my $model, my $serial) = split(/,/,$line);

    if ( is_IP($managedname) == 1 ) {
      next; # wrong entry from the HMC, a problem of the HMC or unconfigured server yet
    }

    $managed_ok=1;
    if ( $managed_systems_exclude ne '' ) {
      @m_excl = split(/:/,$managed_systems_exclude);
      foreach $managedname_exl (@m_excl) {
        chomp($managedname_exl);
        if ( $managedname =~ m/^$managedname_exl$/ ) {
          $managed_ok=0;
        }
      }
    }
    if ( $managed_ok == 0 ) {
      next;
    }

    if ( $type_sam =~ "d" ) {
      $req_time = $act_time - 31536000;
    }
    $file = "pool.rrm";
    if ( "$type" =~ "d" ) {
      if ( -e "$wrkdir/$managedname/$host/pool.rrh") { 
        $file = "pool.rrh";
      }
      #if ( -e "$wrkdir/$managedname/$host/pool.rrm") { 
      #}
      $req_time = $act_time - 86400;
    }
    if ( "$type" =~ "w" ) {
      if ( -e "$wrkdir/$managedname/$host/pool.rrh") { 
        $file = "pool.rrh";
      }
      if ( -e "$wrkdir/$managedname/$host/pool.rrm") { 
        $file = "pool.rrm";
      }
      $req_time = $act_time - 604800;
    }
    if ( "$type" =~ "m" ) {
      if ( -e "$wrkdir/$managedname/$host/pool.rrh") { 
        $file = "pool.rrh";
      }
      if ( -e "$wrkdir/$managedname/$host/pool.rrm") { 
        $file = "pool.rrm";
      }
      $req_time = $act_time - 2764800;
    }
    if ( "$type" =~ "y" ) {
      if ( -e "$wrkdir/$managedname/$host/pool.rrd") { 
        $file = "pool.rrd";
      }
      else {
        if ( -e "$wrkdir/$managedname/$host/pool.rrm") {
          $file = "pool.rrm";
        }
        else {
         if ( -e "$wrkdir/$managedname/$host/pool.rrh") {
           $file = "pool.rrh";
         }
         else {
           next;
         }
       }
     }
    }

    if ( -e  "$wrkdir/$managedname/$host/$file" ) {
      # avoid old servers which do not exist in the period
      my $rrd_upd_time = (stat("$wrkdir/$managedname/$host/$file"))[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }
    }
    else {
      # avoid non-existing managed systems (with no utillization data on)
      next;
    }

    my $managedname_space = $managedname;
    for(my $k = length($managedname); $k < 35; $k++) {
      $managedname_space .= " ";
    }

    # bulid RRDTool cmd
    $cmd .= " DEF:totcyc${i}=\\\"$wrkdir/$managedname/$host/$file\\\":total_pool_cycles:AVERAGE";
    $cmd .= " DEF:uticyc${i}=\\\"$wrkdir/$managedname/$host/$file\\\":utilized_pool_cyc:AVERAGE";
    $cmd .= " DEF:ncpu${i}=\\\"$wrkdir/$managedname/$host/$file\\\":conf_proc_units:AVERAGE";
    $cmd .= " DEF:ncpubor${i}=\\\"$wrkdir/$managedname/$host/$file\\\":bor_proc_units:AVERAGE";
    # if it does not exist for some time period then put 0 there
    $cmd .= " CDEF:cpu${i}=ncpu${i},UN,0,ncpu${i},IF";
    $cmd .= " CDEF:cpubor${i}=ncpubor${i},UN,0,ncpubor${i},IF";
    $cmd .= " CDEF:totcpu${i}=cpu${i},cpubor${i},+";
    $cmd .= " CDEF:cpuutil${i}=uticyc${i},totcyc${i},/";
    $cmd .= " CDEF:cpuutiltot${i}=cpuutil${i},totcpu${i},*";
    $cmd .= " CDEF:utilisa${i}=cpuutil${i},100,*";
    $cmdq .= " $gtype:cpuutiltot${i}$color_hmc[$i]:\\\"$managedname_space\\\"";
    $cmdq .= " GPRINT:cpuutiltot${i}:AVERAGE:\\\"%6.2lf \\\"";
    $cmdq .= " GPRINT:cpuutiltot${i}:MAX:\\\"%6.2lf \\l\\\"";
    $gtype="STACK";
    $i++;
  }
  
  if ($i == 0) {
   # no available managed system
   return 1;
  }
 
  # add count of all CPU in pools
  for ($j=0; $j < $i; $j++) {
    if ( $j == 0 ) {
      $cmd .= " CDEF:tcpu${j}=cpu${j},cpubor${j},+";
    }
    else {
      my $k = $j - 1;
      $cmd .= " CDEF:tcpu_tmp${j}=cpu${j},cpubor${j},+";
      $cmd .= " CDEF:tcpu${j}=tcpu_tmp${j},tcpu${k},+";
    }
  }
  if ( $j > 0 ) { 
    $j--;
  }
  my $cpu_pool_total = "CPUs total available in pools";
  for(my $k = length($cpu_pool_total); $k < 35; $k++) {
    $cpu_pool_total .= " ";
  }
  $cmd .= " LINE2:tcpu${j}#888888:\\\"$cpu_pool_total\\\"";
  #$cmd .= " GPRINT:tcpu${j}:AVERAGE:\\\"%6.2lf \\\"";
  # excluded as it is a bit misleading there there is any even small data gap 
  $cmd .= " GPRINT:tcpu${j}:MAX:\\\"         %6.2lf \\l\\\"";
  $cmd .= $cmdq;
  $cmd .= " HRULE:0#000000";
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  my $FH;
  my $tmp_file="$tmpdir/multi-hmc-$host-$type.cmd";
  open(FH, "> $tmp_file") || error(" Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
  
  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("$host: Multi graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err,  my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("$host: $cmd: Pool multi graph rrdtool error : $ret");
    }
  }
  #unlink ("$tmp_file"); --> do not remove, it is used for details

  return 0;
}

sub hea_run  {
  my $managedname = shift;

  # check whether is configured
  if ($hea == 0) {
    return 0;
  }


    opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0;
    my @files = grep(/.*-.*\.db$/,readdir(DIR));
    closedir(DIR);
    my $file = "";
    my $is_hea = 0;
    foreach $file (@files) {
      if ( $file =~ "HSCL" || $file =~ "VIOSE0") {
       unlink ($file);
       next; # some error
      }
      if ( $is_hea == 0 ) {
        hea_dirs($managedname);
      }
      $is_hea = 1;
      chomp($file);
      (my $trash, my $adapter_id, my $port) = split(/-/,$file);
      $port =~ s/\.db//;
      #print "-- $file $port\n";
      my $rrd = "$wrkdir/$managedname/$host/$file";
      draw_all_hea($managedname,$rrd,$adapter_id,$port);
    }

    # HEA errors only once (all ports in one graph)
    if ( $is_hea == 1 ) {
      hea_dirs($managedname);
      draw_all_hea_error($managedname,$host,@files);
    }

  return 1;
}

sub draw_all_hea_error
{
  my $managedname = shift;
  my $host = shift;
    draw_graph_hea_err ("day","d","MINUTE:60:HOUR:2:HOUR:4:0:%H",$managedname);
    draw_graph_hea_err ("week","w","HOUR:8:DAY:1:DAY:1:86400:%a",$managedname);
    draw_graph_hea_err ("4 weeks","m","DAY:1:DAY:2:DAY:7:0:%U",$managedname);
    draw_graph_hea_err ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$managedname);
  return 0;
}

sub draw_graph_hea_err 
{
  my $text     = shift;
  my $type     = shift;
  my $xgrid    = shift;
  my $managedname = shift;
  my $adapter_id = "";
  my $port = "";
  my $t="COMMENT: ";
  my $t2="COMMENT:\\n";
  my $header = "HEA errors: last $text";
  my $act_time = time();
  my $rrd = "";
  my $i = 0;
  my $j = 0;
  # 48 colors for HEA inp/out errors, max is 16 ports X 2 = 32 colors, 48 is a reserve for the future
  my @color_hea=("#FF8080", "#FFFF80", "#80FF80", "#00FF80", "#80FFFF", "#0080FF", "#FF80C0", "#FF80FF", "#FF0000", "#FFFF00", "#80FF00",
"#00FF40", "#00FFFF", "#0080C0", "#8080C0", "#FF00FF", "#804040", "#FF8040", "#00FF00", "#008080", "#004080", "#8080FF",
"#800040", "#FF0080", "#800000", "#FF8000", "#008000", "#008040", "#0000FF", "#0000A0", "#800080", "#8000FF", "#400000",
"#804000", "#004000", "#004040", "#000080", "#000040", "#400040", "#400080", "#000000", "#808000", "#808040", "#808080",
"#408080", "#C0C0C0", "#400040", "#AAAAAA");

  my $name = "$webdir/$host/$managedname/hea/hea-error-$type";
  #$name = "hea-error";

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "y"  && -f "$name.png" && $upgrade == 0 ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "m" && -f "$name.png" && $upgrade == 0 ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "w" && -f "$name.png" && $upgrade == 0 ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  print "creating graph : $host:$managedname:$type:hea-error\n" if $DEBUG ;

  my $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$STEP_HEA";
  $cmd .= " --lower-limit=0";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --vertical-label=\\\"Error packets per second\\\"";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " --alt-y-grid";
  $cmd .= " --force-rules-legend";
  $cmd .= " COMMENT:\\\"Average in packets\\l\\\"";
  opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0;
  my @files = grep(/.*-.*\.db$/,readdir(DIR));
  closedir(DIR);
  my $file = "";
  foreach $file (@files) {
    if ( $file =~ "HSCL" || $file =~ "VIOSE0" ) {
      unlink ($file);
      next;
    } 
    chomp($file);
    #print "-- $file\n";
    (my $trash, $adapter_id, $port) = split(/-/,$file);
    $port =~ s/\.db//;
    $cmd .= " DEF:recv${i}=\\\"$wrkdir/$managedname/$host/$file\\\":recv_octets_err:AVERAGE";
    $cmd .= " DEF:trans${i}=\\\"$wrkdir/$managedname/$host/$file\\\":trans_octets_err:AVERAGE";
    $cmd .= " LINE1:recv${i}$color_hea[$j]:\\\" recv-$adapter_id-$port-$i  \\\"";
    $cmd .= " GPRINT:recv${i}:AVERAGE:\\\"%5.2lf \\l\\\"";
    $j++;
    $cmd .= " LINE1:trans${i}$color_hea[$j]:\\\"trans-$adapter_id-$port-$i  \\\"";
    $cmd .= " GPRINT:trans${i}:AVERAGE:\\\"%5.2lf \\l\\\"";
    $i++;
    $j++;
  }
  $cmd .= " HRULE:0#000000";
  $cmd =~ s/\\"/"/g;

  my $FH;
  my $tmp_file="/var/tmp/lpar2rrd-hea-$type.tmp-$$";
  open(FH, "> $tmp_file") || error(" Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
 
  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues the refore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("$host: HEA err graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err,  my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("$host: $cmd:Pool multi graph rrdtool error : $ret");
    }
  }
  unlink ("$tmp_file");

  return 0;
}

sub draw_all_hea {
  my $managedname = shift;
  my $rrd = shift;
  my $adapter_id = shift;
  my $port = shift;
  my $cfg = "$CONFIG_HISTORY/$managedname/$host/config.cfg";
  my $hea_cfg = "$wrkdir/$managedname/$host/hea-$adapter_id-$port.html";
  my $first = 0;
  my $act_time = time();
  my $port_group = 0;
  my $phys_port_id = 0;

  if ( $rrd !~ "HSCL" && $rrd !~ "VIOSE0") {
    # It must be here
    draw_graph_hea ("day","d","MINUTE:60:HOUR:2:HOUR:4:0:%H",$managedname,$rrd,$adapter_id,$port);
    draw_graph_hea ("week","w","HOUR:8:DAY:1:DAY:1:86400:%a",$managedname,$rrd,$adapter_id,$port);
    draw_graph_hea ("4 weeks","m","DAY:1:DAY:2:DAY:7:0:%U",$managedname,$rrd,$adapter_id,$port);
    draw_graph_hea ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$managedname,$rrd,$adapter_id,$port);
    draw_graph_hea_summ ("4 weeks","m","DAY:1:DAY:2:DAY:7:0:%U",$managedname,$rrd,$adapter_id,$port);
    draw_graph_hea_summ ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$managedname,$rrd,$adapter_id,$port);
  }

  if ( -f "$cfg" ) {
    my $port_id = $port;
    my $line = "";
    $port_id =~ s/port//g;
    my $ret = substr($port_id,0,1);
    if ($ret =~ /\D/) {
      error ("$host: Wrong input data, HEA port : $port");
      return 1;
     }

    if ($port_id == 1) { $phys_port_id = 0; $port_group = 1; }
    if ($port_id == 2) { $phys_port_id = 1; $port_group = 1; }
    if ($port_id == 3) { $phys_port_id = 0; $port_group = 2; }
    if ($port_id == 4) { $phys_port_id = 1; $port_group = 2; }

    # do not update chart every run
    if ( -f "$hea_cfg" ) {
      my $png_time = (stat("$hea_cfg"))[9];
      if ( ($act_time - $png_time) < $CFG_REFRESH && $upgrade == 0 ) {
        #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
        return 0;
      }
    }


    my @hea_ports=`grep $adapter_id "$cfg" |grep phys_port_id=$phys_port_id|grep " port_group=$port_group"|grep " lpar_name="|sort|uniq|awk -F= '{print \$4,"%",\$8}'|sed -e 's/port_group//g' -e 's/drc_index//g'`;

    foreach $line (@hea_ports) {
      chomp ($line);
      if ( $first == 0 ) {
	if ( -f "$hea_cfg" ) {
	  # backup old one, crate a new one a do diff, if there is any then --> install_html.sh
	  if ( -f "$hea_cfg-old" ) {
	    unlink("$hea_cfg-old");
	  }
	  rename("$hea_cfg","$hea_cfg-old");
	}
        open(FHW, "> $hea_cfg") || error(" Cannot open  $hea_cfg: $!".__FILE__.":".__LINE__) && return 0;
        print FHW "<BR><TABLE border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n";
        print FHW "<TR> <TD><font size=-1><B>LPAR name&nbsp;&nbsp;&nbsp;</B></font></TD> <TD align=\"center\"><font size=-1><B>Logical port</font></B></TD></TR>\n";
        $first = 1;
      }
      (my $lpar, my $lport) = split(/%/,$line);
      $lpar =~ s/ state //;
      print FHW "<TR> <TD><font size=-1>$lpar</font></TD> <TD align=\"center\"><font size=-1>$lport</font></TD></TR>\n";
    }
    if ( $first == 1 ) {
      print FHW "</table><BR>\n";
      close (FHW);
      if ( -f "$hea_cfg-old" ) {
        if (compare("$hea_cfg","$hea_cfg-old") != 0) {
         LoadDataModule::touch ("$hea_cfg-old");
	}
      }
      else {
        LoadDataModule::touch ("$hea_cfg-old 1");
      }
    }

    #my $line = "";
    #open(FHLT, "< $CONFIG_HISTORY/$managedname/$host/config.cfg") || error(" Can't open $CONFIG_HISTORY/$managedname/$host/config.cfg : $!".__FILE__.":".__LINE__) && return 0;
    #foreach $line (<FHLT>) {
      #$line ;
     #}
  }

  return 0;
 
}

# it creates HEA subdirs only when there is any HEA DB
sub hea_dirs {
  my $managedname = shift;

  if (! -d "$webdir/$host/$managedname/hea" ) {
    if (! -d "$webdir/$host" ) {
      print "mkdir          : $host:$managedname $webdir/$host\n" if $DEBUG ;
      mkdir("$webdir/$host", 0755) || error(" Cannot mkdir $webdir/$host: $!".__FILE__.":".__LINE__) && return 0;
    }
    if (! -d "$webdir/$host/$managedname" ) {
      print "mkdir          : $host:$managedname $webdir/$host/$managedname\n" if $DEBUG ;
      mkdir("$webdir/$host/$managedname", 0755) || error(" Cannot mkdir $webdir/$host/$managedname: $!".__FILE__.":".__LINE__) && return 0;
    }
    print "mkdir          : $host:$managedname $webdir/$host/$managedname/hea\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host/$managedname/hea");
    mkdir("$webdir/$host/$managedname/hea" , 0755) || error(" Cannot mkdir $webdir/$host/$managedname/hea: $!".__FILE__.":".__LINE__) && return 0;
  }

}

sub draw_graph_hea {
  my $text     = shift;
  my $type     = shift;
  my $xgrid    = shift;
  my $managedname = shift;
  my $rrd = shift;
  my $adapter_id = shift;
  my $port = shift;
  my $t="COMMENT: ";
  my $t2="COMMENT:\\n";
  my $header = "HEA $port : last $text";
  my $act_time = time();

  my $name = "$webdir/$host/$managedname/hea/$adapter_id.$type.$port";

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "y"  && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "m" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "w" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  if ($type =~ "d") {
      RRDp::cmd qq(last "$rrd");
      my $last_tt = RRDp::read;
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $t ="COMMENT:Updated\\: $l ";
  };

  print "creating graph : $host:$managedname:$type:$adapter_id:$port\n" if $DEBUG ;

  RRDp::cmd qq(graph "$name.png"
      "--title" "$header"
      "--start" "now-1$type"
      "--end" "now-1$type+1$type"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=400"
      "--height=150"
      "--step=$STEP_HEA"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--vertical-label=Bytes/sec"
      "--alt-autoscale-max"
      "--x-grid=$xgrid"
      "--base=1024"
      "--alt-y-grid"
      "DEF:recv=$rrd:recv_octets:AVERAGE"
      "DEF:trans=$rrd:trans_octets:AVERAGE"
      "CDEF:recvkb=recv,1024,/"
      "CDEF:transkb=trans,1024,/"
      "CDEF:trans-neg=0,trans,-"
      "COMMENT:   Average  in kB\\n"
      "AREA:recv#00FF00: Received   "
      "GPRINT:recvkb:AVERAGE: %8.3lf"
      "$t2"
      "AREA:trans-neg#FFFF00: Transmited "
      "GPRINT:transkb:AVERAGE: %8.3lf"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
    my $answer = RRDp::read;
    if ( $$answer =~ "ERROR" ) {
      error ("$host:$managedname Graph rrdtool error : $$answer");
      if ( $$answer =~ "is not an RRD file" ) {
        (my $err,  my $file, my $txt) = split(/'/,$$answer);
        error ("Removing as it seems to be corrupted: $file");
        unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
      }
      else {
        print "Graph rrdtool error : $$answer";;
      }
    }
}

sub draw_graph_hea_summ {
  my $text     = shift;
  my $type     = shift;
  my $xgrid    = shift;
  my $managedname = shift;
  my $rrd = shift;
  my $adapter_id = shift;
  my $port = shift;
  my $act_time = time();


  my $t="COMMENT: ";
  my $t2="COMMENT:\\n";
  my $header = "HEA $port : last $text";
  my $name = "$webdir/$host/$managedname/hea/$adapter_id.$type.summ-$port";

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "y" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:summ-$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "m" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:summ-$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "w" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }
  print "creating graph : $host:$managedname:$type.summ.$adapter_id:$port\n" if $DEBUG ;

  RRDp::cmd qq(graph "$name.png"
      "--title" "$header"
      "--start" "now-1$type"
      "--end" "now-1$type+1$type"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=400"
      "--height=150"
      "--step=86400"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--vertical-label=Bytes"
      "--alt-autoscale-max"
      "--alt-y-grid"
      "--x-grid=$xgrid"
      "--base=1024"
      "DEF:recv=$rrd:recv_octets:AVERAGE"
      "DEF:trans=$rrd:trans_octets:AVERAGE"
      "CDEF:recv_s=recv,86400,*"
      "CDEF:recv_smb=recv_s,1048576,/"
      "CDEF:trans_s=trans,86400,*"
      "CDEF:trans_smb=trans_s,1048576,/"
      "CDEF:trans-neg=0,trans_s,-"
      "COMMENT:   Average  in MB\\n"
      "AREA:recv_s#00FF00: Received   "
      "GPRINT:recv_smb:AVERAGE: %10.1lf"
      "$t2"
      "AREA:trans-neg#FFFF00: Transmited "
      "GPRINT:trans_smb:AVERAGE: %10.1lf"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
    my $answer = RRDp::read;
    if ( $$answer =~ "ERROR" ) {
      error ("$host:$managedname Graph rrdtool error : $$answer");
      if ( $$answer =~ "is not an RRD file" ) {
        (my $err,  my $file, my $txt) = split(/'/,$$answer);
        error ("Removing as it seems to be corrupted: $file");
        unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
      }
      else {
        print "Graph rrdtool error : $$answer";;
      }
    }
}

sub fcs_run  {
  my $managedname = shift;

  # check whether is configured
  if ($hea == 0 ) {
    return 0;
  }
  if ($HMC == 1 || $SDMC == 1 ) {
    return 0;
  }

    opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0;
    my @files = grep(/fcs.*\.db$/,readdir(DIR));
    closedir(DIR);
    my $file = "";
    my $is_fcs = 0;
    foreach $file (@files) {
      if ( $file =~ "HSCL" || $file =~ "VIOSE0" ) {
       unlink ($file);
       next; # some error
      }
      if ( $is_fcs == 0 ) {
        fcs_dirs($managedname);
      }
      $is_fcs = 1;
      chomp($file);
      #print "-- $file\n";
      (my $trash, my $adapter_id, my $port) = split(/-/,$file);
      $port=$file;
      $port =~ s/\.db//;
      my $rrd = "$wrkdir/$managedname/$host/$file";
      draw_all_fcs($managedname,$rrd,$port);
    }

  return 1;
}

sub draw_all_fcs {
  my $managedname = shift;
  my $rrd = shift;
  my $port = shift;

    draw_graph_fcs ("day","d","MINUTE:60:HOUR:2:HOUR:4:0:%H",$managedname,$rrd,$port);
    draw_graph_fcs ("week","w","HOUR:8:DAY:1:DAY:1:86400:%a",$managedname,$rrd,$port);
    draw_graph_fcs ("4 weeks","m","DAY:1:DAY:2:DAY:7:0:%U",$managedname,$rrd,$port);
    draw_graph_fcs ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$managedname,$rrd,$port);
    draw_graph_fcs_summ ("4 weeks","m","DAY:1:DAY:2:DAY:7:0:%U",$managedname,$rrd,$port);
    draw_graph_fcs_summ ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$managedname,$rrd,$port);
  return 0;
}

# it creates HEA subdirs only when there is any HEA DB
sub fcs_dirs {
  my $managedname = shift;

  if (! -d "$webdir/$host/$managedname/fcs" ) {
    if (! -d "$webdir/$host" ) {
      print "mkdir          : $host:$managedname $webdir/$host\n" if $DEBUG ;
      mkdir("$webdir/$host", 0755) || error(" Cannot mkdir $webdir/$host: $!".__FILE__.":".__LINE__) && return 0;
    }
    if (! -d "$webdir/$host/$managedname" ) {
      print "mkdir          : $host:$managedname $webdir/$host/$managedname\n" if $DEBUG ;
      mkdir("$webdir/$host/$managedname", 0755) || error(" Cannot mkdir $webdir/$host/$managedname: $!".__FILE__.":".__LINE__) && return 0;
    }
    print "mkdir          : $host:$managedname $webdir/$host/$managedname/fcs\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host/$managedname/fcs");
    mkdir("$webdir/$host/$managedname/fcs" , 0755) || error(" Cannot mkdir $webdir/$host/$managedname/fcs: $!".__FILE__.":".__LINE__) && return 0;
  }

}

sub draw_graph_fcs {
  my $text     = shift;
  my $type     = shift;
  my $xgrid    = shift;
  my $managedname = shift;
  my $rrd = shift;
  my $port = shift;
  my $t="COMMENT: ";
  my $t2="COMMENT:\\n";
  my $fcsder = "FC $port : last $text";
  my $act_time = time();

  my $name = "$webdir/$host/$managedname/fcs/$port.$type";

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "y" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "m" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "w" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }

  if ($type =~ "d") {
      RRDp::cmd qq(last "$rrd");
      my $last_tt = RRDp::read;
      my $l=localtime($$last_tt);
      # following must be for RRD 1.2+
      $l =~ s/:/\\:/g;
      $t ="COMMENT:Updated\\: $l ";
  };

  print "creating graph : $host:$managedname:$type:$port\n" if $DEBUG ;

  RRDp::cmd qq(graph "$name.png"
      "--title" "$fcsder"
      "--start" "now-1$type"
      "--end" "now-1$type+1$type"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=400"
      "--height=150"
      "--step=$STEP_HEA"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--vertical-label=Bytes/sec"
      "--alt-autoscale-max"
      "--x-grid=$xgrid"
      "--base=1024"
      "--alt-y-grid"
      "DEF:recv=$rrd:recv_bytes:AVERAGE"
      "DEF:trans=$rrd:trans_bytes:AVERAGE"
      "CDEF:recvkb=recv,1024,/"
      "CDEF:transkb=trans,1024,/"
      "CDEF:trans-neg=0,trans,-"
      "COMMENT:   Average  in kB\\n"
      "AREA:recv#00FF00: Received   "
      "GPRINT:recvkb:AVERAGE: %8.3lf"
      "$t2"
      "AREA:trans-neg#FFFF00: Transmited "
      "GPRINT:transkb:AVERAGE: %8.3lf"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
    my $answer = RRDp::read;
    if ( $$answer =~ "ERROR" ) {
      error ("$host:$managedname Graph rrdtool error : $$answer");
      if ( $$answer =~ "is not an RRD file" ) {
        (my $err,  my $file, my $txt) = split(/'/,$$answer);
        error ("Removing as it seems to be corrupted: $file");
        unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
      }
      else {
        print "Graph rrdtool error : $$answer";;
      }
    }
}

sub draw_graph_fcs_summ {
  my $text     = shift;
  my $type     = shift;
  my $xgrid    = shift;
  my $managedname = shift;
  my $rrd = shift;
  my $port = shift;
  my $act_time = time();


  my $t="COMMENT: ";
  my $t2="COMMENT:\\n";
  my $fcsder = "FC $port : last $text";
  my $name = "$webdir/$host/$managedname/fcs/$port-sum.$type";

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "y" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $YEAR_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$port-sum no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "m" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$port-sum no upd\n" if $DEBUG ;
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "w" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
      #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
      return 0;
    }
  }
  print "creating graph : $host:$managedname:$type.summ.$port\n" if $DEBUG ;

  RRDp::cmd qq(graph "$name.png"
      "--title" "$fcsder"
      "--start" "now-1$type"
      "--end" "now-1$type+1$type"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=400"
      "--height=150"
      "--step=86400"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--vertical-label=Bytes"
      "--alt-autoscale-max"
      "--alt-y-grid"
      "--x-grid=$xgrid"
      "--base=1024"
      "DEF:recv=$rrd:recv_bytes:AVERAGE"
      "DEF:trans=$rrd:trans_bytes:AVERAGE"
      "CDEF:recv_s=recv,86400,*"
      "CDEF:recv_smb=recv_s,1048576,/"
      "CDEF:trans_s=trans,86400,*"
      "CDEF:trans_smb=trans_s,1048576,/"
      "CDEF:trans-neg=0,trans_s,-"
      "COMMENT:   Average  in MB\\n"
      "AREA:recv_s#00FF00: Received   "
      "GPRINT:recv_smb:AVERAGE: %10.1lf"
      "$t2"
      "AREA:trans-neg#FFFF00: Transmited "
      "GPRINT:trans_smb:AVERAGE: %10.1lf"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
    my $answer = RRDp::read;
    if ( $$answer =~ "ERROR" ) {
      error ("$host:$managedname Graph rrdtool error : $$answer");
      if ( $$answer =~ "is not an RRD file" ) {
        (my $err,  my $file, my $txt) = split(/'/,$$answer);
        error ("Removing as it seems to be corrupted: $file");
        unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
      }
      else {
        print "Graph rrdtool error : $$answer";;
      }
    }
}

sub cfg_config_change 
{
  # If any change in lpar2rrd.cfg  then must be run install-html.sh
  my $cfg_update = "$tmpdir/cfg_update";

  if ( -f "$cfg_update" ) {
    my $last_line = "";
    open(FHLT, "< $cfg_update") || error(" Can't open $cfg_update: $!".__FILE__.":".__LINE__) && return 0;
    foreach my $line (<FHLT>) {
      $last_line = $line;
    }
    close(FHLT);
    my $png_time = (stat("$basedir/etc/lpar2rrd.cfg"))[9];
    if ($last_line < $png_time ) {
      LoadDataModule::touch ("cfg_config_change: $last_line < $png_time");
      open(FHLT, "> $cfg_update") || error(" Can't open $cfg_update: $!".__FILE__.":".__LINE__) && return 0;
      my $act_time = time();
      print FHLT "$act_time";
      close(FHLT);
    }
  }
  else {
    LoadDataModule::touch ("cfg_config_change: $cfg_update");
    my $png_time = (stat("$basedir/etc/lpar2rrd.cfg"))[9];
    open(FHLT, "> $cfg_update") || error(" Can't open $cfg_update: $!".__FILE__.":".__LINE__) && return 0;
    my $act_time = time();
    print FHLT "$act_time";
    close(FHLT);
    #print "$cfg_update $basedir/etc/lpar2rrd.cfg $png_time\n";
  }

  return 1;

}

sub create_graph_pool_agg
{
  my $lpar_pool_alias = shift;
  my $pool = shift;
  my $name = shift;
  my $managedname = shift;
  my $host = shift;
  my $type_sam = shift;
  my $act_time = shift;
  my $type = shift;
  my $xgrid = shift;
  my $step_new = shift;
  my $text = shift;
  my $req_time = "";
  my $header = "LPARs in CPU pool: $lpar_pool_alias : last $text";
  my $file = "";
  my $i = 0;
  my $lpar = "";
  my $cmd = "";
  my $j = 0;

  $color_indx = 0; # reset colour index

  if ( "$type" =~ "d" ) {
    $req_time = $act_time - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time - 2764800;
  }
 
  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d"  || "$type" =~ "y" ) {
    $req_time = $act_time - 31536000;
  }
  #if ( ! -f "$name-agg.png" ) {
  #  LoadDataModule::touch("create_graph_pool_agg $name-agg.png");
  #} 

  $cmd .= "graph \\\"$name-agg.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"CPU cores\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " COMMENT:\\\"Average utilization in CPU cores\\l\\\"";


  my $gtype="AREA";

  # select pool and its lpars
  my $pool_id_act = $pool;
  $pool_id_act =~ s/SharedPool//g;

  #print "---- $name $pool_id_act : $pool  \n";
  #lpar2rrd@hmc06:~> lshwres -m b750_02-PAN -r procpool -F shared_proc_pool_id,lpar_names
  #0,"upgAIX,mq11,batch13,mq15,hatst02,db31,hi02,db18,db14,db32,scube31,scube14,db48,db46,b75002v2,b75002v1"
  #1,"mqe,batch11,batch15,batch12"

  my $lpar_list = "";
  my $pool_found = 0;
  my $pool_id_tmp = "";
  my $lpar_list_tmp = "";

  foreach my $line (@poollistagg) {
    chomp($line);
    if ( $line eq '' ) {
      # no pool??, something wrong anyway
      if ( -f "$name-agg.png" ) {
        unlink ("$name-agg.png"); # delete old graph to do not stay there unchanged for ever
        LoadDataModule::touch ("$name-agg.png delete");
      }
      return 0;
    }
    if ( $line =~ m/,"/ ) { # more lpars, separator is "
      ($pool_id_tmp, $lpar_list_tmp) = split(/"/,$line);
      $pool_id_tmp =~ s/,//g;
    }
    else { # if there is 1 lpar then separato is no " but ,
      ($pool_id_tmp, $lpar_list_tmp) = split(/,/,$line);
    }

    my $ret = substr($pool_id_tmp,0,1);
    if ($ret =~ /\D/) {  
      error ("$host: 01:Pool aggregated graph error : $pool_id_tmp - $lpar_list_tmp - $ret");
    }
    #if ( $pool_id_tmp == $pool_id_act ) {
    my $len = length($pool_id_tmp) + 1;
    if ( $pool_id_tmp == $pool_id_act ) {
      if ($len < length($line) ) {
        if ( $lpar_list_tmp eq '' ) {
          # there are no lpars in that pool
          if ( -f "$name-agg.png" ) {
            unlink ("$name-agg.png"); # delete old graph to do not stay there unchanged for ever
            LoadDataModule::touch ("$name-agg.png delete");
          }
          return 0
        }
        $lpar_list = $lpar_list_tmp;
        $pool_found = 1;
        last;
      }
      else {
        # there are no lpars in that pool
        if ( -f "$name-agg.png" ) {
          unlink ("$name-agg.png"); # delete old graph to do not stay there unchanged for ever
          LoadDataModule::touch ("$name-agg.png delete");
        }
        return 0
      }
    }
  }
 
  #print "---- $pool_id_tmp $lpar_list_tmp  : $ii $iii\n";

  if ( $pool_found == 0 ) {
    # pool does not have to exist when it is not being used, there are gaps so ignoring that and skip that
    #error ("$host: 02:Pool aggregated graph error : $pool_id_tmp - $pool_id_act - $name ");
    if ( -f "$name-agg.png" ) {
      unlink ("$name-agg.png"); # delete old graph to do not stay there unchanged for ever
      LoadDataModule::touch ("$name-agg.png delete");
    }
    return 1;
  }


  my $lpars_exist=0;
  my @files = split(/,/,$lpar_list);
  foreach $lpar (@files) {
    chomp($lpar);
    if ( ! -f "$wrkdir/$managedname/$host/$lpar.rr$type_sam" ) {
      next;
    }

    $lpars_exist++;
    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = (stat("$wrkdir/$managedname/$host/$lpar.rr$type_sam"))[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    my $lpar_space = $lpar;
    $lpar_space =~ s/\&\&1/\//g; # for those which have a slash

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    for(my $k = length($lpar); $k < 25; $k++) {
      $lpar_space .= " ";
    }

    # Assure that lpar colors is same for each lpar through daily - yearly charts
    my $l;
    my $l_count = 0;
    my $found = -1;
    foreach $l (@keep_color_pools) {
      if ( $l eq '' || $l =~ m/^ $/ ) {
        next;
      }
      if ( $l =~ m/^$lpar$/ ) {
        $found = $l_count;
        last;
      }
      $l_count++;
    }
    if ( $found == -1 ) {
      $keep_color_pools[$l_count] = $lpar;
      $found = $l_count;
    }
    #print "000 $found $lpar $color[$found]\n";

    print "$wrkdir/$managedname/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);

    # bulid RRDTool cmd
    $cmd .= " DEF:cap${i}=\\\"$wrkdir/$managedname/$host/$lpar.rr$type_sam\\\":capped_cycles:AVERAGE";
    $cmd .= " DEF:uncap${i}=\\\"$wrkdir/$managedname/$host/$lpar.rr$type_sam\\\":uncapped_cycles:AVERAGE";
    $cmd .= " DEF:ent${i}=\\\"$wrkdir/$managedname/$host/$lpar.rr$type_sam\\\":entitled_cycles:AVERAGE";
    $cmd .= " DEF:cur${i}=\\\"$wrkdir/$managedname/$host/$lpar.rr$type_sam\\\":curr_proc_units:AVERAGE";
    $cmd .= " CDEF:tot${i}=cap${i},uncap${i},+";
    $cmd .= " CDEF:util${i}=tot${i},ent${i},/";
    $cmd .= " CDEF:utiltotu${i}=util${i},cur${i},*";
    $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";
    $cmd .= " $gtype:utiltot${i}$color[$found]:\\\"$lpar_space\\\"";

    # put carriage return after each second lpar in the legend
    if ($j == 1) {
      $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.3lf \\l\\\"";
      $j = 0;
    }
    else {
      $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.3lf \\l\\\"";
      #$cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.3lf \\t\\\"";
	# --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $j++
    }
    $gtype="STACK";
    $i++;
  }
  if ( "$type" =~ "d" ) {
    if ($j == 1) {
      $cmd .= " COMMENT:\\\" \\l\\\"";
    }
    $cmd .= " COMMENT:\\\"\(Note that for CPU dedicated LPARs is always shown their whole entitlement\)\\\"";
  }
  $cmd .= " HRULE:0#000000";
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  my  $FH;
  my $tmp_file="/var/tmp/lpar2rrd.tmp-$$";
  open(FH, "> $tmp_file") || error(" Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
  
  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("$host: Multi graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err,  my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("$host: $cmd: Multi graph rrdtool error : $ret");
    }
  }
  unlink ("$tmp_file");

  if ( $lpars_exist == 0 && -f "$name-agg.png" ) {
    unlink ("$name-agg.png"); # delete old graph to do not stay there unchanged for ever
    LoadDataModule::touch ("$name-agg.png delete");
  }

  return 0;
}

# open top10 files
sub open_topten_files
{
  my $managedname = shift;
  my $host = shift;
  my $act_time = shift;
  my $run_topten = 0; # run top10 only once a day

  if ( ! -f "tmp/$managedname-$host-d" ) {
    print "TOP10 refresh  : initial : tmp/$managedname-$host-d\n" if $DEBUG ;
    $run_topten = 1;
  }
  else {
    my $png_time = (stat("tmp/$managedname-$host-d"))[9];
    (my $sec,my $min,my $h,my $aday,my $m,my $y,my $wday,my $yday,my $isdst) = localtime(time());
    ($sec,$min,$h,my $png_day,$m,$y,$wday,$yday,$isdst) = localtime($png_time);
    if ( $aday != $png_day ) {
      print "TOP10 refresh  : $aday != $png_day\n" if $DEBUG ;
      $run_topten = 1;
    }
  }

  if ( $run_topten == 1) {
    open(TOPD, "> tmp/$managedname-$host-d") || error(" Can't open tmp/$managedname-$host-d: $!".__FILE__.":".__LINE__) && return 0;
    open(TOPW, "> tmp/$managedname-$host-w-tmp") || error(" Can't open tmp/$managedname-$host-w: $!".__FILE__.":".__LINE__) && return 0;
    open(TOPM, "> tmp/$managedname-$host-m-tmp") || error(" Can't open tmp/$managedname-$host-m: $!".__FILE__.":".__LINE__) && return 0;
    open(TOPY, "> tmp/$managedname-$host-y-tmp") || error(" Can't open tmp/$managedname-$host-y: $!".__FILE__.":".__LINE__) && return 0;
    LoadDataModule::touch ("top10 run");
  }

  return $run_topten;
}

sub close_topten_files
{
  $run_topten = shift;

  if ( $run_topten == 1) {
    close(TOPD);
    close(TOPW);
    close(TOPM);
    close(TOPY);
  }

  return 0;
}

# create server/lpar cfg high level page
sub get_cfg_server
{
  my $managedname = "";
  my $model = "";
  my $serial = "";
  my $managedname_exl = "";
  my $managed_ok = "";
  my @m_excl = "";
  my @mm = ""; #array of all servers and lpars
  my @mm_one = ""; #array of all servers and lpars for sum page of processed server (there are other paths then in mm_sum)
  my @mm_sum = ""; #array of all servers and lpars for sum page
  my $count_s = "";
  my $lpar_c = 0;
  my $server_c = 0;
  my $tds = "<TD";
  my $td = "<TD></TD>";
  my $tde = "</TD>";
  my $tr_start = "<TR>";
  my $tr_stop  = "</TR>";
  my $td_col_run = " bgcolor=\"#80FF80\">";
  my $td_col_donot_run = " bgcolor=\"#FF8080\">";
  my $td_col_pool = " bgcolor=\"#FFFF80\">";
  my $cfg_high_html_rel="config-high.html";
  my $cfg_high_html = "$webdir/$host/$cfg_high_html_rel";
  my $cfg_sum_html_rel = "config-high-sum.html";
  my $cfg_sum_html_rel_tmp = "$cfg_sum_html_rel-tmp";
  my $cfg_sum_html = "$webdir/$host/$cfg_sum_html_rel_tmp";
  my $cfg_sum_html_full = "$webdir/$host/$cfg_sum_html_rel";
  my $act_time = time();
  my @proc_pools = "";
  my $cvs_lpar = "$host-lpar-config.csv";
  my $cvs_srv= "$host-server-config.csv";
  my $cvs = "$webdir/$host/$cvs_lpar";
  my $cvs_tmp = "$webdir/$host/$cvs_lpar-tmp";
  my $cvs_server = "$webdir/$host/$cvs_srv";
  my $managedname_url = "";

  # do not update cfg file if less than 1 day
  if ( -f "$cfg_high_html" && $upgrade == 0 ) {
    my $file_time = (stat("$cfg_high_html"))[9];
    if ( ($act_time - $file_time) < $UPDATE_DAILY ) {
      return 0;
    }
  }

  print "CFG highlevel  : $host \n" if $DEBUG ;

  if (! -d "$webdir/$host" ) {
    print "mkdir          : $host $webdir/$host\n" if $DEBUG ;
    LoadDataModule::touch ("$webdir/$host");
    mkdir("$webdir/$host", 0755) || error(" Cannot mkdir $webdir/$host: $!".__FILE__.":".__LINE__) && return 0;
  }


  open(FHR, "> $cvs_server") || error(" Can't open $cvs_srv".__FILE__.":".__LINE__) && return 0; # detailed cfg in CSV format
  print FHR "HMC;server;type_model;serial;configurable_sys_proc_units;configurable_sys_mem[MB];curr_avail_sys_mem[MB];CPU pool name;reserved CPU units;maximum CPU units;\n"; # CVS header

  open(FHO, "> $cfg_high_html") || error(" Can't open $cfg_high_html".__FILE__.":".__LINE__) && return 0; # detailed cfg
  open(FHH, "> $cfg_sum_html_full") || error(" Can't open $cfg_sum_html_full".__FILE__.":".__LINE__) && return 0; # Sum cfg, it has different link path then sum bellow
  open(FHS, "> $cfg_sum_html") || error(" Can't open $cfg_sum_html".__FILE__.":".__LINE__) && return 0;   # sum cfg, just a table, htm is created in install-html.sh

  my $head ="<html> <head> <title>menu</title> 
   <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\"> 
   <META HTTP-EQUIV=\"pragma\" CONTENT=\"no-cache\">
   <META HTTP-EQUIV=\"Expires\" CONTENT=\"NOW\">
   <META HTTP-EQUIV=\"last modified\" CONTENT=\"NOW\">
   <STYLE TYPE=\"text/css\">
   <!--
   .header, .header TD, .header TH
   {
   background-color:#D3D2D2;
   }
   -->
   </STYLE>
   <style>
   <!--
   a {text-decoration: none}
   -->
   </style>
   </head>\n";
  print FHO $head;
  print FHH $head;
  print FHO "<BODY BGCOLOR=\"#D3D2D2\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#0000FF\" ALINK=\"#FF0000\" >\n";
  print FHH "<BODY BGCOLOR=\"#D3D2D2\" TEXT=\"#000000\" LINK=\"#0000FF\" VLINK=\"#0000FF\" ALINK=\"#FF0000\" >\n";
  print FHO "<A HREF=\"../config-global.htm\" target=\"sample\">Global</A> / <A HREF=\"".$cfg_sum_html_rel."\" target=\"sample\">Summary</A> <br><font size=\"-1\"><A HREF=\"$cvs_lpar\" target=\"_blank\">CSV LPAR</A> / <A HREF=\"$cvs_lpar\" target=\"_blank\">CSV Server</A></font>\n";
  print FHH "<A HREF=\"../config-global.htm\" target=\"sample\">Global</A> / <A HREF=\"".$cfg_high_html_rel."\" target=\"sample\">Detailed</A> <br><font size=\"-1\"><A HREF=\"$cvs_lpar\" target=\"_blank\">CSV LPAR</A>  / <A HREF=\"$cvs_srv\" target=\"_blank\">CSV Server</A></font>\n";
  if ($IVM == 0 ) {
    print FHO "<center><h3>$host</h3></center>\n";
    print FHH "<center><h3>$host</h3></center>\n";
  }
  print FHO "<center><TABLE border=\"1\">\n";
  print FHH "<center><TABLE border=\"1\">\n";
  print FHS "<center><TABLE border=\"1\">\n";

  open(FHC, "> $cvs_tmp") || error(" Can't open $cvs".__FILE__.":".__LINE__) && return 0; # detailed cfg in CSV format
  print FHC "HMC;server;lpar_name;curr_shared_proc_pool_name;curr_proc_mode;curr_sharing_mode;curr_proc_units;curr_procs;curr_uncap_weight;lpar_id;default_profile;min_proc_units;desired_proc_units;max_proc_units;curr_min_proc_units;curr_max_proc_units;min_procs;desired_procs;max_procs;curr_min_procs;curr_max_procs;min_mem;desired_mem;max_mem;curr_min_mem;curr_mem;curr_max_mem;state;lpar_env;os_version\n";
  close (FHC);


  # run that 1st time after the midnight!!
  #     --> due to putting avrg lpar load per last day in stats
 
  # get data for all managed system which are conected to HMC
 
  foreach my $line (@managednamelist) {
    chomp($line);

    if ( $line !~ ".*,.*,.*" ){
      # it must look like : PWR6A-9117-MMA-SN103A4B0,9117-MMA,103A4B0, --> exclude banners
      next;
    }
    if ($line =~ /No results were found/) {
      print "$host does not contain any managed system\n" if $DEBUG ;
      return 0;
    }
    ($managedname, $model, $serial) = split(/,/,$line);

    if ( is_IP($managedname) == 1 ) {
      next; # wrong entry from the HMC, a problem of the HMC or unconfigured server yet
    }

    print "managed system : $host:$managedname (type_model*serial : $model*$serial)\n" if $DEBUG ;
    $managedname_url = $managedname;
    $managedname_url =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
    $managedname_url =~ s/\+/ /g;
    $managedname_url =~ s/\#/%23/g;

 
    $managed_ok=1;
    @pool_list = ""; # clean pool_list for each managed name
    
    # do not exclude here anything
 
    my $cfg = "$wrkdir/$managedname/$host/cpu.cfg";
    if ( ! -f $cfg ) {
      error ("$host:$managedname : does not exist : $cfg");
      next;
    }

    my $cfg_shell = $cfg;
    $cfg_shell =~ s/ /\\ /g;
 
    # what about IVM/SDMC???
    my @lparlist="";
    if ($IVM == 1 ) {
      @lparlist=`$SSH $hmc_user\@$host "lssyscfg -m \\"$managedname\\" -r lpar -F name,state" 2>&1`;
    }
    else {
      @lparlist=`$SSH $hmc_user\@$host "lssyscfg -r lpar -m \\"$managedname\\" -F name,state" 2>&1`;
      @proc_pools=`$SSH $hmc_user\@$host "lshwres -m \\"$managedname\\" -r procpool -F \"name,max_pool_proc_units,curr_reserved_pool_proc_units\"" 2>&1`;
      if ($proc_pools[0] =~ /HSCL/ || $proc_pools[0] =~ /VIOSE0/ ) {
        $proc_pools[0] = '';
      }
    }
    #if ($lparlist[0] =~ /HSCL/ || $lparlist[0] =~ /VIOSE0/ ) || $lparlist[0] eq '' ) {
    if ($lparlist[0] =~ /HSCL/ || $lparlist[0] =~ /VIOSE0/ ) {
      $lparlist[0] = '';
      #my $act_time = localtime();
      #error ("$host:$managedname : $lparlist[0]");
      #next;
    }
 
    # get global CPU and MEM
    my $out_file = "$CONFIG_HISTORY/$managedname/$host/config.cfg";
    my $mcpu ="n/a";
    my $c_mem ="n/a";
    my $a_mem ="n/a";
    my $serial_num ="n/a";
    my $type_model ="n/a";

    if ( -f $out_file ) {
      open(FHG, "< $out_file");
      my @lines_glo = <FHG>;
      print FHR "$host;$managedname";
      foreach  my $linel (@lines_glo) {
        chomp ($linel);
        if ( $linel =~ "HSCL" || $linel =~ "VIOSE0" ) {
          last;
        }
        if ($linel =~ m/^type_model/) {
          $type_model =$linel;
          $type_model =~ s/type_model//;
          $type_model =~ s/ //g;
          print FHR ";$type_model";
          next;
        }
        if ($linel =~ m/^serial_num/) {
          $serial_num =$linel;
          $serial_num =~ s/serial_num//;
          $serial_num =~ s/ //g;
          print FHR ";$serial_num";
          next;
        }
        if ($linel =~ m/^configurable_sys_proc_units/) {
          $mcpu=$linel;
          $mcpu =~ s/configurable_sys_proc_units//;
          $mcpu =~ s/ //g;
          print FHR ";$mcpu";
          next;
        }
        if ($linel =~ m/^configurable_sys_mem/) {
          $c_mem=$linel;
          $c_mem =~ s/configurable_sys_mem//;
          $c_mem =~ s/ //g;
          print FHR ";$c_mem";
          if ( $c_mem > 1024  ) { # conversion to GB if >1GB
             my $l = int($c_mem/1024 + 0.5);
             $c_mem = $l."&nbsp;GB";
           }
           else {
            $c_mem .= "&nbsp;MB"; 
          }
          next;
        }
        if ($linel =~ m/^curr_avail_sys_mem/) {
          $a_mem=$linel;
          $a_mem =~ s/curr_avail_sys_mem//;
          $a_mem =~ s/ //g;
          print FHR ";$a_mem\n";
          if ( $a_mem > 1024  ) { # conversion to GB if >1GB
             my $l = int($a_mem/1024 + 0.5);
             $a_mem = $l."&nbsp;GB";
           }
           else {
            $a_mem .= "&nbsp;MB"; 
          }
          last;
        }
      }
      close (FHG);
    }
    else {
      # server might be switched off --> no global cfg is available
      # or global cfg do not exist from any reason, it will be created nex run (day)
      error ("$host:$managedname : does not exist : $out_file");
    }


    $count_s = 0;
    if ( $IVM == 1 ) {
      $mm[$count_s] .= $tds."><b><center><font size=\"-1\"><A HREF=\"".$managedname_url."/config.html\" target=\"sample\">".$host."</A></center></b></font><table width=\"100%\"><tr><td><font size=\"-1\">Total&nbsp;cores:&nbsp;<b>".$mcpu."</b></font></td></tr><tr><td><font size=\"-1\">Total&nbsp;mem:&nbsp;<b>".$c_mem."</b></font></td><td><font size=\"-1\">Avail&nbsp;mem:&nbsp;<b>".$a_mem."</b></font></td></tr></table>".$tde;
      $mm_sum[$count_s] .= $tds." nowrap title=\"Total&nbsp;cores:&nbsp;".$mcpu."&nbsp;&nbsp;Total&nbsp;mem:&nbsp;".$c_mem."&nbsp;&nbsp;Avail&nbsp;mem:&nbsp;".$a_mem."\"><br><b><center><font size=\"-1\"><A HREF=\"".$host."/".$managedname_url."/config.html\" target=\"sample\">".$host."</A></center></b></font>".$tde;
      $mm_one[$count_s] .= $tds." nowrap title=\"Total&nbsp;cores:&nbsp;".$mcpu."&nbsp;&nbsp;Total&nbsp;mem:&nbsp;".$c_mem."&nbsp;&nbsp;Avail&nbsp;mem:&nbsp;".$a_mem."\"><br><b><center><font size=\"-1\"><A HREF=\"".$managedname_url."/config.html\" target=\"sample\">".$host."</A></center></b></font>".$tde;
    }
    else {
      $mm[$count_s] .= $tds."><b><center><font size=\"-1\"><A HREF=\"".$managedname_url."/config.html\" target=\"sample\">".$managedname."</A></center></b></font><table width=\"100%\"><tr><td><font size=\"-1\">Total&nbsp;cores:&nbsp;<b>".$mcpu."</b></font></td></tr><tr><td><font size=\"-1\">Total&nbsp;mem:&nbsp;<b>".$c_mem."</b></font></td><td><font size=\"-1\">Avail&nbsp;mem:&nbsp;<b>".$a_mem."</b></font></td></tr></table>".$tde;
      $mm_sum[$count_s] .= $tds." nowrap title=\"Total&nbsp;cores:&nbsp;".$mcpu."&nbsp;&nbsp;Total&nbsp;mem:&nbsp;".$c_mem."&nbsp;&nbsp;Avail&nbsp;mem:&nbsp;".$a_mem."\"><b><center><font size=\"-1\"><A HREF=\"".$host."/".$managedname_url."/config.html\" target=\"sample\">".$managedname."</A></center></b></font>".$tde;
      $mm_one[$count_s] .= $tds." nowrap title=\"Total&nbsp;cores:&nbsp;".$mcpu."&nbsp;&nbsp;Total&nbsp;mem:&nbsp;".$c_mem."&nbsp;&nbsp;Avail&nbsp;mem:&nbsp;".$a_mem."\"><b><center><font size=\"-1\"><A HREF=\"".$managedname_url."/config.html\" target=\"sample\">".$managedname."</A></center></b></font>".$tde;
    }
    my $result = "";
    my $result_sum = "";
    my $result_one = "";
    $server_c++;
    my $lpar = "";
 
    my $mem_cfg = "$wrkdir/$managedname/$host/mem.html";
    if ( ! -f $mem_cfg ) {
      error ("$host:$managedname : does not exist : $mem_cfg");
      next;
    }

    open(FHM, "< $mem_cfg") || next;
    my @lines_mem = <FHM>;
    close (FHM);
 
 
    open(FHX, "< $cfg") || next ;
    my @lines_unsorted = sort(<FHX>);
    # case non sensitive sort
    my @lines = sort { lc($a) cmp lc($b) } @lines_unsorted;
    close (FHX);
 
    # proc pools
    #print "--- CFG \n";
    foreach  my $linep (@proc_pools) {
      chomp($linep);
      #print "--- $linep\n";
      if ( $linep eq '' ) {
        last;
      }
      if ( $linep =~ m/^DefaultPool/ ) {
        next;
      }
      if ( $linep =~ "HSCL" || $linep =~ "VIOSE0" ) {
        error ("$host:$managedname : Problem with reading proc pool cfg : $linep");
        last;
      }
      if ( $linep =~ "does not support multiple shared" ) {
        last;
      }

      (my $name_pool, my $max, my $reserved) = split(/,/,$linep);
      if ( $reserved eq '' ) {
        last; # potential error
      }
      print FHR "$host;$managedname;;;;;;$name_pool;$reserved;$max\n";

      $count_s++;
      #print "++ $lpar_c $count_s $server_c \n";
      #print "+++ $lpar_c $count_s $server_c $name_pool $max $reserved \n";
      if ($lpar_c < $count_s && $server_c > 1) {
        $lpar_c = $count_s;
        for (my $i=1; $i < $server_c; $i++) {
          #print "-- $count_s $i\n";
          $mm[$count_s] .= $td;
          $mm_sum[$count_s] .= $td;
          $mm_one[$count_s] .= $td;
        }
      }
      $result = "<TD nowrap ".$td_col_pool."<center><B><font size=\"-1\"><A HREF=\"".$managedname_url."/config.html#CPU_pools\" target=\"sample\">".$name_pool."</A></B></font></center><TABLE border=\"0\" width=\"100%\">";
      $result .= "<tr><td size=\"50%\"><font size=\"-1\">Max&nbsp;CPU:&nbsp;<B>".$max."</font></b></td><td><font size=\"-1\">Reserved&nbsp;CPU:&nbsp;<b>".$reserved."</font></b></td></tr></table></TD>";
      $result_sum = "<TD nowrap title=\"Max&nbsp;CPU:&nbsp;".$max."&nbsp;&nbsp;Reserved&nbsp;CPU:&nbsp;".$reserved."\" ".$td_col_pool."<center><B><font size=\"-1\"><A HREF=\"".$host."/".$managedname_url."/config.html#CPU_pools\" target=\"sample\">".$name_pool."</A></font></B></center></TD>";
      $result_one = "<TD nowrap title=\"Max&nbsp;CPU:&nbsp;".$max."&nbsp;&nbsp;Reserved&nbsp;CPU:&nbsp;".$reserved."\" ".$td_col_pool."<center><B><font size=\"-1\"><A HREF=\"".$managedname_url."/config.html#CPU_pools\" target=\"sample\">".$name_pool."</A></font></B></center></TD>";
      $mm[$count_s] .= $result;
      $mm_sum[$count_s] .= $result_sum;
      $mm_one[$count_s] .= $result_one;
    }

    lpar_details ($managedname,$host,$cvs_tmp);

    foreach  my $linel (@lines ) {
      chomp($linel);
      if ($linel =~ "lpar_name=" ) {
        my @list = split(/,/,$linel);
        my $first = 0;
        my $l     = "";
        my $state = "";
        $result = "";
        $result_sum = "";
        $result_one = "";
        my $cpu_ded = 0;
    	my $phys_cores = "na";
	my $logical = "na";
        my $pool_name = "na";
	my $cpu_weight = "na";
	my $cpu_mode1 = "na";
	my $cpu_mode2 = "na";

        foreach my $item (@list) {
          chomp($item);
          $first++;
          if ($first == 1) {
            (my $name_trash, $lpar) = split(/=/,$item);
            $state = "na";
            foreach $l (@lparlist) {
              if ( $l eq '' ) {
                next;
              }
              chomp ($l);
              if ( $l eq '' ) {
                next;
              }
              (my $name,  my $state_a) = split(/,/,$l);
              if ( $state_a eq '' ) {
                next;
              }
              if (($name =~ quotemeta($lpar)) && (length($name) == length($lpar))) {
                $state = $state_a;
              }
            }
            #print "\n$lpar $state ";
            my $lpar_path = $lpar;
            $lpar_path =~ s/\//\&\&1/g;
            $result = "<center><B><font size=\"-1\"><A HREF=\"".$managedname_url."/config.html#".$lpar."\" target=\"sample\">".$lpar."</A></font></B></center><TABLE border=\"0\" width=\"100%\">";
            $result_sum = "<center><font size=\"-1\"><A HREF=\"".$host."/".$managedname_url."/config.html#".$lpar."\" target=\"sample\">".$lpar."</A></font></center>";
            $result_one = "<center><font size=\"-1\"><A HREF=\"".$managedname_url."/config.html#".$lpar."\" target=\"sample\">".$lpar."</A></font></center>";
            next;
          }
          #if ($item =~ m/^curr_/ && ($item !~ "_max") && ($item !~ "_min") && ($item !~ "curr_shared_proc_pool_id") ){
            my $name_col = "";
            (my $left_c, my $right_c) = split(/=/,$item);
            if ($item =~ m/^curr_shared_proc_pool_name/) {
	      $pool_name = $right_c;
              next;
            }
            if ($item =~ m/^curr_proc_mode/) {
	      $cpu_mode1 = $right_c;
              if ( $right_c =~ m/ded/ ) {
                $cpu_ded = 1;
	      }
              next;
            }
            if ($item =~ m/^curr_proc_units/) {
	      $phys_cores = $right_c;
              next;
            }
            if ($item =~ m/^curr_procs/) {
	      $logical = $right_c;
            }
            if ($item =~ m/^curr_sharing_mode/) {
              if ( $cpu_ded ==  1 ) {
	        $cpu_mode2 = " ";
	      }
	      else {
	        $cpu_mode2 = ",".$right_c;
	      }
              next;
            }
            if ($item =~ m/^curr_uncap_weight/) {
	      $cpu_weight = $right_c;
              next;
            }
        }
        my $mode2 = $cpu_mode2;
        $mode2 =~ s/,//;

        $count_s++;
        #print "++ $lpar_c $count_s $server_c \n";
        if ($lpar_c < $count_s && $server_c > 1) {
          $lpar_c = $count_s;
          for (my $i=1; $i < $server_c; $i++) {
            #print "-- $count_s $i\n";
            $mm[$count_s] .= $td;
            $mm_sum[$count_s] .= $td;
            $mm_one[$count_s] .= $td;
          }
        }

        my $ram = "na";
        foreach my $line_mem (@lines_mem) {
          chomp ($line_mem);
          $line_mem .= " ";
          $line_mem =~ s/<TR> <TD><font size=-1><B>//;
          $line_mem =~ s/<\/B><\/font><\/TD> <TD align=\"center\"><font size=-1>/|/g;
          $line_mem =~ s/<\/font><\/TD> <TD align=\"center\"><font size=-1>/|/g;
          $line_mem =~ s/<\/font><\/TD><\/TR>//g;
          #$line_mem =~ s/<.*//;
          if ( $line_mem !~ m/\|/ ) {
            next;
          }
          #print "$line_mem\n";
          (my $lpar_mem, my $trash1, my $trash2, my $strash3, $ram) = split(/\|/,$line_mem);
          if ("$lpar_mem" eq "$lpar" && length($lpar) == length($lpar_mem)) {
            if ( $ram > 1024  ) { # conversion to GB if >1GB
               my $l = int($ram/1024 + 0.5);
               $ram = $l."&nbsp;GB";
            }
            else {
              $ram .= "&nbsp;MB"; 
            }

            last;
          }
        }
	# get al lpar info into HTML format
        $result .= "<tr><td><font size=\"-1\">Cores:&nbsp;<b>".$phys_cores."</b></font></td>";
        $result .= "    <td><font size=\"-1\">Logical&nbsp;CPU:&nbsp;<b>".$logical."</b></font></td><tr>";
        $result .= "<tr><td><font size=\"-1\">Pool:&nbsp;<b>".$pool_name."</b></font></td>";
        $result .= "    <td><font size=\"-1\">CPU&nbsp;weight:&nbsp;<b>".$cpu_weight."</b></font></td></tr>";
        $result .= "<tr><td><font size=\"-1\">Mode:&nbsp;<b>".$cpu_mode1.$cpu_mode2."</b></font></td>";
        $result .= "    <td><font size=\"-1\">RAM:&nbsp;<b>".$ram."</b></font></td></tr></table>";

        #print "------ $lpar  $state\n";
        if ( $state =~ m/Running/ ) {
          $mm[$count_s] .= $tds.$td_col_run.$result.$tde;
          $mm_sum[$count_s] .= $tds." nowrap title=\"Cores:&nbsp;".$phys_cores."&nbsp;&nbsp;Logical&nbsp;CPU:&nbsp;".$logical."&nbsp;&nbsp;Pool:&nbsp;".$pool_name."&nbsp;&nbsp;CPU&nbsp;weight:".$cpu_weight."&nbsp;&nbsp;Mode:&nbsp;".$cpu_mode1.$cpu_mode2."&nbsp;&nbsp;RAM:&nbsp;".$ram."\" ".$td_col_run.$result_sum.$tde;
          $mm_one[$count_s] .= $tds." nowrap title=\"Cores:&nbsp;".$phys_cores."&nbsp;&nbsp;Logical&nbsp;CPU:&nbsp;".$logical."&nbsp;&nbsp;Pool:&nbsp;".$pool_name."&nbsp;&nbsp;CPU&nbsp;weight:".$cpu_weight."&nbsp;&nbsp;Mode:&nbsp;".$cpu_mode1.$cpu_mode2."&nbsp;&nbsp;RAM:&nbsp;".$ram."\" ".$td_col_run.$result_one.$tde;
        }
        else {
          $mm[$count_s] .= $tds.$td_col_donot_run.$result.$tde;
          $mm_sum[$count_s] .= $tds." nowrap title=\"&nbsp;&nbsp;Cores:&nbsp;".$phys_cores."&nbsp;&nbsp;Logical&nbsp;CPU:&nbsp;".$logical."&nbsp;&nbsp;Pool:&nbsp;".$pool_name."&nbsp;&nbsp;CPU&nbsp;weight:".$cpu_weight."&nbsp;&nbsp;Mode:&nbsp;".$cpu_mode1.$cpu_mode2."&nbsp;&nbsp;RAM:&nbsp;".$ram."\" ".$td_col_donot_run.$result_sum.$tde;
          $mm_one[$count_s] .= $tds." nowrap title=\"&nbsp;&nbsp;Cores:&nbsp;".$phys_cores."&nbsp;&nbsp;Logical&nbsp;CPU:&nbsp;".$logical."&nbsp;&nbsp;Pool:&nbsp;".$pool_name."&nbsp;&nbsp;CPU&nbsp;weight:".$cpu_weight."&nbsp;&nbsp;Mode:&nbsp;".$cpu_mode1.$cpu_mode2."&nbsp;&nbsp;RAM:&nbsp;".$ram."\" ".$td_col_donot_run.$result_one.$tde;
        }
        #print "$mm[$count_s]\n";
      }
    }
    if ( $lpar_c == 0 ) {
      $lpar_c = $count_s;
    }
    else {
      #print "-- $count_s $lpar_c \n";
      #$count_s++;
      for (;$count_s  + 1< $lpar_c + 1;$count_s++) {
      #print "++ $count_s $lpar_c \n";
        $mm[$count_s + 1] .= $td;
        $mm_sum[$count_s + 1] .= $td;
        $mm_one[$count_s + 1] .= $td;
      }
      if ($count_s > $lpar_c) {
        $lpar_c = $count_s;
      }
    }
 
 
  }
 
  foreach my $line (@mm) {
    print FHO "$tr_start $line $tr_stop\n";
  }

  foreach my $line (@mm_sum) {
    print FHS "$tr_start $line $tr_stop\n";
  }

  foreach my $line (@mm_one) {
    print FHH "$tr_start $line $tr_stop\n";
  }

  print FHO "</TABLE>\n";
  print FHH "</TABLE>\n";
  print FHS "</TABLE>\n";
  my $ltime=localtime($act_time);
  print FHO "</center><br><br><table><tr><td $td_col_run <font size=\"-1\"> running</font></td><td $td_col_donot_run <font size=\"-1\"> not running</font></td><td $td_col_pool <font size=\"-1\"> CPU pool</font></td></tr></table><br>\n";
  print FHH "</center><br><br><table><tr><td $td_col_run <font size=\"-1\"> running</font></td><td $td_col_donot_run <font size=\"-1\"> not running</font></td><td $td_col_pool <font size=\"-1\"> CPU pool</font></td></tr></table><br>\n";
  print FHO "<font size=\"-1\">It is updated once a day, last run: $ltime\n</font><br>";
  print FHH "<font size=\"-1\">It is updated once a day, last run: $ltime\n</font><br>";
  print FHO "</BODY></HTML>\n";
  print FHH "</BODY></HTML>\n";

  close (FHO);
  close (FHH);
  close (FHS);
  close (FHR);

  # delete old CVS file and move new one
  if ( -f "$cvs" ) {
    unlink("$cvs") || error("Cannot rm $cvs: $!".__FILE__.":".__LINE__) && return 0;
  }
  rename("$cvs_tmp","$cvs") || error(" Cannot mv $cvs_tmp $cvs: $!".__FILE__.":".__LINE__) && return 0;
 
  return 1;
}
 
# had a problem with filling that in var in LoadDataModule.pm tehrefore it is here
sub procpoolagg
{
  my $host = shift;
  my $managedname = shift;
  
  @poollistagg=`$SSH $hmc_user\@$host "lshwres -r procpool -m \\"$managedname\\" -F shared_proc_pool_id,lpar_names"`;

  return 1;
}


# error handling
sub error
{
  my $text = shift;
  my $act_time = localtime();
  chomp ($text);
  
  print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
}

sub save_cfg_data 
{
  my $managedname = shift;
  my $date = shift;
  my $upgrade = shift;
  my $ret = 0;

  $ret = save_cpu_cfg($managedname,$date,$upgrade);
  if ( $ret > 0 ) {
    return $ret 
  }

  $ret = save_cpu_cfg_global($managedname,$date,$upgrade);
  if ( $ret == 1 ) {
    return 1
  }

  # Include HW info 
  if ( $HWINFO ) {
    save_config($managedname,$date);
  }

  if ( $HMC == 1 || $SDMC == 1 ){ # IVM does not support it
    # Include system changes
    if ( $SYS_CHANGE ) {
      sys_change($managedname);
    }
  }

  return 0;
}

sub rename_server
{
  my $host = shift;
  my $managedname = shift;
  my $model = shift;
  my $serial = shift;

    if (! -d "$wrkdir/$managedname" ) {
      # when managed system is renamed then find the original nale per a sym link with model*serial
      #   and rename it in lpar2rrd as well
      if ( -l "$wrkdir/$model*$serial" ) {
        my $link=readlink("$wrkdir/$model*$serial");

        #my $base = basename($link);
	# basename without direct function
        my @link_l = split(/\//,$link);
        my $base = "";
        foreach my  $m (@link_l) {
          $base = $m;
        }

        print "system renamed : $host:$managedname from $base to $managedname, behave as upgrade \n" if $DEBUG ;
        if ( -f "$link" ) {
          rename("$link","$wrkdir/$managedname")  || error(" Cannot mv $link $wrkdir/$managedname: $!".__FILE__.":".__LINE__) && return 0;
        }
        unlink("$wrkdir/$model*$serial") || error(" Cannot rm $wrkdir/$model*4serial: $!".__FILE__.":".__LINE__) && return 0;
        $upgrade=1; # must be like upgrade behave due to views
      }
      else {
        print "mkdir          : $host:$managedname $wrkdir/$managedname\n" if $DEBUG ;
        mkdir("$wrkdir/$managedname", 0755) || error(" Cannot mkdir $wrkdir/$managedname: $!".__FILE__.":".__LINE__) && return 0;
      }
      LoadDataModule::touch ("$wrkdir/$managedname"); #must be at the end dou to renaming servers
    } 

    # check wherher the symlink is linked to the right targed
    # there could be an issue with symlink prior 3.37 ($managedname dirs could be created from HEA stuff without care about renaming)
    my $managedname_linked = "";
    my $link = "";
    my $link_expected = "$wrkdir/$model*$serial";
    if ( -l "$link_expected" ) {
      $link=readlink("$link_expected");
      # basename without direct function
      my @link_l = split(/\//,$link);
      my $managedname_linked = "";
      foreach my  $m (@link_l) {
        $managedname_linked = $m;
      }
      if ( $managedname =~ m/^$managedname_linked$/ ) {
        # ok, symlink target is properly linked
        return 1;
      }
      else {
        print "symlink correct: $host:$managedname : $link : $link_expected\n" if $DEBUG ;
        unlink ($link_expected);
      }
    }

  return 1;
}

# fill in @lpm_excl_vio for server
sub lpm_exclude_vio
{
  my $host = shift;
  my $managedname = shift;
  my $wrkdir = shift;
  my $lpm_excl= "$wrkdir/$managedname/$host/lpm-exclude.txt";

  if ( $lpm == 0) {
    return 0; # LPM is switched off
  }
  open(FH, "< $lpm_excl") || return 1;
  @lpm_excl_vio = <FH>;
  close(FH);
  return 0;
}

sub lpar_details 
{
  my $managedname = shift;
  my $host = shift;
  my $cvs = shift;

  if ( ! -f "$wrkdir/$managedname/$host/config.cfg" ) {
    error ("does not exist $wrkdir/$managedname/$host/config.cfg");
    return 1;
  }

  open(FHC, ">> $cvs") || error(" Can't open $cvs".__FILE__.":".__LINE__) && return 0; # detailed cfg in CSV format

  open(FCFG, "< $wrkdir/$managedname/$host/config.cfg") || error(" Can't open $wrkdir/$managedname/$host/config.cfg".__FILE__.":".__LINE__) && return 0; 


  my $csv_lpar_name = "na";
  my $csv_lpar_id = "na";
  my $csv_default_profile = "na";
  my $csv_min_proc_units = "na";
  my $csv_desired_proc_units = "na";
  my $csv_max_proc_units = "na";
  my $csv_curr_min_proc_units = "na";
  my $csv_curr_proc_units = "na";
  my $csv_curr_max_proc_units = "na";
  my $csv_min_procs = "na";
  my $csv_desired_procs = "na";
  my $csv_max_procs = "na";
  my $csv_curr_min_procs = "na";
  my $csv_curr_procs = "na";
  my $csv_curr_max_procs = "na";
  my $csv_min_mem = "na";
  my $csv_desired_mem = "na";
  my $csv_max_mem = "na";
  my $csv_curr_min_mem = "na";
  my $csv_curr_mem = "na";
  my $csv_curr_max_mem = "na";
  my $csv_state = "na";
  my $csv_lpar_env = "na";
  my $csv_os_version = "na";
  my $pool_name = "na";
  my $cpu_mode1 = "na";
  my $phys_cores = "na";
  my $logical = "na";
  my $cpu_mode2 = "na";
  my $cpu_weight = "na"; 
  my $cpu_ded = "na";

  my $item = "";
  my $right_c = "";
  my $first = 0;

  #print "000 $wrkdir/$managedname/$host/config.cfg\n";
  foreach  my $linel (<FCFG>) {
    chomp($linel);
    if ( $linel eq '' ) {
       next;
    }

    if ($linel =~ "LPAR config:" ) {
      $first = 1;
      if ( $csv_lpar_name !~ m/^na$/ ) {
        print FHC "$host;$managedname;$csv_lpar_name;$pool_name;$cpu_mode1;$phys_cores;$logical;$cpu_mode2;$cpu_weight;$csv_lpar_id;$csv_default_profile;$csv_min_proc_units;$csv_desired_proc_units;$csv_max_proc_units;$csv_curr_min_proc_units;$csv_curr_max_proc_units;$csv_min_procs;$csv_desired_procs;$csv_max_procs;$csv_curr_min_procs;$csv_curr_max_procs;$csv_min_mem;$csv_desired_mem;$csv_max_mem;$csv_curr_min_mem;$csv_curr_mem;$csv_curr_max_mem;$csv_state;$csv_lpar_env;$csv_os_version\n";
      }
      #print "001 $csv_lpar_name\n";
     
      $csv_lpar_name = "na";
      $csv_lpar_id = "na";
      $csv_default_profile = "na";
      $csv_min_proc_units = "na";
      $csv_desired_proc_units = "na";
      $csv_max_proc_units = "na";
      $csv_curr_min_proc_units = "na";
      $csv_curr_proc_units = "na";
      $csv_curr_max_proc_units = "na";
      $csv_min_procs = "na";
      $csv_desired_procs = "na";
      $csv_max_procs = "na";
      $csv_curr_min_procs = "na";
      $csv_curr_procs = "na";
      $csv_curr_max_procs = "na";
      $csv_min_mem = "na";
      $csv_desired_mem = "na";
      $csv_max_mem = "na";
      $csv_curr_min_mem = "na";
      $csv_curr_mem = "na";
      $csv_curr_max_mem = "na";
      $csv_state = "na";
      $csv_lpar_env = "na";
      $csv_os_version = "na";
      $pool_name = "na";
      $cpu_mode1 = "na";
      $phys_cores = "na";
      $logical = "na";
      $cpu_mode2 = "na";
      $cpu_weight = "na"; 
      $cpu_ded = "na";
      next;
     }
     
     if ( $first == 0 ) {
       next;
     }
 
     $item = "";
     $right_c = "na";
     #print "003 $linel \n";

     if ( $linel !~ m/ / ) {
       next;
     }

     ($item, $right_c) = split(/\s{1,}/,$linel); # if is there one and more spaces or tabelators

     if ( $item eq '' ) {
       next;
     }
     if ( $right_c eq '' ) {
       next;
     }

     # --> this is due to parameter with a space inside like pool name, status ...
     my $line2 = $linel;
     $line2 =~ s/^.*$right_c / /;
     if (length($linel) != length($line2)) {
       $right_c .= $line2;
     }

     #print "004 $linel : $item - $right_c\n";

            if ($item =~ m/^curr_shared_proc_pool_name/) {
              if ( $pool_name !~ m/^na$/ ) {
                next;
              }
	      $pool_name = $right_c;
              next;
            }
            if ($item =~ m/^curr_proc_mode/) {
              if ( $cpu_mode1 !~ m/^na$/ ) {
                next;
              }
	      $cpu_mode1 = $right_c;
              next;
            }
            if ($item =~ m/^curr_proc_units/) {
              if ( $phys_cores !~ m/^na$/ ) {
                next;
              }
	      $phys_cores = $right_c;
              next;
            }
            if ($item =~ m/^curr_procs/) {
              if ( $logical !~ m/^na$/ ) {
                next;
              }
	      $logical = $right_c;
              next;
            }
            if ($item =~ m/^curr_sharing_mode/) {
              if ( $cpu_mode2 !~ m/^na$/ ) {
                next;
              }
	      $cpu_mode2 = $right_c;
              next;
            }
            if ($item =~ m/^curr_uncap_weight/) {
              if ( $cpu_weight !~ m/^na$/ ) {
                next;
              }
	      $cpu_weight = $right_c;
              next;
            }
            if ($item =~ m/^name/) {
              if ( $csv_lpar_name !~ m/^na$/ ) {
                next;
              }
	      $csv_lpar_name = $right_c;
              next;
            }
            if ($item =~ m/^lpar_id/) {
              if ( $csv_lpar_id !~ m/^na$/ ) {
                next;
              }
	      $csv_lpar_id = $right_c;
              next;
            }
            if ($item =~ m/^default_profile/) {
              if ( $csv_default_profile !~ m/^na$/ ) {
                next;
              }
	      $csv_default_profile = $right_c;
              next;
            }
            if ($item =~ m/^min_proc_units/) {
              if ( $csv_min_proc_units !~ m/^na$/ ) {
                next;
              }
	      $csv_min_proc_units = $right_c;
              next;
            }
            if ($item =~ m/^desired_proc_units/) {
              if ( $csv_desired_proc_units !~ m/^na$/ ) {
                next;
              }
	      $csv_desired_proc_units = $right_c;
              next;
            }
            if ($item =~ m/^max_proc_units/) {
              if ( $csv_max_proc_units !~ m/^na$/ ) {
                next;
              }
	      $csv_max_proc_units = $right_c;
              next;
            }
            if ($item =~ m/^curr_min_proc_units/) {
              if ( $csv_curr_min_proc_units !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_min_proc_units = $right_c;
              next;
            }
            if ($item =~ m/^curr_proc_units/) {
              if ( $csv_curr_proc_units !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_proc_units = $right_c;
              next;
            }
            if ($item =~ m/^curr_max_proc_units/) {
              if ( $csv_curr_max_proc_units !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_max_proc_units = $right_c;
              next;
            }
            if ($item =~ m/^min_procs/) {
              if ( $csv_min_procs !~ m/^na$/ ) {
                next;
              }
	      $csv_min_procs = $right_c;
              next;
            }
            if ($item =~ m/^desired_procs/) {
              if ( $csv_desired_procs !~ m/^na$/ ) {
                next;
              }
	      $csv_desired_procs = $right_c;
              next;
            }
            if ($item =~ m/^max_procs/) {
              if ( $csv_max_procs !~ m/^na$/ ) {
                next;
              }
	      $csv_max_procs = $right_c;
              next;
            }
            if ($item =~ m/^curr_min_procs/) {
              if ( $csv_curr_min_procs !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_min_procs = $right_c;
              next;
            }
            if ($item =~ m/^curr_procs/) {
              if ( $csv_curr_procs !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_procs = $right_c;
              next;
            }
            if ($item =~ m/^curr_max_procs/) {
              if ( $csv_curr_max_procs !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_max_procs = $right_c;
              next;
            }
            if ($item =~ m/^min_mem/) {
              if ( $csv_min_mem !~ m/^na$/ ) {
                next;
              }
	      $csv_min_mem = $right_c;
              next;
            }
            if ($item =~ m/^desired_mem/) {
              if ( $csv_desired_mem !~ m/^na$/ ) {
                next;
              }
	      $csv_desired_mem = $right_c;
              next;
            }
            if ($item =~ m/^max_mem/) {
              if ( $csv_max_mem !~ m/^na$/ ) {
                next;
              }
	      $csv_max_mem = $right_c;
              next;
            }
            if ($item =~ m/^curr_min_mem/) {
              if ( $csv_curr_min_mem !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_min_mem = $right_c;
              next;
            }
            if ($item =~ m/^curr_mem/) {
              if ( $csv_curr_mem !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_mem = $right_c;
              next;
            }
            if ($item =~ m/^curr_max_mem/) {
              if ( $csv_curr_max_mem !~ m/^na$/ ) {
                next;
              }
	      $csv_curr_max_mem = $right_c;
              next;
            }
            if ($item =~ m/^state/) {
              if ( $csv_state !~ m/^na$/ ) {
                next;
              }
	      $csv_state = $right_c;
              next;
            }
            if ($item =~ m/^lpar_env/) {
              if ( $csv_lpar_env !~ m/^na$/ ) {
                next;
              }
	      $csv_lpar_env = $right_c;
              next;
            }
            if ($item =~ m/^os_version/) {
              if ( $csv_os_version !~ m/^na$/ ) {
                next;
              }
	      $csv_os_version = $right_c;
              next;
            }
  }

  # print the last lpar
  if ( $csv_lpar_name !~ m/^na$/ ) {
        print FHC "$host;$managedname;$csv_lpar_name;$pool_name;$cpu_mode1;$phys_cores;$logical;$cpu_mode2;$cpu_weight;$csv_lpar_id;$csv_default_profile;$csv_min_proc_units;$csv_desired_proc_units;$csv_max_proc_units;$csv_curr_min_proc_units;$csv_curr_max_proc_units;$csv_min_procs;$csv_desired_procs;$csv_max_procs;$csv_curr_min_procs;$csv_curr_max_procs;$csv_min_mem;$csv_desired_mem;$csv_max_mem;$csv_curr_min_mem;$csv_curr_mem;$csv_curr_max_mem;$csv_state;$csv_lpar_env;$csv_os_version\n";
  }

  close (FHC);
  close (FCFG);


  return 0;
}

# load favourites.cfg into mem (graph of lpars in favorites has to be created regulary)
sub  load_favourites { 
  my $fav_cfg = "$basedir/etc/favourites.cfg";
  my $indx = 0;

  if ( ! -f "$fav_cfg" ) {
    return 0;
  }

  open(FAV, "< $fav_cfg") || error(" Can't open $fav_cfg: $!".__FILE__.":".__LINE__) && return 0;
  foreach my $line (<FAV>) {
    chomp($line);
    if ( $line =~ m/^LPAR:/ ) {
      (my $trash, my $server, my $lpar, my $name) = split(/:/,$line);
      if ( $name eq '' ) {
        next; # use only valid favorites
      }
      $fav_server[$indx] = $server;
      $fav_lpar[$indx] = $lpar;
      #print "001 $server : $lpar\n";
      $indx++;
    }
  }
  close (FAV);
  return 1;
}


sub last_timestamp {
  my $wrkdir = shift;
  my $managedname = shift;
  my $host = shift;
  my $last_file = shift;
  my $source = shift;
  my $t = shift;
  my $last_rec_file = "";
  my $where= "file";

  if ( -f "$wrkdir/$managedname/$host/$last_file" ) {
    $where= "$last_file";
    # read timestamp of last record 
    # this is main loop how to get corectly timestamp of last record!!!
    open(FHLT, "< $wrkdir/$managedname/$host/$last_file") || error(" Can't open $wrkdir/$managedname/$host/$last_file: $!".__FILE__.":".__LINE__) && return 0;
    foreach my $line (<FHLT>) {
      chomp($line);
      $last_rec_file=$line ;
    }
    #print "last rec       : $host:$managedname $last_rec_file : $last_file\n";
    close(FHLT);
    $ret = substr($last_rec_file,0,1);
    if ($last_rec_file eq '' || $ret =~ /\D/) {
      # in case of an issue with last file, remove it and use default 140 min? for further run ...
      error ("Wrong input data, deleting file : $wrkdir/$managedname/$host/$last_file : $last_rec_file");
      unlink ("$wrkdir/$managedname/$host/$last_file");

      # place ther last 2h when an issue with last.txt
      $loadhours = 2;
      $loadmins  = 120;
      $last_rec = $t-7200;
    }
    else {
      $last_rec=str2time($last_rec_file);
      $loadhours = sprintf ("%.0f", ($t - $last_rec)/3600);
      $loadhours++;
      $loadmins  = sprintf ("%.0f", ($t - $last_rec)/60+5); # +5mins to be sure
    }
  }
  else {
    # old not accurate way how to get last time stamp, keeping it here as backup if above temp fails for any reason
    if ( -f "$wrkdir/$managedname/$host/$source.rr$type_sam" ) {
      $where="$source.rr$type_sam - $last_file";
      # find out last record in the db (hourly)
      RRDp::cmd qq(last "$wrkdir/$managedname/$host/$source.rr$type_sam" );
      my $last_rec_raw = RRDp::read;    
      chomp ($$last_rec_raw);
      $last_rec=$$last_rec_raw;
      $loadhours = sprintf ("%.0f", ($t - $last_rec)/3600);
      $loadhours++;
      $loadmins  = sprintf ("%.0f", ($t - $last_rec)/60+5); # +5mins to be sure
      (my $sec,$ivmmin,$ivmh,$ivmd,$ivmm,$ivmy,my $wday,my $yday,my $isdst) = localtime($last_rec);
      $ivmy += 1900;
      $ivmm += 1;
      if ( $loadmins  < 0 ) { # Do not know why, but it sometimes happens!!!
        if ( -f "$wrkdir/$managedname/$host/$source.rrd" ) {
          # find out last record in the db (hourly)
          RRDp::cmd qq(last "$wrkdir/$managedname/$host/$source.rrd" );
          my $last_rec_raw = RRDp::read;    
          chomp ($$last_rec_raw);
          $last_rec=$$last_rec_raw;
          $loadhours = sprintf ("%.0f", ($t - $last_rec)/3600);
          $loadhours++;
          $loadmins  = sprintf ("%.0f", ($t - $last_rec)/60+5); # +5mins to be sure
          error ("$last_file: ++2 $loadhours $last_rec -- $t");
        }
      }
    }
    else {
      if ( -f "$wrkdir/$managedname/$host/$source.rrd" ) {
        $where="$source.rrd - $last_file";
        # find out last record in the db (hourly)
        RRDp::cmd qq(last "$wrkdir/$managedname/$host/$source.rrd" );
        my $last_rec_raw = RRDp::read;    
        chomp ($$last_rec_raw);
        $last_rec=$$last_rec_raw;
        $loadhours = sprintf ("%.0f", ($t - $last_rec)/3600);
        $loadhours++;
        $loadmins  = sprintf ("%.0f", ($t - $last_rec)/60+5); # +5mins to be sure
      }
      else {
        $where="init - $last_file";
        # nothing does not exist --> initial load
        $loadhours = $INIT_LOAD_IN_HOURS_BACK;
        $loadmins  =  $loadhours * 60;
      }
    }
  }

  if ( $loadhours <= 0 || $loadmins <= 0) {	 # something wrong is here
    my $time_string = localtime($last_rec);
    error ("Last rec issue: $host:$managedname $last_file:  $loadhours - $loadmins -  $time_string ($last_rec) : where ($where) : $last_rec_file : 02");
    # place some reasonable defaults : -2 hours
    $loadhours = 2;
    $loadmins = 120;
    $last_rec = time();
    $last_rec = $last_rec - 7200;
  }

  ($sec,$ivmmin,$ivmh,$ivmd,$ivmm,$ivmy,$wday,$yday,$isdst) = localtime($last_rec);
  $ivmy += 1900;
  $ivmm += 1;

  $timerange = " --minutes ".$loadmins." "; # global variable, it must be changed here
  print "last rec 3     : $host:$managedname min:$loadmins , hour:$loadhours, $ivmm/$ivmd/$ivmy $ivmh:$ivmmin : $where - source:$source\n" if $DEBUG ;

  return 0;
}

# LPARs aggregated per a HMC
sub multiview_hmc
{
  my ($host,$name_part,$type,$type_sam,$act_time,$text,$xgrid) = @_;
  my $req_time = 0;
  my $file = "";
  my $i = 0;
  my $lpar = "";
  my $cmd = "";
  my $j = 0;
  my $name = "$webdir/$host/$name_part-$type"; # out graph file name

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "y" && -f "$name.png" ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $YEAR_REFRESH && $upgrade == 0 ) {
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "m" && -f "$name.png" && $upgrade == 0 ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $MONTH_REFRESH ) {
      return 0;
    }
  }

  # do not update charts if there is not new data in RRD DB
  if ( $type =~ "w" && -f "$name.png" && $upgrade == 0 ) {
    my $png_time = (stat("$name.png"))[9];
    if ( ($act_time - $png_time) < $WEEK_REFRESH ) {
      return 0;
    }
  }
  print "creating l_hmc : $host:$type - LPARs aggregated per a HMC\n" if $DEBUG ;

  my $header = "LPARs aggregated per $host: last $text";

  if ( ! -f "$name.png" ) {
    LoadDataModule::touch ("$host:$managedname $wrkdir");
  }

  if ( "$type" =~ "d" ) {
    $req_time = $act_time - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time - 2764800;
  }
  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d"  || "$type" =~ "y" ) {
    $req_time = $act_time - 31536000;
  }


  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$STEP";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"CPU cores\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " COMMENT:\\\"Utilization in CPU cores\\:    average  maximum\\l\\\"";


  my $gtype="AREA";

  foreach my $line (@managednamelist) {
    # goes through all servers of each HMC

    chomp($line);
    if ( $line !~ ".*,.*,.*" ){
      # it must look like : PWR6A-9117-MMA-SN103A4B0,9117-MMA,103A4B0, --> exclude banners
      next;
    }
    if ($line =~ /No results were found/) {
      print "HMC : $host does not contain any managed system\n" if $DEBUG ;
      return 0;
    }
    ($managedname, my $model, my $serial) = split(/,/,$line);

    if ( is_IP($managedname) == 1 ) {
      next; # wrong entry from the HMC, a problem of the HMC or unconfigured server yet
    }

    my $managed_ok=1;
    if ( $managed_systems_exclude ne '' ) {
      my @m_excl = split(/:/,$managed_systems_exclude);
      foreach my $managedname_exl (@m_excl) {
        chomp($managedname_exl);
        if ( $managedname =~ m/^$managedname_exl$/ ) {
          $managed_ok=0;
        }
      }
    }
    if ( $managed_ok == 0 ) {
      next;
    }


    opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host".__FILE__.":".__LINE__) && return 0;
    my @files = "";
    if ( $type_sam =~ "m" ) {
      my @files_unsorted = grep(/\.rrm$/,readdir(DIR));
      @files = sort { lc $a cmp lc $b } @files_unsorted;
    }
    if ( $type_sam =~ "h" ) {
      my @files_unsorted = grep(/\.rrh$/,readdir(DIR));
      @files = sort { lc $a cmp lc $b } @files_unsorted;
    }
    if ( $type_sam =~ "d" ) {
      my @files_unsorted = grep(/\.rrd$/,readdir(DIR));
      @files = sort { lc $a cmp lc $b } @files_unsorted;
      $req_time = $act_time - 31536000;
    }
    closedir(DIR);
  
    foreach $file (@files) {
      # goes through all lpars of each lpar
      chomp($file);
  
      if ($file =~ m/^\.rr/ ) {
        next; # avoid .rrm
      }

      # avoid old lpars which do not exist in the period
      my $rrd_upd_time = (stat("$wrkdir/$managedname/$host/$file"))[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }
  
      $lpar = $file;
      $lpar =~ s/.rrh//;
      $lpar =~ s/.rrm//;
      $lpar =~ s/.rrd//;
      my $lpar_space = $lpar;

      if ( $lpar eq '' ) {
        next; # avoid .rrm --> just to be sure :)
      }
  
      # add spaces to lpar name to have 25 chars total (for formating graph legend)
      $lpar_space =~ s/\&\&1/\//g;
      for(my $k = length($lpar_space); $k < 25; $k++) {
        $lpar_space .= " ";
      }
  
      # Exclude pools and memory
      if ($lpar =~ m/^cod$/ || $lpar =~ m/^pool$/ || $lpar =~ m/^mem$/ || $lpar =~  /^SharedPool[0-9]$/ || $lpar =~  m/^SharedPool[1-9][0-9]$/ || $lpar =~ m/^mem-pool$/) {
        next;
      }
   
      # Assure that lpar colors is same for each lpar through daily - yearly charts
      my $l;
      my $l_count = 0;
      my $found = -1;
      foreach $l (@keep_color_lpar) {
        if ( $l eq '' || $l =~ m/^ $/ ) {
          next;
        }
        if ( $l =~ m/^$lpar$/ ) {
          $found = $l_count;
          last;
        }
        $l_count++;
      }
      if ( $found == -1 ) {
        $keep_color_lpar[$l_count] = $lpar;
        $found = $l_count;
      }
  
      #print "$found $lpar $color[$found]\n";
      print "$wrkdir/$managedname/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);
  
      # bulid RRDTool cmd
      $cmd .= " DEF:cap${i}=\\\"$wrkdir/$managedname/$host/$file\\\":capped_cycles:AVERAGE";
      $cmd .= " DEF:uncap${i}=\\\"$wrkdir/$managedname/$host/$file\\\":uncapped_cycles:AVERAGE";
      $cmd .= " DEF:ent${i}=\\\"$wrkdir/$managedname/$host/$file\\\":entitled_cycles:AVERAGE";
      $cmd .= " DEF:cur${i}=\\\"$wrkdir/$managedname/$host/$file\\\":curr_proc_units:AVERAGE";
  
      $cmd .= " CDEF:tot${i}=cap${i},uncap${i},+";
      $cmd .= " CDEF:util${i}=tot${i},ent${i},/";
      $cmd .= " CDEF:utiltotu${i}=util${i},cur${i},*";
      $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";
      $cmd .= " $gtype:utiltot${i}$color[$found]:\\\"$lpar_space\\\"";
  
      # put carriage return after each second lpar in the legend
      if ($j == 1) {
        $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.2lf \\\"";
        $cmd .= " GPRINT:utiltot${i}:MAX:\\\" %3.2lf \\l\\\"";
        $j = 0;
      }
      else {
        $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.2lf \\\"";
        $cmd .= " GPRINT:utiltot${i}:MAX:\\\" %3.2lf \\l\\\"";
        #$cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.2lf \\t\\\"";
	  # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
        $j++
      }
      $gtype="STACK";
      $i++;
    }
  }
  if ( "$type" =~ "d" ) {
    if ($j == 1) {
      $cmd .= " COMMENT:\\\" \\l\\\"";
    }
    $cmd .= " COMMENT:\\\"\(Note that for CPU dedicated LPARs is always shown their whole entitlement\)\\\"";
  }
  $cmd .= " HRULE:0#000000";
  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  my  $FH;
  my $tmp_file="$tmpdir/multi-hmc-lpar-$host-$type.cmd";
  open(FH, "> $tmp_file") || error(" Can't open $tmp_file : $!".__FILE__.":".__LINE__) && return 0;
  print FH "$cmd\n";
  close (FH);
  
  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret  = `$rrdtool - < "$tmp_file" 2>&1`;
  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error ("$host : Multi graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      (my $err,  my $file, my $txt) = split(/'/,$ret);
      error ("Removing as it seems to be corrupted: $file");
      unlink("$file") || error("Cannot rm $file : $!".__FILE__.":".__LINE__) && return 0;
    }
    else {
      error ("$host: $cmd : Multi graph rrdtool error : $ret");
    }
  }
  #unlink ("$tmp_file"); --> do not remove!!! used for details

  if ( $run_topten == 1 || $upgrade == 1 ) {
    # also do it after the upgrade (or fresh instal what is upgrade in fact as well
    # create a copy for views, all views have to be snapped at the same moment
    copy("$name.png", "$name-$view_suff.png");
  } 

  return 0;
}

# return 1 if the argument is valid IP, otherwise 0
sub is_IP
{
  my $ip = shift;

  if( $ip =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/ && (($1<=255 && $2<=255 && $3<=255 && $4<=255 ))) {
     return 1;
  }
  else {
     return 0;
  }
}


sub isdigit
{
  my $digit = shift;
  my $text = shift;

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;

  if (length($digit_work) == 0) {
    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
}

